@@ -17,8 +17,8 @@
 
 import os
 import logging
-from urllib import basejoin
-from cStringIO import StringIO
+from urllib.parse import urljoin
+from io import BytesIO
 from collections import defaultdict
 from xml.etree import ElementTree as ET
 from copy import copy
@@ -27,7 +27,7 @@ import pkg_resources
 from tg import expose, redirect, flash, validate
 from tg.decorators import without_trailing_slash
 from tg import config as tg_config
-from pylons import request, app_globals as g, tmpl_context as c
+from tg import request, app_globals as g, tmpl_context as c
 from paste.deploy.converters import asbool, asint
 from bson import ObjectId
 from bson.errors import InvalidId
@@ -35,7 +35,7 @@ from formencode import validators as V
 from webob import exc
 from formencode import validators as fev
 
-from ming.orm import session
+from ming.odm import session
 from ming.utils import LazyProperty
 import ew.jinja2_ew as ew
 
@@ -47,13 +47,16 @@ from allura.lib.decorators import require_post, memoize
 from allura.lib.utils import permanent_redirect, ConfigProxy
 from allura import model as M
 from allura.tasks import index_tasks
+import six
+from io import BytesIO
+from allura.model.timeline import ActivityObject
 
 log = logging.getLogger(__name__)
 
 config = ConfigProxy(common_suffix='forgemail.domain')
 
 
-class ConfigOption(object):
+class ConfigOption:
 
     """Definition of a configuration option for an :class:`Application`.
 
@@ -88,7 +91,7 @@ class ConfigOption(object):
         return ew._Jinja2Widget().j2_attrs(self.extra_attrs or {})
 
 
-class SitemapEntry(object):
+class SitemapEntry:
 
     """A labeled URL, which may optionally have
     :class:`children <SitemapEntry>`.
@@ -98,19 +101,27 @@ class SitemapEntry(object):
     """
 
     def __init__(self, label, url=None, children=None, className=None,
-                 ui_icon=None, small=None, tool_name=None, matching_urls=None, extra_html_attrs=None):
-        """Create a new SitemapEntry.
+                 ui_icon=None, small=None, tool_name=None, matching_urls=None, extra_html_attrs=None, mount_point=None):
+        """
+        Create a new SitemapEntry.
 
+        :param label: the name
+        :param url: the url
+        :param children: optional, list of SitemapEntry objects
+        :param className: optional, HTML class
+        :param tool_name: optional, tool_name (used for top-level menu items)
+        :param matching_urls: list of urls to consider "active" in menu display
+        :param extra_html_attrs: dict to show as HTML attributes
+        :param mount_point: used only for tracking project menu admin options
         """
         self.label = label
         self.className = className
-        if url is not None:
-            url = url.encode('utf-8')
         self.url = url
         self.small = small
         self.ui_icon = ui_icon
         self.children = children or []
         self.tool_name = tool_name
+        self.mount_point = mount_point
         self.matching_urls = matching_urls or []
         self.extra_html_attrs = extra_html_attrs or {}
 
@@ -154,14 +165,17 @@ class SitemapEntry(object):
         if callable(lbl):
             lbl = lbl(app)
         if url is not None:
-            url = basejoin(app.url, url)
-        return SitemapEntry(lbl, url, [
-            ch.bind_app(app) for ch in self.children],
-            className=self.className,
-            ui_icon=self.ui_icon,
-            small=self.small,
-            tool_name=self.tool_name,
-            matching_urls=self.matching_urls)
+            url = urljoin(app.url, url)
+        return SitemapEntry(lbl, url,
+                            [ch.bind_app(app) for ch in self.children],
+                            className=self.className,
+                            ui_icon=self.ui_icon,
+                            small=self.small,
+                            tool_name=self.tool_name,
+                            matching_urls=self.matching_urls,
+                            mount_point=app.config.options.mount_point,
+                            extra_html_attrs=self.extra_html_attrs,
+                            )
 
     def extend(self, sitemap_entries):
         """Extend our children with ``sitemap_entries``.
@@ -173,8 +187,8 @@ class SitemapEntry(object):
         children or our copy with the children of the new copy.
 
         """
-        child_index = dict(
-            (ch.label, ch) for ch in self.children)
+        child_index = {
+            ch.label: ch for ch in self.children}
         for e in sitemap_entries:
             lbl = e.label
             match = child_index.get(e.label)
@@ -205,7 +219,7 @@ class SitemapEntry(object):
         )
 
 
-class Application(object):
+class Application(ActivityObject):
 
     """
     The base Allura pluggable application
@@ -271,7 +285,7 @@ class Application(object):
     permissions_desc = {
         'unmoderated_post': 'Post comments without moderation.',
         'post': 'Post comments, subject to moderation.',
-        'moderate': 'Moderate comments.',
+        'moderate': 'Approve and edit all comments.',
         'configure': 'Set label and options. Requires admin permission.',
         'admin': 'Set permissions.',
     }
@@ -347,7 +361,7 @@ class Application(object):
         """
         if self.config.options.get('AllowEmailPosting', True):
             parts = list(reversed(self.url[1:-1].split('/')))
-            return '%s@%s%s' % (parts[0], '.'.join(parts[1:]), config.common_suffix)
+            return '{}@{}{}'.format(parts[0], '.'.join(parts[1:]), config.common_suffix)
         else:
             return tg_config.get('forgemail.return_path')
 
@@ -398,6 +412,8 @@ class Application(object):
 
     @classmethod
     def _installable(cls, tool_name, nbhd, project_tools):
+        if cls.installable is False:  # handle class level `installable = False` declarations
+            return False
         if tool_name.lower() in nbhd.get_prohibited_tools():
             return False
         tools_list = [tool.tool_name.lower() for tool in project_tools]
@@ -504,6 +520,17 @@ class Application(object):
                 project_id=self.project._id,
                 app_config_id=self.config._id)
 
+    def unsubscribe(self, user):
+        """Unsubscribe :class:`user <allura.model.auth.User>` to the
+        :class:`allura.model.notification.Mailbox` for this Application.
+
+        """
+        if user and user != model.User.anonymous():
+            model.Mailbox.unsubscribe(
+                user_id=user._id,
+                project_id=self.project._id,
+                app_config_id=self.config._id)
+
     @classmethod
     def default_options(cls):
         """Return a ``(name, default value)`` mapping of this Application's
@@ -512,9 +539,9 @@ class Application(object):
         :rtype: dict
 
         """
-        return dict(
-            (co.name, co.default)
-            for co in cls.config_options)
+        return {
+            co.name: co.default
+            for co in cls.config_options}
 
     @classmethod
     def options_on_install(cls):
@@ -545,9 +572,7 @@ class Application(object):
         # De-index all the artifacts belonging to this tool in one fell swoop
         index_tasks.solr_del_tool.post(project_id, self.config.options['mount_point'])
 
-        for d in model.Discussion.query.find({
-                'project_id': project_id,
-                'app_config_id': self.config._id}):
+        for d in model.Discussion.query.find({'app_config_id': self.config._id}):
             d.delete()
         self.config.delete()
         session(self.config).flush()
@@ -578,6 +603,15 @@ class Application(object):
             sitemap_without_children.append(sm_copy)
         return sitemap_without_children
 
+    def sitemap_xml(self):
+        """Return a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
+        to add to the sitemap.xml for this Application.
+
+        Default implementation returns the contents of :attr:`main_menu`
+        :return:
+        """
+        return self.main_menu()
+
     def sidebar_menu(self):
         """Return a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
         to render in the left sidebar for this Application.
@@ -600,7 +634,7 @@ class Application(object):
         :return: a list of :class:`WebhookSender <allura.webhooks.WebhookSender>`
         """
         tool_name = self.config.tool_name.lower()
-        webhooks = [w for w in g.entry_points['webhooks'].itervalues()
+        webhooks = [w for w in g.entry_points['webhooks'].values()
                     if tool_name in w.triggered_by]
         return webhooks
 
@@ -643,7 +677,7 @@ class Application(object):
     def admin_menu_collapse_button(self):
         """Returns button for showing/hiding admin sidebar menu"""
         return SitemapEntry(
-            label=u'Admin - {}'.format(self.config.options.mount_label),
+            label=f'Admin - {self.config.options.mount_label}',
             extra_html_attrs={
                 'id': 'sidebar-admin-menu-trigger',
             })
@@ -652,7 +686,7 @@ class Application(object):
     def admin_menu_delete_button(self):
         """Returns button for deleting an app if app can be deleted"""
         anchored_tools = self.project.neighborhood.get_anchored_tools()
-        anchored = self.tool_label.lower() in anchored_tools.keys()
+        anchored = self.tool_label.lower() in list(anchored_tools.keys())
         if self.uninstallable and not anchored:
             return SitemapEntry(
                 label='Delete Everything',
@@ -693,7 +727,7 @@ class Application(object):
         if message.get('filename'):
             # Special case - the actual post may not have been created yet
             log.info('Saving attachment %s', message['filename'])
-            fp = StringIO(message['payload'])
+            fp = BytesIO(six.ensure_binary(message['payload']))
             self.AttachmentClass.save_attachment(
                 message['filename'], fp,
                 content_type=message.get(
@@ -710,11 +744,7 @@ class Application(object):
                 'Existing message_id %s found - saving this as text attachment' %
                 message_id)
 
-            try:
-                fp = StringIO(message['payload'].encode('utf-8'))
-            except UnicodeDecodeError:
-                fp = StringIO(message['payload'])
-
+            fp = BytesIO(six.ensure_binary(message['payload']))
             post.attach(
                 'alternate', fp,
                 content_type=message.get(
@@ -723,7 +753,7 @@ class Application(object):
                 thread_id=thd._id,
                 post_id=message_id)
         else:
-            text = message['payload'] or '--no text body--'
+            text = six.ensure_text(message['payload']) or '--no text body--'
             post = thd.post(
                 message_id=message_id,
                 parent_id=parent_id,
@@ -755,15 +785,15 @@ class Application(object):
 
         Returns dict that will be included in project's API under tools key.
         """
-        return {
+        json = {
             'name': self.config.tool_name,
             'mount_point': self.config.options.mount_point,
-            'url': self.config.url(),
-            'icons': self.icons,
-            'installable': self.installable,
-            'tool_label': self.tool_label,
+            'url': h.absurl(self.config.url()),
             'mount_label': self.config.options.mount_label
         }
+        if self.api_root:
+            json['api_url'] = h.absurl('/rest' + self.config.url())
+        return json
 
     def get_attachment_export_path(self, path='', *args):
         return os.path.join(path, self.config.options.mount_point, *args)
@@ -782,8 +812,27 @@ class Application(object):
             with open(attachment_path.encode('utf8', 'replace'), 'wb') as fl:
                 fl.write(attachment.rfile().read())
 
+    def default_redirect(self):
+        """Redirect to url if first tool in a project. This method raises a
+        redirect exception.
 
-class AdminControllerMixin(object):
+        """
+        return None
+
+    @property
+    def activity_name(self):
+        return self.config.options.mount_label
+
+    @property
+    def activity_url(self):
+        return self.url
+
+    @property
+    def activity_extras(self):
+        return {}
+
+
+class AdminControllerMixin:
     """Provides common functionality admin controllers need"""
     def _before(self, *remainder, **params):
         # Display app's sidebar on admin page, instead of :class:`AdminApp`'s
@@ -809,7 +858,7 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
         """Instantiate this controller for an :class:`app <Application>`.
 
         """
-        super(DefaultAdminController, self).__init__()
+        super().__init__()
         self.app = app
         self.webhooks = WebhooksLookup(app)
 
@@ -830,10 +879,15 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
         user = model.User.by_username(username)
         if not user:
             return dict(error='User "%s" not found' % username)
-        ace = model.ACE.deny(
-            model.ProjectRole.by_user(user, upsert=True)._id, perm, reason)
+        ace = model.ACE.deny(model.ProjectRole.by_user(user, upsert=True)._id, perm, reason)
         if not model.ACL.contains(ace, self.app.acl):
             self.app.acl.append(ace)
+            model.AuditLog.log('{}: blocked user "{}" from permission "{}" for reason: "{}"'.format(
+                self.app.config.options['mount_point'],
+                username,
+                ace.permission,
+                reason,
+            ))
             return dict(user_id=str(user._id), username=user.username, reason=reason)
         return dict(error='User "%s" already blocked' % user.username)
 
@@ -843,7 +897,7 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
     @require_post()
     def unblock_user(self, user_id=None, perm=None, **kw):
         try:
-            user_id = map(ObjectId, user_id)
+            user_id = list(map(ObjectId, user_id))
         except InvalidId:
             user_id = []
         users = model.User.query.find({'_id': {'$in': user_id}}).all()
@@ -856,6 +910,11 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
             if ace:
                 self.app.acl.remove(ace)
                 unblocked.append(str(user._id))
+                model.AuditLog.log('{}: unblocked user "{}" from permission "{}"'.format(
+                    self.app.config.options['mount_point'],
+                    user.username,
+                    ace.permission,
+                ))
         return dict(unblocked=unblocked)
 
     @expose('jinja:allura:templates/app_admin_permissions.html')
@@ -864,11 +923,11 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
         """Render the permissions management web page.
 
         """
-        from ext.admin.widgets import PermissionCard, BlockUser, BlockList
+        from .ext.admin.widgets import PermissionCard, BlockUser, BlockList
         c.card = PermissionCard()
         c.block_user = BlockUser()
         c.block_list = BlockList()
-        permissions = dict((p, []) for p in self.app.permissions)
+        permissions = {p: [] for p in self.app.permissions}
         block_list = defaultdict(list)
         for ace in self.app.config.acl:
             if ace.access == model.ACE.ALLOW:
@@ -879,7 +938,7 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
                     pass
             elif ace.access == model.ACE.DENY:
                 role = model.ProjectRole.query.get(_id=ace.role_id)
-                if role.name is None and role.user:
+                if role and role.name is None and role.user:
                     block_list[ace.permission].append((role.user, ace.reason))
         return dict(
             app=self.app,
@@ -903,9 +962,14 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
 
         """
         require_access(self.app, 'configure')
+        old_label = self.app.config.options['mount_label']
         self.app.config.options['mount_label'] = mount_label
+        model.AuditLog.log('updated label: "{}" => "{}" for {}'.format(
+            old_label,
+            mount_label,
+            self.app.config.options['mount_point']))
         g.post_event('project_menu_updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose('jinja:allura:templates/app_admin_options.html')
     def options(self):
@@ -948,8 +1012,15 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
                 try:
                     val = opt.validate(val)
                 except fev.Invalid as e:
-                    flash(u'{}: {}'.format(opt.name, str(e)), 'error')
+                    flash(f'{opt.name}: {str(e)}', 'error')
                     continue
+                if self.app.config.options[opt.name] != val:
+                    M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                        self.app.config.options['mount_point'],
+                        opt.name,
+                        self.app.config.options[opt.name],
+                        val
+                    ))
                 self.app.config.options[opt.name] = val
             if is_admin:
                 # possibly moving admin mount point
@@ -960,7 +1031,7 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
                          + self.app.config.options.mount_point
                          + '/')
             else:
-                redirect(request.referer)
+                redirect(six.ensure_text(request.referer or '/'))
 
     @without_trailing_slash
     @expose()
@@ -977,9 +1048,9 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
             new_group_ids = args.get('new', [])
             del_group_ids = []
             group_ids = args.get('value', [])
-            if isinstance(new_group_ids, basestring):
+            if isinstance(new_group_ids, str):
                 new_group_ids = [new_group_ids]
-            if isinstance(group_ids, basestring):
+            if isinstance(group_ids, str):
                 group_ids = [group_ids]
 
             for acl in old_acl:
@@ -990,21 +1061,21 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
 
             def get_role(_id):
                 return model.ProjectRole.query.get(_id=ObjectId(_id))
-            groups = map(get_role, group_ids)
-            new_groups = map(get_role, new_group_ids)
-            del_groups = map(get_role, del_group_ids)
+            groups = list(map(get_role, group_ids))
+            new_groups = list(map(get_role, new_group_ids))
+            del_groups = list(map(get_role, del_group_ids))
 
             def group_names(groups):
                 return ', '.join((role.name or '<Unnamed>') for role in groups if role)
 
             if new_groups or del_groups:
-                model.AuditLog.log('updated "%s" permission: "%s" => "%s" for %s' % (
+                model.AuditLog.log('updated "{}" permission: "{}" => "{}" for {}'.format(
                     perm,
                     group_names(groups + del_groups),
                     group_names(groups + new_groups),
                     self.app.config.options['mount_point']))
 
-            role_ids = map(ObjectId, group_ids + new_group_ids)
+            role_ids = list(map(ObjectId, group_ids + new_group_ids))
             self.app.config.acl += [
                 model.ACE.allow(r, perm) for r in role_ids]
 
@@ -1012,13 +1083,14 @@ class DefaultAdminController(BaseController, AdminControllerMixin):
             for ace in old_acl:
                 if (ace.permission == perm) and (ace.access == model.ACE.DENY):
                     self.app.config.acl.append(ace)
-        redirect(request.referer)
+        g.post_event('project_menu_updated')  # since 'read' permission changes can affect what is visible in menu
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class WebhooksLookup(BaseController, AdminControllerMixin):
 
     def __init__(self, app):
-        super(WebhooksLookup, self).__init__()
+        super().__init__()
         self.app = app
 
     @without_trailing_slash
@@ -1042,4 +1114,4 @@ class WebhooksLookup(BaseController, AdminControllerMixin):
         for hook in self.app._webhooks:
             if hook.type == name and hook.controller:
                 return hook.controller(hook, self.app), remainder
-        raise exc.HTTPNotFound, name
+        raise exc.HTTPNotFound(name)
