@@ -21,17 +21,18 @@ from contextlib import contextmanager
 from itertools import groupby
 
 from paste.deploy.converters import asbool
-from pylons import tmpl_context as c, app_globals as g
+from tg import tmpl_context as c, app_globals as g
 from pymongo.errors import DuplicateKeyError, InvalidDocument, OperationFailure
 
-from ming.orm import mapper, session, Mapper
-from ming.orm.declarative import MappedClass
+from ming.odm import mapper, session, Mapper
+from ming.odm.declarative import MappedClass
 
 from allura.tasks.index_tasks import add_artifacts
 from allura.lib.exceptions import CompoundError
 from allura.lib import helpers as h
 from allura.lib import utils
 from . import base
+import six
 
 
 class ShowModelsCommand(base.Command):
@@ -46,14 +47,15 @@ class ShowModelsCommand(base.Command):
         graph = build_model_inheritance_graph()
         for depth, cls in dfs(MappedClass, graph):
             for line in dump_cls(depth, cls):
-                print line
+                print(line)
 
 
 class ReindexCommand(base.Command):
     min_args = 1
     max_args = 1
     usage = '<ini file>'
-    summary = 'Reindex into solr and re-shortlink all artifacts'
+    summary = 'Reindex into solr and re-shortlink all artifacts\n' \
+              'By default both --refs and --solr are enabled, but if you specify just one, the other will be disabled'
     parser = base.Command.standard_parser(verbose=True)
     parser.add_option('-p', '--project', dest='project',  default=None,
                       help='project to reindex')
@@ -127,7 +129,7 @@ class ReindexCommand(base.Command):
                             try:
                                 M.ArtifactReference.from_artifact(a)
                                 M.Shortlink.from_artifact(a)
-                            except:
+                            except Exception:
                                 base.log.exception(
                                     'Making ArtifactReference/Shortlink from %s', a)
                                 continue
@@ -136,7 +138,7 @@ class ReindexCommand(base.Command):
                     M.artifact_orm_session.clear()
                     try:
                         self._chunked_add_artifacts(ref_ids)
-                    except CompoundError, err:
+                    except CompoundError as err:
                         base.log.exception(
                             'Error indexing artifacts:\n%r', err)
                         base.log.error('%s', err.format_error())
@@ -211,7 +213,7 @@ class EnsureIndexCommand(base.Command):
     def command(self):
         from allura import model as M
         main_session_classes = [M.main_orm_session, M.repository_orm_session,
-                                M.task_orm_session]
+                                M.task_orm_session, M.main_explicitflush_orm_session]
         if asbool(self.config.get('activitystream.recording.enabled', False)):
             from activitystream.storage.mingstorage import activity_odm_session
             main_session_classes.append(activity_odm_session)
@@ -234,33 +236,33 @@ class EnsureIndexCommand(base.Command):
                 idx = project_indexes[cname]
             idx.extend(mgr.indexes)
         base.log.info('Updating indexes for main DB')
-        for odm_session, db_indexes in main_indexes.iteritems():
+        for odm_session, db_indexes in main_indexes.items():
             db = odm_session.impl.db
-            for name, indexes in db_indexes.iteritems():
+            for name, indexes in db_indexes.items():
                 self._update_indexes(db[name], indexes)
         base.log.info('Updating indexes for project DB')
         db = M.project_doc_session.db
         base.log.info('... DB: %s', db)
-        for name, indexes in project_indexes.iteritems():
+        for name, indexes in project_indexes.items():
             self._update_indexes(db[name], indexes)
         base.log.info('Done updating indexes')
 
     def _update_indexes(self, collection, indexes):
-        uindexes = dict(
+        uindexes = {
             # convert list to tuple so it's hashable for 'set'
-            (tuple(i.index_spec), i)
+            tuple(i.index_spec): i
             for i in indexes
-            if i.unique)
-        indexes = dict(
-            (tuple(i.index_spec), i)
+            if i.unique}
+        indexes = {
+            tuple(i.index_spec): i
             for i in indexes
-            if not i.unique)
+            if not i.unique}
         prev_indexes = {}
         prev_uindexes = {}
         unique_flag_drop = {}
         unique_flag_add = {}
         try:
-            existing_indexes = collection.index_information().iteritems()
+            existing_indexes = collection.index_information().items()
         except OperationFailure:
             # exception is raised if db or collection doesn't exist yet
             existing_indexes = {}
@@ -279,13 +281,13 @@ class EnsureIndexCommand(base.Command):
                 else:
                     prev_indexes[iname] = keys
 
-        for iname, keys in unique_flag_drop.iteritems():
+        for iname, keys in unique_flag_drop.items():
             self._recreate_index(collection, iname, list(keys), unique=False)
-        for iname, keys in unique_flag_add.iteritems():
+        for iname, keys in unique_flag_add.items():
             self._recreate_index(collection, iname, list(keys), unique=True)
 
         # Ensure all indexes
-        for keys, idx in uindexes.iteritems():
+        for keys, idx in uindexes.items():
             base.log.info('...... ensure %s:%s', collection.name, idx)
             while True:
                 try:
@@ -297,14 +299,14 @@ class EnsureIndexCommand(base.Command):
                         del index_options['unique']
                     collection.ensure_index(idx.index_spec, **index_options)
                     break
-                except DuplicateKeyError, err:
+                except DuplicateKeyError as err:
                     base.log.info('Found dupe key(%s), eliminating dupes', err)
                     self._remove_dupes(collection, idx.index_spec)
-        for keys, idx in indexes.iteritems():
+        for keys, idx in indexes.items():
             base.log.info('...... ensure %s:%s', collection.name, idx)
             collection.ensure_index(idx.index_spec, background=True, **idx.index_options)
         # Drop obsolete indexes
-        for iname, keys in prev_indexes.iteritems():
+        for iname, keys in prev_indexes.items():
             if keys not in indexes:
                 if self.options.clean:
                     base.log.info('...... drop index %s:%s', collection.name, iname)
@@ -312,7 +314,7 @@ class EnsureIndexCommand(base.Command):
                 else:
                     base.log.info('...... potentially unneeded index, could be removed by running with --clean %s:%s',
                                   collection.name, iname)
-        for iname, keys in prev_uindexes.iteritems():
+        for iname, keys in prev_uindexes.items():
             if keys not in uindexes:
                 if self.options.clean:
                     base.log.info('...... drop index %s:%s', collection.name, iname)
@@ -340,7 +342,7 @@ class EnsureIndexCommand(base.Command):
     def _remove_dupes(self, collection, spec):
         iname = collection.create_index(spec)
         fields = [f[0] for f in spec]
-        q = collection.find({}, fields=fields).sort(spec)
+        q = collection.find({}, projection=fields).sort(spec)
 
         def keyfunc(doc):
             return tuple(doc.get(f, None) for f in fields)
@@ -351,12 +353,12 @@ class EnsureIndexCommand(base.Command):
                 base.log.info('Found dupes with %s', key)
                 dupes += [doc['_id'] for doc in docs[1:]]
         collection.drop_index(iname)
-        collection.remove(dict(_id={'$in': dupes}))
+        collection.delete_many(dict(_id={'$in': dupes}))
 
 
 def build_model_inheritance_graph():
-    graph = dict((m.mapped_class, ([], [])) for m in Mapper.all_mappers())
-    for cls, (parents, children) in graph.iteritems():
+    graph = {m.mapped_class: ([], []) for m in Mapper.all_mappers()}
+    for cls, (parents, children) in graph.items():
         for b in cls.__bases__:
             if b not in graph:
                 continue
@@ -367,7 +369,7 @@ def build_model_inheritance_graph():
 
 def dump_cls(depth, cls):
     indent = ' ' * 4 * depth
-    yield indent + '%s.%s' % (cls.__module__, cls.__name__)
+    yield indent + f'{cls.__module__}.{cls.__name__}'
     m = mapper(cls)
     for p in m.properties:
         s = indent * 2 + ' - ' + str(p)
@@ -379,27 +381,4 @@ def dump_cls(depth, cls):
 def dfs(root, graph, depth=0):
     yield depth, root
     for node in graph[root][1]:
-        for r in dfs(node, graph, depth + 1):
-            yield r
-
-
-def pm(etype, value, tb):  # pragma no cover
-    import pdb
-    import traceback
-    try:
-        from IPython.ipapi import make_session
-        make_session()
-        from IPython.Debugger import Pdb
-        sys.stderr.write('Entering post-mortem IPDB shell\n')
-        p = Pdb(color_scheme='Linux')
-        p.reset()
-        p.setup(None, tb)
-        p.print_stack_trace()
-        sys.stderr.write('%s: %s\n' % (etype, value))
-        p.cmdloop()
-        p.forget()
-        # p.interaction(None, tb)
-    except ImportError:
-        sys.stderr.write('Entering post-mortem PDB shell\n')
-        traceback.print_exception(etype, value, tb)
-        pdb.post_mortem(tb)
+        yield from dfs(node, graph, depth + 1)
