@@ -17,30 +17,36 @@
 
 import logging
 import os
-from datetime import datetime, timedelta
+from base64 import b32encode
+from datetime import datetime
 import re
-from urlparse import urlparse, urljoin
+import warnings
+from urllib.parse import urlparse, urljoin
 
 import bson
+import formencode as fe
 import tg
 from tg import expose, flash, redirect, validate, config, session
 from tg.decorators import with_trailing_slash, without_trailing_slash
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request, response
+from tg import tmpl_context as c, app_globals as g
+from tg import request, response
 from webob import exc as wexc
 from paste.deploy.converters import asbool
-from cryptography.hazmat.primitives.twofactor import InvalidToken
+with warnings.catch_warnings():  # ignore py2 CryptographyDeprecationWarning
+    warnings.filterwarnings('ignore')
+    from cryptography.hazmat.primitives.twofactor import InvalidToken
 from beaker.session import _session_id
 
 import allura.tasks.repo_tasks
 from allura import model as M
-from allura.lib import validators as V
-from allura.lib.security import require_authenticated, has_access
+from allura.lib.security import require_authenticated, has_access, is_site_admin
 from allura.lib import helpers as h
 from allura.lib import plugin
+from allura.lib import validators as V
 from allura.lib.decorators import require_post, reconfirm_auth
 from allura.lib.exceptions import InvalidRecoveryCode, MultifactorRateLimitError
 from allura.lib.repository import RepositoryApp
+from allura.lib.security import HIBPClient, HIBPCompromisedCredentials, HIBPClientError
 from allura.lib.widgets import (
     SubscriptionForm,
     OAuthApplicationForm,
@@ -54,11 +60,12 @@ from allura.lib.multifactor import TotpService, RecoveryCodeService
 from allura.lib import utils
 from allura.controllers import BaseController
 from allura.tasks.mail_tasks import send_system_mail_to_user
+import six
 
 log = logging.getLogger(__name__)
 
 
-class F(object):
+class F:
     login_form = LoginForm()
     password_change_form = forms.PasswordChangeForm(action='/auth/preferences/change_password')
     upload_key_form = forms.UploadKeyForm(action='/auth/preferences/upload_sshkey')
@@ -85,6 +92,19 @@ class F(object):
     disable_account_form = DisableAccountForm()
 
 
+def enforce_hibp_password_check(provider, password, failure_redirect_url):
+    if provider.hibp_password_check_enabled():
+        try:
+            HIBPClient.check_breached_password(password)
+
+        except HIBPClientError as ex:
+            log.error("Error invoking HIBP API", exc_info=ex)
+
+        except HIBPCompromisedCredentials:
+            flash('Unsafe Password - Please use a strong, unique password', 'error')
+            redirect(failure_redirect_url)
+
+
 class AuthController(BaseController):
 
     def __init__(self):
@@ -96,9 +116,11 @@ class AuthController(BaseController):
             self.disable = DisableAccountController()
 
     def __getattr__(self, name):
+        if name == 'decoration':
+            raise AttributeError("TG decoratedcontroller calls this during import time, can't do anything complex")
         urls = plugin.UserPreferencesProvider.get().additional_urls()
         if name not in urls:
-            raise AttributeError("'%s' object has no attribute '%s'" % (type(self).__name__, name))
+            raise AttributeError(f"'{type(self).__name__}' object has no attribute '{name}'")
         return urls[name]
 
     @expose()
@@ -112,13 +134,18 @@ class AuthController(BaseController):
     @expose('jinja:allura:templates/login.html')
     @with_trailing_slash
     def index(self, *args, **kwargs):
-        orig_request = request.environ.get('pylons.original_request', None)
+        orig_request = request.environ.get('tg.original_request', None)
         if 'return_to' in kwargs:
             return_to = kwargs.pop('return_to')
         elif orig_request:
             return_to = orig_request.url
         else:
-            return_to = request.referer
+            if request.referer is not None and six.ensure_text(request.referer).split('/')[-1] == 'neighborhood':
+                return_to = '/'
+            elif request.referer:
+                return_to = six.ensure_text(request.referer)
+            else:
+                return_to = None
         c.form = F.login_form
         return dict(return_to=return_to)
 
@@ -140,15 +167,15 @@ class AuthController(BaseController):
         user_record = M.User.query.find(
             {'tool_data.AuthPasswordReset.hash': hash}).first()
         if not user_record:
-            log.info('Reset hash not found: {}'.format(hash))
-            flash('Unable to process reset, please try again')
-            redirect(login_url)
+            log.info(f'Reset hash not found: {hash}')
+            flash('Unable to process password reset', 'error', sticky=True)
+            redirect('/')
         hash_expiry = user_record.get_tool_data(
             'AuthPasswordReset', 'hash_expiry')
         if not hash_expiry or hash_expiry < datetime.utcnow():
-            log.info('Reset hash expired: {} {}'.format(hash, hash_expiry))
-            flash('Unable to process reset, please try again')
-            redirect(login_url)
+            log.info(f'Reset hash expired: {hash} {hash_expiry}')
+            flash('Password reset link is invalid or expired', 'error', sticky=True)
+            redirect('/')
         return user_record
 
     @expose('jinja:allura:templates/forgotten_password.html')
@@ -172,9 +199,17 @@ class AuthController(BaseController):
         if not provider.forgotten_password_process:
             raise wexc.HTTPNotFound()
         user = self._validate_hash(hash)
+        enforce_hibp_password_check(provider, pw, f'/auth/forgotten_password/{hash}')
+
         user.set_password(pw)
         user.set_tool_data('AuthPasswordReset', hash='', hash_expiry='')  # Clear password reset token
+        user.set_tool_data('allura', pwd_reset_preserve_session=session.id)
         h.auditlog_user('Password changed (through recovery process)', user=user)
+        email_body = g.jinja2_env.get_template('allura:templates/mail/password_changed.md').render(dict(
+            user=user,
+            config=config
+        ))
+        send_system_mail_to_user(user, 'Password Changed', email_body)
         flash('Password changed')
         redirect('/auth/?return_to=/')  # otherwise the default return_to would be the forgotten_password referrer page
 
@@ -187,38 +222,36 @@ class AuthController(BaseController):
         if not email:
             redirect('/')
 
-        user_record = M.User.by_email_address(email)
+        user_record = M.User.by_email_address(email, only_confirmed=False)
         allow_non_primary_email_reset = asbool(config.get('auth.allow_non_primary_email_password_reset', True))
 
         if not re.match(r"[^@]+@[^@]+\.[^@]+", email):
-            flash('Enter email in correct format!','error')
+            flash('Enter email in correct format!', 'error')
             redirect('/auth/forgotten_password')
 
-        if not allow_non_primary_email_reset:
+        if user_record and user_record.pending:
+            message = 'If the given email address is on record, '\
+                      'an email has been sent to the account\'s primary email address.'
+            email_record = M.EmailAddress.get(email=provider.get_primary_email_address(user_record=user_record),
+                                              confirmed=False)
+            provider.resend_verification_link(user_record, email_record)
+
+        elif not allow_non_primary_email_reset:
             message = 'If the given email address is on record, '\
                       'a password reset email has been sent to the account\'s primary email address.'
             email_record = M.EmailAddress.get(email=provider.get_primary_email_address(user_record=user_record),
-                                                    confirmed=True)
+                                              confirmed=True)
         else:
             message = 'A password reset email has been sent, if the given email address is on record in our system.'
             email_record = M.EmailAddress.get(email=email, confirmed=True)
 
         if user_record and email_record and email_record.confirmed:
-            hash = h.nonce(42)
-            user_record.set_tool_data('AuthPasswordReset',
-                                      hash=hash,
-                                      hash_expiry=datetime.utcnow() +
-                                      timedelta(seconds=int(config.get('auth.recovery_hash_expiry_period', 600))))
-
-            log.info('Sending password recovery link to %s', email_record.email)
-            subject = '%s Password recovery' % config['site_name']
-            text = g.jinja2_env.get_template('allura:templates/mail/forgot_password.txt').render(dict(
-                user=user_record,
-                config=config,
-                hash=hash,
-            ))
-            send_system_mail_to_user(email_record.email, subject, text)
-        h.auditlog_user('Password recovery link sent to: %s', email, user=user_record)
+            user_record.send_password_reset_email(email_record.email)
+            h.auditlog_user('Password recovery link sent to: %s', email_record.email, user=user_record)
+        elif is_site_admin(c.user):
+            # this can be accessed via a site admin page, and sometimes email records are inconsistent
+            # only site admins may be told if accounts exist or not
+            message = 'Could NOT find email address for user'
         flash(message)
         redirect('/')
 
@@ -251,7 +284,7 @@ class AuthController(BaseController):
     def send_verification_link(self, a):
         addr = M.EmailAddress.get(email=a, claimed_by_user_id=c.user._id)
         confirmed_emails = M.EmailAddress.find(dict(email=a, confirmed=True)).all()
-        confirmed_emails = filter(lambda item: item != addr, confirmed_emails)
+        confirmed_emails = [item for item in confirmed_emails if item != addr]
 
         if addr:
             if any(email.confirmed for email in confirmed_emails):
@@ -261,11 +294,11 @@ class AuthController(BaseController):
             flash('Verification link sent')
         else:
             flash('No such address', 'error')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     def _verify_addr(self, addr, do_auth_check=True):
         confirmed_by_other = M.EmailAddress.find(dict(email=addr.email, confirmed=True)).all() if addr else []
-        confirmed_by_other = filter(lambda item: item != addr, confirmed_by_other)
+        confirmed_by_other = [item for item in confirmed_by_other if item != addr]
 
         if addr and not confirmed_by_other:
             user = addr.claimed_by_user(include_pending=True)
@@ -282,16 +315,26 @@ class AuthController(BaseController):
                         return_to += '?' + request.environ['QUERY_STRING']
                     redirect(login_url, {'return_to': return_to})
 
+            if do_auth_check:
+                # don't send email when do_auth_check=False (e.g. admin panel move)
+                email_body = g.jinja2_env.get_template('allura:templates/mail/email_added.md').render(dict(
+                    user=user,
+                    config=config,
+                    addr=addr.email
+                ))
+                send_system_mail_to_user(user, 'New Email Address Added', email_body)
+
             addr.confirmed = True
             flash('Email address confirmed')
             h.auditlog_user('Email address verified: %s',  addr.email, user=user)
-            if(user.get_pref('email_address') == None):
+            if user.get_pref('email_address') is None:
                 user.set_pref('email_address', addr.email)
             if user.pending:
                 plugin.AuthenticationProvider.get(request).activate_user(user)
                 projectname = plugin.AuthenticationProvider.get(request).user_project_shortname(user)
                 n = M.Neighborhood.query.get(name='Users')
                 n.register_project(projectname, user=user, user_project=True)
+
         else:
             flash('Unknown verification link', 'error')
 
@@ -309,7 +352,7 @@ class AuthController(BaseController):
     @staticmethod
     def _verify_return_to(return_to):
         # protect against any "open redirect" attacks using an external URL
-        if not return_to:
+        if not return_to or '\n' in return_to:
             return_to = '/'
         rt_host = urlparse(urljoin(config['base_url'], return_to)).netloc
         base_host = urlparse(config['base_url']).netloc
@@ -355,7 +398,8 @@ class AuthController(BaseController):
 
         if 'multifactor-username' not in session:
             tg.flash('Your multifactor login was disrupted, please start over.', 'error')
-            redirect('/auth/', return_to=kwargs.get('return_to', ''))
+            plugin.AuthenticationProvider.get(request).logout()  # clears all cookies that might be interfering
+            redirect('/auth/', {'return_to': kwargs.get('return_to', '')})
 
         user = M.User.by_username(session['multifactor-username'])
         try:
@@ -368,11 +412,11 @@ class AuthController(BaseController):
                 recovery.verify_and_remove_code(user, code)
                 h.auditlog_user('Logged in using a multifactor recovery code', user=user)
         except (InvalidToken, InvalidRecoveryCode):
-            c.form_errors['code'] = 'Invalid code, please try again.'
+            request.validation.errors['code'] = 'Invalid code, please try again.'
             h.auditlog_user('Multifactor login - invalid code', user=user)
             return self.multifactor(mode=mode, **kwargs)
         except MultifactorRateLimitError:
-            c.form_errors['code'] = 'Multifactor rate limit exceeded, slow down and try again later.'
+            request.validation.errors['code'] = 'Multifactor rate limit exceeded, slow down and try again later.'
             h.auditlog_user('Multifactor login - rate limit', user=user)
             return self.multifactor(mode=mode, **kwargs)
         else:
@@ -393,7 +437,7 @@ class AuthController(BaseController):
             return '%s does not include a repo mount point' % repo_path
         h.set_context(project.shortname,
                       rest[0], neighborhood=project.neighborhood)
-        if c.app is None or not getattr(c.app, 'repo'):
+        if c.app is None or not getattr(c.app, 'repo', None):
             return 'Cannot find repo at %s' % repo_path
         allura.tasks.repo_tasks.refresh.post()
         return '%r refresh queued.\n' % c.app.repo
@@ -415,7 +459,7 @@ class AuthController(BaseController):
                         continue
                     if not has_access(app, 'write', user, p):
                         continue
-                    repos.append('/%s/%s/%s' % (
+                    repos.append('/{}/{}/{}'.format(
                         app.tool_name.lower(),
                         _unix_group_name(p.neighborhood, p.shortname),
                         app.options['mount_point']))
@@ -478,12 +522,15 @@ class AuthController(BaseController):
     @expose()
     @require_post()
     @without_trailing_slash
-    @validate(V.NullValidator(), error_handler=pwd_expired)
+    @validate(F.password_change_form, error_handler=pwd_expired)
     def pwd_expired_change(self, **kw):
         require_authenticated()
         return_to = kw.get('return_to')
-        kw = F.password_change_form.to_python(kw, None)
         ap = plugin.AuthenticationProvider.get(request)
+        failure_redirect_url = tg.url('/auth/pwd_expired', dict(return_to=return_to))
+
+        enforce_hibp_password_check(ap, kw['pw'], failure_redirect_url)
+
         try:
             expired_username = session.get('expired-username')
             expired_user = M.User.query.get(username=expired_username) if expired_username else None
@@ -493,20 +540,41 @@ class AuthController(BaseController):
 
         except wexc.HTTPUnauthorized:
             flash('Incorrect password', 'error')
-            redirect(tg.url('/auth/pwd_expired', dict(return_to=return_to)))
+            redirect(failure_redirect_url)
+
         flash('Password changed')
         session.pop('pwd-expired', None)
         session['username'] = session.get('expired-username')
         session.pop('expired-username', None)
+        expired_reason = session.pop('expired-reason', None)
 
         session.save()
-        h.auditlog_user('Password reset (via expiration process)')
+        h.auditlog_user(f'Password reset ({expired_reason})')
         if return_to and return_to != request.url:
             redirect(return_to)
         else:
             redirect('/')
 
 
+class AuthRestController:
+
+    @expose('json:')
+    def tools(self, tool_type: str):
+        apps = []
+        for p in c.user.my_projects():
+            for ac in p.app_configs:
+                if ac.tool_name == tool_type and h.has_access(ac, 'read'):
+                    apps.append(p.app_instance(ac))
+
+        return {
+            'tools': [
+                dict(app.__json__(),
+                     project_name=app.project.name,
+                     )
+                for app in apps]
+        }
+
+
 def select_new_primary_addr(user, ignore_emails=[]):
     for obj_e in user.email_addresses:
         obj = user.address_object(obj_e)
@@ -519,7 +587,6 @@ class PreferencesController(BaseController):
     def _check_security(self):
         require_authenticated()
 
-
     @with_trailing_slash
     @expose('jinja:allura:templates/user_prefs.html')
     def index(self, **kw):
@@ -536,6 +603,7 @@ class PreferencesController(BaseController):
         addr = kw.pop('addr', None)
         new_addr = kw.pop('new_addr', None)
         primary_addr = kw.pop('primary_addr', None)
+        old_primary_addr = user.preferences.email_address
         provider = plugin.AuthenticationProvider.get(request)
         for i, (old_a, data) in enumerate(zip(user.email_addresses, addr or [])):
             obj = user.address_object(old_a)
@@ -554,6 +622,13 @@ class PreferencesController(BaseController):
                         primary_addr = None
                         user.set_tool_data('AuthPasswordReset', hash='', hash_expiry='')
                 h.auditlog_user('Email address deleted: %s', user.email_addresses[i], user=user)
+                if not admin:
+                    email_body = g.jinja2_env.get_template('allura:templates/mail/email_removed.md').render(dict(
+                        user=user,
+                        config=config,
+                        addr=user.email_addresses[i]
+                    ))
+                    send_system_mail_to_user(user, 'Email Address Removed', email_body)
                 del user.email_addresses[i]
                 if obj:
                     obj.delete()
@@ -578,7 +653,7 @@ class PreferencesController(BaseController):
                     user.email_addresses.append(em.email)
                     em.claimed_by_user_id = user._id
 
-                    confirmed_emails = filter(lambda email: email.confirmed, claimed_emails)
+                    confirmed_emails = [email for email in claimed_emails if email.confirmed]
                     if not confirmed_emails:
                         if not admin:
                             em.send_verification_link()
@@ -599,7 +674,7 @@ class PreferencesController(BaseController):
         if not primary_addr and not user.get_pref('email_address') and user.email_addresses:
             primary_addr = select_new_primary_addr(user)
         if primary_addr:
-            if user.get_pref('email_address') != primary_addr:
+            if old_primary_addr != primary_addr:
                 if not admin and (not kw.get('password') or not provider.validate_password(user, kw.get('password'))):
                     flash('You must provide your current password to change primary address', 'error')
                     return
@@ -608,6 +683,11 @@ class PreferencesController(BaseController):
                     user.get_pref('email_address'),
                     primary_addr,
                     user=user)
+                if not admin:
+                    email_body = g.jinja2_env.get_template('allura:templates/mail/primary_email_changed.md').render(
+                        dict(user=user, config=config, addr=primary_addr))
+                    # send to previous primary addr
+                    send_system_mail_to_user(old_primary_addr, 'Primary Email Address Changed', email_body)
             user.set_pref('email_address', primary_addr)
             user.set_tool_data('AuthPasswordReset', hash='', hash_expiry='')
 
@@ -631,7 +711,7 @@ class PreferencesController(BaseController):
             c.user.set_pref('display_name', preferences['display_name'])
             if old != preferences['display_name']:
                 h.auditlog_user('Display Name changed %s => %s', old, preferences['display_name'])
-            for k, v in preferences.iteritems():
+            for k, v in preferences.items():
                 if k == 'results_per_page':
                     v = int(v)
                 c.user.set_pref(k, v)
@@ -639,11 +719,12 @@ class PreferencesController(BaseController):
 
     @expose()
     @require_post()
-    @validate(V.NullValidator(), error_handler=index)
+    @validate(F.password_change_form, error_handler=index)
     def change_password(self, **kw):
-        kw = F.password_change_form.to_python(kw, None)
         ap = plugin.AuthenticationProvider.get(request)
         try:
+            enforce_hibp_password_check(ap, kw['pw'], '.')
+
             ap.set_password(c.user, kw['oldpw'], kw['pw'])
             session['_id'] = _session_id()  # new one so even if this session had been intercepted somehow, its invalid
             session.save()
@@ -659,7 +740,7 @@ class PreferencesController(BaseController):
             user=c.user,
             config=config,
         ))
-        send_system_mail_to_user(c.user, u'Password Changed', email_body)
+        send_system_mail_to_user(c.user, 'Password Changed', email_body)
         redirect('.')
 
     @expose()
@@ -668,7 +749,7 @@ class PreferencesController(BaseController):
         ap = plugin.AuthenticationProvider.get(request)
         try:
             ap.upload_sshkey(c.user.username, key)
-        except AssertionError, ae:
+        except AssertionError as ae:
             flash('Error uploading key: %s' % ae, 'error')
         flash('Key uploaded')
         redirect('.')
@@ -677,7 +758,7 @@ class PreferencesController(BaseController):
     @require_post()
     def user_message(self, allow_user_messages=False):
         c.user.set_pref('disable_user_messages', not allow_user_messages)
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose('jinja:allura:templates/user_totp.html')
     @without_trailing_slash
@@ -700,12 +781,14 @@ class PreferencesController(BaseController):
             totp = totp_service.Totp(key)
 
         qr = totp_service.get_qr_code(totp, c.user)
+        key_b32 = b32encode(totp.key).decode('ascii')
         h.auditlog_user('Visited multifactor new TOTP page')
         provider = plugin.AuthenticationProvider.get(request)
 
         return dict(
-            menu = provider.account_navigation(),
+            menu=provider.account_navigation(),
             qr=qr,
+            key_b32=key_b32,
             setup=True,
         )
 
@@ -719,13 +802,15 @@ class PreferencesController(BaseController):
         totp_service = TotpService.get()
         totp = totp_service.get_totp(c.user)
         qr = totp_service.get_qr_code(totp, c.user)
+        key_b32 = b32encode(totp.key).decode('ascii')
         h.auditlog_user('Viewed multifactor TOTP config page')
         provider = plugin.AuthenticationProvider.get(request)
 
         return dict(
             qr=qr,
+            key_b32=key_b32,
             setup=False,
-            menu = provider.account_navigation(),
+            menu=provider.account_navigation(),
         )
 
     @expose('jinja:allura:templates/user_totp.html')
@@ -743,7 +828,7 @@ class PreferencesController(BaseController):
             totp_service.verify(totp, code, c.user)
         except InvalidToken:
             h.auditlog_user('Failed to set up multifactor TOTP (wrong code)')
-            c.form_errors['code'] = 'Invalid code, please try again.'
+            request.validation.errors['code'] = 'Invalid code, please try again.'
             return self.totp_new(**kw)
         else:
             h.auditlog_user('Set up multifactor TOTP')
@@ -754,11 +839,9 @@ class PreferencesController(BaseController):
             del session['totp_new_key']
             session.save()
             tg.flash('Two factor authentication has now been set up.')
-            email_body = g.jinja2_env.get_template('allura:templates/mail/twofactor_enabled.md').render(dict(
-                user=c.user,
-                config=config,
-            ))
-            send_system_mail_to_user(c.user, u'Two-Factor Authentication Enabled', email_body)
+            email_body = g.jinja2_env.get_template('allura:templates/mail/twofactor_enabled.md').render(
+                dict(user=c.user, config=config, ))
+            send_system_mail_to_user(c.user, 'Two-Factor Authentication Enabled', email_body)
             redirect('/auth/preferences/multifactor_recovery')
 
     @expose()
@@ -780,7 +863,7 @@ class PreferencesController(BaseController):
             user=c.user,
             config=config,
         ))
-        send_system_mail_to_user(c.user, u'Two-Factor Authentication Disabled', email_body)
+        send_system_mail_to_user(c.user, 'Two-Factor Authentication Disabled', email_body)
         redirect('.')
 
     @expose()
@@ -793,7 +876,7 @@ class PreferencesController(BaseController):
             user=c.user,
             config=config,
         ))
-        send_system_mail_to_user(c.user, u'Two-Factor Authentication Apps', email_body)
+        send_system_mail_to_user(c.user, 'Two-Factor Authentication Apps', email_body)
 
     @expose('jinja:allura:templates/user_recovery_codes.html')
     @reconfirm_auth
@@ -811,9 +894,12 @@ class PreferencesController(BaseController):
         h.auditlog_user('Viewed multifactor recovery codes')
         provider = plugin.AuthenticationProvider.get(request)
 
+        windows_line_endings = "WINDOWS" in request.headers.get('USER_AGENT', '').upper()
+
         return dict(
             codes=codes,
-            menu = provider.account_navigation(),
+            menu=provider.account_navigation(),
+            windows_line_endings=windows_line_endings
         )
 
     @expose()
@@ -830,7 +916,7 @@ class PreferencesController(BaseController):
             config=config,
         ))
         h.auditlog_user('Regenerated multifactor recovery codes')
-        send_system_mail_to_user(c.user, u'Two-Factor Recovery Codes Regenerated', email_body)
+        send_system_mail_to_user(c.user, 'Two-Factor Recovery Codes Regenerated', email_body)
         tg.flash('Your recovery codes have been regenerated.  Save the new codes!')
         redirect('/auth/preferences/multifactor_recovery')
 
@@ -871,7 +957,7 @@ class UserSkillsController(BaseController):
 
     def __init__(self, category=None):
         self.category = category
-        super(UserSkillsController, self).__init__()
+        super().__init__()
 
     def _check_security(self):
         require_authenticated()
@@ -886,7 +972,7 @@ class UserSkillsController(BaseController):
     @with_trailing_slash
     @expose('jinja:allura:templates/user_skills.html')
     def index(self, **kw):
-        l = []
+        lst = []
         parents = []
         if kw.get('selected_category') is not None:
             selected_skill = M.TroveCategory.query.get(
@@ -894,12 +980,12 @@ class UserSkillsController(BaseController):
         elif self.category:
             selected_skill = self.category
         else:
-            l = M.TroveCategory.query.find(
+            lst = M.TroveCategory.query.find(
                 dict(trove_parent_id=0, show_as_skill=True)).all()
             selected_skill = None
         if selected_skill:
-            l = [scat for scat in selected_skill.subcategories
-                 if scat.show_as_skill]
+            lst = [scat for scat in selected_skill.subcategories
+                   if scat.show_as_skill]
             temp_cat = selected_skill.parent_category
             while temp_cat:
                 parents = [temp_cat] + parents
@@ -907,11 +993,11 @@ class UserSkillsController(BaseController):
         provider = plugin.AuthenticationProvider.get(request)
         menu = provider.account_navigation()
         return dict(
-            skills_list=l,
+            skills_list=lst,
             selected_skill=selected_skill,
             parents=parents,
             menu=menu,
-            add_details_fields=(len(l) == 0))
+            add_details_fields=(len(lst) == 0))
 
     @expose()
     @require_post()
@@ -964,8 +1050,20 @@ class UserContactsController(BaseController):
     def add_social_network(self, **kw):
         require_authenticated()
 
-        if kw['socialnetwork'] == 'Twitter' and not kw['accounturl'].startswith('http'):
-            kw['accounturl'] = 'http://twitter.com/%s' % kw['accounturl'].replace('@', '')
+        validator_map = {
+            'Twitter': V.TwitterValidator(),
+            'Instagram': V.InstagramValidator(),
+            'Facebook': V.FacebookValidator(),
+            'Mastodon': V.FediverseValidator(),
+            'Linkedin': V.LinkedinValidator(),
+        }
+
+        try:
+            Validator = validator_map.get(kw['socialnetwork'])
+            kw['accounturl'] = Validator().to_python(kw['accounturl'])
+        except fe.Invalid as e:
+            flash(e.msg, 'error')
+            redirect('.')
 
         c.user.add_multivalue_pref('socialnetworks',
                                    {'socialnetwork': kw['socialnetwork'], 'accounturl': kw['accounturl']})
@@ -1103,12 +1201,12 @@ class SubscriptionsController(BaseController):
         subscriptions = []
         mailboxes = list(M.Mailbox.query.find(
             dict(user_id=c.user._id, is_flash=False)))
-        projects = dict(
-            (p._id, p) for p in M.Project.query.find(dict(
-                _id={'$in': [mb.project_id for mb in mailboxes]})))
-        app_index = dict(
-            (ac._id, ac) for ac in M.AppConfig.query.find(dict(
-                _id={'$in': [mb.app_config_id for mb in mailboxes]})))
+        projects = {
+            p._id: p for p in M.Project.query.find(dict(
+                _id={'$in': [mb.project_id for mb in mailboxes]}))}
+        app_index = {
+            ac._id: ac for ac in M.AppConfig.query.find(dict(
+                _id={'$in': [mb.app_config_id for mb in mailboxes]}))}
 
         # Add the tools that are already subscribed to by the user.
         for mb in mailboxes:
@@ -1139,19 +1237,19 @@ class SubscriptionsController(BaseController):
                 subscribed=True))
 
         # Dictionary of all projects projects accessible based on a users credentials (user_roles).
-        my_projects = dict((p._id, p) for p in c.user.my_projects())
+        my_projects = {p._id: p for p in c.user.my_projects()}
 
         # Dictionary containing all tools (subscribed and un-subscribed).
         my_tools = M.AppConfig.query.find(dict(
-            project_id={'$in': my_projects.keys()}))
+            project_id={'$in': list(my_projects.keys())}))
 
         # Dictionary containing all the currently subscribed tools for a given user.
-        my_tools_subscriptions = dict(
-            (mb.app_config_id, mb) for mb in M.Mailbox.query.find(dict(
+        my_tools_subscriptions = {
+            mb.app_config_id: mb for mb in M.Mailbox.query.find(dict(
                 user_id=c.user._id,
-                project_id={'$in': projects.keys()},
-                app_config_id={'$in': app_index.keys()},
-                artifact_index_id=None)))
+                project_id={'$in': list(projects.keys())},
+                app_config_id={'$in': list(app_index.keys())},
+                artifact_index_id=None))}
 
         # Add the remaining tools that are eligible for subscription.
         for tool in my_tools:
@@ -1197,7 +1295,18 @@ class SubscriptionsController(BaseController):
         if email_format:
             c.user.set_pref('email_format', email_format)
 
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
+
+    @expose()
+    @require_post()
+    def update_user_notifications(self, allow_umnotif=False):
+        allow_notif = bool(allow_umnotif)
+        c.user.set_pref('mention_notifications', allow_notif)
+        if allow_notif:
+            h.auditlog_user('User mention notifications are enabled')
+        else:
+            h.auditlog_user('User mention notifications are disabled')
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class OAuthController(BaseController):
@@ -1300,7 +1409,11 @@ class DisableAccountController(BaseController):
     def index(self, **kw):
         provider = plugin.AuthenticationProvider.get(request)
         menu = provider.account_navigation()
-        my_projects = c.user.my_projects_by_role_name('Admin')
+        own_user_project = c.user.private_project()
+        my_projects = [
+            p for p in c.user.my_projects_by_role_name('Admin')
+            if p != own_user_project
+        ]
         return {
             'menu': menu,
             'my_projects': my_projects,
