@@ -15,13 +15,16 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from urllib import unquote
+from urllib.parse import unquote, urlsplit, parse_qs
 from datetime import datetime
 import logging
 
-from tg import expose, redirect, validate, request, flash
-from tg.decorators import with_trailing_slash
-from pylons import tmpl_context as c, app_globals as g
+import pymongo
+from tg import expose, redirect, validate, request, flash, response
+from tg.decorators import with_trailing_slash, without_trailing_slash, before_render, before_validate
+from decorator import decorator
+
+from tg import tmpl_context as c, app_globals as g
 from webob import exc
 
 from ming.base import Object
@@ -29,37 +32,42 @@ from ming.odm import session
 from ming.utils import LazyProperty
 
 from allura import model as M
-from base import BaseController
+from .base import BaseController
 from allura.lib import utils
 from allura.lib import helpers as h
-from allura.lib.decorators import require_post
-from allura.lib.security import require_access
+from allura.lib.decorators import require_post, memorable_forget
+from allura.lib.security import has_access, require_access
+
+from allura.tasks import notification_tasks
 
 from allura.lib.widgets import discuss as DW
 from allura.lib.widgets import form_fields as ffw
 
 from allura.model.auth import User
+from allura.model.artifact import ArtifactReference
 from .attachments import AttachmentsController, AttachmentController
 from .feed import FeedArgs, FeedController
+import six
 
 log = logging.getLogger(__name__)
 
 
-class pass_validator(object):
-
+class pass_validator:
     def validate(self, v, s):
         return v
+
+
 pass_validator = pass_validator()
 
 
-class ModelConfig(object):
+class ModelConfig:
     Discussion = M.Discussion
     Thread = M.Thread
     Post = M.Post
     Attachment = M.DiscussionAttachment
 
 
-class WidgetConfig(object):
+class WidgetConfig:
     # Forms
     subscription_form = DW.SubscriptionForm()
     edit_post = DW.EditPost()
@@ -92,7 +100,7 @@ class DiscussionController(BaseController, FeedController):
             self.moderate = ModerationController(self)
 
     def error_handler(self, *args, **kwargs):
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @h.vardec
     @expose()
@@ -107,7 +115,7 @@ class DiscussionController(BaseController, FeedController):
                 thread.unsubscribe()
             session(self.M.Thread)._get().skip_mod_date = True
             session(self.M.Thread)._get().skip_last_updated = True
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     def get_feed(self, project, app, user):
         """Return a :class:`allura.controllers.feed.FeedArgs` object describing
@@ -116,9 +124,18 @@ class DiscussionController(BaseController, FeedController):
         Overrides :meth:`allura.controllers.feed.FeedController.get_feed`.
 
         """
+        def query(since, until, page, limit, **kwargs):
+            if not since and not until and not page:
+                # simplest default case, so make the threads list shorter by grabbing only needed ones
+                discussion_threads = self.discussion.thread_class().query.find(dict(
+                    discussion_id=self.discussion._id,
+                    num_replies={'$gt': 0},  # exclude empty threads (spam/deleted) like ForumController does
+                )).sort([('last_post_date', pymongo.DESCENDING)]).limit(limit)
+            else:
+                discussion_threads = self.discussion.threads
+            return dict(ref_id={'$in': [t.index_id() for t in discussion_threads]})
         return FeedArgs(
-            dict(ref_id={'$in': [t.index_id()
-                 for t in self.discussion.threads]}),
+            query,
             'Recent posts to %s' % self.discussion.name,
             self.discussion.url())
 
@@ -132,8 +149,7 @@ class AppDiscussionController(DiscussionController):
             app_config_id=c.app.config._id)
 
 
-class ThreadsController(BaseController):
-    __metaclass__ = h.ProxiedAttrMeta
+class ThreadsController(BaseController, metaclass=h.ProxiedAttrMeta):
     M = h.attrproxy('_discussion_controller', 'M')
     W = h.attrproxy('_discussion_controller', 'W')
     ThreadController = h.attrproxy(
@@ -154,10 +170,9 @@ class ThreadsController(BaseController):
             raise exc.HTTPNotFound()
 
 
-class ThreadController(BaseController, FeedController):
-    __metaclass__ = h.ProxiedAttrMeta
-    M = h.attrproxy('_discussion_controller', 'M')
-    W = h.attrproxy('_discussion_controller', 'W')
+class ThreadController(BaseController, FeedController, metaclass=h.ProxiedAttrMeta):
+    M: ModelConfig = h.attrproxy('_discussion_controller', 'M')
+    W: WidgetConfig = h.attrproxy('_discussion_controller', 'W')
     ThreadController = h.attrproxy(
         '_discussion_controller', 'ThreadController')
     PostController = h.attrproxy('_discussion_controller', 'PostController')
@@ -172,9 +187,10 @@ class ThreadController(BaseController, FeedController):
     def __init__(self, discussion_controller, thread_id):
         self._discussion_controller = discussion_controller
         self.discussion = discussion_controller.discussion
-        self.thread = self.M.Thread.query.get(_id=thread_id)
+        self.thread = self.M.Thread.query.get(_id=thread_id, app_config_id=c.app.config._id)
         if not self.thread:
-            raise exc.HTTPNotFound
+            url = f'/p/{c.project.shortname}/discussion/{c.forum.shortname}/'
+            utils.permanent_redirect(url)
 
     @expose()
     def _lookup(self, id, *remainder):
@@ -192,6 +208,10 @@ class ThreadController(BaseController, FeedController):
         M.session.artifact_orm_session._get().skip_mod_date = True
         M.session.artifact_orm_session._get().skip_last_updated = True
         count = self.thread.query_posts(page=page, limit=int(limit)).count()
+        if self.thread.num_replies == 0 or all(p.status != 'ok' for p in self.thread.posts):
+            # return status code 404 but still display the page content
+            request.environ['tg.status_code_redirect'] = True
+            response.status_int = 404
         return dict(discussion=self.thread.discussion,
                     thread=self.thread,
                     page=int(page),
@@ -200,30 +220,19 @@ class ThreadController(BaseController, FeedController):
                     show_moderate=kw.get('show_moderate'))
 
     def error_handler(self, *args, **kwargs):
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
+    @memorable_forget()
     @h.vardec
     @expose()
     @require_post()
     @validate(pass_validator, error_handler=error_handler)
     @utils.AntiSpam.validate('Spambot protection engaged')
     def post(self, **kw):
-        require_access(self.thread, 'post')
-        if self.thread.ref:
-            require_access(self.thread.ref.artifact, 'post')
-        kw = self.W.edit_post.to_python(kw, None)
-        if not kw['text']:
-            flash('Your post was not saved. You must provide content.',
-                  'error')
-            redirect(request.referer)
-
-        file_info = kw.get('file_info', None)
-        p = self.thread.add_post(**kw)
-        p.add_multiple_attachments(file_info)
-        if self.thread.artifact:
-            self.thread.artifact.mod_date = datetime.utcnow()
-        flash('Message posted')
-        redirect(request.referer)
+        handle_post_or_reply(thread=self.thread,
+                             edit_widget=self.W.edit_post,
+                             rate_limit=self.rate_limit,
+                             kw=kw)
 
     @expose()
     @require_post()
@@ -232,7 +241,7 @@ class ThreadController(BaseController, FeedController):
         if self.thread.ref:
             require_access(self.thread.ref.artifact, 'post')
         self.thread.labels = labels.split(',')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     def flag_as_spam(self, **kw):
@@ -254,8 +263,27 @@ class ThreadController(BaseController, FeedController):
             self.thread.url())
 
 
-class PostController(BaseController):
-    __metaclass__ = h.ProxiedAttrMeta
+def handle_post_or_reply(thread, edit_widget, rate_limit, kw, parent_post_id=None):
+    require_access(thread, 'post')
+    rate_limit(M.Post, "Comment", redir=six.ensure_text(request.referer or '/'))
+    if thread.ref:
+        require_access(thread.ref.artifact, 'post')
+    kw = edit_widget.to_python(kw, None)  # could raise Invalid, but doesn't seem like it ever does
+    if not kw['text']:
+        flash('Your post was not saved. You must provide content.',
+              'error')
+        redirect(six.ensure_text(request.referer or '/'))
+    file_info = kw.get('file_info', None)
+    p = thread.add_post(parent_id=parent_post_id, **kw)
+    p.add_multiple_attachments(file_info)
+    if thread.artifact:
+        thread.artifact.mod_date = datetime.utcnow()
+    flash('Message posted')
+    notification_tasks.send_usermentions_notification.post(p.index_id(), kw['text'])
+    redirect(six.ensure_text(request.referer or '/'))
+
+
+class PostController(BaseController, metaclass=h.ProxiedAttrMeta):
     M = h.attrproxy('_discussion_controller', 'M')
     W = h.attrproxy('_discussion_controller', 'W')
     ThreadController = h.attrproxy(
@@ -289,11 +317,12 @@ class PostController(BaseController):
     def index(self, version=None, **kw):
         c.post = self.W.post
         if request.method == 'POST':
+            old_text = self.post.text
             require_access(self.post, 'moderate')
-            post_fields = self.W.edit_post.to_python(kw, None)
+            post_fields = self.W.edit_post.to_python(kw, None)  # could raise Invalid, but doesn't seem like it does
             file_info = post_fields.pop('file_info', None)
             self.post.add_multiple_attachments(file_info)
-            for k, v in post_fields.iteritems():
+            for k, v in post_fields.items():
                 try:
                     setattr(self.post, k, v)
                 except AttributeError:
@@ -303,11 +332,12 @@ class PostController(BaseController):
             self.post.last_edit_by_id = c.user._id
             self.thread.is_spam(self.post)  # run spam checker, nothing to do with result yet
             self.post.commit()
+            notification_tasks.send_usermentions_notification.post(self.post.index_id(), kw['text'], old_text)
             g.director.create_activity(c.user, 'modified', self.post,
                                        target=self.post.thread.artifact or self.post.thread,
                                        related_nodes=[self.post.app_config.project],
                                        tags=['comment'])
-            redirect(request.referer)
+            redirect(six.ensure_text(request.referer or '/'))
         elif request.method == 'GET':
             if self.post.deleted:
                 raise exc.HTTPNotFound
@@ -316,8 +346,14 @@ class PostController(BaseController):
                 ss = HC.query.find(
                     {'artifact_id': self.post._id, 'version': int(version)}).first()
                 if not ss:
-                    raise exc.HTTPNotFound
-                post = Object(
+                    url = '/p/{}/discussion/{}/thread/{}/{}'.format(c.project.shortname, c.forum.shortname,
+                                                                    self.thread._id, self._post_slug)
+                    utils.permanent_redirect(url)
+
+                class VersionedSnapshotTempObject(Object):
+                    pass
+
+                post = VersionedSnapshotTempObject(
                     ss.data,
                     acl=self.post.acl,
                     author=self.post.author,
@@ -327,40 +363,87 @@ class PostController(BaseController):
                     attachments=self.post.attachments,
                     related_artifacts=self.post.related_artifacts,
                     parent_security_context=lambda: None,
+                    last_edit_by=lambda: self.post.last_edit_by(),
+                    react_users=self.post.react_users
                 )
             else:
                 post = self.post
             return dict(discussion=self.post.discussion,
                         post=post)
 
-    def error_handler(self, *args, **kwargs):
-        redirect(request.referer)
+    @without_trailing_slash
+    @expose('json:')
+    @require_post()
+    def update_markdown(self, text=None, **kw):
+        if has_access(self.post, 'moderate'):
+            self.post.text = text
+            self.post.edit_count = self.post.edit_count + 1
+            self.post.last_edit_date = datetime.utcnow()
+            self.post.last_edit_by_id = c.user._id
+            self.post.commit()
+            g.director.create_activity(c.user, 'modified', self.post,
+                                       target=self.post.thread.artifact or self.post.thread,
+                                       related_nodes=[self.post.app_config.project],
+                                       tags=['comment'])
+            return {
+                'status': 'success'
+            }
+        else:
+            return {
+                'status': 'no_permission'
+            }
 
+    @expose()
+    @without_trailing_slash
+    def get_markdown(self):
+        return self.post.text
+
+    @expose('json:')
+    @without_trailing_slash
+    @require_post()
+    def post_reaction(self, r, **kw):
+        if c.user.is_anonymous():
+            return {
+                'error': 'no_permission'
+            }
+        status = 'ok'
+        if r in utils.get_reaction_emoji_list():
+            self.post.post_reaction(r, c.user)
+        else:
+            status = 'error'
+        return dict(status=status, counts=self.post.react_counts)
+
+    def error_handler(self, *args, **kwargs):
+        redirect(six.ensure_text(request.referer or '/'))
+
+    @memorable_forget()
     @h.vardec
     @expose()
     @require_post()
     @validate(pass_validator, error_handler=error_handler)
     @utils.AntiSpam.validate('Spambot protection engaged')
     @require_post(redir='.')
-    def reply(self, file_info=None, **kw):
-        require_access(self.thread, 'post')
-        kw = self.W.edit_post.to_python(kw, None)
-        p = self.thread.add_post(parent_id=self.post._id, **kw)
-        p.add_multiple_attachments(file_info)
-        redirect(request.referer)
+    def reply(self, **kw):
+        handle_post_or_reply(thread=self.thread,
+                             parent_post_id=self.post._id,
+                             edit_widget=self.W.edit_post,
+                             rate_limit=self.rate_limit,
+                             kw=kw)
 
     @h.vardec
-    @expose()
+    @expose('json')
     @require_post()
     @validate(pass_validator, error_handler=error_handler)
     def moderate(self, **kw):
         require_access(self.post.thread, 'moderate')
         if kw.pop('delete', None):
-            self.post.delete()
+            self.post.soft_delete()
         elif kw.pop('spam', None):
             self.post.spam()
         elif kw.pop('undo', None):
             prev_status = kw.pop('prev_status', None)
+            if self.post.status == 'spam' and prev_status == 'ok':
+                g.spam_checker.submit_ham(self.post.text, artifact=self.post, user=self.post.author())
             self.post.undo(prev_status)
         elif kw.pop('approve', None):
             if self.post.status != 'ok':
@@ -375,7 +458,7 @@ class PostController(BaseController):
     def attach(self, file_info=None):
         require_access(self.post, 'moderate')
         self.post.add_multiple_attachments(file_info)
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     def _lookup(self, id, *remainder):
@@ -394,8 +477,7 @@ class DiscussionAttachmentsController(AttachmentsController):
     AttachmentControllerClass = DiscussionAttachmentController
 
 
-class ModerationController(BaseController):
-    __metaclass__ = h.ProxiedAttrMeta
+class ModerationController(BaseController, metaclass=h.ProxiedAttrMeta):
     PostModel = M.Post
     M = h.attrproxy('_discussion_controller', 'M')
     W = h.attrproxy('_discussion_controller', 'W')
@@ -454,28 +536,59 @@ class ModerationController(BaseController):
     @expose()
     @require_post()
     def save_moderation(self, post=[], delete=None, spam=None, approve=None, **kw):
+        count = 0
         for p in post:
-            if 'checked' in p:
-                posted = self.PostModel.query.get(
-                    _id=p['_id'],
-                    # make sure nobody hacks the HTML form to moderate other
-                    # posts
-                    discussion_id=self.discussion._id,
-                )
-                if posted:
-                    if delete:
-                        posted.delete()
-                        # If we just deleted the last post in the
-                        # thread, delete the thread.
-                        if posted.thread and posted.thread.num_replies == 0:
-                            posted.thread.delete()
-                    elif spam and posted.status != 'spam':
-                        posted.spam()
-                    elif approve and posted.status != 'ok':
-                        posted.approve()
-                        g.spam_checker.submit_ham(posted.text, artifact=posted, user=posted.author())
-                        posted.thread.post_to_feed(posted)
-        redirect(request.referer)
+            posted = None
+            if isinstance(p, dict):
+                # regular form submit
+                if 'checked' in p:
+                    posted = self.PostModel.query.get(
+                        _id=p['_id'],
+                        # make sure nobody hacks the HTML form to moderate other
+                        # posts
+                        discussion_id=self.discussion._id,
+                    )
+            elif isinstance(p, self.PostModel):
+                # called from save_moderation_bulk_user with models already
+                posted = p
+            else:
+                raise TypeError('post list should be form fields, or Post models')
+
+            if posted:
+                if delete:
+                    # full, for real, delete since this is post was never visible and denied by admin.
+                    posted.delete()
+                    # If we just deleted the last post in the
+                    # thread, delete the thread.
+                    if posted.thread and posted.thread.num_replies == 0:
+                        count += 1
+                        posted.thread.delete()
+                elif spam and posted.status != 'spam':
+                    count += 1
+                    posted.spam()
+                elif approve and posted.status != 'ok':
+                    count += 1
+                    posted.approve()
+                    g.spam_checker.submit_ham(posted.text, artifact=posted, user=posted.author())
+                    posted.thread.post_to_feed(posted)
+        flash('{} {}'.format(h.text.plural(count, 'post', 'posts'),
+                             'deleted' if delete else 'marked as spam' if spam else 'approved'))
+        redirect(six.ensure_text(request.referer or '/'))
+
+    @expose()
+    @require_post()
+    def save_moderation_bulk_user(self, username, **kw):
+        # this is used by post.js as a quick way to deal with all a user's posts
+        user = User.by_username(username)
+        posts = self.PostModel.query.find({
+            'author_id': user._id,
+            'deleted': False,
+            # this is what the main moderation forms does (e.g. single discussion within a forum app)
+            # 'discussion_id': self.discussion._id
+            # but instead want to do all discussions within this app
+            'app_config_id': c.app.config._id
+        })
+        return self.save_moderation(posts, **kw)
 
 
 class PostRestController(PostController):
@@ -490,7 +603,7 @@ class PostRestController(PostController):
     @validate(pass_validator, error_handler=h.json_validation_error)
     def reply(self, **kw):
         require_access(self.thread, 'post')
-        kw = self.W.edit_post.to_python(kw, None)
+        kw = self.W.edit_post.to_python(kw, None)  # could raise Invalid, but doesn't seem like it ever does
         post = self.thread.post(parent_id=self.post._id, **kw)
         self.thread.num_replies += 1
         redirect(post.slug.split('/')[-1] + '/')
@@ -509,7 +622,7 @@ class ThreadRestController(ThreadController):
     @validate(pass_validator, error_handler=h.json_validation_error)
     def new(self, **kw):
         require_access(self.thread, 'post')
-        kw = self.W.edit_post.to_python(kw, None)
+        kw = self.W.edit_post.to_python(kw, None)  # could raise Invalid, but doesn't seem like it ever does
         p = self.thread.add_post(**kw)
         redirect(p.slug + '/')
 
