@@ -18,27 +18,28 @@
 import re
 import logging
 from datetime import datetime, timedelta
-from urllib import unquote
+
+import six
+from urllib.parse import unquote
 
 from bson import ObjectId
 from tg import expose, flash, redirect, validate, request, config, session
 from tg.decorators import with_trailing_slash, without_trailing_slash
-from pylons import tmpl_context as c, app_globals as g
+from tg import tmpl_context as c, app_globals as g
 from paste.deploy.converters import asbool
 from webob import exc
 import jinja2
+import markupsafe
 import pymongo
 
 from ming.utils import LazyProperty
 
 from allura import model as M
 from allura.app import SitemapEntry
-from allura.lib.base import WsgiDispatchController
 from allura.lib import helpers as h
 from allura.lib.helpers import iter_entry_points
 from allura.lib import utils
 from allura.lib.decorators import require_post
-from allura.controllers.error import ErrorController
 from allura.controllers.feed import FeedArgs, FeedController
 from allura.controllers.rest import nbhd_lookup_first_path
 from allura.lib.security import require_access
@@ -47,9 +48,8 @@ from allura.lib.widgets import forms as ff
 from allura.lib.widgets import form_fields as ffw
 from allura.lib.widgets import project_list as plw
 from allura.lib import plugin, exceptions
-from .auth import AuthController
-from .search import SearchController, ProjectBrowseController
-from .static import NewForgeController
+from .search import ProjectBrowseController
+from allura.ext.user_profile.user_main import UserProfileApp
 
 log = logging.getLogger(__name__)
 
@@ -64,7 +64,7 @@ class W:
     award_grant_form = ff.AwardGrantForm
 
 
-class NeighborhoodController(object):
+class NeighborhoodController:
 
     '''Manages a neighborhood of projects.
     '''
@@ -111,7 +111,7 @@ class NeighborhoodController(object):
             else:
                 redirect(c.project.app_configs[0].options.mount_point + '/')
         else:
-            text=g.markdown.cached_convert(
+            text = g.markdown.cached_convert(
                 self.neighborhood, 'homepage'),
 
         c.project_summary = W.project_summary
@@ -153,7 +153,7 @@ class NeighborhoodController(object):
     def add_project(self, **form_data):
         with h.login_overlay():
             require_access(self.neighborhood, 'register')
-        verify = c.form_errors == {'_the_form': u'phone-verification'}
+        verify = request.validation.errors == {'_the_form': 'phone-verification'}
         c.show_phone_verification_overlay = verify
         c.add_project = W.add_project
         form_data.setdefault('tools', W.add_project.default_tools)
@@ -162,10 +162,12 @@ class NeighborhoodController(object):
 
     @expose('jinja:allura:templates/phone_verification_fragment.html')
     def phone_verification_fragment(self, *args, **kw):
+        require_access(self.neighborhood, 'register')
         return {}
 
     @expose('json:')
     def verify_phone(self, number, **kw):
+        require_access(self.neighborhood, 'register')
         p = plugin.ProjectRegistrationProvider.get()
         result = p.verify_phone(c.user, number)
         request_id = result.pop('request_id', None)
@@ -175,25 +177,27 @@ class NeighborhoodController(object):
             session['phone_verification.number_hash'] = number_hash
             session.save()
         if 'error' in result:
-            result['error'] = jinja2.Markup.escape(result['error'])
+            result['error'] = markupsafe.Markup.escape(result['error'])
             result['error'] = h.really_unicode(result['error'])
         return result
 
     @expose('json:')
     def check_phone_verification(self, pin, **kw):
+        require_access(self.neighborhood, 'register')
         p = plugin.ProjectRegistrationProvider.get()
         request_id = session.get('phone_verification.request_id')
         number_hash = session.get('phone_verification.number_hash')
         res = p.check_phone_verification(c.user, request_id, pin, number_hash)
         if 'error' in res:
-            res['error'] = jinja2.Markup.escape(res['error'])
+            res['error'] = markupsafe.Markup.escape(res['error'])
             res['error'] = h.really_unicode(res['error'])
         return res
 
     @expose('json:')
     @validate(W.add_project)
     def check_names(self, **raw_data):
-        return c.form_errors
+        require_access(self.neighborhood, 'register')
+        return request.validation.errors
 
     @h.vardec
     @expose()
@@ -208,11 +212,9 @@ class NeighborhoodController(object):
             require_access(self.neighborhood, 'admin')
         neighborhood = M.Neighborhood.query.get(name=neighborhood)
 
-        project_description = h.really_unicode(
-            project_description or '').encode('utf-8')
-        project_name = h.really_unicode(project_name or '').encode('utf-8')
-        project_unixname = h.really_unicode(
-            project_unixname or '').encode('utf-8').lower()
+        project_description = h.really_unicode(project_description or '')
+        project_name = h.really_unicode(project_name or '')
+        project_unixname = h.really_unicode(project_unixname or '').lower()
         try:
             c.project = neighborhood.register_project(project_unixname,
                                                       project_name=project_name, private_project=private_project)
@@ -240,7 +242,7 @@ class NeighborhoodController(object):
             anchored_tools = neighborhood.get_anchored_tools()
             install_params = []
             for i, tool in enumerate(tools):
-                if (tool.lower() not in anchored_tools.keys()) and (c.project.app_instance(tool) is None):
+                if (tool.lower() not in list(anchored_tools.keys())) and (c.project.app_instance(tool) is None):
                     install_params.append(dict(ep_name=tool, ordinal=i + offset))
             c.project.install_apps(install_params)
         redirect(c.project.script_name + 'admin/?first-visit')
@@ -248,6 +250,8 @@ class NeighborhoodController(object):
     @expose()
     def icon(self, w=None, **kw):
         try:
+            if isinstance(w, list):
+                w = w[0]
             icon = c.project.icon_sized(w=int(w or 48))
         except ValueError as e:
             log.info('Invalid project icon size: %s on %s', e, request.url)
@@ -265,7 +269,7 @@ class NeighborhoodController(object):
         return {
             'options': [{
                 'value': u.username,
-                'label': '%s (%s)' % (u.display_name, u.username)
+                'label': f'{u.display_name} ({u.username})'
             } for u in p.users()]
         }
 
@@ -274,7 +278,7 @@ class NeighborhoodProjectBrowseController(ProjectBrowseController):
 
     def __init__(self, neighborhood=None, category_name=None, parent_category=None):
         self.neighborhood = neighborhood
-        super(NeighborhoodProjectBrowseController, self).__init__(
+        super().__init__(
             category_name=category_name, parent_category=parent_category)
         self.nav_stub = '%sbrowse/' % self.neighborhood.url()
         self.additional_filters = {'neighborhood_id': self.neighborhood._id}
@@ -283,7 +287,8 @@ class NeighborhoodProjectBrowseController(ProjectBrowseController):
     def _lookup(self, category_name, *remainder):
         c.project = self.neighborhood.neighborhood_project
         category_name = unquote(category_name)
-        return NeighborhoodProjectBrowseController(neighborhood=self.neighborhood, category_name=category_name, parent_category=self.category), remainder
+        return NeighborhoodProjectBrowseController(neighborhood=self.neighborhood, category_name=category_name,
+                                                   parent_category=self.category), remainder
 
     @expose('jinja:allura:templates/neighborhood_project_list.html')
     @without_trailing_slash
@@ -303,38 +308,30 @@ class NeighborhoodProjectBrowseController(ProjectBrowseController):
                     limit=limit, page=page, count=count)
 
 
-class HostNeighborhoodController(WsgiDispatchController, NeighborhoodController):
-
-    '''Neighborhood controller with support for use as a root controller, for
-    instance, when using adobe.domain.net (if this is allowed).
-    '''
-
-    auth = AuthController()
-    error = ErrorController()
-    nf = NewForgeController()
-    search = SearchController()
-
-
-class ToolListController(object):
+class ToolListController:
 
     """Renders a list of all tools of a given type in the current project."""
 
     @expose('jinja:allura:templates/tool_list.html')
-    def _default(self, tool_name, page=0, limit=200, **kw):
+    def _default(self, tool_name=None, page=0, limit=200, **kw):
+        if tool_name is None:
+            raise exc.HTTPNotFound
         c.page_list = W.page_list
         tool_name = tool_name.lower()
         entries = c.project.sitemap(included_tools=[tool_name],
-                tools_only=True, per_tool_limit=None)
+                                    tools_only=True, per_tool_limit=None)
         total_entries = len(entries)
         limit, page = h.paging_sanitizer(limit, page, total_entries)
         start = page * limit
+        tool_label = g.entry_points['tool'][tool_name].tool_label if entries else None
         return dict(
             page=page,
             limit=limit,
             total_entries=total_entries,
             entries=entries[start:start + limit],
-            type=g.entry_points['tool'][tool_name].tool_label if entries else None,
-            )
+            type=tool_label,
+            tool_name=h.pluralize_tool_name(tool_label, total_entries),
+        )
 
 
 class ProjectController(FeedController):
@@ -350,12 +347,13 @@ class ProjectController(FeedController):
     @expose()
     def _lookup(self, name, *remainder):
         name = unquote(name)
+        name = six.ensure_text(name)  # we don't support unicode names, but in case a url comes in with one
         if name == '_nav.json':
             return self, ['_nav']
 
         if c.project.deleted:
             if c.user not in c.project.admins():
-                raise exc.HTTPNotFound, name
+                raise exc.HTTPNotFound(name)
         app = c.project.app_instance(name)
 
         if app:
@@ -369,7 +367,12 @@ class ProjectController(FeedController):
             c.project = subproject
             c.app = None
             return ProjectController(), remainder
-        raise exc.HTTPNotFound, name
+
+        if c.project.is_nbhd_project:
+            raise exc.HTTPNotFound(name)
+        else:
+            # if a tool under a project doesn't exist, redirect to the first valid tool instead of 404
+            self.index()
 
     @expose('jinja:allura:templates/members.html')
     @with_trailing_slash
@@ -380,19 +383,17 @@ class ProjectController(FeedController):
         for user in c.project.users():
             roles = M.ProjectRole.query.find(
                 {'_id': {'$in': M.ProjectRole.by_user(user).roles}})
-            roles = set([r.name for r in roles])
-            u = dict(
-                display_name=user.display_name,
-                username=user.username,
-                url=user.url(),
-                roles=', '.join(sorted(roles)))
+            roles = {r.name for r in roles}
+            _user = user
+            _user['roles'] = ', '.join(sorted(roles))
             if 'Admin' in roles:
-                admins.append(u)
+                admins.append(_user)
             elif 'Developer' in roles:
-                developers.append(u)
+                developers.append(_user)
             else:
-                users.append(u)
-        get_username = lambda user: user['username']
+                users.append(_user)
+
+        def get_username(user): return user['username']
         admins = sorted(admins, key=get_username)
         developers = sorted(developers, key=get_username)
         users = sorted(users, key=get_username)
@@ -402,17 +403,16 @@ class ProjectController(FeedController):
         require_access(c.project, 'read')
 
     @expose()
-    @with_trailing_slash
     def index(self, **kw):
-        mount = c.project.first_mount_visible(c.user)
-        activity_enabled = asbool(config.get('activitystream.enabled', False))
+        mount, app = c.project.first_mount_visible(c.user)
         if mount is not None:
-            if 'ac' in mount:
-                redirect(mount['ac'].options.mount_point + '/')
-            elif 'sub' in mount:
-                redirect(mount['sub'].url())
+            if hasattr(app, 'default_redirect'):
+                app.default_redirect()
+            args = dict(redirect_with=exc.HTTPMovedPermanently)
+            # Application has property; Subproject has method
+            redirect(app.url() if callable(app.url) else app.url, **args)
         else:
-            redirect(c.project.app_configs[0].options.mount_point + '/')
+            redirect(c.project.app_configs[0].url(), redirect_with=exc.HTTPMovedPermanently)
 
     def get_feed(self, project, app, user):
         """Return a :class:`allura.controllers.feed.FeedArgs` object describing
@@ -429,6 +429,8 @@ class ProjectController(FeedController):
     @expose()
     def icon(self, w=48, **kw):
         try:
+            if isinstance(w, list):
+                w = w[0]
             icon = c.project.icon_sized(w=int(w))
         except ValueError as e:
             log.info('Invalid project icon size: %s on %s', e, request.url)
@@ -464,7 +466,7 @@ class ProjectController(FeedController):
         return dict(
             users=[
                 dict(
-                    label='%s (%s)' % (u.get_pref('display_name'), u.username),
+                    label='{} ({})'.format(u.get_pref('display_name'), u.username),
                     value=u.username,
                     id=u.username)
                 for u in users])
@@ -479,12 +481,12 @@ class ProjectController(FeedController):
         return {
             'options': [{
                 'value': u.username,
-                'label': '%s (%s)' % (u.display_name, u.username)
+                'label': f'{u.display_name} ({u.username})'
             } for u in users]
         }
 
 
-class ScreenshotsController(object):
+class ScreenshotsController:
 
     @expose()
     def _lookup(self, filename, *args):
@@ -495,7 +497,7 @@ class ScreenshotsController(object):
         return ScreenshotController(filename), args
 
 
-class ScreenshotController(object):
+class ScreenshotController:
 
     def __init__(self, filename):
         self.filename = filename
@@ -541,7 +543,7 @@ def set_nav(neighborhood):
             SitemapEntry('Awards', admin_url + 'accolades')]
 
 
-class NeighborhoodAdminController(object):
+class NeighborhoodAdminController:
 
     def __init__(self, neighborhood):
         self.neighborhood = neighborhood
@@ -665,9 +667,9 @@ class NeighborhoodAdminController(object):
         result = True
         if anchored_tools.strip() != '':
             try:
-                validate_tools = dict(
-                    (tool.split(':')[0].lower(), tool.split(':')[1])
-                    for tool in anchored_tools.replace(' ', '').split(','))
+                validate_tools = {
+                    tool.split(':')[0].lower(): tool.split(':')[1]
+                    for tool in anchored_tools.replace(' ', '').split(',')}
             except Exception:
                 flash('Anchored tools "%s" is invalid' %
                       anchored_tools, 'error')
@@ -682,12 +684,16 @@ class NeighborhoodAdminController(object):
             h.log_if_changed(nbhd, 'anchored_tools', anchored_tools,
                              'update neighborhood anchored tools')
 
-        if icon is not None and icon != '':
+        if icon is not None and icon != b'':
             if self.neighborhood.icon:
                 self.neighborhood.icon.delete()
                 M.ProjectFile.query.remove(dict(project_id=c.project._id, category=re.compile(r'^icon')))
-            M.AuditLog.log('update neighborhood icon')
-            c.project.save_icon(icon.filename, icon.file, content_type=icon.type)
+            save_icon = c.project.save_icon(icon.filename, icon.file, content_type=icon.type)
+            if save_icon:
+                M.AuditLog.log('update neighborhood icon')
+            else:
+                M.AuditLog.log('could not update neighborhood icon')
+                flash("There's a problem with the uploaded image", 'error')
         redirect('overview')
 
     @expose('jinja:allura:templates/neighborhood_help.html')
@@ -700,7 +706,7 @@ class NeighborhoodAdminController(object):
         )
 
 
-class NeighborhoodStatsController(object):
+class NeighborhoodStatsController:
 
     def __init__(self, neighborhood):
         self.neighborhood = neighborhood
@@ -776,7 +782,7 @@ class NeighborhoodStatsController(object):
                     )
 
 
-class NeighborhoodModerateController(object):
+class NeighborhoodModerateController:
 
     def __init__(self, neighborhood):
         self.neighborhood = neighborhood
@@ -837,7 +843,7 @@ class NeighborhoodModerateController(object):
         redirect('.')
 
 
-class NeighborhoodAwardsController(object):
+class NeighborhoodAwardsController:
 
     def __init__(self, neighborhood=None):
         if neighborhood is not None:
@@ -881,7 +887,7 @@ class NeighborhoodAwardsController(object):
                     icon.filename, icon.file, content_type=icon.type,
                     square=True, thumbnail_size=(48, 48),
                     thumbnail_meta=dict(award_id=award._id))
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     @require_post()
@@ -902,10 +908,10 @@ class NeighborhoodAwardsController(object):
             award.comment = comment
             with h.push_context(recipient_q._id):
                 g.post_event('project_updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
-class AwardController(object):
+class AwardController:
 
     def __init__(self, neighborhood=None, award_id=None):
         self.neighborhood = neighborhood
@@ -970,10 +976,10 @@ class AwardController(object):
                     g.post_event('project_updated')
             M.AwardFile.query.remove(dict(award_id=self.award._id))
             self.award.delete()
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
-class GrantController(object):
+class GrantController:
 
     def __init__(self, neighborhood=None, award=None, recipient=None):
         self.neighborhood = neighborhood
@@ -1012,17 +1018,20 @@ class GrantController(object):
         self.grant.delete()
         with h.push_context(self.project._id):
             g.post_event('project_updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
-class ProjectImporterController(object):
+class ProjectImporterController:
 
     def __init__(self, neighborhood, *a, **kw):
-        super(ProjectImporterController, self).__init__(*a, **kw)
+        super().__init__(*a, **kw)
         self.neighborhood = neighborhood
 
     @expose()
-    def _lookup(self, source, *rest):
+    def _lookup(self, source=None, *rest):
+        if source is None:
+            raise exc.HTTPNotFound
+
         # iter_entry_points is a generator with 0 or 1 items, so a loop is the easiest way to handle
         for ep in iter_entry_points('allura.project_importers', source):
             return ep.load()(self.neighborhood), rest
