@@ -18,29 +18,34 @@
 import os
 import logging
 import difflib
+
+from allura.lib.utils import permanent_redirect
 from datetime import datetime
-from urllib import quote, unquote
+from urllib.parse import quote, unquote
 from collections import defaultdict, OrderedDict
 
-from paste.deploy.converters import asbool
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request, response
+
+from ming.utils import LazyProperty
+from paste.deploy.converters import asbool, asint
+from tg import tmpl_context as c, app_globals as g
+from tg import request, response
 from webob import exc
 import tg
 from tg import redirect, expose, flash, validate
 from tg.decorators import with_trailing_slash, without_trailing_slash
 from tg import session as web_session
+import formencode
 from formencode import validators
 from bson import ObjectId
 from ming.base import Object
-from ming.orm import ThreadLocalORMSession, session
+from ming.odm import ThreadLocalODMSession, session
 
 import allura.tasks
 from allura import model as M
 from allura.lib import utils
 from allura.lib import helpers as h
 from allura.lib import widgets as w
-from allura.lib.decorators import require_post
+from allura.lib.decorators import require_post, memorable_forget
 from allura.lib.diff import HtmlSideBySideDiff
 from allura.lib.security import require_access, require_authenticated, has_access
 from allura.lib.widgets import form_fields as ffw
@@ -53,6 +58,9 @@ from allura.controllers.base import DispatchIndex
 from allura.controllers.rest import AppRestControllerMixin
 from allura.controllers.feed import FeedController, FeedArgs
 from .base import BaseController
+import six
+
+from ..app import Application
 
 log = logging.getLogger(__name__)
 
@@ -70,25 +78,20 @@ class RepoRootController(BaseController, FeedController):
     def get_feed(self, project, app, user):
         query = dict(project_id=project._id, app_config_id=app.config._id)
         pname, repo = (project.shortname, app.config.options.mount_label)
-        title = '%s %s changes' % (pname, repo)
-        description = 'Recent changes to %s repository in %s project' % (
+        title = f'{pname} {repo} changes'
+        description = 'Recent changes to {} repository in {} project'.format(
             repo, pname)
         return FeedArgs(query, title, app.url, description=description)
 
     def _check_security(self):
         require_access(c.app, 'read')
 
-    @expose('jinja:allura:templates/markdown_syntax_dialog.html')
-    def markdown_syntax_dialog(self, **kw):
-        """Static page explaining markdown."""
-        return dict()
-
     @with_trailing_slash
     @expose()
     def index(self, offset=0, branch=None, **kw):
         if branch is None:
             branch = c.app.default_branch_name
-        redirect(c.app.repo.url_for_commit(branch, url_type='ref'))
+        permanent_redirect(c.app.repo.url_for_commit(branch, url_type='ref'))
 
     @with_trailing_slash
     @expose('jinja:allura:templates/repo/forks.html')
@@ -100,8 +103,8 @@ class RepoRootController(BaseController, FeedController):
                 repo_path_parts = f.url().strip('/').split('/')
                 links.append(dict(
                     repo_url=f.url(),
-                    repo='%s / %s' % (repo_path_parts[1],
-                                      repo_path_parts[-1]),
+                    repo='{} / {}'.format(repo_path_parts[1],
+                                          repo_path_parts[-1]),
                 ))
         return dict(links=links)
 
@@ -110,7 +113,7 @@ class RepoRootController(BaseController, FeedController):
         allura.tasks.repo_tasks.refresh.post()
         if request.referer:
             flash('Repository is being refreshed')
-            redirect(request.referer)
+            redirect(six.ensure_text(request.referer or '/'))
         else:
             return '%r refresh queued.\n' % c.app.repo
 
@@ -122,12 +125,12 @@ class RepoRootController(BaseController, FeedController):
         if not c.app.forkable:
             raise exc.HTTPNotFound
         from_repo = c.app.repo
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         from_project = c.project
         to_project = M.Project.query.get(_id=ObjectId(project_id))
-        mount_label = mount_label or '%s - %s' % (c.project.name,
-                                                  c.app.config.options.mount_label)
+        mount_label = mount_label or '{} - {}'.format(c.project.name,
+                                                      c.app.config.options.mount_label)
         mount_point = (mount_point or from_project.shortname)
         if request.method != 'POST' or not mount_point:
             return dict(from_repo=from_repo,
@@ -149,22 +152,23 @@ class RepoRootController(BaseController, FeedController):
                     redirect(to_project.url() + mount_point + '/')
                 except exc.HTTPRedirection:
                     raise
-                except Exception, ex:
+                except Exception as ex:
                     flash(str(ex), 'error')
-                    redirect(request.referer)
+                    redirect(six.ensure_text(request.referer or '/'))
 
     @property
     def mr_widget(self):
         source_branches = [
             b.name
-            for b in c.app.repo.get_branches() + c.app.repo.get_tags()]
+            for b in c.app.repo.get_branches() + c.app.repo.get_tags(for_merge_request=True)]
         with c.app.repo.push_upstream_context():
-            target_branches = [
-                b.name
-                for b in c.app.repo.get_branches() + c.app.repo.get_tags()]
+            target_branches = [b.name for b in c.app.repo.get_branches()]
+            subscribed_to_upstream = M.Mailbox.subscribed()
         return SCMMergeRequestWidget(
             source_branches=source_branches,
-            target_branches=target_branches)
+            target_branches=target_branches,
+            show_subscribe_checkbox=not subscribed_to_upstream,
+        )
 
     @without_trailing_slash
     @expose('jinja:allura:templates/repo/request_merge.html')
@@ -182,10 +186,15 @@ class RepoRootController(BaseController, FeedController):
             'target_branch': target_branch,
         }
 
-    @expose()
+    @memorable_forget()
+    @expose('jinja:allura:templates/repo/request_merge.html')  # needed when we "return self.request_merge(...)"
     @require_post()
     def do_request_merge(self, **kw):
-        kw = self.mr_widget.to_python(kw)
+        try:
+            kw = self.mr_widget.to_python(kw)
+        except formencode.Invalid:
+            # trigger error_handler directly
+            return self.request_merge(**kw)
         downstream = dict(
             project_id=c.project._id,
             mount_point=c.app.config.options.mount_point,
@@ -197,7 +206,8 @@ class RepoRootController(BaseController, FeedController):
                 source_branch=kw['source_branch'],
                 summary=kw['summary'],
                 description=kw['description'])
-            mr.subscribe(user=c.user)
+            if kw.get('subscribe'):
+                mr.subscribe(user=c.user)
             M.Notification.post(
                 mr, 'merge_request',
                 subject=mr.email_subject,
@@ -208,7 +218,8 @@ class RepoRootController(BaseController, FeedController):
                 ref_id=mr.index_id(),
             )
             session(t).flush()
-            g.director.create_activity(c.user, 'created', mr,
+            allura.tasks.notification_tasks.send_usermentions_notification.post(mr.index_id(), kw['description'])
+            g.director.create_activity(c.user, 'created', mr, target=mr.app,
                                        related_nodes=[c.project], tags=['merge-request'])
             redirect(mr.url())
 
@@ -226,8 +237,10 @@ class RepoRootController(BaseController, FeedController):
 
     @without_trailing_slash
     @expose('json:')
-    def commit_browser_data(self, start=None, limit=50, **kw):
+    def commit_browser_data(self, start=None, limit=None, **kw):
         log.debug('Start commit_browser_data')
+        if limit is None:
+            limit = int(tg.config.get('scm.view.commit_browser.limit', 500))
 
         if start:
             head_ids = start.split(',')
@@ -255,7 +268,7 @@ class RepoRootController(BaseController, FeedController):
         parents = {}
         children = defaultdict(list)
         dates = {}
-        for row, (oid, ci) in enumerate(commits_by_id.iteritems()):
+        for row, (oid, ci) in enumerate(commits_by_id.items()):
             parents[oid] = list(ci.parent_ids)
             dates[oid] = ci.committed.date
             for p_oid in ci.parent_ids:
@@ -270,7 +283,7 @@ class RepoRootController(BaseController, FeedController):
             url = c.app.repo.url_for_commit(Object(_id=oid))
             msg_split = ci.message.splitlines()
             if msg_split:
-                msg = msg_split[0]
+                msg = h.hide_private_info(msg_split[0])
             else:
                 msg = "No commit message."
             result.append(dict(
@@ -298,13 +311,28 @@ class RepoRestController(RepoRootController, AppRestControllerMixin):
 
     @expose('json:')
     def index(self, **kw):
-        all_commits = c.app.repo._impl.new_commits(all_commits=True)
-        return dict(commit_count=len(all_commits))
+        app: Application = c.app
+        repo: M.Repository = app.repo
+
+        # core fields, shared in other API endpoints
+        resp = app.__json__()
+
+        # more expensive fields that we only show in this individual API endpoint
+        try:
+            all_commits = repo._impl.new_commits(all_commits=True)
+        except Exception:
+            log.exception(f'Error getting commits on {c.app.url}')
+            commit_count = None
+        else:
+            commit_count = len(all_commits)
+        resp['commit_count'] = commit_count
+
+        return resp
 
     @expose('json:')
     def commits(self, rev=None, limit=25, **kw):
         '''
-        Return 25 latest commits   : /rest/p/code/logs/ 
+        Return 25 latest commits   : /rest/p/code/logs/
         Return 25 commits since sha: /rest/p/code/logs/e1a2ad
         Return 04 commits since sha: /rest/p/code/logs/e1a2ad/4
         Return 120 latest commits  : /rest/p/code/logs/?limit=120
@@ -334,8 +362,21 @@ class RepoRestController(RepoRootController, AppRestControllerMixin):
                 for commit in revisions
             ]}
 
+    @expose('json:')
+    def commit_status(self, rev=None, **kwargs):
+        if not g.commit_statuses_enabled:
+            return {'status': 'disabled', 'message': 'check your config file'}
+        params = {x: kwargs.get(x, '').strip() for x in
+                  ['state', 'target_url', 'description', 'context']}
+        params['commit_id'] = rev
+        status = M.CommitStatus.upsert(**params)
+        response = {'status': 'error'}
+        if status:
+            response['status'] = 'success'
+        return response
 
-class MergeRequestsController(object):
+
+class MergeRequestsController:
 
     @with_trailing_slash
     @expose('jinja:allura:templates/repo/merge_requests.html')
@@ -356,7 +397,7 @@ class MergeRequestsController(object):
         return MergeRequestController(num), remainder
 
 
-class MergeRequestController(object):
+class MergeRequestController:
     log_widget = SCMLogWidget(show_paging=False)
     thread_widget = w.Thread(
         page=None, limit=None, page_size=None, count=None,
@@ -399,27 +440,32 @@ class MergeRequestController(object):
             limit=limit,
             count=self.req.discussion_thread.post_count,
             subscribed=subscribed,
+            commits_task_started=False,
         )
-        try:
-            result['commits'] = self.req.commits
-        except Exception:
-            log.info(
-                "Can't get commits for merge request %s",
-                self.req.url(),
-                exc_info=True)
+        if self.req.new_commits is not None:
+            try:
+                result['commits'] = self.req.commits
+            except Exception:
+                log.info(
+                    "Can't get commits for merge request %s",
+                    self.req.url(),
+                    exc_info=True)
+                result['commits'] = []
+                result['error'] = True
+        else:
+            if self.req.commits_task_status() not in ('busy', 'ready'):
+                allura.tasks.repo_tasks.determine_mr_commits.post(self.req._id)
             result['commits'] = []
-            result['error'] = True
+            result['commits_task_started'] = True
         return result
 
     @property
     def mr_widget_edit(self):
         target_branches = [
             b.name
-            for b in c.app.repo.get_branches() + c.app.repo.get_tags()]
+            for b in c.app.repo.get_branches() + c.app.repo.get_tags(for_merge_request=True)]
         with self.req.push_downstream_context():
-            source_branches = [
-                b.name
-                for b in c.app.repo.get_branches() + c.app.repo.get_tags()]
+            source_branches = [b.name for b in c.app.repo.get_branches()]
         return SCMMergeRequestWidget(
             source_branches=source_branches,
             target_branches=target_branches)
@@ -443,12 +489,18 @@ class MergeRequestController(object):
             'summary': self.req['summary']
         }
 
-    @expose()
+    @memorable_forget()
+    @expose('jinja:allura:templates/repo/merge_request_edit.html')   # needed when we "return self.edit(...)"
     @require_post()
     def do_request_merge_edit(self, **kw):
         require_access(self.req, 'write')
-        kw = self.mr_widget_edit.to_python(kw)
+        try:
+            kw = self.mr_widget_edit.to_python(kw)
+        except formencode.Invalid:
+            # trigger error_handler directly
+            return self.edit(**kw)
         changes = OrderedDict()
+        old_text = self.req.description
         if self.req.summary != kw['summary']:
             changes['Summary'] = [self.req.summary, kw['summary']]
             self.req.summary = kw['summary']
@@ -464,15 +516,35 @@ class MergeRequestController(object):
         if self.req.description != kw['description']:
             changes['Description'] = h.unidiff(self.req.description, kw['description'])
             self.req.description = kw['description']
-        with self.req.push_downstream_context():
-            self.req.downstream['commit_id'] = c.app.repo.commit(kw['source_branch'])._id
 
         if changes:
             self.req.add_meta_post(changes=changes)
+            allura.tasks.notification_tasks.send_usermentions_notification.post(self.req.index_id(), kw['description'], old_text)
             g.director.create_activity(c.user, 'updated', self.req,
                                        related_nodes=[c.project], tags=['merge-request'])
+        self.refresh()
 
-        redirect(self.req.url())
+    @without_trailing_slash
+    @expose('json:')
+    @require_post()
+    def update_markdown(self, text=None, **kw):
+        if has_access(self.req, 'write'):
+            self.req.description = text
+            self.req.commit()
+            g.director.create_activity(c.user, 'updated', self.req,
+                                       related_nodes=[c.project], tags=['merge-request'])
+            return {
+                'status': 'success'
+            }
+        else:
+            return {
+                'status': 'no_permission'
+            }
+
+    @expose()
+    @without_trailing_slash
+    def get_markdown(self):
+        return self.req.description
 
     @expose()
     @require_post()
@@ -491,6 +563,7 @@ class MergeRequestController(object):
     def refresh(self, **kw):
         require_access(self.req, 'read')
         with self.req.push_downstream_context():
+            self.req.new_commits = None  # invalidate this cache
             self.req.downstream['commit_id'] = c.app.repo.commit(self.req.source_branch)._id
         redirect(self.req.url())
 
@@ -515,6 +588,21 @@ class MergeRequestController(object):
         """Return result from the cache. Used by js, after task was completed."""
         return {'can_merge': self.req.can_merge()}
 
+    @expose()
+    def commits_html(self, **kw):
+        if self.req.new_commits is not None:
+            with self.req.push_downstream_context():
+                downstream_app = c.app
+            return SCMLogWidget().display(value=self.req.commits, app=downstream_app)
+
+        task_status = self.req.commits_task_status()
+        if task_status is None:
+            raise exc.HTTPNotFound
+        elif task_status == 'error':
+            raise exc.HTTPInternalServerError
+        elif task_status in ('busy', 'ready'):
+            raise exc.HTTPAccepted
+
     @expose('json:')
     @require_post()
     @validate(subscribe_form)
@@ -531,28 +619,33 @@ class MergeRequestController(object):
         }
 
 
-class RefsController(object):
+class RefsController:
 
     def __init__(self, BranchBrowserClass):
         self.BranchBrowserClass = BranchBrowserClass
 
     @expose()
-    def _lookup(self, ref, *remainder):
-        EOR = quote(c.app.END_OF_REF_ESCAPE)
+    def _lookup(self, ref=None, *remainder):
+        if ref is None:
+            raise exc.HTTPNotFound
+        EOR = c.app.END_OF_REF_ESCAPE
         if EOR in remainder:
-            i = remainder.index(quote(c.app.END_OF_REF_ESCAPE))
+            i = remainder.index(EOR)
             ref = '/'.join((ref,) + remainder[:i])
             remainder = remainder[i + 1:]
         return self.BranchBrowserClass(ref), remainder
 
 
-class CommitsController(object):
+class CommitsController:
 
     @expose()
-    def _lookup(self, ci, *remainder):
-        EOR = quote(c.app.END_OF_REF_ESCAPE)
+    def _lookup(self, ci=None, *remainder):
+        if ci is None:
+            raise exc.HTTPNotFound
+        ci = unquote(ci)
+        EOR = c.app.END_OF_REF_ESCAPE
         if EOR in remainder:
-            i = remainder.index(quote(c.app.END_OF_REF_ESCAPE))
+            i = remainder.index(EOR)
             ci = '/'.join((ci,) + remainder[:i])
             remainder = remainder[i + 1:]
         return CommitBrowser(ci), remainder
@@ -597,7 +690,10 @@ class CommitBrowser(BaseController):
         c.revision = revision
         if self._commit is None:
             raise exc.HTTPNotFound
-        self.tree = self.TreeBrowserClass(self._commit, tree=self._commit.tree)
+
+    @LazyProperty
+    def tree(self):
+        return self.TreeBrowserClass(self._commit, tree=self._commit.tree)
 
     @expose('jinja:allura:templates/repo/commit.html')
     @validate(dict(page=validators.Int(if_empty=0, if_invalid=0),
@@ -619,9 +715,17 @@ class CommitBrowser(BaseController):
                     filepath = f['new']
                 else:
                     filepath = f
-                is_text = filepath and tree.get_blob_by_path(filepath) and tree.get_blob_by_path(filepath).has_html_view
+                is_text = False
+                fileobj_type = 'tree'
+                if filepath:
+                    fileobj = tree.get_obj_by_path(filepath)
+                    if isinstance(fileobj, M.repository.Symlink):
+                        fileobj_type = 'symlink'
+                    elif isinstance(fileobj, M.repository.Blob):
+                        fileobj_type = 'blob'
+                        is_text = fileobj.has_html_view
                 result['artifacts'].append(
-                    (t, f, 'blob' if tree.get_blob_by_path(f) else 'tree', is_text)
+                    (t, f, fileobj_type, is_text)
                 )
         count = diffs['total']
         result.update(dict(page=page, limit=limit, count=count))
@@ -632,7 +736,7 @@ class CommitBrowser(BaseController):
         # ('removed', u'bbb.txt', 'tree', None),
         # ('removed', u'ddd.txt', 'tree', None),
         # ('changed', u'ccc.txt', 'blob', True)]
-        result['artifacts'].sort(key=lambda x: x[1]['old'] if(type(x[1]) == dict) else x[1])
+        result['artifacts'].sort(key=lambda x: x[1]['old'] if(isinstance(x[1], dict)) else x[1])
         return result
 
     @expose('jinja:allura:templates/repo/commit_basic.html')
@@ -652,7 +756,7 @@ class CommitBrowser(BaseController):
         status = c.app.repo.get_tarball_status(rev, path)
         if not status and request.method == 'POST':
             allura.tasks.repo_tasks.tarball.post(rev, path)
-            redirect('tarball' + '?path={0}'.format(path) if path else '')
+            redirect('tarball?path={}'.format(h.urlquote(path) if path else ''))
         return dict(commit=self._commit, revision=rev, status=status)
 
     @expose('json:')
@@ -665,8 +769,10 @@ class CommitBrowser(BaseController):
     @expose('jinja:allura:templates/repo/log.html')
     @with_trailing_slash
     @validate(dict(page=validators.Int(if_empty=0, if_invalid=0),
-                   limit=validators.Int(if_empty=25, if_invalid=25)))
-    def log(self, limit=25, path=None, **kw):
+                   limit=validators.Int(if_empty=0, if_invalid=0)))
+    def log(self, limit=0, path=None, **kw):
+        if not limit:
+            limit = int(tg.config.get('scm.view.log.limit', 25))
         is_file = False
         if path:
             is_file = c.app.repo.is_file(path, self._commit._id)
@@ -713,7 +819,7 @@ class TreeBrowser(BaseController, DispatchIndex):
             cutout = len('tree' + self._path)
             if request.path.endswith('/') and not self._path.endswith('/'):
                 cutout += 1
-            tarball_url = quote('%starball' % unquote(request.path)[:-cutout])
+            tarball_url = h.urlquote(request.path_info[:-cutout] + 'tarball')
         return dict(
             repo=c.app.repo,
             commit=self._commit,
@@ -728,9 +834,7 @@ class TreeBrowser(BaseController, DispatchIndex):
         next = h.really_unicode(unquote(next))
         if not rest:
             # Might be a file rather than a dir
-            filename = h.really_unicode(
-                unquote(
-                    request.environ['PATH_INFO'].rsplit('/')[-1]))
+            filename = h.really_unicode(request.path_info.rsplit('/')[-1])
             if filename:
                 try:
                     obj = self._tree[filename]
@@ -742,7 +846,7 @@ class TreeBrowser(BaseController, DispatchIndex):
                         self._tree,
                         filename), rest
         elif rest == ('index', ):
-            rest = (request.environ['PATH_INFO'].rsplit('/')[-1],)
+            rest = (request.path_info.rsplit('/')[-1],)
         try:
             tree = self._tree[next]
         except KeyError:
@@ -778,7 +882,13 @@ class FileBrowser(BaseController):
     @expose('jinja:allura:templates/repo/file.html')
     def index(self, **kw):
         if kw.pop('format', 'html') == 'raw':
-            return self.raw()
+            if self._blob.size > asint(tg.config.get('scm.download.max_file_bytes', 30*1000*1000)):
+                large_size = self._blob.size
+                flash(f'File is {h.do_filesizeformat(large_size)}.  Too large to download.',
+                      'warning', sticky=True)
+                raise exc.HTTPForbidden
+            else:
+                return self.raw()
         elif 'diff' in kw:
             tg.decorators.override_template(
                 self.index, 'jinja:allura:templates/repo/diff.html')
@@ -789,26 +899,31 @@ class FileBrowser(BaseController):
             return self.diff(kw['barediff'], kw.pop('diformat', None), kw.pop('prev_file', None))
         else:
             force_display = 'force' in kw
-            stats = utils.generate_code_stats(self._blob)
+            if self._blob.size > asint(tg.config.get('scm.view.max_file_bytes', 5*1000*1000)):
+                large_size = self._blob.size
+                stats = None
+            else:
+                large_size = False
+                stats = utils.generate_code_stats(self._blob)
             return dict(
                 blob=self._blob,
                 stats=stats,
-                force_display=force_display
+                force_display=force_display,
+                large_size=large_size,
             )
 
-    @expose()
     def raw(self, **kw):
-        content_type = self._blob.content_type.encode('utf-8')
-        filename = self._blob.name.encode('utf-8')
+        content_type = self._blob.content_type
+        filename = self._blob.name
         response.headers['Content-Type'] = ''
-        response.content_type = content_type
+        response.content_type = str(content_type)
         if self._blob.content_encoding is not None:
-            content_encoding = self._blob.content_encoding.encode('utf-8')
+            content_encoding = self._blob.content_encoding
             response.headers['Content-Encoding'] = ''
-            response.content_encoding = content_encoding
+            response.content_encoding = str(content_encoding)
         response.headers.add(
             'Content-Disposition',
-            'attachment;filename="%s"' % filename)
+            'attachment;filename="%s"' % h.urlquote(filename))
         return iter(self._blob)
 
     def diff(self, prev_commit, fmt=None, prev_file=None, **kw):
@@ -819,13 +934,15 @@ class FileBrowser(BaseController):
         :return:
         '''
         try:
-            path, filename = os.path.split(self._blob.path())
             a_ci = c.app.repo.commit(prev_commit)
             a = a_ci.get_path(prev_file or self._blob.path())
+            if not isinstance(a, M.repository.Blob):
+                # could be a Tree (directory) in the previous commit, can't diff that!
+                raise TypeError()
             apath = a.path()
-        except:
+        except Exception:
             # prev commit doesn't have the file
-            a = []
+            a = M.repository.EmptyBlob()
             apath = ''
         b = self._blob
 
@@ -833,20 +950,35 @@ class FileBrowser(BaseController):
             diff = "Cannot display: file marked as a binary type."
             return dict(a=a, b=b, diff=diff)
 
-        la = list(a)
-        lb = list(b)
-        adesc = (u'a' + h.really_unicode(apath)).encode('utf-8')
-        bdesc = (u'b' + h.really_unicode(b.path())).encode('utf-8')
+        if max(a.size, b.size) > asint(tg.config.get('scm.view.max_diff_bytes', 2000000)):
+            # have to check the original file size, not diff size, because difflib._mdiff inside HtmlSideBySideDiff
+            # can take an extremely long time on large files (and its even a generator)
+            diff = 'File too large to view diff'
+            return dict(a=a, b=b, diff=diff)
+
+        # could consider making Blob.__iter__ do unicode conversion?
+        la = [h.really_unicode(line) for line in a]
+        lb = [h.really_unicode(line) for line in b]
+        adesc = 'a' + h.really_unicode(apath)
+        bdesc = 'b' + h.really_unicode(b.path())
 
         if not fmt:
             fmt = web_session.get('diformat', '')
         else:
             web_session['diformat'] = fmt
             web_session.save()
+
         if fmt == 'sidebyside':
             hd = HtmlSideBySideDiff()
             diff = hd.make_table(la, lb, adesc, bdesc)
         else:
+            # always end with a newline, and indicate if original file didn't
+            # doesn't work well with sidebyside (above), but that's ok without this too
+            if la and not la[-1].endswith('\n'):
+                la[-1] += '\n\\ No newline at end of file\n'
+            if lb and not lb[-1].endswith('\n'):
+                lb[-1] += '\n\\ No newline at end of file\n'
+
             diff = ''.join(difflib.unified_diff(la, lb, adesc, bdesc))
         return dict(a=a, b=b, diff=diff)
 
@@ -867,4 +999,5 @@ def topo_sort(children, parents, dates, head_ids):
             else:
                 to_visit.append(p)
 
+
 on_import()
