@@ -25,12 +25,13 @@ from tg.decorators import with_trailing_slash, without_trailing_slash
 import bson
 import tg
 from paste.deploy.converters import aslist
-from pylons import app_globals as g
-from pylons import tmpl_context as c
-from pylons import request
+from tg import app_globals as g
+from tg import tmpl_context as c
+from tg import request
 from formencode import validators, Invalid
 from webob.exc import HTTPNotFound, HTTPFound
-from ming.odm import ThreadLocalORMSession
+from ming.odm import ThreadLocalODMSession
+import paginate
 
 from allura.app import SitemapEntry
 from allura.lib import helpers as h
@@ -38,7 +39,7 @@ from allura.lib import validators as v
 from allura.lib.decorators import require_post
 from allura.lib.plugin import SiteAdminExtension, ProjectRegistrationProvider, AuthenticationProvider
 from allura.lib import search
-from allura.lib.security import require_access, Credentials
+from allura.lib.security import require_site_admin, Credentials
 from allura.lib.widgets import form_fields as ffw
 from allura.ext.admin.widgets import AuditLog
 from allura.lib.widgets import forms
@@ -47,8 +48,8 @@ from allura.command.show_models import dfs, build_model_inheritance_graph
 from allura.scripts.delete_projects import DeleteProjects
 import allura
 
-from urlparse import urlparse
-from webhelpers import paginate
+from urllib.parse import urlparse
+import six
 
 
 log = logging.getLogger(__name__)
@@ -61,19 +62,18 @@ class W:
     admin_search_form = forms.AdminSearchForm
 
 
-class SiteAdminController(object):
+class SiteAdminController:
 
     def __init__(self):
         self.task_manager = TaskManagerController()
-        c.site_admin_sidebar_menu = self.sidebar_menu()
         self.user = AdminUserDetailsController()
         self.delete_projects = DeleteProjectsController()
         self.site_notifications = SiteNotificationController()
 
     def _check_security(self):
-        with h.push_context(config.get('site_admin_project', 'allura'),
-                            neighborhood=config.get('site_admin_project_nbhd', 'Projects')):
-            require_access(c.project, 'admin')
+        require_site_admin(c.user)
+
+        c.site_admin_sidebar_menu = self.sidebar_menu()
 
     @expose()
     def _lookup(self, name, *remainder):
@@ -82,7 +82,7 @@ class SiteAdminController(object):
             controller = admin_extension().controllers.get(name)
             if controller:
                 return controller(), remainder
-        raise HTTPNotFound, name
+        raise HTTPNotFound(name)
 
     def sidebar_menu(self):
         base_url = '/nf/admin/'
@@ -160,8 +160,8 @@ class SiteAdminController(object):
 
             try:
                 ok = self.subscribe_artifact(url, user)
-            except:
-                log.warn("Can't subscribe to artifact", exc_info=True)
+            except Exception:
+                log.warning("Can't subscribe to artifact", exc_info=True)
                 ok = False
 
             if ok:
@@ -219,12 +219,12 @@ class SiteAdminController(object):
                    mount_point=validators.NotEmpty()))
     def reclone_repo(self, prefix=None, shortname=None, mount_point=None, **data):
         if request.method == 'POST':
-            if c.form_errors:
+            if request.validation.errors:
                 error_msg = 'Error: '
-                for msg in list(c.form_errors):
+                for msg in list(request.validation.errors):
                     names = {'prefix': 'Neighborhood prefix', 'shortname':
                              'Project shortname', 'mount_point': 'Repository mount point'}
-                    error_msg += '%s: %s ' % (names[msg], c.form_errors[msg])
+                    error_msg += f'{names[msg]}: {request.validation.errors[msg]} '
                     flash(error_msg, 'error')
                 return dict(prefix=prefix, shortname=shortname, mount_point=mount_point)
             nbhd = M.Neighborhood.query.get(url_prefix='/%s/' % prefix)
@@ -268,6 +268,9 @@ class SiteAdminController(object):
         objects = []
         limit, page, start = g.handle_paging(limit, page, default=25)
         if q:
+            if f in ('username', 'shortname'):
+                # these are always lowercase, so search by lowercase
+                q = q.lower()
             match = search.site_admin_search(model, q, f, rows=limit, start=start)
             if match:
                 count = match.hits
@@ -285,7 +288,7 @@ class SiteAdminController(object):
         def convert_fields(obj):
             # throw the type away (e.g. '_s' from 'url_s')
             result = {}
-            for k,val in obj.iteritems():
+            for k,val in obj.items():
                 name = k.rsplit('_', 1)
                 if len(name) == 2:
                     name = name[0]
@@ -297,7 +300,7 @@ class SiteAdminController(object):
         return {
             'q': q,
             'f': f,
-            'objects': map(convert_fields, objects),
+            'objects': list(map(convert_fields, objects)),
             'count': count,
             'page': page,
             'limit': limit,
@@ -308,7 +311,7 @@ class SiteAdminController(object):
 
     @without_trailing_slash
     @expose('jinja:allura:templates/site_admin_search.html')
-    @validate(validators=dict(q=validators.UnicodeString(if_empty=None),
+    @validate(validators=dict(q=v.UnicodeString(if_empty=None),
                               limit=validators.Int(if_invalid=None),
                               page=validators.Int(if_empty=0, if_invalid=0)))
     def search_projects(self, q=None, f=None, page=0, limit=None, **kw):
@@ -323,7 +326,7 @@ class SiteAdminController(object):
 
     @without_trailing_slash
     @expose('jinja:allura:templates/site_admin_search.html')
-    @validate(validators=dict(q=validators.UnicodeString(if_empty=None),
+    @validate(validators=dict(q=v.UnicodeString(if_empty=None),
                               limit=validators.Int(if_invalid=None),
                               page=validators.Int(if_empty=0, if_invalid=0)))
     def search_users(self, q=None, f=None, page=0, limit=None, **kw):
@@ -338,10 +341,10 @@ class SiteAdminController(object):
         return r
 
 
-class DeleteProjectsController(object):
+class DeleteProjectsController:
     delete_form_validators = dict(
-        projects=validators.UnicodeString(if_empty=None),
-        reason=validators.UnicodeString(if_empty=None),
+        projects=v.UnicodeString(if_empty=None),
+        reason=v.UnicodeString(if_empty=None),
         disable_users=validators.StringBool(if_empty=False))
 
     def remove_comments(self, lines):
@@ -360,12 +363,12 @@ class DeleteProjectsController(object):
         return parsed_projects
 
     def format_parsed_projects(self, projects):
-        template = u'{}    # {}'
+        template = '{}    # {}'
         lines = []
         for input, p, error in projects:
-            comment = u'OK: {}'.format(p.url()) if p else error
+            comment = f'OK: {p.url()}' if p else error
             lines.append(template.format(input, comment))
-        return u'\n'.join(lines)
+        return '\n'.join(lines)
 
     @with_trailing_slash
     @expose('jinja:allura:templates/site_admin_delete_projects.html')
@@ -381,11 +384,11 @@ class DeleteProjectsController(object):
     @validate(validators=delete_form_validators)
     def confirm(self, projects=None, reason=None, disable_users=False, **kw):
         if not projects:
-            flash(u'No projects specified', 'warning')
+            flash('No projects specified', 'warning')
             redirect('.')
         parsed_projects = self.parse_projects(projects)
         projects = self.format_parsed_projects(parsed_projects)
-        edit_link = u'./?projects={}&reason={}&disable_users={}'.format(
+        edit_link = './?projects={}&reason={}&disable_users={}'.format(
             h.urlquoteplus(projects),
             h.urlquoteplus(reason or ''),
             h.urlquoteplus(disable_users))
@@ -401,24 +404,24 @@ class DeleteProjectsController(object):
     @validate(validators=delete_form_validators)
     def really_delete(self, projects=None, reason=None, disable_users=False, **kw):
         if not projects:
-            flash(u'No projects specified', 'warning')
+            flash('No projects specified', 'warning')
             redirect('.')
         projects = self.parse_projects(projects)
         task_params = [p.url().strip('/') for (_, p, _) in projects if p]
         if not task_params:
-            flash(u'Unable to parse at least one project from your input', 'warning')
+            flash('Unable to parse at least one project from your input', 'warning')
             redirect('.')
-        task_params = u' '.join(task_params)
+        task_params = ' '.join(task_params)
         if reason:
-            task_params = u'-r {} {}'.format(pipes.quote(reason), task_params)
+            task_params = f'-r {pipes.quote(reason)} {task_params}'
         if disable_users:
-            task_params = u'--disable-users {}'.format(task_params)
+            task_params = f'--disable-users {task_params}'
         DeleteProjects.post(task_params)
-        flash(u'Delete scheduled', 'ok')
+        flash('Delete scheduled', 'ok')
         redirect('.')
 
 
-class SiteNotificationController(object):
+class SiteNotificationController:
 
     def __init__(self, note=None):
         self.note = note
@@ -452,8 +455,8 @@ class SiteNotificationController(object):
     def new(self, **kw):
         """Render the New SiteNotification form"""
         return dict(
-            form_errors=c.form_errors or {},
-            form_values=c.form_values or {},
+            form_errors=request.validation.errors or {},
+            form_values=request.validation.values or {},
             form_title='New Site Notification',
             form_action='create'
         )
@@ -467,15 +470,15 @@ class SiteNotificationController(object):
             active=active, impressions=impressions, content=content,
             user_role=user_role, page_regex=page_regex,
             page_tool_type=page_tool_type)
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
         redirect('../site_notifications')
 
     @expose('jinja:allura:templates/site_admin_site_notifications_create_update.html')
     def edit(self, **kw):
-        if c.form_values:
+        if request.validation.values:
             return dict(
-                form_errors=c.form_errors or {},
-                form_values=c.form_values or {},
+                form_errors=request.validation.errors or {},
+                form_values=request.validation.values or {},
                 form_title='Edit Site Notification',
                 form_action='update'
             )
@@ -501,23 +504,21 @@ class SiteNotificationController(object):
         self.note.user_role = user_role
         self.note.page_regex = page_regex
         self.note.page_tool_type = page_tool_type
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
         redirect('..')
 
     @expose()
     @require_post()
     def delete(self):
         self.note.delete()
-        ThreadLocalORMSession().flush_all()
-        redirect(request.referer)
+        ThreadLocalODMSession().flush_all()
+        redirect(six.ensure_text(request.referer or '/'))
 
 
-class TaskManagerController(object):
+class TaskManagerController:
 
     def _check_security(self):
-        with h.push_context(config.get('site_admin_project', 'allura'),
-                            neighborhood=config.get('site_admin_project_nbhd', 'Projects')):
-            require_access(c.project, 'admin')
+        require_site_admin(c.user)
 
     @expose('jinja:allura:templates/site_admin_task_list.html')
     @without_trailing_slash
@@ -582,8 +583,8 @@ class TaskManagerController(object):
     def new(self, **kw):
         """Render the New Task form"""
         return dict(
-            form_errors=c.form_errors or {},
-            form_values=c.form_values or {},
+            form_errors=request.validation.errors or {},
+            form_values=request.validation.values or {},
         )
 
     @expose()
@@ -619,12 +620,15 @@ class TaskManagerController(object):
         try:
             task = v.TaskValidator.to_python(task_name)
             doc = task.__doc__ or 'No doc string available'
+            doc = re.sub(r'^usage: ([^-][a-z_-]+ )?',  # remove usage: and possible incorrect binary like "mod_wsgi"
+                         'Enter CLI formatted args above, like "args": ["--foo bar baz"]\n\n',
+                         doc)
         except Invalid as e:
             error = str(e)
         return dict(doc=doc, error=error)
 
 
-class StatsController(object):
+class StatsController:
     """Show neighborhood stats."""
     @expose('jinja:allura:templates/site_admin_stats.html')
     @with_trailing_slash
@@ -640,9 +644,10 @@ class StatsController(object):
         return dict(neighborhoods=neighborhoods)
 
 
-class AdminUserDetailsController(object):
+class AdminUserDetailsController:
 
     @expose('jinja:allura:templates/site_admin_user_details.html')
+    @without_trailing_slash
     def _default(self, username, limit=25, page=0):
         user = M.User.by_username(username)
         if not user or user.is_anonymous():
@@ -693,8 +698,8 @@ class AdminUserDetailsController(object):
             M.AuditLog.comment_user(c.user, comment, user=user)
             flash('Comment added', 'ok')
         else:
-            flash('Can not add comment "%s" for user %s' % (comment, user))
-        redirect(request.referer)
+            flash(f'Can not add comment "{comment}" for user {user}')
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     @require_post()
@@ -703,17 +708,23 @@ class AdminUserDetailsController(object):
         if not user or user.is_anonymous():
             raise HTTPNotFound()
         if status == 'enable' and (user.disabled or user.pending):
-            AuthenticationProvider.get(request).activate_user(user, audit=False)
-            AuthenticationProvider.get(request).enable_user(user)
+            if user.pending:
+                AuthenticationProvider.get(request).activate_user(user,
+                                                                  audit=not user.disabled)  # avoid dupe audits
+            if user.disabled:
+                AuthenticationProvider.get(request).enable_user(user)
             flash('User enabled')
         elif status == 'disable' and not user.disabled:
             AuthenticationProvider.get(request).disable_user(user)
             flash('User disabled')
         elif status == 'pending':
-            AuthenticationProvider.get(request).deactivate_user(user)
-            AuthenticationProvider.get(request).enable_user(user, audit=False)
+            if user.disabled:
+                AuthenticationProvider.get(request).enable_user(user,
+                                                                audit=user.pending)  # skip dupe audits
+            if not user.pending:
+                AuthenticationProvider.get(request).deactivate_user(user)
             flash('Set user status to pending')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     @require_post()
@@ -725,7 +736,7 @@ class AdminUserDetailsController(object):
         AuthenticationProvider.get(request).set_password(user, None, pwd)
         h.auditlog_user('Set random password', user=user)
         flash('Password is set', 'ok')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     @require_post()
@@ -738,7 +749,16 @@ class AdminUserDetailsController(object):
             allura.controllers.auth.AuthController().password_recovery_hash(email)
         except HTTPFound:
             pass  # catch redirect to '/'
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
+
+    @expose()
+    @require_post()
+    def make_password_reset_url(self, username):
+        user = M.User.by_username(username)
+        if not user or user.is_anonymous():
+            raise HTTPNotFound()
+        h.auditlog_user('Generated new password reset URL and shown to admin user', user=user)
+        return user.make_password_reset_url()
 
     @h.vardec
     @expose()
@@ -748,7 +768,7 @@ class AdminUserDetailsController(object):
         if not user or user.is_anonymous():
             raise HTTPNotFound()
         allura.controllers.auth.PreferencesController()._update_emails(user, admin=True, form_params=kw)
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class StatsSiteAdminExtension(SiteAdminExtension):
