@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -16,16 +14,18 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
+from __future__ import annotations
 
+import re
 
 """The application's Globals object"""
 
 import logging
-import cgi
+import html
 import hashlib
 import json
 import datetime
-from urllib import urlencode
+from urllib.parse import urlencode
 from subprocess import Popen, PIPE
 import os
 import time
@@ -39,69 +39,96 @@ import pygments.lexers
 import pygments.formatters
 import pygments.util
 from tg import config
-from pylons import request
-from pylons import tmpl_context as c
+from tg import request
+from tg import tmpl_context as c
 from paste.deploy.converters import asbool, asint, aslist
 from pypeline.markup import markup as pypeline_markup
-from ming.odm import session
+from ming.odm import session, MappedClass
 
 import ew as ew_core
 import ew.jinja2_ew as ew
 from ming.utils import LazyProperty
-from jinja2 import Markup
+from markupsafe import Markup
 
 import allura.tasks.event_tasks
 from allura import model as M
 from allura.lib.markdown_extensions import (
     ForgeExtension,
     CommitMessageExtension,
+    EmojiExtension,
+    UserMentionExtension
 )
 from allura.eventslistener import PostEvent
 
 from allura.lib import gravatar, plugin, utils
 from allura.lib import helpers as h
+from allura.lib.macro import uncacheable_macros_names
 from allura.lib.widgets import analytics
 from allura.lib.security import Credentials
 from allura.lib.solr import MockSOLR, make_solr_from_config
 from allura.model.session import artifact_orm_session
+import six
 
 __all__ = ['Globals']
 
 log = logging.getLogger(__name__)
 
 
-class ForgeMarkdown(markdown.Markdown):
+class ForgeMarkdown:
+
+    def __init__(self, **forge_ext_kwargs):
+        self.forge_ext_kwargs = forge_ext_kwargs
+
+    def make_markdown_instance(self, **forge_ext_kwargs):
+        # for further performance improvements, this might be cachable including forge_ext_kwargs?
+        # but ForgeExtension and the classes it makes often save things on `self` which might not be thread safe
+        # when multiple threads are using the same instance
+        # TestCachedMarkdown re-uses the same instance previously, and had a bug that AddCustomClass wasn't running
+        return markdown.Markdown(
+            extensions=['markdown.extensions.fenced_code', 'markdown.extensions.codehilite',
+                        'markdown.extensions.abbr', 'markdown.extensions.def_list', 'markdown.extensions.footnotes',
+                        'markdown.extensions.md_in_html',
+                        ForgeExtension(**forge_ext_kwargs),
+                        EmojiExtension(),
+                        UserMentionExtension(),
+                        'markdown.extensions.tables', 'markdown.extensions.toc', 'markdown.extensions.nl2br',
+                        'markdown_checklist.extension'],
+            output_format='html')
 
     def convert(self, source, render_limit=True):
-        if render_limit and len(source) > asint(config.get('markdown_render_max_length', 40000)):
+        if render_limit and len(source) > asint(config.get('markdown_render_max_length', 80000)):
             # if text is too big, markdown can take a long time to process it,
             # so we return it as a plain text
             log.info('Text is too big. Skipping markdown processing')
-            escaped = cgi.escape(h.really_unicode(source))
-            return h.html.literal(u'<pre>%s</pre>' % escaped)
+            escaped = html.escape(h.really_unicode(source))
+            return Markup('<pre>%s</pre>' % escaped)
         try:
-            return markdown.Markdown.convert(self, source)
+            return self.make_markdown_instance(**self.forge_ext_kwargs).convert(source)
         except Exception:
             log.info('Invalid markdown: %s  Upwards trace is %s', source,
                      ''.join(traceback.format_stack()), exc_info=True)
             escaped = h.really_unicode(source)
-            escaped = cgi.escape(escaped)
-            return h.html.literal(u"""<p><strong>ERROR!</strong> The markdown supplied could not be parsed correctly.
+            escaped = html.escape(escaped)
+            return Markup("""<p><strong>ERROR!</strong> The markdown supplied could not be parsed correctly.
             Did you forget to surround a code snippet with "~~~~"?</p><pre>%s</pre>""" % escaped)
 
-    def cached_convert(self, artifact, field_name):
-        """Convert ``artifact.field_name`` markdown source to html, caching
+    @LazyProperty
+    def uncacheable_macro_regex(self):
+        regex_names = '|'.join(uncacheable_macros_names())
+        return re.compile(rf"\[\[\s*({regex_names})\b")
+
+    def cached_convert(self, artifact: MappedClass, field_name: str) -> str:
+        """
+        Convert ``artifact.field_name`` markdown source to html, caching
         the result if the render time is greater than the defined threshold.
 
+        :param artifact: often an artifact, but also can be Neighborhood, OAuthConsumerToken, etc
         """
         source_text = getattr(artifact, field_name)
-        # Check if contents macro and never cache
-        if "[[" in source_text:
-            return self.convert(source_text)
         cache_field_name = field_name + '_cache'
         cache = getattr(artifact, cache_field_name, None)
         if not cache:
-            log.warn(
+            log.warning(
                 'Skipping Markdown caching - Missing cache field "%s" on class %s',
                 field_name, artifact.__class__.__name__)
             return self.convert(source_text)
@@ -112,7 +139,7 @@ class ForgeMarkdown(markdown.Markdown):
         if cache.md5 is not None:
             md5 = hashlib.md5(source_text.encode('utf-8')).hexdigest()
             if cache.md5 == md5 and getattr(cache, 'fix7528', False) == bugfix_rev:
-                return h.html.literal(cache.html)
+                return Markup(cache.html)
 
         # Convert the markdown and time the result.
         start = time.time()
@@ -124,9 +151,23 @@ class ForgeMarkdown(markdown.Markdown):
             threshold = float(threshold) if threshold else None
         except ValueError:
             threshold = None
-            log.warn('Skipping Markdown caching - The value for config param '
+            log.warning('Skipping Markdown caching - The value for config param '
                      '"markdown_cache_threshold" must be a float.')
 
+        # Check if contains macro and never cache
+        if self.uncacheable_macro_regex.search(source_text):
+            if render_time > float(config.get('markdown_cache_threshold.nocache', 0.5)):
+                try:
+                    details = artifact.index_id()
+                except Exception:
+                    details = str(artifact._id)
+                try:
+                    details += ' ' + artifact.url()
+                except Exception:
+                    pass
+                log.info(f'Not saving markdown cache since it has a dynamic macro.  Took {render_time:.03}s on {details}')
+            return html
+
         if threshold is not None and render_time > threshold:
             # Save the cache
             if md5 is None:
@@ -134,13 +175,19 @@ class ForgeMarkdown(markdown.Markdown):
             cache.md5, cache.html, cache.render_time = md5, html, render_time
             cache.fix7528 = bugfix_rev  # flag to indicate good caches created after [#7528] and other critical bugs were fixed.
 
-            with utils.skip_mod_date(artifact.__class__), \
-                 utils.skip_last_updated(artifact.__class__):
-                session(artifact).flush(artifact)
+            try:
+                sess = session(artifact)
+            except AttributeError:
+                # this can happen if a non-artifact object is used
+                log.exception('Could not get session for %s', artifact)
+            else:
+                with utils.skip_mod_date(artifact.__class__), \
+                     utils.skip_last_updated(artifact.__class__):
+                    sess.flush(artifact)
         return html
 
 
-class Globals(object):
+class Globals:
 
     """Container for objects available throughout the life of the application.
 
@@ -189,7 +236,7 @@ class Globals(object):
         self.pypeline_markup = pypeline_markup
 
         # Setup analytics
-        accounts = config.get('ga.account', 'UA-XXXXX-X')
+        accounts = config.get('ga.account', '')
         accounts = accounts.split(' ')
         self.analytics = analytics.GoogleAnalytics(accounts=accounts)
 
@@ -256,7 +303,8 @@ class Globals(object):
                     value = ep.load()
                 except Exception:
                     log.exception('Could not load entry point [%s] %s', section_name, ep)
-                d[ep.name] = value
+                else:
+                    d[ep.name] = value
             return d
 
         class entry_point_loading_dict(dict):
@@ -287,7 +335,7 @@ class Globals(object):
 
         # Set listeners to update stats
         statslisteners = []
-        for name, ep in self.entry_points['stats'].iteritems():
+        for name, ep in self.entry_points['stats'].items():
             statslisteners.append(ep())
         self.statsUpdater = PostEvent(statslisteners)
 
@@ -312,7 +360,7 @@ class Globals(object):
         if asbool(config.get('activitystream.recording.enabled', False)):
             return activitystream.director()
         else:
-            class NullActivityStreamDirector(object):
+            class NullActivityStreamDirector:
 
                 def connect(self, *a, **kw):
                     pass
@@ -337,6 +385,20 @@ class Globals(object):
             return NullActivityStreamDirector()
 
     def post_event(self, topic, *args, **kwargs):
+        if 'flush_immediately' not in kwargs:
+            try:
+                env = request.environ
+            except AttributeError:
+                script_without_ming_middleware = True
+            else:
+                script_without_ming_middleware = env['PATH_INFO'] == '--script--'
+            if script_without_ming_middleware:
+                kwargs['flush_immediately'] = True
+            else:
+                # within tasks and web requests, ming middleware will flush everything to mongo
+                # so best to *not* flush immediately and let all db writes happen in order
+                # so there's no chance of an event being created and started while the initiating code is still running
+                kwargs['flush_immediately'] = False
         allura.tasks.event_tasks.event.post(topic, *args, **kwargs)
 
     @LazyProperty
@@ -358,7 +420,10 @@ class Globals(object):
         limit = self.manage_paging_preference(limit, default)
         limit = max(int(limit), 1)
         limit = min(limit, asint(config.get('limit_param_max', 500)))
-        page = max(int(page), 0)
+        try:
+            page = max(int(page), 0)
+        except ValueError:
+            page = 0
         start = page * int(limit)
         return (limit, page, start)
 
@@ -368,7 +433,11 @@ class Globals(object):
                 limit = default
             else:
                 limit = c.user.get_pref('results_per_page') or default
-        return int(limit)
+        try:
+            limit = int(limit)
+        except ValueError:
+            limit = default
+        return limit
 
     def document_class(self, neighborhood):
         classes = ''
@@ -385,50 +454,43 @@ class Globals(object):
     def highlight(self, text, lexer=None, filename=None):
         if not text:
             if lexer == 'diff':
-                return h.html.literal('<em>File contents unchanged</em>')
-            return h.html.literal('<em>Empty file</em>')
+                return Markup('<em>File contents unchanged</em>')
+            return Markup('<em>Empty file</em>')
         # Don't use line numbers for diff highlight's, as per [#1484]
         if lexer == 'diff':
-            formatter = pygments.formatters.HtmlFormatter(
-                cssclass='codehilite', linenos=False)
+            formatter = pygments.formatters.HtmlFormatter(cssclass='codehilite', linenos=False)
         else:
             formatter = self.pygments_formatter
+        text = h.really_unicode(text)
         if lexer is None:
-            try:
-                lexer = pygments.lexers.get_lexer_for_filename(
-                    filename, encoding='chardet')
-            except pygments.util.ClassNotFound:
-                # no highlighting, but we should escape, encode, and wrap it in
-                # a <pre>
-                text = h.really_unicode(text)
-                text = cgi.escape(text)
-                return h.html.literal(u'<pre>' + text + u'</pre>')
+            if len(text) < asint(config.get('scm.view.max_syntax_highlight_bytes', 500000)):
+                try:
+                    lexer = pygments.lexers.get_lexer_for_filename(filename, encoding='chardet')
+                except pygments.util.ClassNotFound:
+                    pass
         else:
-            lexer = pygments.lexers.get_lexer_by_name(
-                lexer, encoding='chardet')
-        return h.html.literal(pygments.highlight(text, lexer, formatter))
+            lexer = pygments.lexers.get_lexer_by_name(lexer, encoding='chardet')
 
-    def forge_markdown(self, **kwargs):
-        '''return a markdown.Markdown object on which you can call convert'''
-        return ForgeMarkdown(
-            # 'fenced_code'
-            extensions=['fenced_code', 'codehilite',
-                        ForgeExtension(
-                            **kwargs), 'tables', 'toc', 'nl2br'],
-            output_format='html4')
+        if lexer is None or len(text) >= asint(config.get('scm.view.max_syntax_highlight_bytes', 500000)):
+            # no highlighting, but we should escape, encode, and wrap it in
+            # a <pre>
+            text = html.escape(text)
+            return Markup('<pre>' + text + '</pre>')
+        else:
+            return Markup(pygments.highlight(text, lexer, formatter))
 
     @property
     def markdown(self):
-        return self.forge_markdown()
+        return ForgeMarkdown()
 
     @property
     def markdown_wiki(self):
-        if c.project.is_nbhd_project:
-            return self.forge_markdown(wiki=True, macro_context='neighborhood-wiki')
-        elif c.project.is_user_project:
-            return self.forge_markdown(wiki=True, macro_context='userproject-wiki')
+        if c.project and c.project.is_nbhd_project:
+            return ForgeMarkdown(wiki=True, macro_context='neighborhood-wiki')
+        elif c.project and c.project.is_user_project:
+            return ForgeMarkdown(wiki=True, macro_context='userproject-wiki')
         else:
-            return self.forge_markdown(wiki=True)
+            return ForgeMarkdown(wiki=True)
 
     @property
     def markdown_commit(self):
@@ -436,12 +498,14 @@ class Globals(object):
 
         """
         app = getattr(c, 'app', None)
-        return ForgeMarkdown(extensions=[CommitMessageExtension(app), 'nl2br'],
-                             output_format='html4')
+        return markdown.Markdown(
+            extensions=[CommitMessageExtension(app), EmojiExtension(), 'markdown.extensions.nl2br'],
+            output_format='html',
+        )
 
     @property
     def production_mode(self):
-        return asbool(config.get('debug')) == False
+        return asbool(config.get('debug')) is False
 
     @LazyProperty
     def user_message_time_interval(self):
@@ -464,7 +528,7 @@ class Globals(object):
         p1 = Popen(['hostname', '-s'], stdout=PIPE)
         server_name = p1.communicate()[0].strip()
         p1.wait()
-        return server_name
+        return six.ensure_text(server_name)
 
     @property
     def tool_icon_css(self):
@@ -479,7 +543,7 @@ class Globals(object):
                 url = self.theme.app_icon_url(tool_name.lower(), size)
                 css += '.ui-icon-tool-%s-%i {background: url(%s) no-repeat;}\n' % (
                     tool_name, size, url)
-        return css, hashlib.md5(css).hexdigest()
+        return css, hashlib.md5(css.encode('utf-8')).hexdigest()
 
     @property
     def resource_manager(self):
@@ -500,12 +564,12 @@ class Globals(object):
     def register_app_css(self, href, **kw):
         app = kw.pop('app', c.app)
         self.resource_manager.register(
-            ew.CSSLink('tool/%s/%s' % (app.config.tool_name.lower(), href), **kw))
+            ew.CSSLink(f'tool/{app.config.tool_name.lower()}/{href}', **kw))
 
     def register_app_js(self, href, **kw):
         app = kw.pop('app', c.app)
         self.resource_manager.register(
-            ew.JSLink('tool/%s/%s' % (app.config.tool_name.lower(), href), **kw))
+            ew.JSLink(f'tool/{app.config.tool_name.lower()}/{href}', **kw))
 
     def register_theme_css(self, href, **kw):
         self.resource_manager.register(ew.CSSLink(self.theme_href(href), **kw))
@@ -525,6 +589,13 @@ class Globals(object):
             base = request.scheme + base
         return base + resource
 
+    @property
+    def user_profile_urls_with_profile_path(self):
+        return asbool(config['user_profile_url_with_profile_path'])
+
+    def user_profile_disabled_tools(self):
+        return aslist(config.get('user_prefs.disabled_tools',''), sep=',')
+
     def app_static(self, resource, app=None):
         base = config['static.url_base']
         app = app or c.app
@@ -536,7 +607,7 @@ class Globals(object):
         'h.set_context() is preferred over this method'
         if isinstance(pid_or_project, M.Project):
             c.project = pid_or_project
-        elif isinstance(pid_or_project, basestring):
+        elif isinstance(pid_or_project, str):
             raise TypeError('need a Project instance, got %r' % pid_or_project)
         elif pid_or_project is None:
             c.project = None
@@ -548,17 +619,12 @@ class Globals(object):
         'h.set_context() is preferred over this method'
         c.app = c.project.app_instance(name)
 
-    def postload_contents(self):
-        text = '''
-'''
-        return json.dumps(dict(text=text))
-
     def year(self):
         return datetime.datetime.utcnow().year
 
     @LazyProperty
     def noreply(self):
-        return unicode(config.get('noreply', 'noreply@%s' % config['domain']))
+        return str(config.get('noreply', 'noreply@%s' % config['domain']))
 
     @property
     def build_key(self):
@@ -587,7 +653,7 @@ class Globals(object):
             return False
 
         allura_path = os.path.dirname(os.path.dirname(__file__))
-        image_full_path = '%s/public/nf/images/%s' % (
+        image_full_path = '{}/public/nf/images/{}'.format(
             allura_path, logo['image_path'])
 
         if not os.path.isfile(image_full_path):
@@ -602,12 +668,15 @@ class Globals(object):
             "image_height": logo['image_height']
         }
 
+    @property
+    def commit_statuses_enabled(self):
+        return asbool(config['scm.commit_statuses'])
 
-class Icon(object):
+class Icon:
 
     def __init__(self, css, title=None):
         self.css = css
-        self.title = title or u''
+        self.title = title or ''
 
     def render(self, show_title=False, extra_css=None, closing_tag=True, tag='a', **kw):
         title = kw.get('title') or self.title
@@ -619,9 +688,9 @@ class Icon(object):
             attrs['href'] = '#'
         attrs.update(kw)
         attrs = ew._Jinja2Widget().j2_attrs(attrs)
-        visible_title = u''
+        visible_title = ''
         if show_title:
-            visible_title = u'&nbsp;{}'.format(Markup.escape(title))
-        closing_tag = u'</{}>'.format(tag) if closing_tag else u''
-        icon = u'<{} {}><i class="{}"></i>{}{}'.format(tag, attrs, self.css, visible_title, closing_tag)
+            visible_title = f'&nbsp;{Markup.escape(title)}'
+        closing_tag = f'</{tag}>' if closing_tag else ''
+        icon = f'<{tag} {attrs}><i class="{self.css}"></i>{visible_title}{closing_tag}'
         return Markup(icon)
