@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -16,33 +14,38 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
+from __future__ import annotations
+
 import base64
 import sys
 import os
 import os.path
 import difflib
-import urllib
-import urllib2
+import jinja2
+
+import urllib.request
+import urllib.parse
+import urllib.error
 import re
+import unicodedata
 import json
 import logging
 import string
 import random
-import cPickle as pickle
+import pickle as pickle
 from hashlib import sha1
 from datetime import datetime, timedelta
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 import shlex
 import socket
 from functools import partial
-from cStringIO import StringIO
+from io import BytesIO
+import html
 
+import emoji
 import tg
-import genshi.template
-try:
-    import cchardet as chardet
-except ImportError:
-    import chardet
+import six
+import cchardet as chardet
 import pkg_resources
 from formencode.validators import FancyValidator
 from dateutil.parser import parse
@@ -50,24 +53,28 @@ from bson import ObjectId
 from paste.deploy import appconfig
 from pymongo.errors import InvalidId
 from contextlib import contextmanager
-from pylons import tmpl_context as c, app_globals as g
-from pylons import response, request
+from tg import tmpl_context as c, app_globals as g
+from tg import response, request
 from tg.decorators import before_validate
 from formencode.variabledecode import variable_decode
 import formencode
-from jinja2 import Markup
-from jinja2.filters import contextfilter, escape
+from markupsafe import Markup
+from jinja2.filters import escape, do_filesizeformat
+from jinja2.utils import pass_context, htmlsafe_json_dumps
 from paste.deploy.converters import asbool, aslist, asint
-
-from webhelpers import date, feedgenerator, html, number, misc, text
+from webhelpers2 import date, text
 from webob.exc import HTTPUnauthorized
 
 from allura.lib import exceptions as exc
-from allura.lib import AsciiDammit
 from allura.lib import utils
+from allura.lib import validators
+import urllib.parse as urlparse
+from urllib.parse import urlencode
+import math
+from webob.multidict import MultiDict
 
 # import to make available to templates, don't delete:
-from .security import has_access, is_allowed_by_role
+from .security import has_access, is_allowed_by_role, is_site_admin
 
 
 log = logging.getLogger(__name__)
@@ -130,7 +137,7 @@ def make_safe_path_portion(ustr, relaxed=True):
     """Return an ascii representation of ``ustr`` that conforms to mount point
     naming :attr:`rules <re_tool_mount_point_fragment>`.
 
-    Will return an empty string if no char in ``ustr`` is latin1-encodable.
+    Will return an empty string if no char in ``ustr`` is ascii-encodable.
 
     :param relaxed: Use relaxed mount point naming rules (allows more
         characters. See :attr:`re_relaxed_tool_mount_point_fragment`.
@@ -140,8 +147,8 @@ def make_safe_path_portion(ustr, relaxed=True):
     regex = (re_relaxed_tool_mount_point_fragment if relaxed else
              re_tool_mount_point_fragment)
     ustr = really_unicode(ustr)
-    s = ustr.encode('latin1', 'ignore')
-    s = AsciiDammit.asciiDammit(s)
+    s = ustr.encode('ascii', 'ignore')
+    s = six.ensure_text(s)
     if not relaxed:
         s = s.lower()
     s = '-'.join(regex.findall(s))
@@ -149,8 +156,43 @@ def make_safe_path_portion(ustr, relaxed=True):
     return s
 
 
-def escape_json(data):
-    return json.dumps(data).replace('<', '\u003C')
+def escape_json(data) -> str:
+    # Templates should use `|tojson` instead of this
+    return str(htmlsafe_json_dumps(data))  # str() to keep previous behavior of being str, not MarkupSafe
+
+
+def querystring(request, url_params):
+    """
+    add/update/remove url parameters. When a value is set to None the key will
+    be removed from the final constructed url.
+
+    :param request: request object
+    :param url_params: dict with the params that should be updated/added/deleted.
+    :return: a full url with updated url parameters.
+    """
+    params = urlparse.parse_qs(request.query_string)
+    params.update(url_params)
+    for param in list(params.keys()):
+        if params[param] is None:
+            del params[param]
+    # flatten dict values
+    params = {k: v[0] if isinstance(v, list) else v for k, v in params.items()}
+    url_parts = urlparse.urlparse(request.url)
+    url = url_parts._replace(query=urlencode(params)).geturl()
+    return url
+
+
+def ceil(number):
+    return math.ceil(number)
+
+
+def strip_bad_unicode(s):
+    """
+    xml doesn't like some control characters: https://www.w3.org/TR/REC-xml/#charsets
+    :param s:
+    :return:
+    """
+    return re.sub('[\x00-\x08\x0B\x0C\x0E-\x1F]', '', s)
 
 
 def monkeypatch(*objs):
@@ -160,37 +202,61 @@ def monkeypatch(*objs):
     return patchem
 
 
-def urlquote(url, safe="/"):
+def urlquote(url, safe=b"/"):
     try:
-        return urllib.quote(str(url), safe=safe)
+        return urllib.parse.quote(str(url), safe=safe)
     except UnicodeEncodeError:
-        return urllib.quote(url.encode('utf-8'), safe=safe)
+        return urllib.parse.quote(url.encode('utf-8'), safe=safe)
 
 
-def urlquoteplus(url, safe=""):
+def urlquoteplus(url, safe=b""):
     try:
-        return urllib.quote_plus(str(url), safe=safe)
+        return urllib.parse.quote_plus(str(url), safe=safe)
     except UnicodeEncodeError:
-        return urllib.quote_plus(url.encode('utf-8'), safe=safe)
+        return urllib.parse.quote_plus(url.encode('utf-8'), safe=safe)
+
+
+def urlquote_path_only(url):
+    """
+    Given a relative url like /fö/bar/?sdf&sdf
+    urlquote only the path portion of it, leaving any querystring or target hash unquoted
+    :param url:
+    :return:
+    """
+    if '?' in url:
+        url_path, url_joiner, url_remainder = url.partition('?')
+    elif '#' in url:
+        url_path, url_joiner, url_remainder = url.partition('#')
+    else:
+        url_path = url
+        url_joiner = url_remainder = ''
+    return urlquote(url_path) + url_joiner + url_remainder
 
 
 def _attempt_encodings(s, encodings):
     if s is None:
-        return u''
+        return ''
     for enc in encodings:
         try:
             if enc is None:
-                return unicode(s)  # try default encoding
+                if six.PY3 and isinstance(s, bytes):
+                    # special handling for bytes (avoid b'asdf' turning into "b'asfd'")
+                    return s.decode('utf-8')
+                return str(s)  # try default encoding, and handle other types like int, etc
             else:
-                return unicode(s, enc)
+                return str(s, enc)
         except (UnicodeDecodeError, LookupError):
             pass
     # Return the repr of the str -- should always be safe
-    return unicode(repr(str(s)))[1:-1]
+    return str(repr(str(s)))[1:-1]
 
 
 def really_unicode(s):
+    if isinstance(s, str):
+        # default case.  Also lets Markup() instances be preserved
+        return s
     # Try to guess the encoding
+
     def encodings():
         yield None
         yield 'utf-8'
@@ -231,14 +297,6 @@ def make_neighborhoods(ids):
     return _make_xs('Neighborhood', ids)
 
 
-def make_projects(ids):
-    return _make_xs('Project', ids)
-
-
-def make_users(ids):
-    return _make_xs('User', ids)
-
-
 def make_roles(ids):
     return _make_xs('ProjectRole', ids)
 
@@ -247,9 +305,9 @@ def _make_xs(X, ids):
     from allura import model as M
     X = getattr(M, X)
     ids = list(ids)
-    results = dict(
-        (r._id, r)
-        for r in X.query.find(dict(_id={'$in': ids})))
+    results = {
+        r._id: r
+        for r in X.query.find(dict(_id={'$in': ids}))}
     result = (results.get(i) for i in ids)
     return (r for r in result if r is not None)
 
@@ -263,9 +321,11 @@ def make_app_admin_only(app):
 
 @contextmanager
 def push_config(obj, **kw):
+    # if you need similar for a dict, use mock.patch.dict
+
     saved_attrs = {}
     new_attrs = []
-    for k, v in kw.iteritems():
+    for k, v in kw.items():
         try:
             saved_attrs[k] = getattr(obj, k)
         except AttributeError:
@@ -274,7 +334,7 @@ def push_config(obj, **kw):
     try:
         yield obj
     finally:
-        for k, v in saved_attrs.iteritems():
+        for k, v in saved_attrs.items():
             setattr(obj, k, v)
         for k in new_attrs:
             delattr(obj, k)
@@ -288,12 +348,24 @@ def sharded_path(name, num_parts=2):
 
 
 def set_context(project_shortname_or_id, mount_point=None, app_config_id=None, neighborhood=None):
+    """
+    Set ``c.project`` and ``c.app`` globals
+
+    :param project_id: _id or shortname of a project
+    :type project_id: ObjectId|str
+    :param mount_point: mount point to set c.app by
+    :type mount_point: str
+    :param app_config_id: alternative to mount_point parameter
+    :type app_config_id: ObjectId|str
+    :param neighborhood: neighborhood full name, required if project is specified by shortname
+    :type neighborhood: str
+    """
     from allura import model
     try:
         p = model.Project.query.get(_id=ObjectId(str(project_shortname_or_id)))
     except InvalidId:
         p = None
-    if p is None and type(project_shortname_or_id) != ObjectId:
+    if p is None and not isinstance(project_shortname_or_id, ObjectId):
         if neighborhood is None:
             raise TypeError('neighborhood is required; it must not be None')
         if not isinstance(neighborhood, model.Neighborhood):
@@ -321,7 +393,7 @@ def set_context(project_shortname_or_id, mount_point=None, app_config_id=None, n
     if app_config_id is None:
         c.app = p.app_instance(mount_point)
     else:
-        if isinstance(app_config_id, basestring):
+        if isinstance(app_config_id, str):
             app_config_id = ObjectId(app_config_id)
         app_config = model.AppConfig.query.get(_id=app_config_id)
         c.app = p.app_instance(app_config)
@@ -329,6 +401,19 @@ def set_context(project_shortname_or_id, mount_point=None, app_config_id=None, n
 
 @contextmanager
 def push_context(project_id, mount_point=None, app_config_id=None, neighborhood=None):
+    """
+    A context manager to set ``c.project`` and ``c.app`` globals temporarily.
+    To set ``c.user`` or others, use ``push_config(c, user=...)``
+
+    :param project_id: _id or shortname of a project
+    :type project_id: ObjectId|str
+    :param mount_point: mount point to set c.app by
+    :type mount_point: str
+    :param app_config_id: alternative to mount_point parameter
+    :type app_config_id: ObjectId|str
+    :param neighborhood: neighborhood full name, required if project is specified by shortname
+    :type neighborhood: str
+    """
     project = getattr(c, 'project', ())
     app = getattr(c, 'app', ())
     set_context(project_id, mount_point, app_config_id, neighborhood)
@@ -348,16 +433,16 @@ def push_context(project_id, mount_point=None, app_config_id=None, neighborhood=
 def encode_keys(d):
     '''Encodes the unicode keys of d, making the result
     a valid kwargs argument'''
-    return dict(
-        (k.encode('utf-8'), v)
-        for k, v in d.iteritems())
+    return {
+        six.ensure_str(k): v
+        for k, v in d.items()}
 
 
 def vardec(fun):
     def vardec_hook(remainder, params):
-        new_params = variable_decode(dict(
-            (k, v) for k, v in params.items()
-            if re_clean_vardec_key.match(k)))
+        new_params = variable_decode({
+            k: v for k, v in params.items()
+            if re_clean_vardec_key.match(k)})
         params.update(new_params)
     before_validate(vardec_hook)(fun)
     return fun
@@ -373,7 +458,7 @@ def convert_bools(conf, prefix=''):
     :return: dict
     '''
     def convert_value(val):
-        if isinstance(val, basestring):
+        if isinstance(val, str):
             if val.strip().lower() == 'true':
                 return True
             elif val.strip().lower() == 'false':
@@ -382,7 +467,7 @@ def convert_bools(conf, prefix=''):
 
     return {
         k: (convert_value(v) if k.startswith(prefix) else v)
-        for k, v in conf.iteritems()
+        for k, v in conf.items()
     }
 
 
@@ -391,8 +476,10 @@ def nonce(length=4):
 
 
 def cryptographic_nonce(length=40):
+    rand_bytes = os.urandom(length)
+    rand_ints = tuple(rand_bytes)
     hex_format = '%.2x' * length
-    return hex_format % tuple(map(ord, os.urandom(length)))
+    return hex_format % rand_ints
 
 
 def random_password(length=20, chars=string.ascii_uppercase + string.digits):
@@ -403,13 +490,15 @@ def ago(start_time, show_date_after=7):
     """
     Return time since starting time as a rounded, human readable string.
     E.g., "3 hours ago"
+    Also works with future times
+    E.g., "in 3 hours"
     """
 
     if start_time is None:
         return 'unknown'
     granularities = ['century', 'decade', 'year', 'month', 'day', 'hour', 'minute', 'second']
     end_time = datetime.utcnow()
-    if show_date_after is not None and end_time - start_time > timedelta(days=show_date_after):
+    if show_date_after is not None and abs(end_time - start_time) > timedelta(days=show_date_after):
         return start_time.strftime('%Y-%m-%d')
 
     while True:
@@ -418,7 +507,11 @@ def ago(start_time, show_date_after=7):
         rounded_to_one_granularity = 'and' not in ago
         if rounded_to_one_granularity:
             break
-    return ago + ' ago'
+
+    if (end_time - start_time).total_seconds() >= 0:
+        return ago + ' ago'
+    else:
+        return 'in ' + ago
 
 
 def ago_ts(timestamp):
@@ -428,22 +521,22 @@ def ago_ts(timestamp):
 def ago_string(s):
     try:
         return ago(parse(s, ignoretz=True))
-    except (ValueError, AttributeError):
+    except (ValueError, AttributeError, TypeError):
         return 'unknown'
 
 
 class DateTimeConverter(FancyValidator):
 
-    def _to_python(self, value, state):
+    def _convert_to_python(self, value, state):
         try:
             return parse(value)
-        except ValueError:
+        except (ValueError, TypeError):
             if self.if_invalid != formencode.api.NoDefault:
                 return self.if_invalid
             else:
                 raise
 
-    def _from_python(self, value, state):
+    def _convert_from_python(self, value, state):
         return value.isoformat()
 
 
@@ -475,14 +568,18 @@ def diff_text(t1, t2, differ=None):
     if differ is None:
         differ = difflib.SequenceMatcher(None, t1_words, t2_words)
     result = []
+
+    def escape_list(words_list):
+        return [html.escape(words) for words in words_list]
+
     for tag, i1, i2, j1, j2 in differ.get_opcodes():
         if tag in ('delete', 'replace'):
-            result += ['<del>'] + t1_words[i1:i2] + ['</del>']
+            result += ['<del>'] + escape_list(t1_words[i1:i2]) + ['</del>']
         if tag in ('insert', 'replace'):
-            result += ['<ins>'] + t2_words[j1:j2] + ['</ins>']
+            result += ['<ins>'] + escape_list(t2_words[j1:j2]) + ['</ins>']
         if tag == 'equal':
-            result += t1_words[i1:i2]
-    return ' '.join(result).replace('\n', '<br/>\n')
+            result += escape_list(t1_words[i1:i2])
+    return Markup(' '.join(result).replace('\n', '<br/>\n'))
 
 
 def gen_message_id(_id=None):
@@ -493,29 +590,29 @@ def gen_message_id(_id=None):
     else:
         parts = ['mail']
     if getattr(c, 'app', None):
-        addr = '%s.%s' % (_id, c.app.config.options['mount_point'])
+        addr = '{}.{}'.format(_id, c.app.config.options['mount_point'])
     else:
         addr = _id
-    return '%s@%s.%s' % (
+    return '{}@{}.{}'.format(
         addr, '.'.join(reversed(parts)), tg.config['domain'])
 
 
 class ProxiedAttrMeta(type):
 
     def __init__(cls, name, bases, dct):
-        for v in dct.itervalues():
+        for v in dct.values():
             if isinstance(v, attrproxy):
                 v.cls = cls
 
 
-class attrproxy(object):
+class attrproxy:
     cls = None
 
     def __init__(self, *attrs):
         self.attrs = attrs
 
     def __repr__(self):
-        return '<attrproxy on %s for %s>' % (
+        return '<attrproxy on {} for {}>'.format(
             self.cls, self.attrs)
 
     def __get__(self, obj, klass=None):
@@ -536,18 +633,18 @@ class attrproxy(object):
 class promised_attrproxy(attrproxy):
 
     def __init__(self, promise, *attrs):
-        super(promised_attrproxy, self).__init__(*attrs)
+        super().__init__(*attrs)
         self._promise = promise
 
     def __repr__(self):
-        return '<promised_attrproxy for %s>' % (self.attrs,)
+        return f'<promised_attrproxy for {self.attrs}>'
 
     def __getattr__(self, name):
         cls = self._promise()
         return getattr(cls, name)
 
 
-class proxy(object):
+class proxy:
 
     def __init__(self, obj):
         self._obj = obj
@@ -565,31 +662,19 @@ class fixed_attrs_proxy(proxy):
     provided during object construction, returns it's value. Otherwise proxies
     to obj.
     """
+
     def __init__(self, obj, **kw):
         self._obj = obj
-        for k, v in kw.iteritems():
+        for k, v in kw.items():
             setattr(self, k, v)
 
 
-def render_genshi_plaintext(template_name, **template_vars):
-    assert os.path.exists(template_name)
-    fd = open(template_name)
-    try:
-        tpl_text = fd.read()
-    finally:
-        fd.close()
-    filepath = os.path.dirname(template_name)
-    tt = genshi.template.NewTextTemplate(tpl_text,
-                                         filepath=filepath, filename=template_name)
-    stream = tt.generate(**template_vars)
-    return stream.render(encoding='utf-8').decode('utf-8')
-
-
 @tg.expose(content_type='text/plain')
 def json_validation_error(controller, **kwargs):
+    exc = request.validation.exception
     result = dict(status='Validation Error',
-                  errors=c.validation_exception.unpack_errors(),
-                  value=c.validation_exception.value,
+                  errors={fld: str(err) for fld, err in exc.error_dict.items()},
+                  value=exc.value,
                   params=kwargs)
     response.status = 400
     return json.dumps(result, indent=2)
@@ -600,7 +685,7 @@ def pop_user_notifications(user=None):
     if user is None:
         user = c.user
     mbox = M.Mailbox.query.get(user_id=user._id, is_flash=True)
-    if mbox:
+    if mbox and mbox.queue:
         notifications = M.Notification.query.find(
             dict(_id={'$in': mbox.queue}))
         mbox.queue = []
@@ -616,116 +701,30 @@ def config_with_prefix(d, prefix):
     with the prefix stripped
     '''
     plen = len(prefix)
-    return dict((k[plen:], v) for k, v in d.iteritems()
-                if k.startswith(prefix))
+    return {k[plen:]: v for k, v in d.items()
+            if k.startswith(prefix)}
 
 
-@contextmanager
-def twophase_transaction(*engines):
-    connections = [
-        e.contextual_connect()
-        for e in engines]
-    txns = []
-    to_rollback = []
-    try:
-        for conn in connections:
-            txn = conn.begin_twophase()
-            txns.append(txn)
-            to_rollback.append(txn)
-        yield
-        to_rollback = []
-        for txn in txns:
-            txn.prepare()
-            to_rollback.append(txn)
-        for txn in txns:
-            txn.commit()
-    except:
-        for txn in to_rollback:
-            txn.rollback()
-        raise
-
-
-class log_action(object):
-    extra_proto = dict(
-        action=None,
-        action_type=None,
-        tool_type=None,
-        tool_mount=None,
-        project=None,
-        neighborhood=None,
-        username=None,
-        url=None,
-        ip_address=None)
-
-    def __init__(self, logger, action):
-        self._logger = logger
-        self._action = action
-
-    def log(self, level, message, *args, **kwargs):
-        kwargs = dict(kwargs)
-        extra = kwargs.setdefault('extra', {})
-        meta = kwargs.pop('meta', {})
-        kwpairs = extra.setdefault('kwpairs', {})
-        for k, v in meta.iteritems():
-            kwpairs['meta_%s' % k] = v
-        extra.update(self._make_extra())
-        self._logger.log(level, self._action + ': ' + message, *args, **kwargs)
-
-    def info(self, message, *args, **kwargs):
-        self.log(logging.INFO, message, *args, **kwargs)
-
-    def debug(self, message, *args, **kwargs):
-        self.log(logging.DEBUG, message, *args, **kwargs)
-
-    def error(self, message, *args, **kwargs):
-        self.log(logging.ERROR, message, *args, **kwargs)
-
-    def critical(self, message, *args, **kwargs):
-        self.log(logging.CRITICAL, message, *args, **kwargs)
-
-    def exception(self, message, *args, **kwargs):
-        self.log(logging.EXCEPTION, message, *args, **kwargs)
-
-    def warning(self, message, *args, **kwargs):
-        self.log(logging.EXCEPTION, message, *args, **kwargs)
-    warn = warning
-
-    def _make_extra(self):
-        result = dict(self.extra_proto, action=self._action)
-        try:
-            if hasattr(c, 'app') and c.app:
-                result['tool_type'] = c.app.config.tool_name
-                result['tool_mount'] = c.app.config.options['mount_point']
-            if hasattr(c, 'project') and c.project:
-                result['project'] = c.project.shortname
-                result['neighborhood'] = c.project.neighborhood.name
-            if hasattr(c, 'user') and c.user:
-                result['username'] = c.user.username
-            else:
-                result['username'] = '*system'
-            try:
-                result['url'] = request.url
-                result['ip_address'] = utils.ip_address(request)
-            except TypeError:
-                pass
-            return result
-        except:
-            self._logger.warning(
-                'Error logging to rtstats, some info may be missing', exc_info=True)
-            return result
-
-
-def paging_sanitizer(limit, page, total_count=sys.maxint, zero_based_pages=True):
+def paging_sanitizer(limit, page, total_count=sys.maxsize, zero_based_pages=True):
     """Return limit, page - both converted to int and constrained to
     valid ranges based on total_count.
 
     Useful for sanitizing limit and page query params.
+
+    See also g.handle_paging which also checks prefs
     """
-    limit = max(int(limit), 1)
+    try:
+        limit = max(int(limit), 1)
+    except ValueError:
+        limit = 25
     limit = min(limit, asint(tg.config.get('limit_param_max', 500)))
-    max_page = (total_count / limit) + (1 if total_count % limit else 0)
+    max_page = (total_count // limit) + (1 if total_count % limit else 0)
     max_page = max(0, max_page - (1 if zero_based_pages else 0))
-    page = min(max(int(page or 0), (0 if zero_based_pages else 1)), max_page)
+    try:
+        page = int(page or 0)
+    except ValueError:
+        page = 0
+    page = min(max(page, (0 if zero_based_pages else 1)), max_page)
     return limit, page
 
 
@@ -733,7 +732,7 @@ def _add_inline_line_numbers_to_text(txt):
     markup_text = '<div class="codehilite"><pre>'
     for line_num, line in enumerate(txt.splitlines(), 1):
         markup_text = markup_text + \
-            '<span id="l%s" class="code_block"><span class="lineno">%s</span> %s</span>' % (
+            '<span id="l{}" class="code_block"><span class="lineno">{}</span> {}</span>'.format(
                 line_num, line_num, line)
     markup_text = markup_text + '</pre></div>'
     return markup_text
@@ -747,7 +746,7 @@ def _add_table_line_numbers_to_text(txt):
 
     def _len_to_str_column(l, start=1):
         max_num = l + start
-        return '\n'.join(map(_prepend_whitespaces, range(start, max_num), [max_num] * l))
+        return '\n'.join(map(_prepend_whitespaces, list(range(start, max_num)), [max_num] * l))
 
     lines = txt.splitlines(True)
     linenumbers = '<td class="linenos"><div class="linenodiv"><pre>' + \
@@ -756,7 +755,7 @@ def _add_table_line_numbers_to_text(txt):
         linenumbers + '<td class="code"><div class="codehilite"><pre>'
     for line_num, line in enumerate(lines, 1):
         markup_text = markup_text + \
-            '<span id="l%s" class="code_block">%s</span>' % (line_num, line)
+            f'<span id="l{line_num}" class="code_block">{line}</span>'
     markup_text = markup_text + '</pre></div></td></tr></tbody></table>'
     return markup_text
 
@@ -767,16 +766,17 @@ TABLE = 'table'
 
 def render_any_markup(name, txt, code_mode=False, linenumbers_style=TABLE):
     """
-    renders markdown using allura enhacements if file is in markdown format
+    renders markdown using allura enhancements if file is in markdown format
     renders any other markup format using the pypeline
     Returns jinja-safe text
     """
-    if txt == '':
+    if not txt:
         txt = '<p><em>Empty File</em></p>'
     else:
         fmt = g.pypeline_markup.can_render(name)
+        txt = really_unicode(txt)
         if fmt == 'markdown':
-            txt = g.markdown.convert(really_unicode(txt))
+            txt = g.markdown.convert(txt)
         else:
             txt = g.pypeline_markup.render(name, txt)
         if not fmt:
@@ -788,39 +788,12 @@ def render_any_markup(name, txt, code_mode=False, linenumbers_style=TABLE):
                 txt = '<pre>%s</pre>' % txt
     return Markup(txt)
 
-# copied from jinja2 dev
-# latest release, 2.6, implements this incorrectly
-# can remove and use jinja2 implementation after upgrading to 2.7
 
-
-def do_filesizeformat(value, binary=False):
-    """Format the value like a 'human-readable' file size (i.e. 13 kB,
-4.1 MB, 102 Bytes, etc). Per default decimal prefixes are used (Mega,
-Giga, etc.), if the second parameter is set to `True` the binary
-prefixes are used (Mebi, Gibi).
-"""
-    bytes = float(value)
-    base = binary and 1024 or 1000
-    prefixes = [
-        (binary and 'KiB' or 'kB'),
-        (binary and 'MiB' or 'MB'),
-        (binary and 'GiB' or 'GB'),
-        (binary and 'TiB' or 'TB'),
-        (binary and 'PiB' or 'PB'),
-        (binary and 'EiB' or 'EB'),
-        (binary and 'ZiB' or 'ZB'),
-        (binary and 'YiB' or 'YB')
-    ]
-    if bytes == 1:
-        return '1 Byte'
-    elif bytes < base:
-        return '%d Bytes' % bytes
-    else:
-        for i, prefix in enumerate(prefixes):
-            unit = base ** (i + 2)
-            if bytes < unit:
-                return '%.1f %s' % ((base * bytes / unit), prefix)
-        return '%.1f %s' % ((base * bytes / unit), prefix)
+@pass_context
+def subrender_jinja_filter(context, value):
+    _template = context.eval_ctx.environment.from_string(value)
+    result = _template.render(**context)
+    return result
 
 
 def nl2br_jinja_filter(value):
@@ -844,7 +817,7 @@ def get_tool_packages(tool_name):
     app = g.entry_points['tool'].get(tool_name.lower())
     if not app:
         return []
-    classes = [c for c in app.mro() if c not in (Application, object)]
+    classes = [c for c in app.mro() if c not in Application.mro()]
     return [cls.__module__.split('.')[0] for cls in classes]
 
 
@@ -866,7 +839,8 @@ def datetimeformat(value, format='%Y-%m-%d %H:%M:%S'):
 
 @contextmanager
 def log_output(log):
-    class Writer(object):
+    # TODO: replace with contextlib.redirect_stdout and redirect_stderr?
+    class Writer:
 
         def __init__(self, func):
             self.func = func
@@ -958,7 +932,7 @@ def topological_sort(items, partial_order):
 
 @contextmanager
 def ming_config(**conf):
-    """Temporarily swap in a new ming configuration, restoring the previous
+    r"""Temporarily swap in a new ming configuration, restoring the previous
     one when the contextmanager exits.
 
     :param \*\*conf: keyword arguments defining the new ming configuration
@@ -968,11 +942,11 @@ def ming_config(**conf):
     from ming.session import Session
     datastores = Session._datastores
     try:
-        ming.configure(**conf)
+        utils.configure_ming(conf)
         yield
     finally:
         Session._datastores = datastores
-        for name, session in Session._registry.iteritems():
+        for name, session in Session._registry.items():
             session.bind = datastores.get(name, None)
             session._name = name
 
@@ -991,13 +965,14 @@ def ming_config_from_ini(ini_path):
         yield
 
 
+def shlex_split(string):
+    # py2/3 compatibility
+    return [six.ensure_text(s) for s in shlex.split(six.ensure_str(string))]
+
+
 def split_select_field_options(field_options):
     try:
-        # shlex have problems with parsing unicode,
-        # it's better to pass properly encoded byte-string
-        field_options = shlex.split(field_options.encode('utf-8'))
-        # convert splitted string back to unicode
-        field_options = map(really_unicode, field_options)
+        field_options = shlex_split(field_options)
     except ValueError:
         field_options = field_options.split()
         # After regular split field_options might contain a " characters,
@@ -1029,7 +1004,7 @@ def null_contextmanager(returning=None, *args, **kw):
     yield returning
 
 
-class exceptionless(object):
+class exceptionless:
 
     '''Decorator making the decorated function return 'error_result' on any
     exceptions rather than propagating exceptions up the stack
@@ -1051,11 +1026,22 @@ class exceptionless(object):
                         'Error calling %s(args=%s, kwargs=%s): %s',
                         fname, args, kwargs, str(e))
                 return self.error_result
-        inner.__name__ = fname
+        inner.__name__ = str(fname)
         return inner
 
 
-def urlopen(url, retries=3, codes=(408, 500, 502, 503, 504), timeout=None):
+class NoRedirectToInternal(urllib.request.HTTPRedirectHandler):
+    def redirect_request(self, req: urllib.request.Request, fp, code, msg, headers, newurl: str):
+        if not asbool(tg.config.get('urlopen_allow_internal_hostnames', 'false')):
+            validators.URLIsPrivate().to_python(newurl, None)
+        return super().redirect_request(req, fp, code, msg, headers, newurl)
+
+
+opener = urllib.request.build_opener(NoRedirectToInternal)
+urllib.request.install_opener(opener)
+
+
+def urlopen(url: str | urllib.request.Request, retries=3, codes=(408, 500, 502, 503, 504), timeout=None):
     """Open url, optionally retrying if an error is encountered.
 
     Socket and other IO errors will always be retried if retries > 0.
@@ -1065,12 +1051,22 @@ def urlopen(url, retries=3, codes=(408, 500, 502, 503, 504), timeout=None):
     :param codes: HTTP error codes that should be retried.
 
     """
+    if isinstance(url, urllib.request.Request):
+        url_str =  url.full_url
+    else:
+        url_str = url
+    if not url_str.startswith(('http://', 'https://')):
+        raise ValueError(f'URL must be http(s), got {url_str}')
+    if not asbool(tg.config.get('urlopen_allow_internal_hostnames', 'false')):
+        # will raise error if hostname resolves to private address space:
+        validators.URLIsPrivate().to_python(url_str, None)
+
     attempts = 0
     while True:
         try:
-            return urllib2.urlopen(url, timeout=timeout)
-        except IOError as e:
-            no_retry = isinstance(e, urllib2.HTTPError) and e.code not in codes
+            return urllib.request.urlopen(url, timeout=timeout)
+        except OSError as e:
+            no_retry = isinstance(e, urllib.error.HTTPError) and e.code not in codes
             if attempts < retries and not no_retry:
                 attempts += 1
                 continue
@@ -1079,11 +1075,17 @@ def urlopen(url, retries=3, codes=(408, 500, 502, 503, 504), timeout=None):
                     url_string = url.get_full_url()  # if url is Request obj
                 except Exception:
                     url_string = url
+                if hasattr(e, 'filename') and url_string != e.filename:
+                    url_string += f' => {e.filename}'
                 if timeout is None:
                     timeout = socket.getdefaulttimeout()
+                if getattr(e, 'fp', None):
+                    body = e.fp.read()
+                else:
+                    body = ''
                 log.exception(
                     'Failed after %s retries on url with a timeout of %s: %s: %s',
-                    attempts, timeout, url_string, e)
+                    attempts, timeout, url_string, body[:250])
                 raise e
 
 
@@ -1111,6 +1113,9 @@ def plain2markdown(txt, preserve_multiple_spaces=False, has_html_entities=False)
     return txt
 
 
+OrderedDefaultDict = defaultdict  # py3.7+ dicts are always ordered
+
+
 def iter_entry_points(group, *a, **kw):
     """Yields entry points that have not been disabled in the config.
 
@@ -1132,10 +1137,10 @@ def iter_entry_points(group, *a, **kw):
                 if ep.name not in disabled]
 
     def unique_eps(entry_points):
-        by_name = defaultdict(list)
+        by_name = OrderedDefaultDict(list)
         for ep in entry_points:
             by_name[ep.name].append(ep)
-        for name, eps in by_name.iteritems():
+        for name, eps in by_name.items():
             ep_count = len(eps)
             if ep_count == 1:
                 yield eps[0]
@@ -1143,9 +1148,9 @@ def iter_entry_points(group, *a, **kw):
                 yield subclass(eps)
 
     def subclass(entry_points):
-        loaded = dict((ep, ep.load()) for ep in entry_points)
-        for ep, cls in loaded.iteritems():
-            others = loaded.values()[:]
+        loaded = {ep: ep.load() for ep in entry_points}
+        for ep, cls in loaded.items():
+            others = list(loaded.values())[:]
             others.remove(cls)
             if all([issubclass(cls, other) for other in others]):
                 return ep
@@ -1208,20 +1213,6 @@ def login_overlay(exceptions=None):
         c.show_login_overlay = True
 
 
-def get_filter(ctx, filter_name):
-    """
-    Gets a named Jinja2 filter, passing through
-    any context requested by the filter.
-    """
-    filter_ = ctx.environment.filters[filter_name]
-    if getattr(filter_, 'contextfilter', False):
-        return partial(filter_, ctx)
-    elif getattr(filter_, 'evalcontextfilter', False):
-        return partial(filter_, ctx.eval_ctx)
-    elif getattr(filter_, 'environmentfilter', False):
-        return partial(filter_, ctx.environment)
-
-
 def unidiff(old, new):
     """Returns unified diff between `one` and `two`."""
     return '\n'.join(difflib.unified_diff(
@@ -1241,9 +1232,9 @@ def auditlog_user(message, *args, **kwargs):
     """
     from allura import model as M
     ip_address = utils.ip_address(request)
-    message = 'IP Address: {}\nUser-Agent: {}\n'.format(ip_address, request.user_agent) + message
+    message = f'IP Address: {ip_address}\nUser-Agent: {request.user_agent}\n' + message
     if c.user and kwargs.get('user') and kwargs['user'] != c.user:
-        message = 'Done by user: {}\n'.format(c.user.username) + message
+        message = f'Done by user: {c.user.username}\n' + message
     return M.AuditLog.log_user(message, *args, **kwargs)
 
 
@@ -1285,12 +1276,89 @@ def rate_limit(cfg_opt, artifact_count, start_date, exception=None):
                 raise exception()
 
 
-def base64uri(content_or_image, image_format='PNG', mimetype='image/png'):
+def base64uri(content_or_image, image_format='PNG', mimetype='image/png', windows_line_endings=False):
     if hasattr(content_or_image, 'save'):
-        output = StringIO()
+        output = BytesIO()
         content_or_image.save(output, format=image_format)
         content = output.getvalue()
     else:
         content = content_or_image
-    data = base64.b64encode(content)
-    return 'data:{};base64,{}'.format(mimetype, data)
\ No newline at end of file
+
+    if windows_line_endings:
+        content = content.replace('\n', '\r\n')
+
+    data = six.ensure_text(base64.b64encode(six.ensure_binary(content)))
+    return f'data:{mimetype};base64,{data}'
+
+
+def slugify(name, allow_periods=False):
+    """
+    Returns a tuple with slug and lowered slug based on name
+    """
+    RE_NON_ALPHA_ETC = re.compile(r'[^.\w]+' if allow_periods else r'[^\w]+')
+    slug = RE_NON_ALPHA_ETC.sub('-',  # replace non ". alphanum_" sequences into single -
+                                unicodedata.normalize('NFKD', name).encode('ascii', 'ignore').decode().replace("'", '')  # asciify & strip apostophes.   https://stackoverflow.com/a/53261200
+                                ).strip('-')  # leading - or trailing - gets removed
+    return slug, slug.lower()
+
+
+email_re = re.compile(r'(([a-z0-9_]|\-|\.)+)@([\w\.-]+)', re.IGNORECASE)
+
+
+def hide_private_info(message):
+    if asbool(tg.config.get('hide_private_info', 'true')) and message:
+        hidden = email_re.sub(r'\1@...', message)
+        if type(message) not in (str,):
+            # custom subclass like markupsafe.Markup, convert to that type again
+            hidden = type(message)(hidden)
+        return hidden
+    else:
+        return message
+
+
+def emojize(text):
+    """Coverts emoji codes to unicode emojis"""
+    return emoji.emojize(text, language="alias")
+
+
+def get_current_reaction(react_users_dict):
+    """Return current selected reaction for given react_users dict"""
+    return utils.get_key_from_value(react_users_dict, c.user.username)
+
+
+def username_project_url(user_or_username):
+    from allura.lib import plugin
+
+    url = None
+
+    if not user_or_username:
+        return url
+
+    if isinstance(user_or_username, str):
+        class UserName:
+            def __init__(self, username):
+                self.username = username
+        username = user_or_username
+        auth_provider = plugin.AuthenticationProvider.get(request)
+        try:
+            # in 99% of cases, we can get away without a DB lookup
+            url = auth_provider.user_project_url(UserName(username))
+        except AttributeError:
+            user = auth_provider.by_username(username)
+            url = user.url()
+    else:
+        user = user_or_username
+        url = user.url()
+
+    return f'{url}profile/'
+
+
+def pluralize_tool_name(tool_name: string, count: int):
+    pluralize_tools = ['Wiki', 'Discussion', 'Blog']
+    if tool_name is not None and tool_name in pluralize_tools:
+        return f"{tool_name}{'s'[:count^1]}"
+    return tool_name
+
+def parse_fediverse_address(username: str):
+    pieces = username.split('@')
+    return f'https://{pieces[-1]}/@{pieces[1]}'
