@@ -15,77 +15,105 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-import cgi
+from __future__ import annotations
+
+import html
 import random
-import shlex
 import logging
 import traceback
+from dataclasses import dataclass
+from typing import Callable
+
+import six.moves.urllib.parse
+import six.moves.urllib.request
+import six.moves.urllib.error
+
 import oembed
 import jinja2
+import markupsafe
 from operator import attrgetter
-from urlparse import urlparse, urlunparse
+from urllib.parse import urlparse, urlunparse
 
 import pymongo
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request
+from tg import tmpl_context as c, app_globals as g
+from tg import request
 from paste.deploy.converters import asint
-from BeautifulSoup import BeautifulSoup
+from bs4 import BeautifulSoup
 
+from allura.lib.utils import socket_default_timeout
 from . import helpers as h
 from . import security
+import six
 
 log = logging.getLogger(__name__)
 
-_macros = {}
+
+@dataclass
+class RegisteredMacro:
+    func: Callable
+    context: str
+    cacheable: bool
 
 
-class macro(object):
+_macros: dict[str, RegisteredMacro] = {}
 
-    def __init__(self, context=None):
+
+def uncacheable_macros_names():
+    return [name for name, m in _macros.items() if not m.cacheable]
+
+
+class macro:
+    """
+    Decorator to declare a function is a custom markdown macro
+    """
+
+    def __init__(self, context: str = None, cacheable: bool = False):
+        """
+        :param context: either "neighborhood-wiki" or "userproject-wiki" to limit the macro to be used in those contexts
+        :param cacheable: indicates if its ok to cache the macro's output permanently
+        """
         self._context = context
+        self._cacheable = cacheable
 
     def __call__(self, func):
-        _macros[func.__name__] = (func, self._context)
+        _macros[func.__name__] = RegisteredMacro(func, self._context, self._cacheable)
         return func
 
 
-class parse(object):
+class parse:
 
     def __init__(self, context):
         self._context = context
 
     def __call__(self, s):
+        if s.startswith('quote '):
+            return '[[' + s[len('quote '):] + ']]'
         try:
-            if s.startswith('quote '):
-                return '[[' + s[len('quote '):] + ']]'
-            try:
-                parts = [unicode(x, 'utf-8')
-                         for x in shlex.split(s.encode('utf-8'))]
-                if not parts:
-                    return '[[' + s + ']]'
-                macro = self._lookup_macro(parts[0])
-                if not macro:
-                    return '[[' + s + ']]'
-                for t in parts[1:]:
-                    if '=' not in t:
-                        return '[-%s: missing =-]' % ' '.join(parts)
-                args = dict(t.split('=', 1) for t in parts[1:])
-                response = macro(**h.encode_keys(args))
-                return response
-            except (ValueError, TypeError) as ex:
-                log.warn('macro error.  Upwards stack is %s',
-                         ''.join(traceback.format_stack()),
-                         exc_info=True)
-                msg = cgi.escape(u'[[%s]] (%s)' % (s, repr(ex)))
-                return '\n<div class="error"><pre><code>%s</code></pre></div>' % msg
-        except Exception, ex:
-            raise
-            return '[[Error parsing %s: %s]]' % (s, ex)
+            parts = h.shlex_split(s)
+            if not parts:
+                return '[[' + s + ']]'
+            macro = self._lookup_macro(parts[0])
+            if not macro:
+                return '[[' + s + ']]'
+            for t in parts[1:]:
+                if '=' not in t:
+                    return '[-%s: missing =-]' % ' '.join(parts)
+            args = dict(t.split('=', 1) for t in parts[1:])
+            response = macro(**h.encode_keys(args))
+            return response
+        except (ValueError, TypeError) as ex:
+            log.warning('macro error.  Upwards stack is %s',
+                     ''.join(traceback.format_stack()),
+                     exc_info=True)
+            msg = html.escape(f'[[{s}]] ({repr(ex)})')
+            return '\n<div class="error"><pre><code>%s</code></pre></div>' % msg
 
     def _lookup_macro(self, s):
-        macro, context = _macros.get(s, (None, None))
-        if context is None or context == self._context:
-            return macro
+        macro = _macros.get(s)
+        if not macro:
+            return None
+        elif macro.context is None or macro.context == self._context:
+            return macro.func
         else:
             return None
 
@@ -149,14 +177,15 @@ def project_blog_posts(max_number=5, sort='timestamp', summary=False, mount_poin
         'state': 'published',
     })
     posts = posts.sort(sort, pymongo.DESCENDING).limit(int(max_number)).all()
-    output = ((dict(
-        href=post.url(),
-        title=post.title,
-        author=post.author().display_name,
-        ago=h.ago(post.timestamp),
-        description=summary and '&nbsp;' or g.markdown.cached_convert(post, 'text')))
+    output = [
+        dict(href=post.url(),
+             title=post.title,
+             author=post.author().display_name,
+             ago=h.ago(post.timestamp),
+             description=summary and '&nbsp;' or g.markdown.cached_convert(post, 'text'))
         for post in posts if security.has_access(post, 'read', project=post.app.project)() and
-        security.has_access(post.app.project, 'read', project=post.app.project)())
+            security.has_access(post.app.project, 'read', project=post.app.project)()
+    ]
     posts = BlogPosts(posts=output)
     g.resource_manager.register(posts)
     response = posts.display(posts=output)
@@ -231,7 +260,7 @@ def get_projects_for_macro(
             # 2. Choose a random sample of those _ids
             # 3. Do an ORM query to fetch the objects with those _ids
             # 4. Shuffle the results
-            from ming.orm import mapper
+            from ming.odm import mapper
             m = mapper(M.Project)
             collection = M.main_doc_session.db[m.collection.m.collection_name]
             docs = list(collection.find(q, {'_id': 1}))
@@ -309,16 +338,7 @@ def project_screenshots():
     from allura.lib.widgets.project_list import ProjectScreenshots
     ps = ProjectScreenshots()
     g.resource_manager.register(ps)
-    response = ps.display(project=c.project)
-    return response
-
-
-@macro()
-def gittip_button(username):
-    from allura.lib.widgets.macros import GittipButton
-    button = GittipButton(username=username)
-    g.resource_manager.register(button)
-    response = button.display(username=username)
+    response = ps.display(project=c.project, h=h)
     return response
 
 
@@ -356,7 +376,7 @@ def include_file(repo, path=None, rev=None, **kw):
     try:
         file = app.repo.commit(rev).get_path(path)
     except Exception:
-        return "[[include can't find file %s in revision %s]]" % (path, rev)
+        return f"[[include can't find file {path} in revision {rev}]]"
 
     text = ''
     if file.has_pypeline_view:
@@ -364,7 +384,7 @@ def include_file(repo, path=None, rev=None, **kw):
     elif file.has_html_view:
         text = g.highlight(file.text, filename=file.name)
     else:
-        return "[[include can't display file %s in revision %s]]" % (path, rev)
+        return f"[[include can't display file {path} in revision {rev}]]"
 
     from allura.lib.widgets.macros import Include
     sb = Include()
@@ -399,15 +419,15 @@ def include(ref=None, repo=None, **kw):
     return response
 
 
-@macro()
+@macro(cacheable=True)
 def img(src=None, **kw):
-    attrs = ('%s="%s"' % t for t in kw.iteritems())
+    attrs = ('%s="%s"' % t for t in kw.items())
     included = request.environ.setdefault('allura.macro.att_embedded', set())
     included.add(src)
     if '://' in src:
-        return '<img src="%s" %s/>' % (src, ' '.join(attrs))
+        return '<img src="{}" {}/>'.format(src, ' '.join(attrs))
     else:
-        return '<img src="./attachment/%s" %s/>' % (src, ' '.join(attrs))
+        return '<img src="./attachment/{}" {}/>'.format(src, ' '.join(attrs))
 
 
 @macro()
@@ -444,23 +464,39 @@ def members(limit=20):
     return response
 
 
-@macro()
+@macro(cacheable=True)
 def embed(url=None):
     consumer = oembed.OEmbedConsumer()
-    endpoint = oembed.OEmbedEndpoint(
-        'http://www.youtube.com/oembed', ['http://*.youtube.com/*', 'https://*.youtube.com/*'])
+    endpoint = oembed.OEmbedEndpoint('https://www.youtube.com/oembed',
+                                     ['http://*.youtube.com/*', 'https://*.youtube.com/*',
+                                      'http://*.youtube-nocookie.com/*', 'https://*.youtube-nocookie.com/*',
+                                      ])
     consumer.addEndpoint(endpoint)
-    try:
-        html = consumer.embed(url)['html']
-    except oembed.OEmbedNoEndpoint:
-        html = None
+
+    # workaround for https://github.com/abarmat/python-oembed/pull/9 not being implemented yet
+    with socket_default_timeout(5):
+
+        try:
+            html = consumer.embed(url)['html']
+        except oembed.OEmbedNoEndpoint:
+            html = None
+        except oembed.OEmbedError:
+            log.exception(f'Could not embed: {url}')
+            return f'Could not embed: {url}'
+        except six.moves.urllib.error.HTTPError as e:
+            if e.code in (403, 404):
+                return 'Video not available'
+            else:
+                log.exception(f'Could not embed: {url}')
+                return f'Could not embed: {url}'
 
     if html:
         # youtube has a trailing ")" at the moment
         html = html.rstrip(')')
 
         # convert iframe src from http to https, to avoid mixed security blocking when used on an https page
-        html = BeautifulSoup(html)
+        # and convert to youtube-nocookie.com
+        html = BeautifulSoup(html, 'html.parser')
         embed_url = html.find('iframe').get('src')
         if embed_url:
             embed_url = urlparse(embed_url)
@@ -468,7 +504,8 @@ def embed(url=None):
                 embed_url = urlunparse(['https'] + list(embed_url[1:]))
             else:
                 embed_url = embed_url.geturl()
+            embed_url = embed_url.replace('www.youtube.com', 'www.youtube-nocookie.com')
             html.find('iframe')['src'] = embed_url
-        return jinja2.Markup('<p>%s</p>' % html)
+        return markupsafe.Markup('<p>%s</p>' % html)
 
     return '[[embed url=%s]]' % url
