@@ -17,7 +17,10 @@
 
 import re
 import logging
-from urlparse import urljoin
+from typing import List
+import xml.etree.ElementTree as etree
+
+from urllib.parse import urljoin
 
 from tg import config
 from bs4 import BeautifulSoup
@@ -25,21 +28,47 @@ import html5lib
 import html5lib.serializer
 import html5lib.filters.alphabeticalattributes
 import markdown
+import emoji
+from markupsafe import Markup
 
 from . import macro
 from . import helpers as h
 from allura import model as M
-from allura.lib.utils import ForgeHTMLSanitizer
+from allura.lib.utils import ForgeHTMLSanitizerFilter, is_nofollow_url
+import six
 
 log = logging.getLogger(__name__)
 
-PLAINTEXT_BLOCK_RE = re.compile(
-    r'(?P<bplain>\[plain\])(?P<code>.*?)(?P<eplain>\[\/plain\])',
-    re.MULTILINE | re.DOTALL
-)
 
 MACRO_PATTERN = r'\[\[([^\]\[]+)\]\]'
 
+# SHORT_REF_RE copied from markdown pre 3.0
+SHORT_REF_RE = markdown.inlinepatterns.NOIMG + r'\[([^\]]+)\]'
+
+# FORGE_LINK_RE copied from markdown pre 3.0's LINK_RE
+# TODO: replace these with newer approach, see ForgeLinkPattern
+NOBRACKET = r'[^\]\[]*'  # if not using regex-as-re-globally, must change "*" to {0,50} for performance mitigation
+BRK = (
+    r'\[(' +
+    (NOBRACKET + r'(\[')*6 +
+    (NOBRACKET + r'\])*')*6 +
+    NOBRACKET + r')\]'
+)
+FORGE_LINK_RE = markdown.inlinepatterns.NOIMG + BRK + \
+    r'''\(\s*(<.*?>|((?:(?:\(.*?\))|[^\(\)]))*?)\s*((['"])(.*?)\12\s*)?\)'''
+
+
+def clear_markdown_registry(reg: markdown.util.Registry, keep: List[str] = []):
+    keep_items = {}
+    for name in keep:
+        keep_items[name] = reg[name]
+
+    # this resets Registry's internal data structures to be empty
+    reg.__init__()
+
+    for name, item in keep_items.items():
+        reg.register(item, name, 50)  # arbitrary priority :(
+
 
 class CommitMessageExtension(markdown.Extension):
 
@@ -66,36 +95,40 @@ class CommitMessageExtension(markdown.Extension):
     """
 
     def __init__(self, app):
-        markdown.Extension.__init__(self)
+        super().__init__()
         self.app = app
         self._use_wiki = False
 
-    def extendMarkdown(self, md, md_globals):
+    def extendMarkdown(self, md):
         md.registerExtension(self)
         # remove default preprocessors and add our own
-        md.preprocessors.clear()
-        md.preprocessors['trac_refs'] = PatternReplacingProcessor(TracRef1(), TracRef2(), TracRef3(self.app))
+        clear_markdown_registry(md.preprocessors)
+
+        # The last param of .register() is priority. Higher vals go first.
+
+        md.preprocessors.register(PatternReplacingProcessor(TracRef1(), TracRef2(), TracRef3(self.app)), 'trac_refs', 0)
+
         # remove all inlinepattern processors except short refs and links
-        md.inlinePatterns.clear()
-        md.inlinePatterns["link"] = markdown.inlinepatterns.LinkPattern(markdown.inlinepatterns.LINK_RE, md)
-        md.inlinePatterns['short_reference'] = ForgeLinkPattern(markdown.inlinepatterns.SHORT_REF_RE, md, ext=self)
+        clear_markdown_registry(md.inlinePatterns, keep=['link'])
+        md.inlinePatterns.register(ForgeLinkPattern(SHORT_REF_RE, md, ext=self), 'short_reference', 0)
+
         # remove all default block processors except for paragraph
-        md.parser.blockprocessors.clear()
-        md.parser.blockprocessors['paragraph'] = markdown.blockprocessors.ParagraphProcessor(md.parser)
+        clear_markdown_registry(md.parser.blockprocessors, keep=['paragraph'])
         # wrap artifact link text in square brackets
         self.forge_link_tree_processor = ForgeLinkTreeProcessor(md)
-        md.treeprocessors['links'] = self.forge_link_tree_processor
+        md.treeprocessors.register(self.forge_link_tree_processor, 'links', 0)
+
         # Sanitize HTML
-        md.postprocessors['sanitize_html'] = HTMLSanitizer()
+        md.postprocessors.register(HTMLSanitizer(), 'sanitize_html', 3)
         # Put a class around markdown content for custom css
-        md.postprocessors['add_custom_class'] = AddCustomClass()
-        md.postprocessors['mark_safe'] = MarkAsSafe()
+        md.postprocessors.register(AddCustomClass(), 'add_custom_class', 2)
+        md.postprocessors.register(MarkAsSafe(), 'mark_safe', 1)
 
     def reset(self):
         self.forge_link_tree_processor.reset()
 
 
-class Pattern(object):
+class Pattern:
 
     """Base class for regex patterns used by the :class:`PatternReplacingProcessor`.
 
@@ -238,51 +271,98 @@ class PatternReplacingProcessor(markdown.preprocessors.Preprocessor):
 class ForgeExtension(markdown.Extension):
 
     def __init__(self, wiki=False, email=False, macro_context=None):
-        markdown.Extension.__init__(self)
+        super().__init__()
         self._use_wiki = wiki
         self._is_email = email
         self._macro_context = macro_context
 
-    def extendMarkdown(self, md, md_globals):
+    def extendMarkdown(self, md):
         md.registerExtension(self)
-        # allow markdown within e.g. <div markdown>...</div>  More info at:
-        # https://github.com/waylan/Python-Markdown/issues/52
-        md.preprocessors['html_block'].markdown_in_raw = True
-        md.preprocessors.add('plain_text_block', PlainTextPreprocessor(md), "_begin")
-        md.preprocessors.add('macro_include', ForgeMacroIncludePreprocessor(md), '_end')
+        md.preprocessors.register(ForgeMacroIncludePreprocessor(md), 'macro_include', -99)
+
+        # The last param of .register() is priority. Higher vals go first.
+
         # this has to be before the 'escape' processor, otherwise weird
         # placeholders are inserted for escaped chars within urls, and then the
         # autolink can't match the whole url
-        md.inlinePatterns.add('autolink_without_brackets', AutolinkPattern(r'(http(?:s?)://[a-zA-Z0-9./\-\\_%?&=+#;~:!]+)', md), '<escape')
+        md.inlinePatterns.register(AutolinkPattern(r'(http(?:s?)://[a-zA-Z0-9./\-\\_%?&=+#;~:!@]+)', md),
+                                   'autolink_without_brackets',
+                                   185)  # was '<escape' and 'escape' is priority 180; great num runs first, so: 185
         # replace the link pattern with our extended version
-        md.inlinePatterns['link'] = ForgeLinkPattern(markdown.inlinepatterns.LINK_RE, md, ext=self)
-        md.inlinePatterns['short_reference'] = ForgeLinkPattern(markdown.inlinepatterns.SHORT_REF_RE, md, ext=self)
+        md.inlinePatterns.register(ForgeLinkPattern(FORGE_LINK_RE, md, ext=self), 'link', 160)
+        md.inlinePatterns.register(ForgeLinkPattern(SHORT_REF_RE, md, ext=self), 'short_reference', 130)
         # macro must be processed before links
-        md.inlinePatterns.add('macro', ForgeMacroPattern(MACRO_PATTERN, md, ext=self), '<link')
+        md.inlinePatterns.register(ForgeMacroPattern(MACRO_PATTERN, md, ext=self), 'macro', 165)  # similar to above
+
         self.forge_link_tree_processor = ForgeLinkTreeProcessor(md)
-        md.treeprocessors['links'] = self.forge_link_tree_processor
+        md.treeprocessors.register(self.forge_link_tree_processor, 'links', 0)
         # Sanitize HTML
-        md.postprocessors['sanitize_html'] = HTMLSanitizer()
+        md.postprocessors.register(HTMLSanitizer(), 'sanitize_html', 5)
         # Rewrite all relative links that don't start with . to have a '../' prefix
-        md.postprocessors['rewrite_relative_links'] = RelativeLinkRewriter(make_absolute=self._is_email)
+        md.postprocessors.register(RelativeLinkRewriter(make_absolute=self._is_email), 'rewrite_relative_links', 4)
         # Put a class around markdown content for custom css
-        md.postprocessors['add_custom_class'] = AddCustomClass()
-        md.postprocessors['mark_safe'] = MarkAsSafe()
+        md.postprocessors.register(AddCustomClass(), 'add_custom_class', 3)
+        md.postprocessors.register(MarkAsSafe(), 'mark_safe', 2)
 
     def reset(self):
         self.forge_link_tree_processor.reset()
 
 
-class ForgeLinkPattern(markdown.inlinepatterns.LinkPattern):
+class EmojiExtension(markdown.Extension):
+
+    EMOJI_RE = r'(:[a-zA-Z0-9\+\-_&.ô’Åéãíç()!#\*]+:)'
+
+    def extendMarkdown(self, md):
+        md.registerExtension(self)
+        md.inlinePatterns.register(EmojiInlinePattern(self.EMOJI_RE), 'emoji', 0)
+
+
+class EmojiInlinePattern(markdown.inlinepatterns.Pattern):
+
+    def handleMatch(self, m):
+        emoji_code = m.group(2)
+        return emoji.emojize(emoji_code, language="alias")
+
+
+class UserMentionExtension(markdown.Extension):
+
+    UM_RE = r'\B(@(?![0-9]+$)(?!-)[a-z0-9_-]{2,14}[a-z0-9_])'
+
+    def extendMarkdown(self, md):
+        md.registerExtension(self)
+        md.inlinePatterns.register(UserMentionInlinePattern(self.UM_RE), 'user_mentions', 0)
+
+
+class UserMentionInlinePattern(markdown.inlinepatterns.Pattern):
+
+    def handleMatch(self, m):
+        user_name = m.group(2).replace("@", "")
+        user = M.User.by_username(user_name)
+        result = None
+
+        if user and not user.pending and not user.disabled:
+            result = etree.Element('a')
+            result.text = "@%s" % user_name
+            result.set('href', user.url())
+            result.set('class', 'user-mention')
+        else:
+            result = "@%s" % user_name
+        return result
+
+
+class ForgeLinkPattern(markdown.inlinepatterns.Pattern):
+    # TODO: convert from extending Pattern to extending InlineProcessor
+    #  which is how core Markdown library in 3.0 made its base link parsing much faster.
+    # https://github.com/Python-Markdown/markdown/commit/d18c3d0acab0e7469c3284c897afcb61f9dd1fea
 
     artifact_re = re.compile(r'((.*?):)?((.*?):)?(.+)')
 
     def __init__(self, *args, **kwargs):
         self.ext = kwargs.pop('ext')
-        markdown.inlinepatterns.LinkPattern.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def handleMatch(self, m):
-        el = markdown.util.etree.Element('a')
+        el = etree.Element('a')
         el.text = m.group(2)
         is_link_with_brackets = False
         try:
@@ -290,6 +370,8 @@ class ForgeLinkPattern(markdown.inlinepatterns.LinkPattern):
         except IndexError:
             href = m.group(2)
             is_link_with_brackets = True
+            if el.text == 'x' or el.text == ' ':  # skip [ ] and [x] for markdown checklist
+                return '[' + el.text + ']'
         try:
             title = m.group(13)
         except IndexError:
@@ -301,7 +383,7 @@ class ForgeLinkPattern(markdown.inlinepatterns.LinkPattern):
                 return '[TOC]'  # skip TOC
             if self.artifact_re.match(href):
                 href, classes = self._expand_alink(href, is_link_with_brackets)
-            el.set('href', self.sanitize_url(self.unescape(href.strip())))
+            el.set('href', self.unescape(href.strip()))
             el.set('class', classes)
         else:
             el.set('href', '')
@@ -312,7 +394,7 @@ class ForgeLinkPattern(markdown.inlinepatterns.LinkPattern):
 
         if 'notfound' in classes and not self.ext._use_wiki:
             text = el.text
-            el = markdown.util.etree.Element('span')
+            el = etree.Element('span')
             el.text = '[%s]' % text
         return el
 
@@ -343,47 +425,16 @@ class ForgeLinkPattern(markdown.inlinepatterns.LinkPattern):
         return href, classes
 
 
-class PlainTextPreprocessor(markdown.preprocessors.Preprocessor):
-
-    '''
-    This was used earlier for [plain] tags that the Blog tool's rss importer
-    created, before html2text did good escaping of all special markdown chars.
-    Can be deprecated.
-    '''
-
-    def run(self, lines):
-        text = "\n".join(lines)
-        while 1:
-            res = PLAINTEXT_BLOCK_RE.finditer(text)
-            for m in res:
-                code = self._escape(m.group('code'))
-                placeholder = self.markdown.htmlStash.store(code, safe=True)
-                text = '%s%s%s' % (
-                    text[:m.start()], placeholder, text[m.end():])
-                break
-            else:
-                break
-        return text.split("\n")
-
-    def _escape(self, txt):
-        """ basic html escaping """
-        txt = txt.replace('&', '&amp;')
-        txt = txt.replace('<', '&lt;')
-        txt = txt.replace('>', '&gt;')
-        txt = txt.replace('"', '&quot;')
-        return txt
-
-
 class ForgeMacroPattern(markdown.inlinepatterns.Pattern):
 
     def __init__(self, *args, **kwargs):
         self.ext = kwargs.pop('ext')
         self.macro = macro.parse(self.ext._macro_context)
-        markdown.inlinepatterns.Pattern.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def handleMatch(self, m):
         html = self.macro(m.group(2))
-        placeholder = self.markdown.htmlStash.store(html)
+        placeholder = self.md.htmlStash.store(html)
         return placeholder
 
 
@@ -396,7 +447,7 @@ class ForgeLinkTreeProcessor(markdown.treeprocessors.Treeprocessor):
         self.alinks = []
 
     def run(self, root):
-        for node in root.getiterator('a'):
+        for node in root.iter('a'):
             if 'alink' in node.get('class', '').split() and node.text:
                 node.text = '[' + node.text + ']'
         return root
@@ -408,7 +459,7 @@ class ForgeLinkTreeProcessor(markdown.treeprocessors.Treeprocessor):
 class MarkAsSafe(markdown.postprocessors.Postprocessor):
 
     def run(self, text):
-        return h.html.literal(text)
+        return Markup(text)
 
 
 class AddCustomClass(markdown.postprocessors.Postprocessor):
@@ -423,7 +474,8 @@ class RelativeLinkRewriter(markdown.postprocessors.Postprocessor):
         self._make_absolute = make_absolute
 
     def run(self, text):
-        soup = BeautifulSoup(text, 'html5lib')  # 'html.parser' parser gives weird </li> behaviour with test_macro_members
+        soup = BeautifulSoup(text,
+                             'html5lib')  # 'html.parser' parser gives weird </li> behaviour with test_macro_members
 
         if self._make_absolute:
             rewrite = self._rewrite_abs
@@ -435,7 +487,7 @@ class RelativeLinkRewriter(markdown.postprocessors.Postprocessor):
             rewrite(link, 'src')
 
         # html5lib parser adds html/head/body tags, so output <body> without its own tags
-        return unicode(soup.body)[len('<body>'):-len('</body>')]
+        return str(soup.body)[len('<body>'):-len('</body>')]
 
     def _rewrite(self, tag, attr):
         val = tag.get(attr)
@@ -446,11 +498,11 @@ class RelativeLinkRewriter(markdown.postprocessors.Postprocessor):
             # just make sure no spaces
             val = val.replace(' ', '%20')
             tag[attr] = val
-        if '://' in val:
-            for domain in re.split(r'\s*,\s*', config.get('nofollow_exempt_domains', '')):
-                if domain and domain in val:
-                    return
+        if 'markdown_syntax' in val:
             tag['rel'] = 'nofollow'
+        if '://' in val:
+            if is_nofollow_url(val):
+                tag['rel'] = 'nofollow'
             return
         if val.startswith('/'):
             return
@@ -472,32 +524,36 @@ class RelativeLinkRewriter(markdown.postprocessors.Postprocessor):
 class HTMLSanitizer(markdown.postprocessors.Postprocessor):
 
     def run(self, text):
-        parser = html5lib.HTMLParser(tokenizer=ForgeHTMLSanitizer)
-        parsed = parser.parse(text)
-        serializer = html5lib.serializer.HTMLSerializer()
-        walker = html5lib.getTreeWalker("etree")
-        stream = html5lib.filters.alphabeticalattributes.Filter(walker(parsed))
-        out = ''.join(serializer.serialize(stream))
-        return out
+        parsed = html5lib.parseFragment(text)
+
+        # if we didn't have to customize our sanitization, could just do:
+        # return html5lib.serialize(parsed, sanitize=True)
+
+        # instead we do the same steps as that function,
+        # but add our ForgeHTMLSanitizerFilter instead of sanitize=True which would use the standard one
+        TreeWalker = html5lib.treewalkers.getTreeWalker("etree")
+        walker = TreeWalker(parsed)
+        walker = ForgeHTMLSanitizerFilter(walker)  # this is our custom step
+        s = html5lib.serializer.HTMLSerializer()
+        return s.render(walker)
 
 
 class AutolinkPattern(markdown.inlinepatterns.Pattern):
 
     def __init__(self, pattern, markdown_instance=None):
-        markdown.inlinepatterns.Pattern.__init__(
-            self, pattern, markdown_instance)
+        super().__init__(pattern, markdown_instance)
         # override the complete regex, requiring the preceding text (.*?) to end
         # with whitespace or beginning of line "\s|^"
-        self.compiled_re = re.compile("^(.*?\s|^)%s(.*?)$" % pattern,
+        self.compiled_re = re.compile(r"^(.*?\s|^)%s(.*?)$" % pattern,
                                       re.DOTALL | re.UNICODE)
 
     def handleMatch(self, mo):
         old_link = mo.group(2)
-        result = markdown.util.etree.Element('a')
+        result = etree.Element('a')
         result.text = old_link
         # since this is run before the builtin 'escape' processor, we have to
         # do our own unescaping
-        for char in markdown.Markdown.ESCAPED_CHARS:
+        for char in self.md.ESCAPED_CHARS:
             old_link = old_link.replace('\\' + char, char)
         result.set('href', old_link)
         return result
