@@ -18,21 +18,27 @@
 """
 This module provides the security predicates used in decorating various models.
 """
+
+import six
+import sys
 import logging
 from collections import defaultdict
+import hashlib
+import requests
 
-from pylons import tmpl_context as c
-from pylons import request
+from tg import tmpl_context as c
+from tg import request
 from webob import exc
 from itertools import chain
 from ming.utils import LazyProperty
+import tg
 
 from allura.lib.utils import TruthyCallable
 
 log = logging.getLogger(__name__)
 
 
-class Credentials(object):
+class Credentials:
 
     '''
     Role graph logic & caching
@@ -95,10 +101,10 @@ class Credentials(object):
                 'project_id': {'$in': project_ids},
                 'name': None})
             q = chain(q0, q1)
-        roles_by_project = dict((pid, []) for pid in project_ids)
+        roles_by_project = {pid: [] for pid in project_ids}
         for role in q:
             roles_by_project[role['project_id']].append(role)
-        for pid, roles in roles_by_project.iteritems():
+        for pid, roles in roles_by_project.items():
             self.users[user_id, pid] = RoleCache(self, roles)
 
     def load_project_roles(self, *project_ids):
@@ -110,10 +116,10 @@ class Credentials(object):
             return
         q = self.project_role.find({
             'project_id': {'$in': project_ids}})
-        roles_by_project = dict((pid, []) for pid in project_ids)
+        roles_by_project = {pid: [] for pid in project_ids}
         for role in q:
             roles_by_project[role['project_id']].append(role)
-        for pid, roles in roles_by_project.iteritems():
+        for pid, roles in roles_by_project.items():
             self.projects[pid] = RoleCache(self, roles)
 
     def project_roles(self, project_id):
@@ -158,7 +164,7 @@ class Credentials(object):
         return role.userids_that_reach
 
 
-class RoleCache(object):
+class RoleCache:
     '''
     An iterable collection of :class:`ProjectRoles <allura.model.auth.ProjectRole>` that is cached after first use
     '''
@@ -172,7 +178,7 @@ class RoleCache(object):
         self.q = q
 
     def find(self, **kw):
-        tests = kw.items()
+        tests = list(kw.items())
 
         def _iter():
             for r in self:
@@ -193,14 +199,14 @@ class RoleCache(object):
         return None
 
     def __iter__(self):
-        return self.index.itervalues()
+        return iter(self.index.values())
 
     def __len__(self):
         return len(self.index)
 
     @LazyProperty
     def index(self):
-        return dict((r['_id'], r) for r in self.q)
+        return {r['_id']: r for r in self.q}
 
     @LazyProperty
     def named(self):
@@ -245,8 +251,8 @@ class RoleCache(object):
     @LazyProperty
     def reaching_roles(self):
         def _iter():
-            to_visit = self.index.items()
-            project_ids = set([r['project_id'] for _id, r in to_visit])
+            to_visit = list(self.index.items())
+            project_ids = {r['project_id'] for _id, r in to_visit}
             pr_index = {r['_id']: r for r in self.cred.project_role.find({
                 'project_id': {'$in': list(project_ids)},
                 'user_id': None,
@@ -437,7 +443,7 @@ def all_allowed(obj, user_or_role=None, project=None):
                         denied[role_id].add(ace.permission)
         obj = obj.parent_security_context()
     if M.ALL_PERMISSIONS in perms:
-        return set([M.ALL_PERMISSIONS])
+        return {M.ALL_PERMISSIONS}
     return perms
 
 
@@ -490,6 +496,29 @@ def require_authenticated():
         raise exc.HTTPUnauthorized()
 
 
+def is_site_admin(user):
+    '''
+    :param allura.model.User user:
+    :rtype: bool
+    '''
+    from allura.lib import helpers as h
+
+    if user.is_anonymous():
+        return False
+
+    with h.push_context(tg.config.get('site_admin_project', 'allura'),
+                        neighborhood=tg.config.get('site_admin_project_nbhd', 'Projects')):
+        return has_access(c.project, 'admin', user=user)
+
+
+def require_site_admin(user):
+    from allura.lib import helpers as h
+
+    with h.push_context(tg.config.get('site_admin_project', 'allura'),
+                        neighborhood=tg.config.get('site_admin_project_nbhd', 'Projects')):
+        return require_access(c.project, 'admin', user=user)
+
+
 def simple_grant(acl, role_id, permission):
     from allura.model.types import ACE
     for ace in acl:
@@ -505,3 +534,70 @@ def simple_revoke(acl, role_id, permission):
             remove.append(i)
     for i in reversed(remove):
         acl.pop(i)
+
+
+class HIBPClientError(Exception):
+    """
+    Represents an unexpected failure consuming the HIBP API
+    """
+    pass
+
+
+class HIBPCompromisedCredentials(Exception):
+    """
+    Raised when a sha1 hash of a password is found to be compromised according to HIBP
+    """
+
+    def __init__(self, count, partial_hash):
+        self.count = count
+        self.partial_hash = partial_hash
+
+
+class HIBPClient:
+
+    @classmethod
+    def check_breached_password(cls, password):
+        """
+        Checks the Have I Been Pwned API for a known compromised password.
+        Raises a named HIBPCompromisedCredentials exception if any found
+        :param password: user-supplied password
+        """
+        result = 0
+        try:
+            # sha1 it
+            sha_1 = hashlib.sha1(password.encode('utf-8')).hexdigest()
+
+            # first 5 for HIBP API
+            sha_1_first_5 = sha_1[:5]
+
+            # hit HIBP API
+            headers = {'User-Agent': '{}-pwnage-checker'.format(tg.config.get('site_name', 'Allura'))}
+            resp = requests.get(f'https://api.pwnedpasswords.com/range/{sha_1_first_5}', timeout=1,
+                                headers=headers)
+            # check results
+            result = cls.scan_response(resp, sha_1)
+
+        except Exception as ex:
+            raise HIBPClientError from ex
+
+        if result:
+            raise HIBPCompromisedCredentials(result, sha_1_first_5)
+
+    @classmethod
+    def scan_response(self, resp, sha):
+        """
+        Scans an API result from HIBP matching the supplied SHA
+        :return: The entry count HIBP has of the password; 0 if not present
+        """
+        sha_remainder = sha[5:]
+
+        entries = resp.text.split('\r\n')
+        try:
+            entry = [e for e in entries if e.startswith(sha_remainder.upper())][0]
+            vals = entry.split(':')
+            result = vals[1]
+            result = int(result)
+        except IndexError:
+            result = 0
+
+        return result
