@@ -14,52 +14,80 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
-from contextlib import contextmanager
+from __future__ import annotations
 
+import base64
+from collections.abc import Iterable, Mapping, MutableMapping
+from contextlib import contextmanager
 import time
 import string
 import hashlib
 import binascii
 import logging.handlers
-import codecs
-
-from html5lib.constants import tokenTypes
-
-from ming.odm import session
 import os.path
 import datetime
 import random
 import mimetypes
 import re
+from typing import Type, TypeVar
 import magic
 from itertools import groupby
 import operator as op
 import collections
+import ming
+from urllib.parse import urlparse
+import six.moves.urllib.request
+import six.moves.urllib.parse
+import six.moves.urllib.error
+import socket
 
 import tg
-import pylons
+import emoji
 import json
-from formencode import Invalid
+from collections import OrderedDict
+
+from bs4 import BeautifulSoup
+from tg import redirect, app_globals as g
 from tg.decorators import before_validate
-from pylons.controllers.util import etag_cache
+from tg.controllers.util import etag_cache
 from paste.deploy.converters import asbool, asint
-from webhelpers.html import literal
+from markupsafe import Markup
 from webob import exc
 from pygments.formatters import HtmlFormatter
 from setproctitle import getproctitle
-import html5lib.sanitizer
-
+import html5lib.filters.sanitizer
 from ew import jinja2_ew as ew
 from ming.utils import LazyProperty
 from ming.odm.odmsession import ODMCursor
+from ming.odm import session
+import six
+
+
+T = TypeVar('T')
+
 
 MARKDOWN_EXTENSIONS = ['.markdown', '.mdown', '.mkdn', '.mkd', '.md']
 
 
+def clean_ming_config(config):
+    # delete replicaSet=''
+    for key in list(config.keys()):
+        if '.replicaSet' in key and not config[key]:
+            del config[key]
+        elif 'mongo_host' in key and 'replicaSet=&' in config[key]:
+            config[key] = config[key].replace('replicaSet=&', '')
+    return config
+
+
+def configure_ming(conf):
+    conf = clean_ming_config(conf)
+    ming.configure(**conf)
+
+
 def permanent_redirect(url):
     try:
         tg.redirect(url)
-    except exc.HTTPFound, err:
+    except exc.HTTPFound as err:
         raise exc.HTTPMovedPermanently(location=err.location)
 
 
@@ -76,7 +104,7 @@ def guess_mime_type(filename):
     return content_type
 
 
-class ConfigProxy(object):
+class ConfigProxy:
 
     '''Wrapper for loading config values at module-scope so we don't
     have problems when a module is imported before tg.config is initialized
@@ -95,7 +123,7 @@ class ConfigProxy(object):
         return asbool(self.get(key))
 
 
-class lazy_logger(object):
+class lazy_logger:
 
     '''Lazy instatiation of a logger, to ensure that it does not get
     created before logging is configured (which would make it disabled)'''
@@ -112,61 +140,9 @@ class lazy_logger(object):
             raise AttributeError(name)
         return getattr(self._logger, name)
 
+
 log = lazy_logger(__name__)
 
-class TimedRotatingHandler(logging.handlers.BaseRotatingHandler):
-
-    def __init__(self, strftime_pattern):
-        self.pattern = strftime_pattern
-        self.last_filename = self.current_filename()
-        logging.handlers.BaseRotatingHandler.__init__(
-            self, self.last_filename, 'a')
-
-    def current_filename(self):
-        return os.path.abspath(datetime.datetime.utcnow().strftime(self.pattern))
-
-    def shouldRollover(self, record):
-        'Inherited from BaseRotatingFileHandler'
-        return self.current_filename() != self.last_filename
-
-    def doRollover(self):
-        self.stream.close()
-        self.baseFilename = self.current_filename()
-        if self.encoding:
-            self.stream = codecs.open(self.baseFilename, 'w', self.encoding)
-        else:
-            self.stream = open(self.baseFilename, 'w')
-
-
-class StatsHandler(TimedRotatingHandler):
-    fields = (
-        'action', 'action_type', 'tool_type', 'tool_mount', 'project', 'neighborhood',
-        'username', 'url', 'ip_address')
-
-    def __init__(self,
-                 strftime_pattern,
-                 module='allura',
-                 page=1,
-                 **kwargs):
-        self.page = page
-        self.module = module
-        TimedRotatingHandler.__init__(self, strftime_pattern)
-
-    def emit(self, record):
-        if not hasattr(record, 'action'):
-            return
-        kwpairs = dict(
-            module=self.module,
-            page=self.page)
-        for name in self.fields:
-            kwpairs[name] = getattr(record, name, None)
-        kwpairs.update(getattr(record, 'kwpairs', {}))
-        record.kwpairs = ','.join(
-            '%s=%s' % (k, v) for k, v in sorted(kwpairs.iteritems())
-            if v is not None)
-        record.exc_info = None  # Never put tracebacks in the rtstats log
-        TimedRotatingHandler.emit(self, record)
-
 
 class CustomWatchedFileHandler(logging.handlers.WatchedFileHandler):
 
@@ -179,11 +155,12 @@ class CustomWatchedFileHandler(logging.handlers.WatchedFileHandler):
         """
         title = getproctitle()
         if title.startswith('taskd:'):
-            record.name = "{0}:{1}".format(title, record.name)
-        return super(CustomWatchedFileHandler, self).format(record)
+            record.name = f"{title}:{record.name}"
+        return super().format(record)
 
 
-def chunked_find(cls, query=None, pagesize=1024, sort_key='_id', sort_dir=1):
+def chunked_find(cls: type[T], query: dict | None = None, pagesize: int = 1024, sort_key: str | None = '_id',
+                 sort_dir: int = 1) -> Iterable[Iterable[T]]:
     '''
     Execute a mongo query against the specified class, yield some results at
     a time (avoids mongo cursor timeouts if the total result set is very large).
@@ -205,41 +182,33 @@ def chunked_find(cls, query=None, pagesize=1024, sort_key='_id', sort_dir=1):
         else:
             # skipping requires scanning, even for an indexed query
             q = cls.query.find(query).limit(pagesize).skip(pagesize * page)
-        results = (q.all())
+        results = q.all()
         if not results:
             break
         if sort_key:
             max_id = results[-1][sort_key]
         yield results
+        if len(results) < pagesize:
+            break
         page += 1
 
 
-def lsub_utf8(s, n):
-    '''Useful for returning n bytes of a UTF-8 string, rather than characters'''
-    while len(s) > n:
-        k = n
-        while (ord(s[k]) & 0xc0) == 0x80:
-            k -= 1
-        return s[:k]
-    return s
-
-
 def chunked_list(l, n):
     """ Yield successive n-sized chunks from l.
     """
-    for i in xrange(0, len(l), n):
+    for i in range(0, len(l), n):
         yield l[i:i + n]
 
 
 def chunked_iter(iterable, max_size):
     '''return iterable 'chunks' from the iterable of max size max_size'''
     eiter = enumerate(iterable)
-    keyfunc = lambda (i, x): i // max_size
+    keyfunc = lambda i_x: i_x[0] // max_size
     for _, chunk in groupby(eiter, keyfunc):
         yield (x for i, x in chunk)
 
 
-class AntiSpam(object):
+class AntiSpam:
 
     '''Helper class for bot-protecting forms'''
     honey_field_template = string.Template('''<p class="$honey_class">
@@ -250,18 +219,19 @@ class AntiSpam(object):
     def __init__(self, request=None, num_honey=2, timestamp=None, spinner=None):
         self.num_honey = num_honey
         if request is None or request.method == 'GET':
-            self.request = pylons.request
+            self.request = tg.request
             self.timestamp = timestamp if timestamp else int(time.time())
             self.spinner = spinner if spinner else self.make_spinner()
             self.timestamp_text = str(self.timestamp)
-            self.spinner_text = self._wrap(self.spinner)
+            self.spinner_text = six.ensure_text(self._wrap(self.spinner))
         else:
             self.request = request
             self.timestamp_text = request.params['timestamp']
             self.spinner_text = request.params['spinner']
             self.timestamp = int(self.timestamp_text)
             self.spinner = self._unwrap(self.spinner_text)
-        self.spinner_ord = map(ord, self.spinner)
+        trans_fn = int
+        self.spinner_ord = list(map(trans_fn, self.spinner))
         self.random_padding = [random.randint(0, 255) for x in self.spinner]
         self.honey_class = self.enc(self.spinner_text, css_safe=True)
 
@@ -277,27 +247,26 @@ class AntiSpam(object):
 
     @staticmethod
     def _wrap(s):
-        '''Encode a string to make it HTML id-safe (starts with alpha, includes
+        '''Encode bytes to make it HTML id-safe (starts with alpha, includes
         only digits, hyphens, underscores, colons, and periods).  Luckily, base64
         encoding doesn't use hyphens, underscores, colons, nor periods, so we'll
         use these characters to replace its plus, slash, equals, and newline.
         '''
-        tx_tbl = string.maketrans('+/', '-_')
-        s = binascii.b2a_base64(s)
-        s = s.rstrip('=\n')
-        s = s.translate(tx_tbl)
-        s = 'X' + s
+        s = base64.b64encode(six.ensure_binary(s))
+        s = s.rstrip(b'=\n')
+        s = s.replace(b'+', b'-').replace(b'/', b'_')
+        s = b'X' + s
         return s
 
     @staticmethod
     def _unwrap(s):
-        tx_tbl = string.maketrans('-_', '+/')
         s = s[1:]
-        s = str(s).translate(tx_tbl)
+        s = six.ensure_binary(s)
+        s = s.replace(b'-', b'+').replace(b'_', b'/')
         i = len(s) % 4
         if i > 0:
-            s += '=' * (4 - i)
-        s = binascii.a2b_base64(s + '\n')
+            s += b'=' * (4 - i)
+        s = base64.b64decode(s + b'\n')
         return s
 
     def enc(self, plain, css_safe=False):
@@ -307,17 +276,25 @@ class AntiSpam(object):
         '''
         # Plain starts with its length, includes the ordinals for its
         #   characters, and is padded with random data
+
+        # limit to plain ascii, which should be sufficient for field names
+        # I don't think the logic below would work with non-ascii multi-byte text anyway
+        plain.encode('ascii')
+
         plain = ([len(plain)]
-                 + map(ord, plain)
+                 + list(map(ord, plain))
                  + self.random_padding[:len(self.spinner_ord) - len(plain) - 1])
         enc = ''.join(chr(p ^ s) for p, s in zip(plain, self.spinner_ord))
+        enc = six.ensure_binary(enc)
         enc = self._wrap(enc)
+        enc = six.ensure_text(enc)
         if css_safe:
             enc = ''.join(ch for ch in enc if ch.isalpha())
         return enc
 
     def dec(self, enc):
         enc = self._unwrap(enc)
+        enc = six.ensure_text(enc)
         enc = list(map(ord, enc))
         plain = [e ^ s for e, s in zip(enc, self.spinner_ord)]
         plain = plain[1:1 + plain[0]]
@@ -330,7 +307,7 @@ class AntiSpam(object):
         for fldno in range(self.num_honey):
             fld_name = self.enc('honey%d' % (fldno))
             fld_id = self.enc('honey%d%d' % (self.counter, fldno))
-            yield literal(self.honey_field_template.substitute(
+            yield Markup(self.honey_field_template.substitute(
                 honey_class=self.honey_class,
                 fld_id=fld_id,
                 fld_name=fld_name))
@@ -350,13 +327,13 @@ class AntiSpam(object):
         octets = self.client_ip.split('.')
         ip_chunk = '.'.join(octets[0:3])
         plain = '%d:%s:%s' % (
-            timestamp, ip_chunk, pylons.config.get('spinner_secret', 'abcdef'))
-        return hashlib.sha1(plain).digest()
+            timestamp, ip_chunk, tg.config.get('spinner_secret', 'abcdef'))
+        return hashlib.sha1(six.ensure_binary(plain)).digest()
 
     @classmethod
     def validate_request(cls, request=None, now=None, params=None):
         if request is None:
-            request = pylons.request
+            request = tg.request
         if params is None:
             params = request.params
         new_params = dict(params)
@@ -371,29 +348,32 @@ class AntiSpam(object):
                     now = time.time()
                 if obj.timestamp > now + 5:
                     raise ValueError('Post from the future')
-                if now - obj.timestamp > 24 * 60 * 60:
+                if now - obj.timestamp > int(tg.config.get('spam.form_post_expiration', 24 * 60 * 60)):
                     raise ValueError('Post from the distant past')
                 if obj.spinner != expected_spinner:
                     raise ValueError('Bad spinner value')
-                for k in new_params.keys():
+                for k in list(new_params.keys()):
                     try:
                         new_params[obj.dec(k)] = new_params[k]
                         new_params.pop(k)
-                    except:
+                    except Exception as ex:
                         pass
                 for fldno in range(obj.num_honey):
-                    value = new_params.pop('honey%s' % fldno)
+                    try:
+                        value = new_params.pop('honey%s' % fldno)
+                    except KeyError:
+                        raise ValueError('Missing honeypot field: honey%s' % fldno)
                     if value:
                         raise ValueError('Value in honeypot field: %s' % value)
             except Exception as ex:
                 attrs = dict(now=now, obj=vars(obj) if obj else None)
-                log.info('Form validation failure: {}'.format(attrs))
+                log.info(f'Form validation failure: {attrs}')
                 log.info('Error is', exc_info=ex)
-                raise ex
+                raise
         return new_params
 
     @classmethod
-    def validate(cls, error_msg):
+    def validate(cls, error_msg, error_url=None):
         '''Controller decorator to raise Invalid errors if bot protection is engaged'''
         def antispam_hook(remainder, params):
             '''Converts various errors in validate_request to a single Invalid message'''
@@ -405,19 +385,18 @@ class AntiSpam(object):
                     # request.params is immutable, but will reflect changes to request.POST
                     tg.request.POST.update(new_params)
             except (ValueError, TypeError, binascii.Error):
-                testing = pylons.request.environ.get('paste.testing', False)
-                if testing:
+                testing = tg.request.environ.get('paste.testing', False)
+                if testing and not tg.request.environ.get('regular_antispam_err_handling_even_when_tests'):
                     # re-raise so we can see problems more easily
                     raise
                 else:
                     # regular antispam failure handling
                     tg.flash(error_msg, 'error')
-                    raise Invalid(error_msg, params, None)
+                    redirect(error_url or six.ensure_text(tg.request.referer or '.'))
         return before_validate(antispam_hook)
 
 
-class TruthyCallable(object):
-
+class TruthyCallable:
     '''
     Wraps a callable to make it truthy in a boolean context.
 
@@ -430,11 +409,19 @@ class TruthyCallable(object):
     def __call__(self, *args, **kw):
         return self.callable(*args, **kw)
 
-    def __nonzero__(self):
+    def __bool__(self):
         return self.callable()
 
+    def __eq__(self, other):
+        if other is True and bool(self):
+            return True
+        elif other is False and not bool(self):
+            return True
+        else:
+            return NotImplemented
 
-class TransformedDict(collections.MutableMapping):
+
+class TransformedDict(MutableMapping):
 
     """
     A dictionary which applies an arbitrary
@@ -476,23 +463,9 @@ def postmortem_hook(etype, value, tb):  # pragma no cover
     import sys
     import pdb
     import traceback
-    try:
-        from IPython.ipapi import make_session
-        make_session()
-        from IPython.Debugger import Pdb
-        sys.stderr.write('Entering post-mortem IPDB shell\n')
-        p = Pdb(color_scheme='Linux')
-        p.reset()
-        p.setup(None, tb)
-        p.print_stack_trace()
-        sys.stderr.write('%s: %s\n' % (etype, value))
-        p.cmdloop()
-        p.forget()
-        # p.interaction(None, tb)
-    except ImportError:
-        sys.stderr.write('Entering post-mortem PDB shell\n')
-        traceback.print_exception(etype, value, tb)
-        pdb.post_mortem(tb)
+    sys.stderr.write('Entering post-mortem PDB shell\n')
+    traceback.print_exception(etype, value, tb)
+    pdb.post_mortem(tb)
 
 
 class LineAnchorCodeHtmlFormatter(HtmlFormatter):
@@ -508,21 +481,23 @@ class LineAnchorCodeHtmlFormatter(HtmlFormatter):
         num = self.linenostart
         yield 0, ('<pre' + (style and ' style="%s"' % style) + '>')
         for tup in inner:
-            yield (tup[0], '<div id="l%s" class="code_block">%s</div>' % (num, tup[1]))
+            yield (tup[0], f'<div id="l{num}" class="code_block">{tup[1]}</div>')
             num += 1
         yield 0, '</pre>'
 
 
 def generate_code_stats(blob):
+    from allura.lib import helpers as h
+
     stats = {'line_count': 0,
              'code_size': 0,
              'data_line_count': 0}
-    code = blob.text
+    code = h.really_unicode(blob.text)
     lines = code.split('\n')
     stats['code_size'] = blob.size
     stats['line_count'] = len(lines)
     spaces = re.compile(r'^\s*$')
-    stats['data_line_count'] = sum([1 for l in lines if not spaces.match(l)])
+    stats['data_line_count'] = sum(1 for l in lines if not spaces.match(l))
     return stats
 
 
@@ -544,66 +519,101 @@ def serve_file(fp, filename, content_type, last_modified=None,
                cache_expires=None, size=None, embed=True, etag=None):
     '''Sets the response headers and serves as a wsgi iter'''
     if not etag and filename and last_modified:
-        etag = u'{0}?{1}'.format(filename, last_modified).encode('utf-8')
+        # must be latin1, no unicode
+        etag = filename.encode("latin1", "backslashreplace").decode('latin1') + f'?{last_modified}'
     if etag:
         etag_cache(etag)
-    pylons.response.headers['Content-Type'] = ''
-    pylons.response.content_type = content_type.encode('utf-8')
-    pylons.response.cache_expires = cache_expires or asint(
+    tg.response.headers['Content-Type'] = ''
+    tg.response.content_type = str(content_type)
+    tg.response.cache_expires = cache_expires or asint(
         tg.config.get('files_expires_header_secs', 60 * 60))
-    pylons.response.last_modified = last_modified
+    tg.response.last_modified = last_modified
     if size:
-        pylons.response.content_length = size
-    if 'Pragma' in pylons.response.headers:
-        del pylons.response.headers['Pragma']
-    if 'Cache-Control' in pylons.response.headers:
-        del pylons.response.headers['Cache-Control']
+        tg.response.content_length = size
+    if 'Pragma' in tg.response.headers:
+        del tg.response.headers['Pragma']
+    if 'Cache-Control' in tg.response.headers:
+        del tg.response.headers['Cache-Control']
     if not embed:
-        pylons.response.headers.add(
+        from allura.lib import helpers as h
+        tg.response.headers.add(
             'Content-Disposition',
-            'attachment;filename="%s"' % filename.encode('utf-8'))
+            'attachment;filename="%s"' % h.urlquote(filename))
     # http://code.google.com/p/modwsgi/wiki/FileWrapperExtension
     block_size = 4096
     if 'wsgi.file_wrapper' in tg.request.environ:
         return tg.request.environ['wsgi.file_wrapper'](fp, block_size)
     else:
-        return iter(lambda: fp.read(block_size), '')
+        return iter(lambda: fp.read(block_size), b'')
 
 
-class ForgeHTMLSanitizer(html5lib.sanitizer.HTMLSanitizer):
-    # remove some elements from the sanitizer whitelist
-    # <form> and <input> could be used for a social engineering attack to construct a form
-    # others are just unexpected and confusing, and have no need to be used in markdown
-    _form_elements = ('button', 'datalist', 'fieldset', 'form', 'input', 'label', 'legend', 'meter', 'optgroup',
-                      'option', 'output', 'progress', 'select', 'textarea')
-    _forge_acceptable_elements = [e for e in html5lib.sanitizer.HTMLSanitizer.acceptable_elements
-                                  if e not in (_form_elements)]
-    allowed_elements = _forge_acceptable_elements \
-                       + html5lib.sanitizer.HTMLSanitizer.mathml_elements \
-                       + html5lib.sanitizer.HTMLSanitizer.svg_elements
+class ForgeHTMLSanitizerFilter(html5lib.filters.sanitizer.Filter):
 
-    # srcset is used in our own project_list/project_summary widgets which are used as macros so go through markdown
-    allowed_attributes = html5lib.sanitizer.HTMLSanitizer.allowed_attributes + ['srcset']
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        # remove some elements from the sanitizer whitelist
+        # <form> and <input> could be used for a social engineering attack to construct a form
+        # others are just unexpected and confusing, and have no need to be used in markdown
+        ns_html = html5lib.constants.namespaces['html']
+        _form_elements = {(ns_html, 'button'),
+                          (ns_html, 'datalist'),
+                          (ns_html, 'fieldset'),
+                          (ns_html, 'form'),
+                          (ns_html, 'input'),
+                          (ns_html, 'label'),
+                          (ns_html, 'legend'),
+                          (ns_html, 'meter'),
+                          (ns_html, 'optgroup'),
+                          (ns_html, 'option'),
+                          (ns_html, 'output'),
+                          (ns_html, 'progress'),
+                          (ns_html, 'select'),
+                          (ns_html, 'textarea'),
+                          }
+        _extra_allowed_elements = {
+            (ns_html, 'summary'),
+        }
+        self.allowed_elements = (set(html5lib.filters.sanitizer.allowed_elements) | _extra_allowed_elements) - _form_elements
 
-    valid_iframe_srcs = ('https://www.youtube.com/embed/', 'https://www.gittip.com/')
+        # srcset is used in our own project_list/project_summary widgets
+        # which are used as macros so go through markdown
+        self.allowed_attributes = html5lib.filters.sanitizer.allowed_attributes | {(None, 'srcset')}
 
-    _prev_token_was_ok_iframe = False
+        self.valid_iframe_srcs = ('https://www.youtube.com/embed/',
+                                  'https://www.youtube-nocookie.com/embed/',
+                                  )
+        self._prev_token_was_ok_iframe = False
 
     def sanitize_token(self, token):
-        if 'iframe' in self.allowed_elements:
-            self.allowed_elements.remove('iframe')
+        """
+        Allow iframe tags if the src attribute matches our list of valid sources.
+        Allow input tags if the type attribute matches "checkbox"
+        Otherwise use default sanitization.
+        """
+
+        iframe_el = (html5lib.constants.namespaces['html'], 'iframe')
+        self.allowed_elements.discard(iframe_el)
         ok_opening_iframe = False
 
+        input_el = (html5lib.constants.namespaces['html'], 'input')
+        self.allowed_elements.discard(input_el)
+
         if token.get('name') == 'iframe':
-            attrs = dict(token.get('data'))
-            if attrs.get('src', '').startswith(self.valid_iframe_srcs):
-                self.allowed_elements.append('iframe')
+            attrs = token.get('data') or {}
+            if attrs.get((None, 'src'), '').startswith(self.valid_iframe_srcs):
+                self.allowed_elements.add(iframe_el)
                 ok_opening_iframe = True
-            elif token.get('type') == tokenTypes["EndTag"] and self._prev_token_was_ok_iframe:
-                self.allowed_elements.append('iframe')
+            elif token.get('type') == "EndTag" and self._prev_token_was_ok_iframe:
+                self.allowed_elements.add(iframe_el)
 
         self._prev_token_was_ok_iframe = ok_opening_iframe
-        return super(ForgeHTMLSanitizer, self).sanitize_token(token)
+
+        if token.get('name') == 'input':
+            attrs = token.get('data') or {}
+            if attrs.get((None, 'type'), '') == "checkbox":
+                self.allowed_elements.add(input_el)
+
+        return super().sanitize_token(token)
 
 
 def ip_address(request):
@@ -631,6 +641,7 @@ class EmptyCursor(ODMCursor):
 
     def next(self):
         raise StopIteration
+    __next__ = next
 
     def options(self, **kw):
         return self
@@ -656,14 +667,14 @@ class DateJSONEncoder(json.JSONEncoder):
 
 
 def clean_phone_number(number):
-    pattern = re.compile('\W+')
+    pattern = re.compile(r'\W+')
     number = pattern.sub('', number)
     return number
 
 
 def phone_number_hash(number):
     number = clean_phone_number(number)
-    return hashlib.sha1(number).hexdigest()
+    return hashlib.sha1(number.encode('utf-8')).hexdigest()
 
 
 @contextmanager
@@ -723,14 +734,14 @@ def is_ajax(request):
     return False
 
 
-class JSONForExport(tg.jsonify.GenericJSON):
+class JSONForExport(tg.jsonify.JSONEncoder):
     def default(self, obj):
         if hasattr(obj, '__json__') and callable(obj.__json__):
             try:
                 return obj.__json__(is_export=True)
             except TypeError:
                 return obj.__json__()
-        return super(JSONForExport, self).default(obj)
+        return super().default(obj)
 
 
 @contextmanager
@@ -740,3 +751,88 @@ def umask(new_mask):
         yield
     finally:
         os.umask(cur_mask)
+
+
+def get_reaction_emoji_list():
+    """ Get reactions emoji list from .ini file. If not defined there get fixed
+    default emoji list """
+    emo_list = tg.config.get('reactions.emoji_list')
+    if emo_list is None:
+        emo_list = [':+1:', ':-1:', ':smile:', ':tada:', ':confused:', ':heart:']
+    else:
+        emo_list = emo_list.split(',')
+    return emo_list
+
+
+def get_reactions_json():
+    """ Returns global reactions json """
+    j = OrderedDict()
+    for em in get_reaction_emoji_list():
+        j[em] = emoji.emojize(em, language='alias')
+    return json.dumps(j)
+
+
+def get_usernames_from_md(text):
+    """ Returns a unique usernames set from a text """
+    usernames = set()
+    html_text = g.markdown.convert(text)
+    soup = BeautifulSoup(html_text, 'html.parser')
+    for mention in soup.select('a.user-mention'):
+        usernames.add(mention.get_text().replace('@', ''))
+    return usernames
+
+
+def get_key_from_value(d, val):
+    """ Get key from given value. return empty str if not exists """
+    for k, v in d.items():
+        if val in v:
+            return k
+    return ''
+
+
+def is_nofollow_url(url):
+    url_domain = urlparse(url).hostname
+    ok_domains = re.split(r'\s*,\s*', tg.config.get('nofollow_exempt_domains', '')) + [tg.config['domain']]
+    return url_domain and url_domain not in ok_domains
+
+
+def smart_str(s):
+    '''
+    Returns a bytestring version of 's' from any type
+    '''
+    if isinstance(s, bytes):
+        return s
+    else:
+        return str(s).encode('utf-8')
+
+
+def generate_smart_str(params):
+    if isinstance(params, Mapping):
+        params_list = params.items()
+    else:
+        params_list = params
+    for key, value in params_list:
+        yield smart_str(key), smart_str(value)
+
+
+def urlencode(params):
+    """
+    A version of Python's urllib.urlencode() function that can operate on
+    unicode strings. The parameters are first case to UTF-8 encoded strings and
+    then encoded as per normal.
+    """
+    return six.moves.urllib.parse.urlencode([i for i in generate_smart_str(params)])
+
+
+def close_ipv4_addrs(ip1, ip2):
+    return ip1.split('.')[0:3] == ip2.split('.')[0:3]
+
+
+@contextmanager
+def socket_default_timeout(timeout):
+    orig_timeout = socket.getdefaulttimeout()
+    socket.setdefaulttimeout(timeout)
+    try:
+        yield
+    finally:
+        socket.setdefaulttimeout(orig_timeout)
