@@ -18,7 +18,7 @@
 import logging
 import warnings
 
-from pylons import app_globals as g, tmpl_context as c
+from tg import app_globals as g, tmpl_context as c
 from formencode import validators as fev
 import formencode
 import ew as ew_core
@@ -27,6 +27,7 @@ from pytz import common_timezones, country_timezones, country_names
 from paste.deploy.converters import aslist, asint, asbool
 import tg
 from tg import config
+from markupsafe import Markup
 
 from allura.lib import validators as V
 from allura.lib import helpers as h
@@ -34,6 +35,8 @@ from allura.lib import plugin
 from allura.lib.widgets import form_fields as ffw
 from allura.lib import exceptions as forge_exc
 from allura import model as M
+import six
+from functools import reduce
 
 
 log = logging.getLogger(__name__)
@@ -52,7 +55,7 @@ class _HTMLExplanation(ew.InputField):
 
 class NeighborhoodProjectShortNameValidator(fev.FancyValidator):
 
-    def _validate_shortname(self, shortname, neighborhood, state):
+    def _validate_shortname(self, shortname, neighborhood, state, permit_legacy=False):
         if not h.re_project_name.match(shortname):
             raise forge_exc.ProjectShortnameInvalid(
                 'Please use 3-15 small letters, numbers, and dashes.',
@@ -66,7 +69,7 @@ class NeighborhoodProjectShortNameValidator(fev.FancyValidator):
                 'This project name is taken.',
                 shortname, state)
 
-    def to_python(self, value, state=None, check_allowed=True, neighborhood=None):
+    def to_python(self, value, state=None, check_allowed=True, neighborhood=None, permit_legacy=False):
         """
         Validate a project shortname.
 
@@ -75,10 +78,9 @@ class NeighborhoodProjectShortNameValidator(fev.FancyValidator):
         otherwise disallowed.
         """
         if neighborhood is None:
-            neighborhood = M.Neighborhood.query.get(
-                name=state.full_dict['neighborhood'])
-        value = h.really_unicode(value or '').encode('utf-8').lower()
-        self._validate_shortname(value, neighborhood, state)
+            neighborhood = M.Neighborhood.query.get(name=state.full_dict['neighborhood'])
+        value = h.really_unicode(value or '')
+        self._validate_shortname(value, neighborhood, state, permit_legacy=permit_legacy)
         if check_allowed:
             self._validate_allowed(value, neighborhood, state)
         return value
@@ -96,18 +98,18 @@ class ForgeForm(ew.SimpleForm):
         target=None)
 
     def display_label(self, field, label_text=None):
-        ctx = super(ForgeForm, self).context_for(field)
+        ctx = super().context_for(field)
         label_text = (
             label_text
             or ctx.get('label')
             or getattr(field, 'label', None)
             or ctx['name'])
-        html = '<label for="%s">%s</label>' % (
+        html = '<label for="{}">{}</label>'.format(
             ctx['id'], label_text)
-        return h.html.literal(html)
+        return Markup(html)
 
     def context_for(self, field):
-        ctx = super(ForgeForm, self).context_for(field)
+        ctx = super().context_for(field)
         if self.antispam:
             ctx['rendered_name'] = g.antispam.enc(ctx['name'])
         return ctx
@@ -116,21 +118,20 @@ class ForgeForm(ew.SimpleForm):
         ctx = self.context_for(field)
         display = field.display(**ctx)
         if ctx['errors'] and field.show_errors and not ignore_errors:
-            display = "%s<div class='error'>%s</div>" % (display,
-                                                         ctx['errors'])
-        return h.html.literal(display)
+            display = "{}<div class='error'>{}</div>".format(display,
+                                                             ctx['errors'])
+        return Markup(display)
 
-    def display_field_by_idx(self, idx, ignore_errors=False):
-        warnings.warn(
-            'ForgeForm.display_field_by_idx is deprecated; use '
-            'ForgeForm.display_field() instead', DeprecationWarning)
-        field = self.fields[idx]
-        ctx = self.context_for(field)
-        display = field.display(**ctx)
-        if ctx['errors'] and field.show_errors and not ignore_errors:
-            display = "%s<div class='error'>%s</div>" % (display,
-                                                         ctx['errors'])
-        return display
+
+class ForgeFormResponsive(ForgeForm):
+    def __init__(self):
+        super().__init__()
+        # use alternate template if responsive overrides are on, but not actually using template override for this
+        # since that would change all forms, and we want to just do individual ones right now
+        for tmpl_override_ep in h.iter_entry_points('allura.theme.override'):
+            if tmpl_override_ep.name == 'responsive':
+                self.template = 'jinja:allura:templates_responsive/widgets/forge_form.html'
+                break
 
 
 class PasswordChangeBase(ForgeForm):
@@ -141,23 +142,37 @@ class PasswordChangeBase(ForgeForm):
             ew.PasswordField(
                 name='pw',
                 label='New Password',
-                validator=fev.UnicodeString(
+                attrs=dict(
+                    minlength=asint(tg.config.get('auth.min_password_len', 6)),
+                    maxlength=asint(tg.config.get('auth.max_password_len', 30)),
+                    required=True,
+                    autocomplete='new-password',
+                ),
+                validator=V.UnicodeString(
                     not_empty=True,
                     min=asint(tg.config.get('auth.min_password_len', 6)),
                     max=asint(tg.config.get('auth.max_password_len', 30)))),
             ew.PasswordField(
                 name='pw2',
                 label='New Password (again)',
-                validator=fev.UnicodeString(not_empty=True)),
+                validator=V.UnicodeString(not_empty=True),
+                attrs=dict(
+                    required=True,
+                    autocomplete='new-password',
+                )
+            ),
         ]
 
     @ew_core.core.validator
     def to_python(self, value, state):
-        d = super(PasswordChangeBase, self).to_python(value, state)
+        d = super().to_python(value, state)
         if d['pw'] != d['pw2']:
             raise formencode.Invalid('Passwords must match', value, state)
         return d
 
+    def resources(self):
+        yield ew.JSLink('js/password-validator.js')
+
 
 class PasswordChangeForm(PasswordChangeBase):
 
@@ -167,24 +182,40 @@ class PasswordChangeForm(PasswordChangeBase):
             ew.PasswordField(
                 name='oldpw',
                 label='Old Password',
-                validator=fev.UnicodeString(not_empty=True)),
+                validator=V.UnicodeString(not_empty=True),
+                attrs=dict(
+                    required=True,
+                    autocomplete='current-password',
+                ),
+            ),
             ew.PasswordField(
                 name='pw',
                 label='New Password',
-                validator=fev.UnicodeString(
+                attrs=dict(
+                    minlength=asint(tg.config.get('auth.min_password_len', 6)),
+                    maxlength=asint(tg.config.get('auth.max_password_len', 30)),
+                    required=True,
+                    autocomplete='new-password',
+                ),
+                validator=V.UnicodeString(
                     not_empty=True,
                     min=asint(tg.config.get('auth.min_password_len', 6)),
                     max=asint(tg.config.get('auth.max_password_len', 30)))),
             ew.PasswordField(
                 name='pw2',
                 label='New Password (again)',
-                validator=fev.UnicodeString(not_empty=True)),
+                validator=V.UnicodeString(not_empty=True),
+                attrs=dict(
+                    required=True,
+                    autocomplete='new-password',
+                ),
+            ),
             ew.HiddenField(name='return_to'),
         ]
 
     @ew_core.core.validator
     def to_python(self, value, state):
-        d = super(PasswordChangeForm, self).to_python(value, state)
+        d = super().to_python(value, state)
         if d['oldpw'] == d['pw']:
             raise formencode.Invalid(
                 'Your old and new password should not be the same',
@@ -221,23 +252,21 @@ class PersonalDataForm(ForgeForm):
                 label='Gender',
                 options=[ew.Option(py_value=v, label=v, selected=False)
                          for v in ['Male', 'Female', 'Unknown', 'Other']],
-                validator=formencode.All(
-                    V.OneOfValidator(['Male', 'Female', 'Unknown', 'Other']),
-                    fev.UnicodeString(not_empty=True))),
+                validator=V.OneOfValidator(['Male', 'Female', 'Unknown', 'Other'])),
             ew.SingleSelectField(
                 name='country',
                 label='Country of residence',
                 validator=V.MapValidator(country_names, not_empty=False),
                 options=[ew.Option(py_value=" ", label=" -- Unknown -- ", selected=False)] +
                         [ew.Option(py_value=c, label=n, selected=False)
-                         for c, n in sorted(country_names.items(),
-                                            key=lambda (k, v): v)],
-                attrs={'onchange': 'selectTimezone(this.value)'}),
+                         for c, n in sorted(list(country_names.items()),
+                                            key=lambda k_v: k_v[1])],
+                ),
             ew.TextField(
                 name='city',
                 label='City of residence',
                 attrs=dict(value=None),
-                validator=fev.UnicodeString(not_empty=False)),
+                validator=V.UnicodeString(not_empty=False)),
             ew.SingleSelectField(
                 name='timezone',
                 label='Timezone',
@@ -257,12 +286,12 @@ class PersonalDataForm(ForgeForm):
 
         self._fields = self.fields
 
-        birthdate_field = filter(lambda x: x.name == 'birthdate', self._fields)
+        birthdate_field = [x for x in self._fields if x.name == 'birthdate']
 
-        sex_field = filter(lambda x: x.name == 'sex', self._fields)[0]
-        country_field = filter(lambda x: x.name == 'country', self._fields)[0]
-        city_field = filter(lambda x: x.name == 'city', self._fields)[0]
-        timezone_field = filter(lambda x: x.name == 'timezone', self._fields)[0]
+        sex_field = [x for x in self._fields if x.name == 'sex'][0]
+        country_field = [x for x in self._fields if x.name == 'country'][0]
+        city_field = [x for x in self._fields if x.name == 'city'][0]
+        timezone_field = [x for x in self._fields if x.name == 'timezone'][0]
 
         for opt in sex_field.options:
             if opt.label == user.sex:
@@ -304,13 +333,8 @@ class PersonalDataForm(ForgeForm):
 
         yield ew.JSScript('''
 var $allTimezones = $("#tz").clone();
-var $t = {};
-''' +
-                          reduce(_append, [
-                              '$t["' + el + '"] = ' + str([name.encode('utf-8')
-                                                           for name in country_timezones[el]]) + ";\n"
-                              for el in country_timezones]) + '''
-function selectTimezone($country){
+var $t = ''' + h.escape_json(dict(country_timezones)) + ''';
+function selectTimeZ($country){
      if($country == " "){
          $("#tz").replaceWith($allTimezones);
      }
@@ -320,7 +344,9 @@ function selectTimezone($country){
              $("#tz").append($("<option></option>").attr("value", value).text(value))
          })
      }
-}''')
+};
+$("select[name='country']").on("change", function(e){ selectTimeZ(e.target.value) });
+''')
 
 
 class AddTelNumberForm(ForgeForm):
@@ -330,7 +356,7 @@ class AddTelNumberForm(ForgeForm):
         newnumber = ew.TextField(
             label='New telephone number',
             attrs={'value': ''},
-            validator=fev.UnicodeString(not_empty=True))
+            validator=V.UnicodeString(not_empty=True))
 
     def display(self, **kw):
         initial_value = kw.get('initial_value', '')
@@ -360,7 +386,7 @@ class SkypeAccountForm(ForgeForm):
         skypeaccount = ew.TextField(
             label='Skype account',
             attrs={'value': ''},
-            validator=fev.UnicodeString(not_empty=False))
+            validator=V.UnicodeString(not_empty=False))
 
     def display(self, **kw):
         initial_value = kw.get('initial_value', '')
@@ -399,7 +425,7 @@ class RemoveTextValueForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveTextValueForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["oldvalue"] = kw.get('oldvalue', '')
         return d
 
@@ -410,19 +436,21 @@ class AddSocialNetworkForm(ForgeForm):
     @property
     def fields(self):
         socialnetworks = aslist(tg.config.get('socialnetworks',
-                                              ['Facebook', 'Linkedin', 'Twitter', 'Google+']),
+                                              ['Facebook', 'Linkedin', 'Twitter',]),
                                 ',')
+
         return [
             ew.SingleSelectField(
                 name='socialnetwork',
                 label='Social network',
-                validator=fev.UnicodeString(not_empty=True),
+                validator=V.OneOfValidator(socialnetworks),
                 options=[ew.Option(py_value=name, label=name)
                          for name in socialnetworks]),
             ew.TextField(
                 name='accounturl',
                 label='Account url',
-                validator=fev.UnicodeString(not_empty=True))
+                validator=V.UnicodeString(not_empty=True),
+            )
         ]
 
 
@@ -454,7 +482,7 @@ class RemoveSocialNetworkForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveSocialNetworkForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["account"] = kw.get('account', '')
         d["socialnetwork"] = kw.get('socialnetwork', '')
         return d
@@ -465,22 +493,18 @@ class AddInactivePeriodForm(ForgeForm):
     class fields(ew_core.NameList):
         startdate = ew.TextField(
             label='Start date',
-            validator=formencode.All(
-                V.DateValidator(),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.DateValidator())
         enddate = ew.TextField(
             label='End date',
-            validator=formencode.All(
-                V.DateValidator(),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.DateValidator())
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(AddInactivePeriodForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         if d['startdate'] > d['enddate']:
-                raise formencode.Invalid(
-                    'Invalid period: start date greater than end date.',
-                    kw, state)
+            raise formencode.Invalid(
+                'Invalid period: start date greater than end date.',
+                kw, state)
         return d
 
 
@@ -511,7 +535,7 @@ class RemoveInactivePeriodForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveInactivePeriodForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d['startdate'] = V.convertDate(kw.get('startdate', ''))
         d['enddate'] = V.convertDate(kw.get('enddate', ''))
         return d
@@ -524,28 +548,22 @@ class AddTimeSlotForm(ForgeForm):
             label='Weekday',
             options=[ew.Option(py_value=wd, label=wd)
                      for wd in weekdays],
-            validator=formencode.All(
-                V.OneOfValidator(weekdays),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.OneOfValidator(weekdays))
         starttime = ew.TextField(
             label='Start time',
-            validator=formencode.All(
-                V.TimeValidator(),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.TimeValidator())
         endtime = ew.TextField(
             label='End time',
-            validator=formencode.All(
-                V.TimeValidator(),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.TimeValidator())
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(AddTimeSlotForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         if (d['starttime']['h'], d['starttime']['m']) > \
            (d['endtime']['h'], d['endtime']['m']):
-                raise formencode.Invalid(
-                    'Invalid period: start time greater than end time.',
-                    kw, state)
+            raise formencode.Invalid(
+                'Invalid period: start time greater than end time.',
+                kw, state)
         return d
 
 
@@ -581,7 +599,7 @@ class RemoveTimeSlotForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveTimeSlotForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["weekday"] = kw.get('weekday', None)
         d['starttime'] = V.convertTime(kw.get('starttime', ''))
         d['endtime'] = V.convertTime(kw.get('endtime', ''))
@@ -602,6 +620,9 @@ class RemoveTroveCategoryForm(ForgeForm):
                     ew.LinkField(
                         text=cat.fullname,
                         href="/categories/%s" % cat.trove_cat_id),
+                    ew.HTMLField(
+                        text=cat.shortname,
+                        attrs={'disabled': True, 'value': cat.shortname}),
                     ew.SubmitButton(
                         show_errors=False,
                         attrs={'value': 'Remove'})],
@@ -613,7 +634,7 @@ class RemoveTroveCategoryForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveTroveCategoryForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["categoryid"] = kw.get('categoryid')
         if d["categoryid"]:
             d["categoryid"] = int(d['categoryid'])
@@ -629,7 +650,12 @@ class AddTroveCategoryForm(ForgeForm):
             show_errors=False)
         categoryname = ew.TextField(
             label="Category name",
-            validator=fev.UnicodeString(not_empty=True))
+            attrs={},
+            validator=V.UnicodeString(not_empty=True))
+        shortname = ew.TextField(
+            label="Short name",
+            validator=V.UnicodeString(),
+            attrs={'placeholder': 'optional; unique identifier'})
 
     def display(self, **kw):
         upper_category = kw.get('uppercategory_id', 0)
@@ -639,7 +665,7 @@ class AddTroveCategoryForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(AddTroveCategoryForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["uppercategory_id"] = kw.get('uppercategory_id', 0)
         return d
 
@@ -651,19 +677,17 @@ class AddUserSkillForm(ForgeForm):
         selected_skill = ew.HiddenField(
             attrs={'value': ''},
             show_errors=False,
-            validator=fev.UnicodeString(not_empty=True))
+            validator=V.UnicodeString(not_empty=True))
         level = ew.SingleSelectField(
             label="Level of knowledge",
             options=[
                 ew.Option(py_value="low", label="Low level"),
                 ew.Option(py_value="medium", label="Medium level"),
                 ew.Option(py_value="high", label="Advanced level")],
-            validator=formencode.All(
-                V.OneOfValidator(['low', 'medium', 'high']),
-                fev.UnicodeString(not_empty=True)))
+            validator=V.OneOfValidator(['low', 'medium', 'high']))
         comment = ew.TextArea(
             label="Additional comments",
-            validator=fev.UnicodeString(not_empty=False),
+            validator=V.UnicodeString(not_empty=False),
             attrs={'rows': 5, 'cols': 30})
 
     def display(self, **kw):
@@ -688,9 +712,7 @@ class SelectSubCategoryForm(ForgeForm):
         self.fields['selected_category'].options = [
             ew.Option(py_value=el.trove_cat_id, label=el.fullname) for el in categories
         ]
-        self.fields['selected_category'].validator = formencode.All(
-            V.OneOfValidator(categories),
-            fev.UnicodeString(not_empty=True))
+        self.fields['selected_category'].validator = V.OneOfValidator(categories)
         return super(ForgeForm, self).display(**kw)
 
 
@@ -724,7 +746,7 @@ class RemoveSkillForm(ForgeForm):
 
     @ew_core.core.validator
     def to_python(self, kw, state):
-        d = super(RemoveSkillForm, self).to_python(kw, state)
+        d = super().to_python(kw, state)
         d["categoryid"] = kw.get('categoryid', None)
         return d
 
@@ -742,17 +764,13 @@ class RegistrationForm(ForgeForm):
         username = ew.TextField(
             name='username',
             label='Desired Username',
-            validator=fev.Regex(
-                h.re_project_name))
-        username.validator._messages['invalid'] = (
-            'Usernames must include only small letters, numbers, and dashes.'
-            ' They must also start with a letter and be at least 3 characters'
-            ' long.')
+            validator=plugin.AuthenticationProvider.get(None).username_validator(),
+        )
         fields = [
             ew.TextField(
                 name='display_name',
                 label='Displayed Name',
-                validator=fev.UnicodeString(not_empty=True)),
+                validator=V.UnicodeString(not_empty=True)),
             username,
         ]
         if asbool(config.get('auth.require_email_addr', False)):
@@ -764,20 +782,21 @@ class RegistrationForm(ForgeForm):
             ew.PasswordField(
                 name='pw',
                 label='New Password',
-                validator=fev.UnicodeString(
+                attrs=dict(minlength=asint(tg.config.get('auth.min_password_len', 6)), maxlength=asint(tg.config.get('auth.max_password_len', 30))),
+                validator=V.UnicodeString(
                     not_empty=True,
                     min=asint(tg.config.get('auth.min_password_len', 6)),
                     max=asint(tg.config.get('auth.max_password_len', 30)))),
             ew.PasswordField(
                 name='pw2',
                 label='New Password (again)',
-                validator=fev.UnicodeString(not_empty=True)),
+                validator=V.UnicodeString(not_empty=True)),
         ]
         return fields
 
     @ew_core.core.validator
     def to_python(self, value, state):
-        d = super(RegistrationForm, self).to_python(value, state)
+        d = super().to_python(value, state)
         value['username'] = username = value['username'].lower()
         if M.User.by_username(username):
             raise formencode.Invalid(
@@ -787,11 +806,25 @@ class RegistrationForm(ForgeForm):
             raise formencode.Invalid('Passwords must match', value, state)
         return d
 
+    def resources(self):
+        yield ew.JSLink('js/password-validator.js')
+
 
 class AdminForm(ForgeForm):
     template = 'jinja:allura:templates/widgets/admin_form.html'
 
 
+class AdminFormResponsive(ForgeForm):
+    def __init__(self):
+        super().__init__()
+        # use alternate template if responsive overrides are on, but not actually using template override for this
+        # since that would change all forms, and we want to just do individual ones right now
+        for tmpl_override_ep in h.iter_entry_points('allura.theme.override'):
+            if tmpl_override_ep.name == 'responsive':
+                self.template = 'jinja:allura:templates_responsive/widgets/admin_form.html'
+                break
+
+
 class NeighborhoodOverviewForm(ForgeForm):
     template = 'jinja:allura:templates/widgets/neighborhood_overview_form.html'
 
@@ -806,7 +839,7 @@ class NeighborhoodOverviewForm(ForgeForm):
         project_template = ffw.AutoResizeTextarea(
             validator=V.JsonValidator(if_empty=''))
         icon = ew.FileField(attrs={'accept': 'image/*'})
-        tracking_id = ew.TextField()
+        tracking_id = ew.TextField(attrs=(dict(placeholder='UA-123456-0', pattern='UA-[0-9]+-[0-9]+')))
         project_list_url = ew.TextField()
         anchored_tools = ffw.AutoResizeTextarea()
         prohibited_tools = ffw.AutoResizeTextarea()
@@ -818,7 +851,7 @@ class NeighborhoodOverviewForm(ForgeForm):
         else:
             self.list_color_inputs = False
             self.color_inputs = []
-        return super(NeighborhoodOverviewForm, self).from_python(value, state)
+        return super().from_python(value, state)
 
     def display_field(self, field, ignore_errors=False):
         if field.name == "css" and self.list_color_inputs:
@@ -834,7 +867,6 @@ class NeighborhoodOverviewForm(ForgeForm):
                            '<td class="right"><div class="%(ctx_name)s-%(inp_name)s-inp"><table class="input_inner">' \
                            '<tr><td><input type="text" class="%(inp_type)s" name="%(ctx_name)s-%(inp_name)s" ' \
                            'value="%(inp_value)s"></td><td>%(inp_additional)s</td></tr></table></div></td></tr>\n' % {
-                               'ctx_id': ctx['id'],
                                'ctx_name': ctx['name'],
                                'inp_name': inp['name'],
                                'inp_value': inp['value'],
@@ -845,16 +877,16 @@ class NeighborhoodOverviewForm(ForgeForm):
             display += '</table>'
 
             if ctx['errors'] and field.show_errors and not ignore_errors:
-                display = "%s<div class='error'>%s</div>" % (display,
-                                                             ctx['errors'])
+                display = "{}<div class='error'>{}</div>".format(display,
+                                                                 ctx['errors'])
 
-            return h.html.literal(display)
+            return Markup(display)
         else:
-            return super(NeighborhoodOverviewForm, self).display_field(field, ignore_errors)
+            return super().display_field(field, ignore_errors)
 
     @ew_core.core.validator
     def to_python(self, value, state):
-        d = super(NeighborhoodOverviewForm, self).to_python(value, state)
+        d = super().to_python(value, state)
         neighborhood = M.Neighborhood.query.get(name=d.get('name', None))
         if neighborhood and neighborhood.features['css'] == "picker":
             css_form_dict = {}
@@ -866,8 +898,7 @@ class NeighborhoodOverviewForm(ForgeForm):
         return d
 
     def resources(self):
-        for r in super(NeighborhoodOverviewForm, self).resources():
-            yield r
+        yield from super().resources()
         yield ew.CSSLink('css/colorPicker.css')
         yield ew.CSSLink('css/jqfontselector.css')
         yield ew.CSSScript('''
@@ -923,13 +954,13 @@ class NeighborhoodAddProjectForm(ForgeForm):
         submit_text='Start',
         neighborhood=None)
     # tools installed by default
-    default_tools = [u'wiki', u'git', u'tickets', u'discussion']
+    default_tools = ['wiki', 'git', 'tickets', 'discussion']
 
     @property
     def fields(self):
         provider = plugin.ProjectRegistrationProvider.get()
         tools_options = []
-        for ep, tool in g.entry_points["tool"].iteritems():
+        for ep, tool in g.entry_points["tool"].items():
             if tool.status == 'production' and tool._installable(tool_name=ep,
                                                                  nbhd=c.project.neighborhood,
                                                                  project_tools=[]):
@@ -942,7 +973,7 @@ class NeighborhoodAddProjectForm(ForgeForm):
             ew.InputField(name='project_name', label='Project Name',
                           field_type='text',
                           validator=formencode.All(
-                              fev.UnicodeString(not_empty=True, max=40),
+                              V.UnicodeString(not_empty=True, max=40),
                               V.MaxBytesValidator(max=40)),
                           ),
             ew.InputField(name='project_unixname',
@@ -957,15 +988,14 @@ class NeighborhoodAddProjectForm(ForgeForm):
 
     @ew_core.core.validator
     def validate(self, value, state=None):
-        value = super(NeighborhoodAddProjectForm, self).validate(value, state)
+        value = super().validate(value, state)
         provider = plugin.ProjectRegistrationProvider.get()
         if not provider.phone_verified(c.user, c.project.neighborhood):
-            raise formencode.Invalid(u'phone-verification', value, None)
+            raise formencode.Invalid('phone-verification', value, None)
         return value
 
     def resources(self):
-        for r in super(NeighborhoodAddProjectForm, self).resources():
-            yield r
+        yield from super().resources()
         yield ew.CSSLink('css/add_project.css')
         neighborhood = g.antispam.enc('neighborhood')
         project_name = g.antispam.enc('project_name')
@@ -1029,7 +1059,7 @@ class NeighborhoodAddProjectForm(ForgeForm):
                     }
                 });
                 var suggest_name = function(project_name) {
-                    return project_name.replace(/[^A-Za-z0-9]+/g, '-').toLowerCase();
+                    return project_name.replace(/[^A-Za-z0-9]+/g, '-').toLowerCase().replace(/^-+|-+$/g, '');
                 };
                 var check_names = function() {
                     var data = {
@@ -1076,7 +1106,7 @@ class MoveTicketForm(ForgeForm):
 
     def __init__(self, *args, **kwargs):
         trackers = kwargs.pop('trackers', [])
-        super(MoveTicketForm, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fields.tracker.options = (
             [ew.Option(py_value=v, label=l, selected=s)
              for v, l, s in sorted(trackers, key=lambda x: x[1])])
@@ -1089,7 +1119,7 @@ class CsrfForm(ew.SimpleForm):
         return [ew.HiddenField(name='_session_id')]
 
     def context_for(self, field):
-        ctx = super(CsrfForm, self).context_for(field)
+        ctx = super().context_for(field)
         if field.name == '_session_id':
             ctx['value'] = tg.request.cookies.get('_session_id') or tg.request.environ['_session_id']
         return ctx
@@ -1100,7 +1130,7 @@ class AwardGrantForm(ForgeForm):
     def __init__(self, *args, **kw):
         self._awards = kw.pop('awards', [])
         self._project_select_url = kw.pop('project_select_url', '')
-        super(AwardGrantForm, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
 
     def award_options(self):
         return [ew.Option(py_value=a.short, label=a.short) for a in self._awards]
@@ -1122,7 +1152,7 @@ class AwardGrantForm(ForgeForm):
             ew.TextArea(
                 name='comment',
                 label='Comment',
-                validator=fev.UnicodeString(not_empty=False),
+                validator=V.UnicodeString(not_empty=False),
                 attrs={'rows': 5, 'cols': 30}),
         ]
 
@@ -1136,7 +1166,7 @@ class AdminSearchForm(ForgeForm):
         submit_text=None)
 
     def __init__(self, fields, *args, **kw):
-        super(AdminSearchForm, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
         self._fields = fields
 
     @property
@@ -1160,7 +1190,7 @@ class AdminSearchForm(ForgeForm):
             ])]
 
     def context_for(self, field):
-        ctx = super(AdminSearchForm, self).context_for(field)
+        ctx = super().context_for(field)
         if field.name is None and not ctx.get('value'):
             # RowField does not pass context down to the children :(
             render_ctx = ew_core.widget_context.render_context
