@@ -14,38 +14,44 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
+from __future__ import annotations
 
 import logging
 from collections import defaultdict
 from datetime import datetime
+import typing
 
 import pymongo
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request
+from tg import tmpl_context as c, app_globals as g
+from tg import request
 from ming import schema as S
-from ming.orm import state, session
-from ming.orm import FieldProperty, ForeignIdProperty, RelationProperty
-from ming.orm.declarative import MappedClass
+from ming.odm import state, session
+from ming.odm import FieldProperty, ForeignIdProperty, RelationProperty
+from ming.odm.declarative import MappedClass
 from ming.utils import LazyProperty
-from webhelpers import feedgenerator as FG
+import feedgenerator as FG
 
 from allura.lib import helpers as h
 from allura.lib import security
 from allura.lib import utils
 from allura.lib import plugin
 from allura.lib import exceptions as forge_exc
-
+from allura.lib.decorators import memoize
 from allura.lib.search import SearchIndexable
 from .session import main_orm_session
 from .session import project_orm_session
-
 from .session import artifact_orm_session
-from .index import ArtifactReference
+from .index import ArtifactReference, Shortlink
 from .types import ACL, MarkdownCache
 from .project import AppConfig
 from .notification import MailFooter
 
-from filesystem import File
+from .filesystem import File
+import six
+
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+    from allura.model import BaseAttachment
 
 log = logging.getLogger(__name__)
 
@@ -81,6 +87,8 @@ class Artifact(MappedClass, SearchIndexable):
             if c.project and not skip_last_updated:
                 c.project.last_updated = datetime.utcnow()
 
+    query: 'Query[Artifact]'
+
     type_s = 'Generic Artifact'
 
     # Artifact base schema
@@ -139,6 +147,9 @@ class Artifact(MappedClass, SearchIndexable):
         Artifact.
 
         """
+        if hasattr(self, '_ref'):
+            return self._ref
+
         return ArtifactReference.from_artifact(self)
 
     @LazyProperty
@@ -156,6 +167,9 @@ class Artifact(MappedClass, SearchIndexable):
         :return: list of :attr:`allura.model.index.ArtifactReference._id`'s
 
         """
+        if hasattr(self, '_backrefs'):
+            return self._backrefs
+
         q = ArtifactReference.query.find(dict(references=self.index_id()))
         return [aref._id for aref in q]
 
@@ -233,16 +247,24 @@ class Artifact(MappedClass, SearchIndexable):
             app_config_id=self.app_config._id,
             artifact_index_id=self.index_id())
 
-    def subscribed(self, user=None):
+    @memoize  # since its called many times from edit_post.html within threaded comments
+    def subscribed(self, user=None, include_parents=True) -> bool:
         from allura.model import Mailbox
         if user is None:
             user = c.user
-        return Mailbox.subscribed(
-            user_id=user._id,
-            project_id=self.app_config.project_id,
-            app_config_id=self.app_config._id,
-            artifact=self,
-        )
+        if user.is_anonymous():
+            return False
+        user_proj_app_q = dict(user_id=user._id,
+                               project_id=self.app_config.project_id,
+                               app_config_id=self.app_config._id)
+        art_subscribed = Mailbox.subscribed(artifact=self, **user_proj_app_q)
+        if art_subscribed:
+            return True
+        if include_parents:
+            tool_subscribed = Mailbox.subscribed(**user_proj_app_q)
+            if tool_subscribed:
+                return True
+        return False
 
     def primary(self):
         """If an artifact is a "secondary" artifact (discussion of a ticket, for
@@ -267,7 +289,7 @@ class Artifact(MappedClass, SearchIndexable):
 
         """
         if subject:
-            return 'mailto:%s?subject=[%s:%s:%s] Re: %s' % (
+            return 'mailto:{}?subject=[{}:{}:{}] Re: {}'.format(
                 self.email_address,
                 self.app_config.project.shortname,
                 self.app_config.options.mount_point,
@@ -329,6 +351,14 @@ class Artifact(MappedClass, SearchIndexable):
             snippet_s='',
             deleted_b=self.deleted)
 
+    @property
+    def type_name(self):
+        """
+        :return: a presentation name for this type of artifact
+        :rtype: str
+        """
+        return self.type_s.lower()
+
     def url(self):
         """Return the URL for this Artifact.
 
@@ -357,6 +387,30 @@ class Artifact(MappedClass, SearchIndexable):
         """
         return self.shorthand_id()
 
+    def _get_discussion_thread(self):
+        from .discuss import Thread
+        threads = Thread.query.find(dict(ref_id=self.index_id())).all()
+        if not threads:
+            return None
+
+        if len(threads) == 1:
+            return threads[0]
+
+        # there should not be multiple threads, we'll merge them
+        destination = threads.pop()
+        for thread in threads:
+            for post in thread.posts:
+                post.thread_id = destination._id
+                destination.num_replies += 1
+                destination.last_post_date = max(destination.last_post_date, post.mod_date)
+                session(post).flush(post)
+                session(post).expunge(post)  # so thread.posts ref later in the code doesn't use stale posts
+            Thread.query.remove({'_id': thread._id})  # NOT thread.delete() since that would remove its posts too
+            thread.attachment_class().query.update({'thread_id': thread._id},
+                                                   {'$set': {'thread_id': destination._id}},
+                                                   multi=True)
+        return destination
+
     def get_discussion_thread(self, data=None):
         """Return the discussion thread and parent_id for this artifact.
 
@@ -364,31 +418,14 @@ class Artifact(MappedClass, SearchIndexable):
 
         """
         from .discuss import Thread
-        threads = Thread.query.find(dict(ref_id=self.index_id())).all()
-        if not threads:
+        thread = self._get_discussion_thread()
+        if not thread:
             idx = self.index()
-            t = Thread.new(
+            thread = Thread.new(
                 app_config_id=self.app_config_id,
                 discussion_id=self.app_config.discussion_id,
                 ref_id=idx['id'],
                 subject='%s discussion' % h.get_first(idx, 'title'))
-        elif len(threads) == 1:
-            t = threads[0]
-        else:
-            # there should not be multiple threads, we'll merge them
-            destination = threads.pop()
-            for thread in threads:
-                for post in thread.posts:
-                    post.thread_id = destination._id
-                    destination.num_replies += 1
-                    destination.last_post_date = max(destination.last_post_date, post.mod_date)
-                    session(post).flush(post)
-                    session(post).expunge(post)  # so thread.posts ref later in the code doesn't use stale posts
-                Thread.query.remove({'_id': thread._id})  # NOT thread.delete() since that would remove its posts too
-                thread.attachment_class().query.update({'thread_id': thread._id},
-                                                       {'$set': {'thread_id': destination._id}},
-                                                       multi=True)
-            t = destination
 
         parent_id = None
         if data:
@@ -396,7 +433,7 @@ class Artifact(MappedClass, SearchIndexable):
             if in_reply_to:
                 parent_id = in_reply_to[0]
 
-        return t, parent_id
+        return thread, parent_id
 
     @LazyProperty
     def discussion_thread(self):
@@ -419,7 +456,7 @@ class Artifact(MappedClass, SearchIndexable):
 
         :param filename: file name
         :param fp: a file-like object (implements ``read()``)
-        :param \*\*kw: passed through to Attachment class constructor
+        :param kw: passed through to Attachment class constructor
 
         """
         att = self.attachment_class().save_attachment(
@@ -429,16 +466,43 @@ class Artifact(MappedClass, SearchIndexable):
 
     @LazyProperty
     def attachments(self):
-        atts = self.attachment_class().query.find(dict(
-            app_config_id=self.app_config_id, artifact_id=self._id, type='attachment')).all()
-        return utils.unique_attachments(atts)
+        return self._get_attachments()
+
+    def _get_attachments(self, unique_files_only=True, include_thumbnails=False) -> list[BaseAttachment]:
+        if hasattr(self, '_attachments'):
+            atts = self._attachments
+        else:
+            try:
+                atts = self.attachment_class().query.find({
+                    'app_config_id': self.app_config_id,
+                    'artifact_id': {
+                        # some artifact_ids are ObjectIds and some are strings
+                        '$in': [self._id, str(self._id)]
+                    },
+                    'type': {'$in': ['attachment'] + (['thumbnail'] if include_thumbnails else [])}
+                }).all()
+            except NotImplementedError:
+                atts = []
+        if unique_files_only:
+            atts = utils.unique_attachments(atts)
+        return atts
 
     def delete(self):
         """Delete this Artifact.
 
         """
-        ArtifactReference.query.remove(dict(_id=self.index_id()))
-        super(Artifact, self).delete()
+        thread = self._get_discussion_thread()
+        if thread:
+            thread.delete()
+        for att in self._get_attachments(unique_files_only=False, include_thumbnails=True):
+            att.delete()
+            session(att).flush(att)
+        idx_id = self.index_id()
+        ArtifactReference.query.remove(dict(_id=idx_id))
+        Shortlink.query.remove(dict(ref_id=idx_id))
+        super().delete()
+        session(self).flush(self)
+        session(self).imap.expunge(self)
 
     def get_mail_footer(self, notification, toaddr):
         allow_email_posting = self.app.config.options.get('AllowEmailPosting', True)
@@ -455,7 +519,7 @@ class Artifact(MappedClass, SearchIndexable):
         False otherwise
         """
         pkg = cls.__module__.split('.', 1)[0]
-        opt = u'{}.rate_limits'.format(pkg)
+        opt = f'{pkg}.rate_limits'
 
         def count_in_app():
             return cls.query.find(dict(app_config_id=app_config._id)).count()
@@ -484,6 +548,8 @@ class Snapshot(Artifact):
                    'author.id',
                    ]
 
+    query: 'Query[Snapshot]'
+
     _id = FieldProperty(S.ObjectId)
     artifact_id = FieldProperty(S.ObjectId)
     artifact_class = FieldProperty(str)
@@ -515,10 +581,10 @@ class Snapshot(Artifact):
         return result
 
     def original(self):
-        raise NotImplemented('original')  # pragma no cover
+        raise NotImplementedError('original')  # pragma no cover
 
     def shorthand_id(self):
-        return '%s#%s' % (self.original().shorthand_id(), self.version)
+        return f'{self.original().shorthand_id()}#{self.version}'
 
     def clear_user_data(self):
         """ Redact author data for a given user """
@@ -557,17 +623,19 @@ class VersionedArtifact(Artifact):
         name = 'versioned_artifact'
         history_class = Snapshot
 
+    query: 'Query[VersionedArtifact]'
+
     version = FieldProperty(S.Int, if_missing=0)
 
     def commit(self, update_stats=True):
         '''Save off a snapshot of the artifact and increment the version #'''
         try:
             ip_address = utils.ip_address(request)
-        except:
+        except Exception:
             ip_address = '0.0.0.0'
         data = dict(
             artifact_id=self._id,
-            artifact_class='%s.%s' % (
+            artifact_class='{}.{}'.format(
                 self.__class__.__module__,
                 self.__class__.__name__),
             author=dict(
@@ -606,7 +674,7 @@ class VersionedArtifact(Artifact):
             n = self.version + n + 1
         ss = self.__mongometa__.history_class.query.get(
             artifact_id=self._id,
-            artifact_class='%s.%s' % (
+            artifact_class='{}.{}'.format(
                 self.__class__.__module__,
                 self.__class__.__name__),
             version=n)
@@ -617,7 +685,7 @@ class VersionedArtifact(Artifact):
     def revert(self, version):
         ss = self.get_version(version)
         old_version = self.version
-        for k, v in ss.data.iteritems():
+        for k, v in ss.data.items():
             setattr(self, k, v)
         self.version = old_version
 
@@ -637,7 +705,7 @@ class VersionedArtifact(Artifact):
 
     def delete(self):
         # remove history so that the snapshots aren't left orphaned
-        super(VersionedArtifact, self).delete()
+        super().delete()
         HC = self.__mongometa__.history_class
         HC.query.remove(dict(artifact_id=self._id))
 
@@ -646,6 +714,7 @@ class VersionedArtifact(Artifact):
         if 'user' in kwargs:
             def distinct_artifacts_by_user():
                 # count distinct items, not total (e.g. many edits to a single wiki page doesn't count against you)
+                # query history here, as regular base artifacts have no author information
                 HC = cls.__mongometa__.history_class
                 artifacts = HC.query.find({'author.id': kwargs['user']._id}).distinct('artifact_id')
                 """
@@ -657,11 +726,10 @@ class VersionedArtifact(Artifact):
                 """
                 return len(artifacts)
             kwargs['count_by_user'] = distinct_artifacts_by_user
-        return super(VersionedArtifact, cls).is_limit_exceeded(*args, **kwargs)
+        return super().is_limit_exceeded(*args, **kwargs)
 
 
 class Message(Artifact):
-
     """
     A message
 
@@ -673,6 +741,9 @@ class Message(Artifact):
     class __mongometa__:
         session = artifact_orm_session
         name = 'message'
+
+    query: 'Query[Message]'
+
     type_s = 'Generic Message'
 
     _id = FieldProperty(str, if_missing=h.gen_message_id)
@@ -721,6 +792,9 @@ class AwardFile(File):
     class __mongometa__:
         session = main_orm_session
         name = 'award_file'
+
+    query: 'Query[AwardFile]'
+
     award_id = FieldProperty(S.ObjectId)
 
 
@@ -730,6 +804,9 @@ class Award(Artifact):
         session = main_orm_session
         name = 'award'
         indexes = ['short']
+
+    query: 'Query[Award]'
+
     type_s = 'Generic Award'
 
     from .project import Neighborhood
@@ -773,6 +850,9 @@ class AwardGrant(Artifact):
         session = main_orm_session
         name = 'grant'
         indexes = ['short']
+
+    query: 'Query[AwardGrant]'
+
     type_s = 'Generic Award Grant'
 
     _id = FieldProperty(S.ObjectId)
@@ -821,12 +901,12 @@ class AwardGrant(Artifact):
 
 class RssFeed(FG.Rss201rev2Feed):
     def rss_attributes(self):
-        attrs = super(RssFeed, self).rss_attributes()
+        attrs = super().rss_attributes()
         attrs['xmlns:atom'] = 'http://www.w3.org/2005/Atom'
         return attrs
 
     def add_root_elements(self, handler):
-        super(RssFeed, self).add_root_elements(handler)
+        super().add_root_elements(handler)
         if self.feed['feed_url'] is not None:
             handler.addQuickElement('atom:link', '', {
                 'rel': 'self',
@@ -853,14 +933,17 @@ class Feed(MappedClass):
              ('app_config_id', pymongo.ASCENDING),
              ('pubdate', pymongo.DESCENDING)),
             # used in ext/user_profile/user_main.py for user feeds
-            'author_link',
+            (('author_link', pymongo.ASCENDING),
+             ('pubdate', pymongo.DESCENDING)),
             # used in project feed
             (('project_id', pymongo.ASCENDING),
              ('pubdate', pymongo.DESCENDING)),
         ]
 
+    query: 'Query[Feed]'
+
     _id = FieldProperty(S.ObjectId)
-    ref_id = ForeignIdProperty('ArtifactReference')
+    ref_id: str = ForeignIdProperty('ArtifactReference')
     neighborhood_id = ForeignIdProperty('Neighborhood')
     project_id = ForeignIdProperty('Project')
     app_config_id = ForeignIdProperty('AppConfig')
@@ -882,11 +965,11 @@ class Feed(MappedClass):
         self.author_name = ""
         self.author_link = ""
         title_parts = self.title.partition(" modified by ")
-        self.title = u"".join(title_parts[0:2]) + (u"<REDACTED>" if title_parts[2] else '')
+        self.title = "".join(title_parts[0:2]) + ("<REDACTED>" if title_parts[2] else '')
 
     @classmethod
     def from_username(cls, username):
-        return cls.query.find({'author_link': u"/u/{}/".format(username)}).all()
+        return cls.query.find({'author_link': f"/u/{username}/"}).all()
 
     @classmethod
     def has_access(cls, artifact):
@@ -916,7 +999,7 @@ class Feed(MappedClass):
         if author_name is None:
             author_name = author.get_pref('display_name')
         if title is None:
-            title = '%s modified by %s' % (
+            title = '{} modified by {}'.format(
                 h.get_first(idx, 'title'), author_name)
         if description is None:
             description = title
@@ -944,9 +1027,9 @@ class Feed(MappedClass):
     @classmethod
     def feed(cls, q, feed_type, title, link, description,
              since=None, until=None, page=None, limit=None):
-        "Produces webhelper.feedgenerator Feed"
-        d = dict(title=title, link=h.absurl(link),
-                 description=description, language=u'en',
+        "Produces feedgenerator Feed"
+        d = dict(title=title, link=h.absurl(h.urlquote(link)),
+                 description=description, language='en',
                  feed_url=request.url)
         if feed_type == 'atom':
             feed = FG.Atom1Feed(**d)
@@ -954,6 +1037,8 @@ class Feed(MappedClass):
             feed = RssFeed(**d)
         limit, page = h.paging_sanitizer(limit or 10, page)
         query = defaultdict(dict)
+        if callable(q):
+            q = q(since, until, page, limit)
         query.update(q)
         if since is not None:
             query['pubdate']['$gte'] = since
@@ -965,7 +1050,7 @@ class Feed(MappedClass):
         cur = cur.skip(limit * page)
         for r in cur:
             feed.add_item(title=r.title,
-                          link=h.absurl(r.link.encode('utf-8')),
+                          link=h.absurl(h.urlquote_path_only(r.link)),
                           pubdate=r.pubdate,
                           description=r.description,
                           unique_id=h.absurl(r.unique_id),
@@ -982,6 +1067,8 @@ class VotableArtifact(MappedClass):
         session = main_orm_session
         name = 'vote'
 
+    query: 'Query[VotableArtifact]'
+
     votes = FieldProperty(int, if_missing=0)
     votes_up = FieldProperty(int, if_missing=0)
     votes_down = FieldProperty(int, if_missing=0)
@@ -1050,12 +1137,68 @@ class VotableArtifact(MappedClass):
         }
 
 
+class ReactableArtifact(MappedClass):
+
+    """Reaction support for the Artifact. Use as a mixin."""
+
+    react_counts = FieldProperty({str: None}, if_missing=dict())
+    # dict to store reaction counts
+    react_users = FieldProperty({str: None}, if_missing=dict())
+    # dict to store reactions vs usernames
+
+    def post_reaction(self, r, user):
+        current_reaction = self.user_reacted(user)
+        if current_reaction is None:
+            # no prev reactions. simply append
+            if r in self.react_users:
+                self.react_users[r].append(user.username)
+            else:
+                self.react_users[r] = [user.username]
+            self.update_react_count(r)
+        elif current_reaction == r:
+            # prev=current so remove
+            self.react_users[r].remove(user.username)
+            self.update_react_count(r, add=False)
+            if len(self.react_users[r]) == 0:
+                self.react_users.pop(r)
+        else:
+            # prev!=currnet so remove prev then append
+            self.react_users[current_reaction].remove(user.username)
+            if r in self.react_users:
+                self.react_users[r].append(user.username)
+            else:
+                self.react_users[r] = [user.username]
+            self.update_react_count(current_reaction, add=False)
+            self.update_react_count(r)
+            if len(self.react_users[current_reaction]) == 0:
+                self.react_users.pop(current_reaction)
+
+    def user_reacted(self, user):
+        for i in self.react_users:
+            if user.username in self.react_users[i]:
+                return i
+        return
+
+    def update_react_count(self, r, add=True):
+        i = 1
+        if not add:
+            i = -1
+        if r in self.react_counts:
+            self.react_counts[r] += i
+            if self.react_counts[r] == 0:
+                self.react_counts.pop(r)
+        else:
+            self.react_counts[r] = 1
+
+
 class MovedArtifact(Artifact):
 
     class __mongometa__:
         session = artifact_orm_session
         name = 'moved_artifact'
 
+    query: 'Query[MovedArtifact]'
+
     _id = FieldProperty(S.ObjectId)
     app_config_id = ForeignIdProperty(
         'AppConfig', if_missing=lambda: c.app.config._id)
@@ -1072,8 +1215,10 @@ class SpamCheckResult(MappedClass):
             ('user_id', 'result'),
         ]
 
+    query: 'Query[SpamCheckResult]'
+
     _id = FieldProperty(S.ObjectId)
-    ref_id = ForeignIdProperty('ArtifactReference')
+    ref_id: str = ForeignIdProperty('ArtifactReference')
     ref = RelationProperty('ArtifactReference', via='ref_id')
     project_id = ForeignIdProperty('Project')
     project = RelationProperty('Project', via='project_id')
