@@ -15,88 +15,55 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
+from __future__ import annotations
+
 import logging
-import urllib
 import calendar
-from urlparse import urlparse
+import typing
+
+import six
+from markupsafe import Markup
+from urllib.parse import urlparse
 from email import header
 from hashlib import sha256
 from datetime import timedelta, datetime, time
-
 import os
 import re
+
 from pytz import timezone
 import pymongo
+from pymongo.errors import DuplicateKeyError
+from bson import ObjectId
 from tg import config
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request
+from tg import tmpl_context as c, app_globals as g
+from tg import request
 from ming import schema as S
-from ming import Field, collection
-from ming.orm import session, state
-from ming.orm import FieldProperty, RelationProperty, ForeignIdProperty
-from ming.orm.declarative import MappedClass
-from ming.orm.ormsession import ThreadLocalORMSession
+from ming import Field
+from ming.odm import session, state
+from ming.odm import FieldProperty, RelationProperty, ForeignIdProperty
+from ming.odm.declarative import MappedClass
+from ming.odm.odmsession import ThreadLocalODMSession
 from ming.utils import LazyProperty
 
-import types
 import allura.tasks.mail_tasks
 from allura.lib import helpers as h
 from allura.lib import plugin
 from allura.lib import utils
 from allura.lib.decorators import memoize
 from allura.lib.search import SearchIndexable
-from .session import main_orm_session, main_doc_session
+from .session import main_orm_session, main_explicitflush_orm_session
 from .session import project_orm_session
 from .timeline import ActivityNode, ActivityObject
 
 
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+    from allura.model.project import Project
+
+
 log = logging.getLogger(__name__)
 
 
-def smart_str(s, encoding='utf-8', strings_only=False, errors='strict'):
-    """
-    Returns a bytestring version of 's', encoded as specified in 'encoding'.
-
-    If strings_only is True, don't convert (some) non-string-like objects.
-
-    This function was borrowed from Django
-    """
-    if strings_only and isinstance(s, (types.NoneType, int)):
-        return s
-    elif not isinstance(s, basestring):
-        try:
-            return str(s)
-        except UnicodeEncodeError:
-            if isinstance(s, Exception):
-                # An Exception subclass containing non-ASCII data that doesn't
-                # know how to print itself properly. We shouldn't raise a
-                # further exception.
-                return ' '.join([smart_str(arg, encoding, strings_only,
-                                           errors) for arg in s])
-            return unicode(s).encode(encoding, errors)
-    elif isinstance(s, unicode):
-        r = s.encode(encoding, errors)
-        return r
-    elif s and encoding != 'utf-8':
-        return s.decode('utf-8', errors).encode(encoding, errors)
-    else:
-        return s
-
-
-def generate_smart_str(params):
-    for (key, value) in params:
-        yield smart_str(key), smart_str(value)
-
-
-def urlencode(params):
-    """
-    A version of Python's urllib.urlencode() function that can operate on
-    unicode strings. The parameters are first case to UTF-8 encoded strings and
-    then encoded as per normal.
-    """
-    return urllib.urlencode([i for i in generate_smart_str(params)])
-
-
 class AlluraUserProperty(ForeignIdProperty):
     '''
     Specialized ForeignIdProperty for users, specifically to set allow_none=True
@@ -105,11 +72,11 @@ class AlluraUserProperty(ForeignIdProperty):
     '''
 
     def __init__(self, **kwargs):
-        super(AlluraUserProperty, self).__init__('User', allow_none=True, **kwargs)
+        super().__init__('User', allow_none=True, **kwargs)
 
 
 class EmailAddress(MappedClass):
-    re_format = re.compile('^.*\s+<(.*)>\s*$')
+    re_format = re.compile(r'^.*\s+<(.*)>\s*$')
 
     class __mongometa__:
         name = 'email_address'
@@ -117,11 +84,16 @@ class EmailAddress(MappedClass):
         indexes = ['nonce', ]
         unique_indexes = [('email', 'claimed_by_user_id'), ]
 
+    query: Query[EmailAddress]
+
     _id = FieldProperty(S.ObjectId)
     email = FieldProperty(str)
     claimed_by_user_id = FieldProperty(S.ObjectId, if_missing=None)
     confirmed = FieldProperty(bool, if_missing=False)
     nonce = FieldProperty(str)
+    valid_address = FieldProperty(bool, if_missing=None)
+    valid_details = FieldProperty(S.Anything, if_missing=None)
+    valid_check_date = FieldProperty(datetime, if_missing=None)
 
     @classmethod
     def get(cls, **kw):
@@ -171,7 +143,7 @@ class EmailAddress(MappedClass):
         if '@' in addr:
             try:
                 user, domain = addr.strip().split('@')
-                return '%s@%s' % (user, domain.lower())
+                return f'{user}@{domain.lower()}'
             except ValueError:
                 return addr.strip()
         else:
@@ -189,7 +161,7 @@ class EmailAddress(MappedClass):
             ))
 
             allura.tasks.mail_tasks.send_system_mail_to_user(self.email,
-                                                             u'%s - Email address claim attempt' % config['site_name'],
+                                                             '%s - Email address claim attempt' % config['site_name'],
                                                              text)
 
     def set_nonce_hash(self):
@@ -205,15 +177,15 @@ please visit the following URL:
 
 %s
 ''' % (self.email,
-       self.claimed_by_user(include_pending=True).username,
-       h.absurl('/auth/verify_addr?a={}'.format(h.urlquote(self.nonce))),
+            self.claimed_by_user(include_pending=True).username,
+            h.absurl(f'/auth/verify_addr?a={h.urlquote(self.nonce)}'),
        )
         log.info('Verification email:\n%s', text)
         allura.tasks.mail_tasks.sendsimplemail.post(
             fromaddr=g.noreply,
             reply_to=g.noreply,
             toaddr=self.email,
-            subject=u'%s - Email address verification' % config['site_name'],
+            subject='%s - Email address verification' % config['site_name'],
             message_id=h.gen_message_id(),
             text=text)
 
@@ -223,6 +195,8 @@ class AuthGlobals(MappedClass):
         name = 'auth_globals'
         session = main_orm_session
 
+    query: Query[AuthGlobals]
+
     _id = FieldProperty(int)
     next_uid = FieldProperty(int, if_missing=10000)
 
@@ -271,12 +245,14 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
     class __mongometa__:
         name = 'user'
         session = main_orm_session
-        indexes = ['tool_data.sfx.userid', 'tool_data.AuthPasswordReset.hash']
+        indexes = ['tool_data.AuthPasswordReset.hash']
         unique_indexes = ['username']
         custom_indexes = [
             dict(fields=('tool_data.phone_verification.number_hash',), sparse=True),
         ]
 
+    query: Query[User]
+
     type_s = 'User'
 
     _id = FieldProperty(S.ObjectId)
@@ -284,7 +260,8 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
     username = FieldProperty(str)
     email_addresses = FieldProperty([str])
     password = FieldProperty(str)
-    last_password_updated = FieldProperty(datetime)
+    last_password_updated = FieldProperty(datetime)  # to access, use AuthProvider's get_last_password_updated
+    reg_date = FieldProperty(datetime)  # to access, use user.registration_date()
     projects = FieldProperty(S.Deprecated)
     # full mount point: prefs dict
     tool_preferences = FieldProperty(S.Deprecated)
@@ -298,11 +275,13 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         email_address=str,
         email_format=str,
         disable_user_messages=bool,
+        mention_notifications=bool,
         multifactor=bool,
+        message_reply_real_address=bool,
     ))
     # Additional top-level fields can/should be accessed with get/set_pref also
     # Not sure why we didn't put them within the 'preferences' dictionary :(
-    display_name = FieldPropertyDisplayName(str)
+    display_name: str = FieldPropertyDisplayName(str)
     # Personal data
     sex = FieldProperty(
         S.OneOf('Male', 'Female', 'Other', 'Unknown',
@@ -343,12 +322,16 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         session_ip=str,
         session_ua=str))
 
+    def __repr__(self):
+        return ('<User username={s.username!r} display_name={s.display_name!r} _id={s._id!r} '
+                'disabled={s.disabled!r} pending={s.pending!r}>'.format(s=self))
+
     def index(self):
         provider = plugin.AuthenticationProvider.get(None)  # no need in request here
-        localization = '%s/%s' % (
+        localization = '{}/{}'.format(
             self.get_pref('localization')['country'],
             self.get_pref('localization')['city'])
-        socialnetworks = ' '.join(['%s: %s' % (n['socialnetwork'], n['accounturl'])
+        socialnetworks = ' '.join(['{}: {}'.format(n['socialnetwork'], n['accounturl'])
                                    for n in self.get_pref('socialnetworks')])
         fields = dict(
             id=self.index_id(),
@@ -380,6 +363,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
             last_access_session_date_dt=self.last_access['session_date'],
             last_access_session_ip_s=self.last_access['session_ip'],
             last_access_session_ua_t=self.last_access['session_ua'],
+            user_registration_date_dt=self.registration_date(),
         )
         return dict(provider.index_user(self), **fields)
 
@@ -405,6 +389,49 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
             self.last_access['session_ua'] = user_agent
             session(self).flush(self)
 
+    def add_login_detail(self, detail):
+        try:
+            session(detail).flush(detail)
+        except DuplicateKeyError:
+            session(detail).expunge(detail)
+
+    def backfill_login_details(self, auth_provider):
+        # ".*" at start of regex and the DOTALL flag is needed only for the test, which uses mim
+        # Fixed in ming f9f69d3c, so once we upgrade to 0.6.1+ we can remove it
+        msg_regex = re.compile(r'.*^({})'.format('|'.join([re.escape(line_prefix)
+                                                           for line_prefix
+                                                           in auth_provider.trusted_auditlog_line_prefixes])),
+                               re.MULTILINE | re.DOTALL)
+        for auditlog in AuditLog.for_user(self, message=msg_regex):
+            if not msg_regex.search(auditlog.message):
+                continue
+            login_detail = auth_provider.login_details_from_auditlog(auditlog)
+            if login_detail:
+                self.add_login_detail(login_detail)
+
+    def send_password_reset_email(self, email_address=None, subject_tmpl='{site_name} Password recovery'):
+        if email_address is None:
+            email_address = self.get_pref('email_address')
+        reset_url = self.make_password_reset_url()
+
+        log.info('Sending password recovery link to %s', email_address)
+        subject = subject_tmpl.format(site_name=config['site_name'])
+        text = g.jinja2_env.get_template('allura:templates/mail/forgot_password.txt').render(dict(
+            user=self,
+            config=config,
+            reset_url=reset_url,
+        ))
+        allura.tasks.mail_tasks.send_system_mail_to_user(email_address, subject, text)
+
+    def make_password_reset_url(self):
+        hash = h.nonce(42)
+        expiry_delta = timedelta(seconds=int(config.get('auth.recovery_hash_expiry_period', 600)))
+        self.set_tool_data('AuthPasswordReset',
+                           hash=hash,
+                           hash_expiry=datetime.utcnow() + expiry_delta)
+        reset_url = h.absurl(f'/auth/forgotten_password/{hash}')
+        return reset_url
+
     def can_send_user_message(self):
         """Return true if User is permitted to send a mesage to another user.
 
@@ -432,7 +459,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
                 timedelta(seconds=g.user_message_time_interval) -
                 datetime.utcnow())
 
-    def send_user_message(self, user, subject, message, cc):
+    def send_user_message(self, user, subject, message, cc, reply_to_real_address, sender_email_address):
         """Send a user message (email) to ``user``.
 
         """
@@ -444,16 +471,46 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
             'base_url': config['base_url'],
             'user': c.user,
         }
+        real_address = user.preferences.email_address
+        reply_to = self.get_pref('email_address')
+        if reply_to_real_address:
+            reply_to = sender_email_address
         allura.tasks.mail_tasks.sendsimplemail.post(
             toaddr=user.get_pref('email_address'),
             fromaddr=self.get_pref('email_address'),
-            reply_to=self.get_pref('email_address'),
+            reply_to=reply_to,
             message_id=h.gen_message_id(),
             subject=subject,
             text=tmpl.render(tmpl_context),
             cc=cc)
         self.sent_user_message_times.append(datetime.utcnow())
 
+    def send_user_mention_notification(self, mentioned_by, artifact):
+        """Send user mention notification to {self} user.
+
+        """
+        tmpl = g.jinja2_env.get_template('allura:templates/mail/usermentions_email.md')
+        subject = '[{}:{}] Your name was mentioned'.format(
+            c.project.shortname, c.app.config.options.mount_point)
+        item_url = artifact.url()
+        if artifact.type_s == 'Post':
+            item_url = artifact.url_paginated()
+        tmpl_context = {
+            'site_domain': config['domain'],
+            'base_url': config['base_url'],
+            'user': c.user,
+            'artifact_link': h.absurl(item_url),
+            'artifact_linktext': artifact.link_text(),
+            'mentioned_by': mentioned_by
+        }
+        allura.tasks.mail_tasks.sendsimplemail.post(
+            toaddr=self.get_pref('email_address'),
+            fromaddr=g.noreply,
+            reply_to=g.noreply,
+            message_id=h.gen_message_id(),
+            subject=subject,
+            text=tmpl.render(tmpl_context))
+
     @property
     def activity_name(self):
         return self.display_name or self.username
@@ -572,22 +629,22 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         This includes any special handling via the :class:`~allura.lib.plugin.AuthenticationProvider` to determine
         the proper user-project name
         '''
-        return '/%s/' % plugin.AuthenticationProvider.get(request).user_project_shortname(self)
+        return plugin.AuthenticationProvider.get(request).user_project_url(self)
 
     @memoize
     def icon_url(self, gravatar_default_url=None, return_more=False):
         icon_url = None
         try:
             private_project = self.private_project()
-        except:
-            log.warn('Error getting/creating user-project for %s',
-                     self.username, exc_info=True)
+        except Exception:
+            log.warning('Error getting/creating user-project for %s',
+                        self.username, exc_info=True)
             private_project = None
         icon_source = None
         if private_project and private_project.icon:
-            icon_url = self.url() + 'user_icon'
+            icon_url = config.get('static.icon_base', '') + self.url() + 'user_icon'
             icon_source = 'local'
-        elif self.preferences.email_address:
+        elif self.preferences.email_address and h.asbool(config.get('use_gravatar')):
             gravatar_args = {}
             if gravatar_default_url:
                 gravatar_args['d'] = gravatar_default_url
@@ -616,13 +673,16 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         return u
 
     @classmethod
-    def by_email_address(cls, addr):
-        addrs = EmailAddress.find(dict(email=addr, confirmed=True))
-        users = [ea.claimed_by_user() for ea in addrs]
+    def by_email_address(cls, addr, only_confirmed=True):
+        q = dict(email=addr)
+        if only_confirmed:
+            q['confirmed'] = True
+        addrs = EmailAddress.find(q)
+        users = [ea.claimed_by_user(not only_confirmed) for ea in addrs]
         users = [u for u in users if u is not None]
         if len(users) > 1:
-            log.warn('Multiple active users matching confirmed email %s %s. '
-                     'Using first one', [u.username for u in users], addr)
+            log.warning('Multiple active users matching confirmed email: %s %s. '
+                        'Using first one', [u.username for u in users], addr)
         return users[0] if len(users) > 0 else None
 
     @classmethod
@@ -635,7 +695,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         return plugin.AuthenticationProvider.get(request).by_username(name)
 
     def get_tool_data(self, tool, key, default=None):
-        return self.tool_data.get(tool, {}).get(key, None)
+        return self.tool_data.get(tool, {}).get(key, default)
 
     def set_tool_data(self, tool, **kw):
         d = self.tool_data.setdefault(tool, {})
@@ -661,6 +721,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
 
         auth_provider = plugin.AuthenticationProvider.get(request)
         user = auth_provider.register_user(doc)
+        user.set_pref('mention_notifications', True)
         if user and 'display_name' in doc:
             user.set_pref('display_name', doc['display_name'])
         if user:
@@ -681,7 +742,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         '''
         Returns the personal user-project for the user
         '''
-        if self.disabled or self.pending:
+        if self.disabled or self.pending or self.is_anonymous():
             return None
 
         from allura import model as M
@@ -698,7 +759,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
                 'completely deleting user project (was already flagged as deleted) %s',
                 project_shortname)
             p.delete()
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
             p = None
         if not p and not self.is_anonymous():
             # create user-project on demand if it is missing
@@ -710,9 +771,9 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
     def script_name(self):
         return '/u/' + self.username + '/'
 
-    def my_projects(self):
+    def my_projects(self) -> typing.Iterable[Project]:
         if self.is_anonymous():
-            return
+            return []
         roles = g.credentials.user_roles(user_id=self._id)
         # filter out projects to which the user belongs to no named groups (i.e., role['roles'] is empty)
         projects = [r['project_id'] for r in roles if r['roles']]
@@ -726,7 +787,7 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
         that role.
         """
         if self.is_anonymous():
-            return
+            return []
         reaching_role_ids = list(
             g.credentials.user_roles(user_id=self._id).reaching_ids_set)
         reaching_roles = ProjectRole.query.find(
@@ -736,21 +797,54 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
 
         return Project.query.find({'_id': {'$in': projects}, 'deleted': False}).all()
 
+    def my_merge_requests(self):
+        if self.is_anonymous():
+            return
+
+        from .repository import MergeRequest
+
+        return MergeRequest.query.find({'creator_id': self._id}).sort('mod_date', pymongo.DESCENDING)
+
     def set_password(self, new_password):
         return plugin.AuthenticationProvider.get(request).set_password(
             self, None, new_password)
 
     @classmethod
+    @memoize
     def anonymous(cls):
-        return User.query.get(_id=None)
+        return cls.anonymous_uncached()
+
+    @classmethod
+    def anonymous_uncached(cls):
+        anon = cls(
+            _id=None,
+            username='*anonymous',
+            display_name='Anonymous')
+        session(anon).expunge(anon)  # don't save this transient Anon record
+        return anon
+
+    def __eq__(self, o):
+        # maybe could do all _id equal, but not sure.  Just supporting *anonymous user for now
+        if self._id is None:
+            return isinstance(o, User) and hasattr(o, '_id') and o._id == self._id
+        else:
+            return super().__eq__(o)
+
+    def __hash__(self):
+        # Since we've implemented __eq__ we need to provide a __hash__ implementation
+        # https://docs.python.org/3/reference/datamodel.html#object.__hash__
+        if self._id is None:
+            return 0
+        else:
+            return super().__hash__()
 
     def is_anonymous(self):
         return self._id is None or self.username == ''
 
     def email_address_header(self):
         h = header.Header()
-        h.append(u'"%s" ' % self.get_pref('display_name'))
-        h.append(u'<%s>' % self.get_pref('email_address'))
+        h.append('"{}"{}'.format(self.get_pref('display_name'), ''))
+        h.append('<%s>' % self.get_pref('email_address'))
         return h
 
     def update_notifications(self):
@@ -769,30 +863,18 @@ class User(MappedClass, ActivityNode, ActivityObject, SearchIndexable):
 
     def registration_date(self):
         p = plugin.AuthenticationProvider.get(request)
-        d = p.user_registration_date(self)
-        # provider's user_registration_date returns aware datetime (in UTC)
-        # but we're using naive UTC time everywhere
-        d = datetime.utcfromtimestamp(calendar.timegm(d.utctimetuple()))
-        return d
-
-
-class OldProjectRole(MappedClass):
-    class __mongometa__:
-        session = project_orm_session
-        name = 'user'
-        unique_indexes = [('user_id', 'project_id', 'name')]
+        return p.user_registration_date(self)
 
 
 class ProjectRole(MappedClass):
     """
-    Per-project roles, called "Groups" in the UI.
-    This can be a proxy for a single user.  It can also inherit roles.
+    The roles that a single user holds in a project.
+    Also the named roles (called "Groups" in the UI) are in this model (and can include other named roles)
 
-    :var user_id: used if this role is for a single user
-    :var project_id:
-    :var name:
-    :var roles: a list of other :class:`ProjectRole` ``ObjectId`` values.  These roles are delegated through the
-                current role.
+    :var user_id: used if this role is for a single user.  Empty for named roles
+    :var project_id: the project id
+    :var name: named roles (like Admin, Developer, custom-names-too)
+    :var roles: a list of other :class:`ProjectRole` ``ObjectId`` values which this user/group has access to
     """
 
     class __mongometa__:
@@ -805,8 +887,10 @@ class ProjectRole(MappedClass):
             ('roles',),
         ]
 
+    query: Query[ProjectRole]
+
     _id = FieldProperty(S.ObjectId)
-    user_id = AlluraUserProperty(if_missing=None)
+    user_id: ObjectId = AlluraUserProperty(if_missing=None)
     project_id = ForeignIdProperty('Project', if_missing=None)
     name = FieldProperty(str)
     roles = FieldProperty([S.ObjectId])
@@ -816,7 +900,7 @@ class ProjectRole(MappedClass):
 
     def __init__(self, **kw):
         assert 'project_id' in kw, 'Project roles must specify a project id'
-        super(ProjectRole, self).__init__(**kw)
+        super().__init__(**kw)
 
     def display(self):
         if self.name:
@@ -896,8 +980,8 @@ class ProjectRole(MappedClass):
     @property
     def user(self):
         if (self.user_id is None
-            and self.name
-            and self.name != '*anonymous'):
+                and self.name
+                and self.name != '*anonymous'):
             return None
         return User.query.get(_id=self.user_id)
 
@@ -928,22 +1012,45 @@ class ProjectRole(MappedClass):
                                     user_id={'$ne': None}, roles=self._id)).all()
 
 
-audit_log = collection(
-    'audit_log', main_doc_session,
-    Field('_id', S.ObjectId()),
-    Field('project_id', S.ObjectId, if_missing=None,
-          index=True),  # main view of audit log queries by project_id
-    Field('user_id', S.ObjectId, if_missing=None, index=True),
-    Field('timestamp', datetime, if_missing=datetime.utcnow),
-    Field('url', str),
-    Field('message', str))
+class AuditLog(MappedClass):
+    class __mongometa__:
+        session = main_orm_session
+        name = 'audit_log'
+        indexes = [
+            'project_id',
+            'user_id',
+        ]
 
+    query: Query[AuditLog]
+
+    _id = FieldProperty(S.ObjectId)
+    project_id = ForeignIdProperty('Project', if_missing=None)
+    project = RelationProperty('Project')
+    user_id: ObjectId = AlluraUserProperty()
+    user = RelationProperty('User')
+    timestamp = FieldProperty(datetime, if_missing=datetime.utcnow)
+    url = FieldProperty(str)
+    message = FieldProperty(str)
 
-class AuditLog(object):
     @property
     def timestamp_str(self):
         return self.timestamp.strftime('%Y-%m-%d %H:%M:%S')
 
+    @property
+    def message_html(self):
+        standard_metadata_prefixes = (
+            'Done by user:',
+            'IP Address:',
+            'User-Agent:',
+        )
+        with_br = h.nl2br_jinja_filter(self.message)
+        message_bold = '<br>\n'.join([
+            line if line.startswith(standard_metadata_prefixes) else f'<strong>{line}</strong>'
+            for line in
+            with_br.split('<br>\n')
+        ])
+        return Markup(message_bold)
+
     @property
     def url_str(self):
         scheme, netloc, path, params, query, fragment = urlparse(self.url)
@@ -960,7 +1067,12 @@ class AuditLog(object):
     def log(cls, message, *args, **kwargs):
         project = kwargs.pop('project') if 'project' in kwargs else c.project
         user = kwargs.pop('user', c.user)
-        url = kwargs.pop('url', request.url)
+        url = kwargs.pop('url', '')
+        if not url:
+            try:
+                url = request.url
+            except AttributeError:
+                pass
         if args:
             message = message % args
         elif kwargs:
@@ -971,8 +1083,8 @@ class AuditLog(object):
         return cls(project_id=pid, user_id=user._id, url=url, message=message)
 
     @classmethod
-    def for_user(cls, user):
-        return cls.query.find(dict(project_id=None, user_id=user._id))
+    def for_user(cls, user, **kwargs):
+        return cls.query.find(dict(project_id=None, user_id=user._id, **kwargs))
 
     @classmethod
     def log_user(cls, message, *args, **kwargs):
@@ -981,12 +1093,32 @@ class AuditLog(object):
 
     @classmethod
     def comment_user(cls, by, message, *args, **kwargs):
-        message = u'Comment by %s: %s' % (by.username, message)
+        message = f'Comment by {by.username}: {message}'
         return cls.log_user(message, *args, **kwargs)
 
 
-main_orm_session.mapper(AuditLog, audit_log, properties=dict(
-    project_id=ForeignIdProperty('Project'),
-    project=RelationProperty('Project'),
-    user_id=AlluraUserProperty(),
-    user=RelationProperty('User')))
+class UserLoginDetails(MappedClass):
+    """
+    Store unique entries for users' previous login details.
+
+    Used to help determine if new logins are suspicious or not
+    """
+
+    class __mongometa__:
+        name = 'user_login_details'
+        session = main_explicitflush_orm_session
+        indexes = ['user_id']
+        unique_indexes = [('user_id', 'ip', 'ua'),  # DuplicateKeyError checked in add_login_detail
+                          ]
+
+    query: Query[UserLoginDetails]
+
+    _id = FieldProperty(S.ObjectId)
+    user_id: ObjectId = AlluraUserProperty(required=True)
+    ip = FieldProperty(str)
+    ua = FieldProperty(str)
+    extra = FieldProperty({
+        str: S.Anything
+    })
+
+    user = RelationProperty('User')
