@@ -14,34 +14,40 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
-
+from __future__ import annotations
 import os
 import logging
 from datetime import datetime
+import typing
 
 import jinja2
+import markupsafe
 import pymongo
 from pymongo.errors import DuplicateKeyError
-from pylons import tmpl_context as c, app_globals as g
+from tg import tmpl_context as c, app_globals as g
 import tg
 
 from ming import schema
-from ming.orm.base import session
-from ming.orm.property import (FieldProperty, RelationProperty,
+from ming.odm.base import session
+from ming.odm.property import (FieldProperty, RelationProperty,
                                ForeignIdProperty)
 from ming.utils import LazyProperty
+from bson import ObjectId
 
 from allura.lib import helpers as h
 from allura.lib import security
 from allura.lib.security import require_access, has_access
 from allura.lib import utils
 from allura.model.notification import Notification, Mailbox
-from .artifact import Artifact, ArtifactReference, VersionedArtifact, Snapshot, Message, Feed
+from .artifact import Artifact, ArtifactReference, VersionedArtifact, Snapshot, Message, Feed, ReactableArtifact
 from .attachments import BaseAttachment
 from .auth import User, ProjectRole, AlluraUserProperty
 from .timeline import ActivityObject
 from .types import MarkdownCache
 
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+
 log = logging.getLogger(__name__)
 
 
@@ -49,6 +55,9 @@ class Discussion(Artifact, ActivityObject):
 
     class __mongometa__:
         name = 'discussion'
+
+    query: 'Query[Discussion]'
+
     type_s = 'Discussion'
 
     parent_id = FieldProperty(schema.Deprecated)
@@ -78,11 +87,11 @@ class Discussion(Artifact, ActivityObject):
         return 'discussion %s' % self.name
 
     @classmethod
-    def thread_class(cls):
+    def thread_class(cls) -> Thread:
         return cls.threads.related
 
     @classmethod
-    def post_class(cls):
+    def post_class(cls) -> Post:
         return cls.posts.related
 
     @classmethod
@@ -118,12 +127,27 @@ class Discussion(Artifact, ActivityObject):
             text=self.description)
         return result
 
+    def attach(self, filename, fp, **kw):
+        """Attach a file to this Artifact.
+
+        :param filename: file name
+        :param fp: a file-like object (implements ``read()``)
+        :param kw: passed through to Attachment class constructor
+
+        Override's Artifact.attach to use a str artifact_id
+        """
+        att = self.attachment_class().save_attachment(
+            filename=filename,
+            fp=fp, artifact_id=str(self._id), **kw)
+        return att
+
     def delete(self):
-        # Delete all the threads, posts, and artifacts
-        self.thread_class().query.remove(dict(discussion_id=self._id))
-        self.post_class().query.remove(dict(discussion_id=self._id))
-        self.attachment_class().remove(dict(discussion_id=self._id))
-        super(Discussion, self).delete()
+        # Delete all the threads
+        for thread in self.threads:
+            thread.delete()
+        # threads handle deleting posts
+        # super() handles artifacts, attachments, etc.
+        super().delete()
 
     def find_posts(self, **kw):
         q = dict(kw, discussion_id=self._id, deleted=False)
@@ -142,11 +166,14 @@ class Thread(Artifact, ActivityObject):
              ('mod_date', pymongo.DESCENDING)),
             ('discussion_id',),
         ]
+
+    query: 'Query[Thread]'
+
     type_s = 'Thread'
 
-    _id = FieldProperty(str, if_missing=lambda: h.nonce(8))
+    _id = FieldProperty(str, if_missing=lambda: h.nonce(10))
     discussion_id = ForeignIdProperty(Discussion)
-    ref_id = ForeignIdProperty('ArtifactReference')
+    ref_id: str = ForeignIdProperty('ArtifactReference')
     subject = FieldProperty(str, if_missing='')
     num_replies = FieldProperty(int, if_missing=0)
     num_views = FieldProperty(int, if_missing=0)
@@ -200,7 +227,9 @@ class Thread(Artifact, ActivityObject):
                         author_icon_url=h.absurl(p.author().icon_url()),
                         timestamp=p.timestamp,
                         last_edited=p.last_edit_date,
-                        attachments=self.attachment_for_export(p) if is_export else self.attachments_for_json(p))
+                        attachments=self.attachment_for_export(p) if is_export else self.attachments_for_json(p),
+                        is_meta=p.is_meta,
+                        )
                    for p in self.query_posts(status='ok', style='chronological', limit=limit, page=page)
                    ]
         )
@@ -229,11 +258,11 @@ class Thread(Artifact, ActivityObject):
                 continue
 
     @classmethod
-    def discussion_class(cls):
+    def discussion_class(cls) -> Discussion:
         return cls.discussion.related
 
     @classmethod
-    def post_class(cls):
+    def post_class(cls) -> Post:
         return cls.posts.related
 
     @classmethod
@@ -293,7 +322,7 @@ class Thread(Artifact, ActivityObject):
         roles = [r.name for r in c.project.named_roles]
         spam_check_text = post.text
         if self.include_subject_in_spam_check(post):
-            spam_check_text = self.subject + u'\n' + spam_check_text
+            spam_check_text = self.subject + '\n' + spam_check_text
         spammy = g.spam_checker.check(spam_check_text, artifact=post, user=c.user)
         if c.user in c.project.users_with_role(*roles):
             # always run the check, so it's logged.  But don't act on it for admins/developers of their own project
@@ -303,11 +332,11 @@ class Thread(Artifact, ActivityObject):
 
     def post(self, text, message_id=None, parent_id=None, notify=True,
              notification_text=None, timestamp=None, ignore_security=False,
-             is_meta=False, **kw):
+             is_meta=False, subscribe=False, **kw):
         if not ignore_security:
             require_access(self, 'post')
-        if self.ref_id and self.artifact:
-            self.artifact.subscribe()
+        if subscribe:
+            self.primary().subscribe()
         if message_id is None:
             message_id = h.gen_message_id()
         parent = parent_id and self.post_class().query.get(_id=parent_id)
@@ -326,11 +355,22 @@ class Thread(Artifact, ActivityObject):
         if message_id is not None:
             kwargs['_id'] = message_id
         post = self.post_class()(**kwargs)
-        if ignore_security or not self.is_spam(post) and has_access(self, 'unmoderated_post')():
+
+        if ignore_security or is_meta:
+            spammy = False
+        else:
+            spammy = self.is_spam(post)
+        # unmoderated post -> autoapprove
+        # unmoderated post but is spammy -> don't approve it, it goes into moderation
+        # moderated post -> moderation
+        # moderated post but is spammy -> mark as spam
+        if ignore_security or (not spammy and has_access(self, 'unmoderated_post')):
             log.info('Auto-approving message from %s', c.user.username)
             file_info = kw.get('file_info', None)
             post.approve(file_info, notify=notify,
                          notification_text=notification_text)
+        elif not has_access(self, 'unmoderated_post') and spammy:
+            post.spam(submit_spam_feedback=False)  # no feedback since we're marking as spam automatically not manually
         else:
             self.notify_moderators(post)
         return post
@@ -338,7 +378,7 @@ class Thread(Artifact, ActivityObject):
     def notify_moderators(self, post):
         ''' Notify moderators that a post needs approval [#2963] '''
         artifact = self.artifact or self
-        subject = '[%s:%s] Moderation action required' % (
+        subject = '[{}:{}] Moderation action required'.format(
             c.project.shortname, c.app.config.options.mount_point)
         author = post.author()
         url = self.discussion_class().query.get(_id=self.discussion_id).url()
@@ -412,10 +452,8 @@ class Thread(Artifact, ActivityObject):
             q = q.limit(limit)
         return q
 
-    def find_posts(self, page=None, limit=None, timestamp=None,
-                   style='threaded'):
-        return self.query_posts(page=page, limit=limit,
-                                timestamp=timestamp, style=style).all()
+    def find_posts(self, *args, **kwargs):
+        return self.query_posts(*args, **kwargs).all()
 
     def url(self):
         # Can't use self.discussion because it might change during the req
@@ -437,8 +475,7 @@ class Thread(Artifact, ActivityObject):
     def delete(self):
         for p in self.post_class().query.find(dict(thread_id=self._id)):
             p.delete()
-        self.attachment_class().remove(dict(thread_id=self._id))
-        super(Thread, self).delete()
+        super().delete()
 
     def spam(self):
         """Mark this thread as spam."""
@@ -451,10 +488,12 @@ class PostHistory(Snapshot):
     class __mongometa__:
         name = 'post_history'
 
+    query: 'Query[PostHistory]'
+
     artifact_id = ForeignIdProperty('Post')
 
     @classmethod
-    def post_class(cls):
+    def post_class(cls) -> Post:
         return cls.artifact_id.related
 
     def original(self):
@@ -463,7 +502,7 @@ class PostHistory(Snapshot):
     def shorthand_id(self):
         original = self.original()
         if original:
-            return '%s#%s' % (original.shorthand_id(), self.version)
+            return f'{original.shorthand_id()}#{self.version}'
         else:
             return None
 
@@ -481,7 +520,7 @@ class PostHistory(Snapshot):
         return result
 
 
-class Post(Message, VersionedArtifact, ActivityObject):
+class Post(Message, VersionedArtifact, ActivityObject, ReactableArtifact):
 
     class __mongometa__:
         name = 'post'
@@ -489,8 +528,16 @@ class Post(Message, VersionedArtifact, ActivityObject):
         indexes = [
             # used in general lookups, last_post, etc
             ('discussion_id', 'status', 'timestamp'),
-            'thread_id'
+            # for update_stats()
+            ('discussion_id', 'deleted', 'status'),
+            # for update_stats() and thread_id in general
+            ('thread_id', 'status', 'deleted'),
+            # for find_posts/query_posts, including full_slug sort which is useful on super big threads
+            ('deleted', 'discussion_id', 'thread_id', 'full_slug'),
         ]
+
+    query: 'Query[Post]'
+
     type_s = 'Post'
 
     thread_id = ForeignIdProperty(Thread)
@@ -499,7 +546,7 @@ class Post(Message, VersionedArtifact, ActivityObject):
     status = FieldProperty(schema.OneOf('ok', 'pending', 'spam',
                                         if_missing='pending'))
     last_edit_date = FieldProperty(datetime, if_missing=None)
-    last_edit_by_id = AlluraUserProperty()
+    last_edit_by_id: ObjectId = AlluraUserProperty()
     edit_count = FieldProperty(int, if_missing=0)
     spam_check_id = FieldProperty(str, if_missing='')
     text_cache = FieldProperty(MarkdownCache)
@@ -559,7 +606,7 @@ class Post(Message, VersionedArtifact, ActivityObject):
         # For activity summary, convert Post text to html,
         # strip all tags, and truncate
         LEN = 500
-        summary = jinja2.Markup.escape(
+        summary = markupsafe.Markup.escape(
             g.markdown.cached_convert(self, 'text')).striptags()
         if len(summary) > LEN:
             split = max(summary.find(' ', LEN), LEN)
@@ -568,9 +615,9 @@ class Post(Message, VersionedArtifact, ActivityObject):
         return d
 
     def index(self):
-        result = super(Post, self).index()
+        result = super().index()
         result.update(
-            title='Post by %s on %s' % (
+            title='Post by {} on {}'.format(
                 self.author().username, self.subject),
             name_s=self.subject,
             type_s='Post',
@@ -578,11 +625,11 @@ class Post(Message, VersionedArtifact, ActivityObject):
         return result
 
     @classmethod
-    def discussion_class(cls):
+    def discussion_class(cls) -> Discussion:
         return cls.discussion.related
 
     @classmethod
-    def thread_class(cls):
+    def thread_class(cls) -> Thread:
         return cls.thread.related
 
     @classmethod
@@ -605,15 +652,10 @@ class Post(Message, VersionedArtifact, ActivityObject):
                     subject = getattr(artifact, 'email_subject', '')
         return subject or '(no subject)'
 
-    @LazyProperty
-    def attachments(self):
-        atts = self.attachment_class().query.find(dict(
-            post_id=self._id, type='attachment')).all()
-        return utils.unique_attachments(atts)
-
     def add_multiple_attachments(self, file_info):
         if isinstance(file_info, list):
-            map(self.add_attachment, file_info)
+            for fi in file_info:
+                self.add_attachment(fi)
         else:
             self.add_attachment(file_info)
 
@@ -691,17 +733,17 @@ class Post(Message, VersionedArtifact, ActivityObject):
                 traverse(posts)
                 return len(q)
 
-            page = find_i(posts) / limit
+            page = find_i(posts) // limit
 
         slug = h.urlquote(self.slug)
         url = self.main_url()
         if page == 0:
-            return '%s?limit=%s#%s' % (url, limit, slug)
-        return '%s?limit=%s&page=%s#%s' % (url, limit, page, slug)
+            return f'{url}?limit={limit}#{slug}'
+        return f'{url}?limit={limit}&page={page}#{slug}'
 
     def shorthand_id(self):
         if self.thread:
-            return '%s#%s' % (self.thread.shorthand_id(), self.slug)
+            return f'{self.thread.shorthand_id()}#{self.slug}'
         else:  # pragma no cover
             return None
 
@@ -715,6 +757,11 @@ class Post(Message, VersionedArtifact, ActivityObject):
             return 'Re: ' + (self.subject or '(no subject)')
 
     def delete(self):
+        thread = self.thread
+        super().delete()
+        thread.update_stats()
+
+    def soft_delete(self):
         self.deleted = True
         session(self).flush(self)
         self.thread.update_stats()
@@ -726,11 +773,12 @@ class Post(Message, VersionedArtifact, ActivityObject):
         author = self.author()
         author_role = ProjectRole.by_user(
             author, project=self.project, upsert=True)
-        security.simple_grant(
-            self.acl, author_role._id, 'moderate')
+        if not author.is_anonymous():
+            security.simple_grant(
+                self.acl, author_role._id, 'moderate')
         self.commit()
         if (c.app.config.options.get('PostingPolicy') == 'ApproveOnceModerated'
-                and author._id != None):
+                and author._id is not None):
             security.simple_grant(
                 self.acl, author_role._id, 'unmoderated_post')
         if notify:
@@ -763,7 +811,7 @@ class Post(Message, VersionedArtifact, ActivityObject):
             # this means artifact was not auto approved, and all the
             # subscribers did not receive notification. Now, moderator approved
             # artifact/post, so we should re-send actual notification
-            msg_id = u'approved-' + msg_id
+            msg_id = 'approved-' + msg_id
             n = Notification.query.get(_id=msg_id)
             if n:
                 # 'approved' notification also exists, re-send
@@ -785,9 +833,10 @@ class Post(Message, VersionedArtifact, ActivityObject):
             else:  # Send if no extra checks required
                 n.send_simple(artifact.monitoring_email)
 
-    def spam(self):
+    def spam(self, submit_spam_feedback=True):
         self.status = 'spam'
-        g.spam_checker.submit_spam(self.text, artifact=self, user=self.author())
+        if submit_spam_feedback:
+            g.spam_checker.submit_spam(self.text, artifact=self, user=self.author())
         session(self).flush(self)
         self.thread.update_stats()
 
@@ -809,6 +858,8 @@ class DiscussionAttachment(BaseAttachment):
         polymorphic_identity = 'DiscussionAttachment'
         indexes = ['filename', 'discussion_id', 'thread_id', 'post_id']
 
+    query: 'Query[DiscussionAttachment]'
+
     discussion_id = FieldProperty(schema.ObjectId)
     thread_id = FieldProperty(str)
     post_id = FieldProperty(str)
