@@ -1,4 +1,3 @@
-# coding=utf-8
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -16,38 +15,46 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
+from __future__ import annotations
+
 import logging
 from calendar import timegm
 from collections import Counter, OrderedDict
+from collections.abc import Iterable
+from hashlib import sha256
+import typing
 from datetime import datetime
 from copy import deepcopy
-import urllib
+import six.moves.urllib.request
+import six.moves.urllib.parse
+import six.moves.urllib.error
 import re
 from xml.etree import ElementTree as ET
 
 from tg import config
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request
+from tg import tmpl_context as c, app_globals as g
+from tg import request
 from paste.deploy.converters import asbool, aslist
 import formencode as fe
 from webob import exc
 import PIL
 
-from allura.lib.decorators import memoize
 from ming import schema as S
 from ming.utils import LazyProperty
-from ming.orm import ThreadLocalORMSession
-from ming.orm import session, state, MapperExtension
-from ming.orm import FieldProperty, RelationProperty, ForeignIdProperty
-from ming.orm.declarative import MappedClass
+from ming.odm import ThreadLocalODMSession
+from ming.odm import session, state, MapperExtension
+from ming.odm import FieldProperty, RelationProperty, ForeignIdProperty
+from ming.odm.declarative import MappedClass
 
 from allura.lib import helpers as h
 from allura.lib import plugin
 from allura.lib import exceptions
 from allura.lib import security
 from allura.lib import validators as v
+from allura.lib.decorators import memoize
 from allura.lib.security import has_access
 from allura.lib.search import SearchIndexable
+from allura.lib.utils import is_nofollow_url
 from allura.model.types import MarkdownCache
 
 from .session import main_orm_session
@@ -58,7 +65,14 @@ from .timeline import ActivityNode, ActivityObject
 from .types import ACL, ACE
 from .monq_model import MonQTask
 
-from filesystem import File
+from .filesystem import File
+import six
+
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+    from allura.model import AppConfig
+    from allura.app import Application
+
 
 log = logging.getLogger(__name__)
 
@@ -74,6 +88,8 @@ class ProjectFile(File):
         session = main_orm_session
         indexes = [('project_id', 'category')]
 
+    query: Query[ProjectFile]
+
     project_id = FieldProperty(S.ObjectId)
     category = FieldProperty(str)
     caption = FieldProperty(str)
@@ -86,6 +102,8 @@ class ProjectCategory(MappedClass):
         session = main_orm_session
         name = 'project_category'
 
+    query: Query[ProjectCategory]
+
     _id = FieldProperty(S.ObjectId)
     parent_id = FieldProperty(S.ObjectId, if_missing=None)
     name = FieldProperty(str)
@@ -104,13 +122,13 @@ class ProjectCategory(MappedClass):
 class TroveCategoryMapperExtension(MapperExtension):
 
     def after_insert(self, obj, state, sess):
-        g.post_event('trove_category_created', obj.trove_cat_id)
+        g.post_event('trove_category_created', obj.trove_cat_id, flush_immediately=True)
 
     def after_update(self, obj, state, sess):
-        g.post_event('trove_category_updated', obj.trove_cat_id)
+        g.post_event('trove_category_updated', obj.trove_cat_id, flush_immediately=True)
 
     def after_delete(self, obj, state, sess):
-        g.post_event('trove_category_deleted', obj.trove_cat_id)
+        g.post_event('trove_category_deleted', obj.trove_cat_id, flush_immediately=True)
 
 
 class TroveCategory(MappedClass):
@@ -121,6 +139,8 @@ class TroveCategory(MappedClass):
         extensions = [TroveCategoryMapperExtension]
         indexes = ['trove_cat_id', 'trove_parent_id', 'shortname', 'fullpath']
 
+    query: Query[TroveCategory]
+
     _id = FieldProperty(S.ObjectId)
     trove_cat_id = FieldProperty(int, if_missing=None)
     trove_parent_id = FieldProperty(int, if_missing=None)
@@ -151,28 +171,18 @@ class TroveCategory(MappedClass):
             trove = trove.parent_category
         return trove.shortname
 
-    @LazyProperty
-    def ancestors(self):
-        ancestors = []
-        trove = self
-        while trove:
-            ancestors.append(trove)
-            trove = trove.parent_category
-        return ancestors
-
-    @LazyProperty
-    def breadcrumbs(self):
-        url = '/directory/'
-        crumbs = []
-        for trove in reversed(self.ancestors[:-1]):
-            url += trove.shortname + '/'
-            crumbs.append((trove.fullname, url))
-        return crumbs
-
     @property
     def fullpath_within_type(self):
         'remove first section of full path, and use nicer separator'
-        return u' » '.join(self.fullpath.split(' :: ')[1:])
+        return ' » '.join(self.fullpath.split(' :: ')[1:])
+
+    @classmethod
+    def by_trove_cat_ids(cls, *ids):
+        return cls.query.find({'trove_cat_id': {'$in': ids}}).all()
+
+    @classmethod
+    def by_trove_fullpaths(cls, *ids):
+        return cls.query.find({'fullpath': {'$in': ids}}).all()
 
     def __json__(self):
         return dict(
@@ -183,6 +193,20 @@ class TroveCategory(MappedClass):
         )
 
 
+class ProjectNameFieldProperty(FieldProperty):
+    """
+    Make project names be the username instead of u/whatever, when a user-project.
+    Particularly nice if the username and user-project name don't match exactly.
+    (This is a python "descriptor")
+    """
+    def __get__(self, instance, cls=None):
+        if instance:
+            owning_user = instance.user_project_of
+            if owning_user:
+                return owning_user.username
+        return super().__get__(instance, cls)
+
+
 class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     '''
     Projects contain tools, subprojects, and their own metadata.  They live
@@ -205,6 +229,8 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             ('neighborhood_id', 'is_nbhd_project', 'deleted')]
         unique_indexes = [('neighborhood_id', 'shortname')]
 
+    query: Query[Project]
+
     type_s = 'Project'
 
     # Project schema
@@ -212,7 +238,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     parent_id = FieldProperty(S.ObjectId, if_missing=None)
     neighborhood_id = ForeignIdProperty(Neighborhood)
     shortname = FieldProperty(str)
-    name = FieldProperty(str)
+    name: str = ProjectNameFieldProperty(str)
     show_download_button = FieldProperty(S.Deprecated)
     short_description = FieldProperty(str, if_missing='')
     summary = FieldProperty(str, if_missing='')
@@ -227,15 +253,13 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     removal = FieldProperty(str, if_missing='')
     moved_to_url = FieldProperty(str, if_missing='')
     removal_changed_date = FieldProperty(datetime, if_missing=datetime.utcnow)
-    export_controlled = FieldProperty(bool, if_missing=False)
-    export_control_type = FieldProperty(str, if_missing=None)
     database = FieldProperty(S.Deprecated)
     database_uri = FieldProperty(S.Deprecated)
     is_root = FieldProperty(bool)
     acl = FieldProperty(ACL(permissions=_perms_init))
     neighborhood_invitations = FieldProperty([S.ObjectId])
     neighborhood = RelationProperty(Neighborhood)
-    app_configs = RelationProperty('AppConfig')
+    app_configs: Iterable[AppConfig] = RelationProperty('AppConfig')
     category_id = FieldProperty(S.ObjectId, if_missing=None)
     deleted = FieldProperty(bool, if_missing=False)
     labels = FieldProperty([str])
@@ -256,6 +280,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     tracking_id = FieldProperty(str, if_missing='')
     is_nbhd_project = FieldProperty(bool, if_missing=False)
     features = FieldProperty([str])
+    rating = FieldProperty(float, if_missing=0)
 
     # transient properties
     notifications_disabled = False
@@ -296,7 +321,8 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         return result
 
     def troves_by_type(self, trove_type):
-        troves = getattr(self, 'trove_%s' % trove_type)
+        trove_key = 'trove_%s' % trove_type
+        troves = getattr(self, trove_key) if hasattr(self, trove_key) else None
         if troves:
             return TroveCategory.query.find({'_id': {'$in': troves}}).all()
         else:
@@ -336,10 +362,14 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         else:
             return url
 
-    def url(self):
+    def url(self, use_userproject_shortname=False):
         if self.is_nbhd_project:
             return self.neighborhood.url()
         shortname = self.shortname[len(self.neighborhood.shortname_prefix):]
+        if self.neighborhood.url_prefix == '/u/' and not use_userproject_shortname:
+            user = self.user_project_of
+            if user:
+                return user.url()
         url = self.neighborhood.url_prefix + shortname + '/'
         if url.startswith('//'):
             try:
@@ -349,6 +379,10 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         else:
             return url
 
+    def icon_url(self):
+        icon_url = config.get('static.icon_base', '') + self.url() + 'icon'
+        return icon_url
+
     def best_download_url(self):
         provider = plugin.ProjectRegistrationProvider.get()
         return provider.best_download_url(self)
@@ -365,10 +399,24 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             thumbnail_meta=dict(project_id=self._id, category='icon'),
             save_original=True,
             original_meta=dict(project_id=self._id, category='icon_original'),
+            convert_bmp=True,
         )
-        # store the dimensions so we don't have to read the whole image each time we need to know
-        icon_orig_img = PIL.Image.open(icon_orig.rfile())
-        self.set_tool_data('allura', icon_original_size=icon_orig_img.size)
+        if icon_orig:
+            # store the dimensions so we don't have to read the whole image each time we need to know
+            icon_orig_img = PIL.Image.open(icon_orig.rfile())
+
+            self.set_tool_data('allura', icon_original_size=icon_orig_img.size)
+
+            try:
+                # calc and save icon file hash, for better cache busting purposes
+                file_input.seek(0)
+                file_bytes = file_input.read()
+                file_sha256 = sha256(file_bytes).hexdigest()
+                self.set_tool_data('allura', icon_sha256=file_sha256)
+            except Exception as ex:
+                log.exception(f'Failed to calculate sha256 for icon file for {self.shortname}')
+            return True
+        return False
 
     @property
     def icon(self):
@@ -376,13 +424,13 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
 
     @memoize
     def icon_sized(self, w):
-        allowed_sizes = map(int, aslist(config.get('project_icon_sizes', '16 24 32 48 64 72 96')))
+        allowed_sizes = list(map(int, aslist(config.get('project_icon_sizes', '16 24 32 48 64 72 96'))))
         if w not in allowed_sizes:
-            raise ValueError('Width must be one of {} (see project_icon_sizes in your .ini file)'.format(allowed_sizes))
+            raise ValueError(f'Width must be one of {allowed_sizes} (see project_icon_sizes in your .ini file)')
         if w == DEFAULT_ICON_WIDTH:
             icon_cat_name = 'icon'
         else:
-            icon_cat_name = 'icon-{}'.format(w)
+            icon_cat_name = f'icon-{w}'
         sized = ProjectFile.query.get(project_id=self._id, category=icon_cat_name)
         if not sized and w != DEFAULT_ICON_WIDTH:
             orig = self.icon_original
@@ -449,15 +497,23 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     def is_user_project(self):
         return self.is_root and self.shortname.startswith('u/')
 
-    @LazyProperty
+    @property
     def user_project_of(self):
+        '''
+        If this is a user-project, return the User, else None
+        '''
+        return self.get_userproject_user()
+
+    def get_userproject_user(self, include_disabled=False):
         '''
         If this is a user-project, return the User, else None
         '''
         user = None
         if self.is_user_project:
-            user = plugin.AuthenticationProvider.get(
-                request).user_by_project_shortname(self.shortname[2:])
+            user = plugin.AuthenticationProvider.get(request).user_by_project_shortname(
+                self.shortname[2:],  # strip leading u/ prefix
+                include_disabled=include_disabled,
+            )
         return user
 
     @LazyProperty
@@ -466,90 +522,33 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             return self
         return self.parent_project.root_project
 
-    @LazyProperty
-    def project_hierarchy(self):
-        if not self.is_root:
-            return self.root_project.project_hierarchy
-        projects = set([self])
-        while True:
-            new_projects = set(
-                self.query.find(dict(parent_id={'$in': [p._id for p in projects]})))
-            new_projects.update(projects)
-            if new_projects == projects:
-                break
-            projects = new_projects
-        return projects
-
     @property
     def category(self):
         return ProjectCategory.query.find(dict(_id=self.category_id)).first()
 
-    def roleids_with_permission(self, name):
-        roles = set()
-        for p in self.parent_iter():
-            for ace in p.acl:
-                if ace.permission == name and ace.access == ACE.allow:
-                    roles.add(ace.role_id)
-        return list(roles)
-
-    @classmethod
-    def menus(cls, projects):
-        '''Return a dict[project_id] = sitemap of sitemaps, efficiently'''
-        from allura.app import SitemapEntry
-        pids = [p._id for p in projects]
-        project_index = dict((p._id, p) for p in projects)
-        entry_index = dict((pid, []) for pid in pids)
-        q_subprojects = cls.query.find(dict(
-            parent_id={'$in': pids},
-            deleted=False))
-        for sub in q_subprojects:
-            entry_index[sub.parent_id].append(
-                dict(ordinal=sub.ordinal, entry=SitemapEntry(sub.name, sub.url())))
-        q_app_configs = AppConfig.query.find(dict(
-            project_id={'$in': pids}))
-        for ac in q_app_configs:
-            App = ac.load()
-            project = project_index[ac.project_id]
-            app = App(project, ac)
-            if app.is_visible_to(c.user):
-                for sm in app.main_menu():
-                    entry = sm.bind_app(app)
-                    entry.ui_icon = 'tool-%s' % ac.tool_name
-                    ordinal = ac.options.get('ordinal', 0)
-                    entry_index[ac.project_id].append(
-                        {'ordinal': ordinal, 'entry': entry})
-
-        sitemaps = dict((pid, []) for pid in pids)
-        for pid, entries in entry_index.iteritems():
-            entries.sort(key=lambda e: e['ordinal'])
-            sitemap = sitemaps[pid]
-            for e in entries:
-                sitemap.append(e['entry'])
-        return sitemaps
-
     @classmethod
     def icon_urls(cls, projects):
         '''Return a dict[project_id] = icon_url, efficiently'''
-        project_index = dict((p._id, p) for p in projects)
-        result = dict((p._id, None) for p in projects)
+        project_index = {p._id: p for p in projects}
+        result = {p._id: None for p in projects}
         for icon in ProjectFile.query.find(dict(
-                project_id={'$in': result.keys()},
+                project_id={'$in': list(result.keys())},
                 category='icon')):
-            result[icon.project_id] = project_index[icon.project_id].url() + 'icon'
+            result[icon.project_id] = project_index[icon.project_id].icon_url()
         return result
 
     @classmethod
     def accolades_index(cls, projects):
         '''Return a dict[project_id] = list of accolades, efficiently'''
         from .artifact import AwardGrant
-        result = dict((p._id, []) for p in projects)
+        result = {p._id: [] for p in projects}
         for award in AwardGrant.query.find(dict(
-                granted_to_project_id={'$in': result.keys()})):
+                granted_to_project_id={'$in': list(result.keys())})):
             result[award.granted_to_project_id].append(award)
         return result
 
     def sitemap(self, excluded_tools=None, included_tools=None,
-            tools_only=False, per_tool_limit=SITEMAP_PER_TOOL_LIMIT):
+            tools_only=False, per_tool_limit=SITEMAP_PER_TOOL_LIMIT, xml=False):
         """
         Return the project sitemap.
 
@@ -567,6 +566,10 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             Max number of entries included in the sitemap for a single tool
             type. Use `None` to include all.
 
+        :param bool xml:
+            If True, return sitemap entries for use in the sitemap.xml
+            instead of site navigation.
+
         """
         from allura.app import SitemapEntry
         entries = []
@@ -587,8 +590,9 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                 ordinal = sub.ordinal + delta_ordinal
                 if ordinal > max_ordinal:
                     max_ordinal = ordinal
+                mount_point = sub.shortname.split('/')[-1]
                 entries.append({'ordinal': sub.ordinal + delta_ordinal,
-                               'entry': SitemapEntry(sub.name, sub.url())})
+                                'entry': SitemapEntry(sub.name, sub.url(), mount_point=mount_point)})
 
         for ac in self.app_configs + [a.config for a in new_tools]:
             if per_tool_limit:
@@ -620,12 +624,18 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             else:
                 app = App(self, ac)
             if app.is_visible_to(c.user):
-                for sm in app.main_menu():
+                if xml:
+                    sms = app.sitemap_xml()
+                else:
+                    sms = app.main_menu()
+                for sm in sms:
                     entry = sm.bind_app(app)
                     entry.tool_name = ac.tool_name
                     entry.ui_icon = 'tool-%s' % entry.tool_name.lower()
-                    if not self.is_nbhd_project and (entry.tool_name.lower() in anchored_tools.keys()):
-                        ordinal = anchored_tools.keys().index(
+                    if is_nofollow_url(entry.url):
+                        entry.extra_html_attrs.update({'rel': 'nofollow'})
+                    if not self.is_nbhd_project and (entry.tool_name.lower() in list(anchored_tools.keys())):
+                        ordinal = list(anchored_tools.keys()).index(
                             entry.tool_name.lower())
                     elif ac.tool_name == 'admin':
                         ordinal = 100
@@ -660,7 +670,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         i = 0
         new_tools = []
         if not self.is_nbhd_project:
-            for tool, label in anchored_tools.iteritems():
+            for tool, label in anchored_tools.items():
                 if (tool not in installed_tools) and (self.app_instance(tool) is None):
                     try:
                         new_tools.append(
@@ -670,39 +680,45 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                 i += 1
         return new_tools
 
-    def nav_data(self, admin_options=False):
+    def nav_data(self, admin_options=False, navbar_entries=None):
+        """
+        Return data about project nav entries
+
+        :param bool admin_options: include admin options?
+        :param navbar_entries: for performance, include this if you already have grouped_navbar_entries data
+        :return:
+        """
         from allura.ext.admin.admin_main import ProjectAdminRestController
 
         grouping_threshold = self.get_tool_data('allura', 'grouping_threshold', 1)
         anchored_tools = self.neighborhood.get_anchored_tools()
         children = []
 
-        def make_entry(s, mount_point):
+        def make_entry(s, app_config):
             entry = dict(name=s.label,
                          url=s.url,
                          icon=s.ui_icon or 'tool-admin',
                          tool_name=s.tool_name or 'sub',
-                         mount_point=mount_point,
-                         is_anchored=s.tool_name in anchored_tools.keys(),
+                         mount_point=s.mount_point,
+                         is_anchored=s.tool_name in list(anchored_tools.keys()),
                          )
-            if admin_options and s.tool_name and mount_point:
+            if admin_options and app_config:
                 try:
-                    entry['admin_options'] = ProjectAdminRestController().admin_options(mount_point)['options']
+                    entry['admin_options'] = ProjectAdminRestController().admin_options(app_config)['options']
                 except exc.HTTPError:
-                    log.debug('Could not get admin_options mount_point for tool: %s', s.url)
+                    log.debug('Could not get admin_options mount_point for tool: %s', s.url, exc_info=True)
             if admin_options and not s.tool_name:
                 entry['admin_options'] = [dict(text='Subproject Admin', href=s.url + 'admin', className=None)]
             return entry
 
-        for s in self.grouped_navbar_entries():
+        if navbar_entries is None:
+            navbar_entries = self.grouped_navbar_entries()
+        tools_by_mount = {ac.options.mount_point: ac for ac in self.app_configs if ac.options.mount_point}
+        for s in navbar_entries:
+            entry = make_entry(s, tools_by_mount.get(s.mount_point))
             if s.children:
-                mount_point = None
-            else:
-                _offset = -2 if s.url.endswith("/") else -1
-                mount_point = s.url.split('/')[_offset]
-            entry = make_entry(s, mount_point=mount_point)
-            if s.children:
-                entry['children'] = [make_entry(child, mount_point=child.url.split('/')[-2]) for child in s.children]
+                entry['children'] = [make_entry(child, tools_by_mount.get(child.mount_point))
+                                     for child in s.children]
             children.append(entry)
 
         response = dict(grouping_threshold=grouping_threshold, menu=children)
@@ -741,13 +757,13 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                 if tool_name not in grouped_nav:
                     child = deepcopy(e)
                     # change label to be the tool name (type)
-                    e.label = g.entry_points['tool'][
-                        tool_name].tool_label + u' \u25be'
+                    e.label = g.entry_points['tool'][tool_name].tool_label + ' \u25be'
                     # add tool url to list of urls that will match this nav entry
                     # have to do this before changing the url to the list page
                     e.matching_urls.append(e.url)
                     # change url to point to tool list page
                     e.url = self.url() + '_list/' + tool_name
+                    e.mount_point = None
                     e.children.append(child)
                     grouped_nav[tool_name] = e
                 else:
@@ -759,15 +775,16 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                     elif len(grouped_nav[tool_name].children) == SITEMAP_PER_TOOL_LIMIT - 1:
                         e.url = self.url() + '_list/' + tool_name
                         e.label = 'More...'
+                        e.mount_point = None
+                        e.extra_html_attrs = {}
                         grouped_nav[tool_name].children.append(e)
-        return grouped_nav.values()
+        return list(grouped_nav.values())
 
     def parent_iter(self):
         yield self
         pp = self.parent_project
         if pp:
-            for p in pp.parent_iter():
-                yield p
+            yield from pp.parent_iter()
 
     @property
     def subprojects(self):
@@ -815,7 +832,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             mount_points[App] = mount_point
 
         # count mount point names
-        mount_point_counts = Counter(mount_points.values())
+        mount_point_counts = Counter(list(mount_points.values()))
 
         # install each app with unique names
         for app_params in apps_params:
@@ -837,7 +854,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         try:
             return opt.validate(value)
         except fe.Invalid as e:
-            raise exceptions.ToolError(u'{}: {}'.format(opt.name, str(e)))
+            raise exceptions.ToolError(f'{opt.name}: {str(e)}')
 
     def last_ordinal_value(self):
         last_menu_item = self.ordered_mounts(include_hidden=True)[-1]
@@ -877,7 +894,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         options['mount_label'] = mount_label or App.default_mount_label or mount_point
         options['ordinal'] = int(ordinal)
         options_on_install = {o.name: o for o in App.options_on_install()}
-        for o, val in override_options.iteritems():
+        for o, val in override_options.items():
             if o in options_on_install:
                 val = self._validate_tool_option(options_on_install[o], val)
             options[o] = val
@@ -891,7 +908,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             app.install(self)
         return app
 
-    def uninstall_app(self, mount_point):
+    def uninstall_app(self, mount_point: str):
         app = self.app_instance(mount_point)
         if app is None:
             return
@@ -900,7 +917,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         with h.push_config(c, project=self, app=app):
             app.uninstall(self)
 
-    def app_instance(self, mount_point_or_config):
+    def app_instance(self, mount_point_or_config: AppConfig | str):
         if isinstance(mount_point_or_config, AppConfig):
             app_config = mount_point_or_config
         else:
@@ -913,12 +930,12 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         else:
             return App(self, app_config)
 
-    def app_config(self, mount_point):
+    def app_config(self, mount_point: str):
         return AppConfig.query.find({
             'project_id': self._id,
             'options.mount_point': mount_point}).first()
 
-    def app_config_by_tool_type(self, tool_type):
+    def app_config_by_tool_type(self, tool_type: str):
         for ac in self.app_configs:
             if ac.tool_name == tool_type:
                 return ac
@@ -929,7 +946,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             provider.shortname_validator.to_python(
                 name, check_allowed=False, neighborhood=self.neighborhood)
         except exceptions.Invalid:
-            raise exceptions.ToolError, 'Mount point "%s" is invalid' % name
+            raise exceptions.ToolError('Mount point "%s" is invalid' % name)
         return provider.register_subproject(self, name, user or c.user, install_apps, project_name=project_name)
 
     def ordered_mounts(self, include_hidden=False):
@@ -948,8 +965,8 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         for ac in self.app_configs:
             App = g.entry_points['tool'].get(ac.tool_name)
             if include_hidden or App and not App.hidden:
-                if not self.is_nbhd_project and (ac.tool_name.lower() in anchored_tools.keys()):
-                    ordinal = anchored_tools.keys().index(ac.tool_name.lower())
+                if not self.is_nbhd_project and (ac.tool_name.lower() in list(anchored_tools.keys())):
+                    ordinal = list(anchored_tools.keys()).index(ac.tool_name.lower())
                 else:
                     ordinal = int(ac.options.get('ordinal', 0)) + i
                 result.append({'ordinal': int(ordinal), 'ac': ac})
@@ -962,11 +979,11 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             if 'sub' in mount:
                 sub = mount['sub']
                 if has_access(sub, 'read', user):
-                    return mount
+                    return mount, sub
             elif 'ac' in mount:
                 app = self.app_instance(mount['ac'])
                 if app.is_visible_to(user):
-                    return mount
+                    return mount, app
         return None
 
     def next_mount_point(self, include_hidden=False):
@@ -1003,7 +1020,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         uids = [uid for uid in named_roles.userids_that_reach if uid is not None]
         return list(User.query.find({'_id': {'$in': uids}, 'disabled': False, 'pending': False}))
 
-    def users_with_role(self, *role_names):
+    def users_with_role(self, *role_names) -> list[User]:
         """Return all users in this project that have at least one of the roles
         specified.
 
@@ -1046,9 +1063,9 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         if apps is None:
             apps = []
             if is_user_project:
-                apps += [('Wiki', 'wiki', 'Wiki'),
-                         ('profile', 'profile', 'Profile'),
-                         ]
+                if 'wiki' not in g.user_profile_disabled_tools():
+                    apps.append(('Wiki', 'wiki', 'Wiki'))
+                apps.append(('profile', 'profile', 'Profile'))
             apps += [
                 ('admin', 'admin', 'Admin'),
                 ('search', 'search', 'Search'),
@@ -1087,15 +1104,33 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                 self.install_app(ep_name, mount_point, label, ordinal=i)
             self.database_configured = True
             self.notifications_disabled = False
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
 
     def add_user(self, user, role_names):
-        'Convenience method to add member with the given role(s).'
+        '''Convenience method to add member with the given role(s).'''
         pr = ProjectRole.by_user(user, project=self, upsert=True)
         for role_name in role_names:
             r = ProjectRole.by_name(role_name, self)
             pr.roles.append(r._id)
 
+    def remove_user(self, user, role_names=None):
+        '''Convenience method to add member with the given role(s).'''
+        pr = ProjectRole.by_user(user, project=self)
+        if not pr:
+            return
+
+        if not role_names or not isinstance(role_names, Iterable):
+            ProjectRole.query.remove({'_id': pr._id})
+            return
+
+        for role_name in role_names:
+            r = ProjectRole.by_name(role_name, self)
+            if r._id in pr.roles:
+                pr.roles.remove(r._id)
+
+        if not pr.roles:
+            pr.remove()
+
     @property
     def twitter_handle(self):
         return self.social_account('Twitter').accounturl
@@ -1104,10 +1139,23 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
     def facebook_page(self):
         return self.social_account('Facebook').accounturl
 
+    @property
+    def fediverse_address(self):
+        return self.social_account('Fediverse').accounturl
+
+    @classmethod
+    def fediverse_parse_address(cls, username):
+        pieces = username.split('@')
+        return f'https://{pieces[-1]}/@{pieces[1]}'
+
+    @property
+    def instagram_page(self):
+        return self.social_account('Instagram').accounturl
+
     def social_account(self, socialnetwork):
         try:
-            account = (
-                sn for sn in self.socialnetworks if sn.socialnetwork == socialnetwork).next()
+            account = next(
+                sn for sn in self.socialnetworks if sn.socialnetwork == socialnetwork)
         except StopIteration:
             return None
         else:
@@ -1123,7 +1171,11 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
                 accounturl=accounturl
             ))
 
-    def bulk_export_path(self):
+    def bulk_export_path(self, rootdir):
+        """
+        :param rootdir: a directory path, using {nbhd} {project} or {c..} substitution vars if desired
+        :return: a bulk export path for the current project
+        """
         shortname = self.shortname
         if self.is_nbhd_project:
             shortname = self.url().strip('/')
@@ -1131,7 +1183,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             shortname = self.shortname.split('/')[1]
         elif not self.is_root:
             shortname = self.shortname.split('/')[0]
-        return config['bulk_export_path'].format(
+        return rootdir.format(
             nbhd=self.neighborhood.url_prefix.strip('/'),
             project=shortname,
             c=c,
@@ -1150,7 +1202,8 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         elif not self.is_root:
             shortname = self.shortname.split('/')[1]
 
-        return config['bulk_export_filename'].format(project=shortname, date=datetime.utcnow())
+        filename_format = config['bulk_export_filename']
+        return filename_format.format(project=shortname, date=datetime.utcnow())
 
     def bulk_export_status(self):
         '''
@@ -1172,7 +1225,7 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
         try:
             _private = self.private
         except Exception:
-            log.warn('Error getting self.private on project {}'.format(self.shortname), exc_info=True)
+            log.warning(f'Error getting self.private on project {self.shortname}', exc_info=True)
             _private = False
         fields = dict(id=self.index_id(),
                       title='Project %s' % self.name,
@@ -1237,10 +1290,10 @@ class Project(SearchIndexable, MappedClass, ActivityNode, ActivityObject):
             screenshots=[
                 dict(
                     url=h.absurl(self.url() + 'screenshot/' +
-                                 urllib.quote(ss.filename.encode('utf8'))),
+                                 six.moves.urllib.parse.quote(ss.filename.encode('utf8'))),
                     thumbnail_url=h.absurl(
                         self.url(
-                        ) + 'screenshot/' + urllib.quote(ss.filename.encode('utf8')) + '/thumb'),
+                        ) + 'screenshot/' + six.moves.urllib.parse.quote(ss.filename.encode('utf8')) + '/thumb'),
                     caption=ss.caption,
                 )
                 for ss in self.get_screenshots()
@@ -1357,6 +1410,8 @@ class AppConfig(MappedClass, ActivityObject):
             'options.import_id',
             ('options.mount_point', 'project_id')]
 
+    query: Query[AppConfig]
+
     # AppConfig schema
     _id = FieldProperty(S.ObjectId)
     project_id = ForeignIdProperty(Project)
@@ -1386,7 +1441,7 @@ class AppConfig(MappedClass, ActivityObject):
         '''ACL processing should terminate at the AppConfig'''
         return None
 
-    def load(self):
+    def load(self) -> typing.Type[Application]:
         """
         :returns: the related :class:`Application <allura.app.Application>` class
         """
