@@ -15,18 +15,26 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
+
+import six
 from datetime import datetime
+import typing
 from tg import config
 from paste.deploy.converters import asbool
 
 from ming import schema as S
-from ming.orm import Mapper
-from ming.orm import FieldProperty
-from ming.orm.declarative import MappedClass
+from ming.odm import Mapper
+from ming.odm import FieldProperty
+from ming.odm.declarative import MappedClass
 from datetime import timedelta
 import difflib
 
 from allura.model.session import main_orm_session
+from allura.lib import helpers as h
+from functools import reduce
+
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
 
 
 class Stats(MappedClass):
@@ -36,6 +44,8 @@ class Stats(MappedClass):
         session = main_orm_session
         unique_indexes = ['_id']
 
+    query: 'Query[Stats]'
+
     _id = FieldProperty(S.ObjectId)
 
     visible = FieldProperty(bool, if_missing=True)
@@ -87,44 +97,6 @@ class Stats(MappedClass):
         min_date = config.get('userstats.start_date', '0001-1-1')
         return max(datetime.strptime(min_date, '%Y-%m-%d'), self.registration_date)
 
-    def getCodeContribution(self):
-        days = (datetime.today() - self.start_date).days
-        if not days:
-            days = 1
-        for val in self['general']:
-            if val['category'] is None:
-                for commits in val['commits']:
-                    if commits['language'] is None:
-                        if days > 30:
-                            return round(float(commits.lines) / days * 30, 2)
-                        else:
-                            return float(commits.lines)
-        return 0
-
-    def getDiscussionContribution(self):
-        days = (datetime.today() - self.start_date).days
-        if not days:
-            days = 1
-        for val in self['general']:
-            if val['category'] is None:
-                for artifact in val['messages']:
-                    if artifact['messagetype'] is None:
-                        tot = artifact.created + artifact.modified
-                        if days > 30:
-                            return round(float(tot) / days * 30, 2)
-                        else:
-                            return float(tot)
-        return 0
-
-    def getTicketsContribution(self):
-        for val in self['general']:
-            if val['category'] is None:
-                tickets = val['tickets']
-                if tickets.assigned == 0:
-                    return 0
-                return round(float(tickets.solved) / tickets.assigned, 2)
-        return 0
-
     def getCommits(self, category=None):
         i = getElementIndex(self.general, category=category)
         if i is None:
@@ -178,29 +150,18 @@ class Stats(MappedClass):
                 n, lines = 0, 0
             else:
                 n, lines = entry.commits[i].number, entry.commits[i].lines
-            if cat != None:
+            if cat is not None:
                 cat = TroveCategory.query.get(_id=cat)
             by_cat[cat] = dict(number=n, lines=lines)
         return by_cat
 
-    # For the moment, commit stats by language are not used, since each project
-    # can be linked to more than one programming language and we don't know how
-    # to which programming language should be credited a line of code modified
-    # within a project including two or more languages.
-    def getCommitsByLanguage(self):
-        i = getElementIndex(self.general, category=None)
-        if i is None:
-            return dict(number=0, lines=0)
-        return dict([(el.language, dict(lines=el.lines, number=el.number))
-                     for el in self.general[i].commits])
-
     def getArtifactsByCategory(self, detailed=False):
         from allura.model.project import TroveCategory
 
         by_cat = {}
         for entry in self.general:
             cat = entry.category
-            if cat != None:
+            if cat is not None:
                 cat = TroveCategory.query.get(_id=cat)
             if detailed:
                 by_cat[cat] = entry.messages
@@ -217,9 +178,9 @@ class Stats(MappedClass):
         if i is None:
             return {}
         entry = self.general[i].messages
-        by_type = dict([(el.messagetype, dict(created=el.created,
-                                              modified=el.modified))
-                        for el in entry])
+        by_type = {el.messagetype: dict(created=el.created,
+                                              modified=el.modified)
+                        for el in entry}
         return by_type
 
     def getTicketsByCategory(self):
@@ -228,7 +189,7 @@ class Stats(MappedClass):
         by_cat = {}
         for entry in self.general:
             cat = entry.category
-            if cat != None:
+            if cat is not None:
                 cat = TroveCategory.query.get(_id=cat)
             a, s = entry.tickets.assigned, entry.tickets.solved
             r, time = entry.tickets.solved, entry.tickets.totsolvingtime
@@ -263,30 +224,11 @@ class Stats(MappedClass):
                          if cat in el.categories + [None]]
             n = len(lineslist)
             lines = sum(lineslist)
-            if cat != None:
+            if cat is not None:
                 cat = TroveCategory.query.get(_id=cat)
             by_cat[cat] = dict(number=n, lines=lines)
         return by_cat
 
-    def getLastMonthCommitsByLanguage(self):
-        from allura.model.project import TroveCategory
-
-        self.checkOldArtifacts()
-        seen = set()
-        langlist = [el.language for el in self.general
-                    if el.language not in seen and not seen.add(el.language)]
-
-        by_lang = {}
-        for lang in langlist:
-            lineslist = [el.lines for el in self.lastmonth.commits
-                         if lang in el.programming_languages + [None]]
-            n = len(lineslist)
-            lines = sum(lineslist)
-            if lang != None:
-                lang = TroveCategory.query.get(_id=lang)
-            by_lang[lang] = dict(number=n, lines=lines)
-        return by_lang
-
     def getLastMonthArtifacts(self, category=None, art_type=None):
         self.checkOldArtifacts()
         cre, mod = reduce(
@@ -316,26 +258,6 @@ class Stats(MappedClass):
             by_type[t] = dict(created=cre, modified=mod)
         return by_type
 
-    def getLastMonthArtifactsByCategory(self):
-        from allura.model.project import TroveCategory
-
-        self.checkOldArtifacts()
-        seen = set()
-        catlist = [el.category for el in self.general
-                   if el.category not in seen and not seen.add(el.category)]
-
-        by_cat = {}
-        for cat in catlist:
-            cre, mod = reduce(
-                addtuple,
-                [(int(el.created), 1 - int(el.created))
-                 for el in self.lastmonth.messages
-                 if cat in el.categories + [None]], (0, 0))
-            if cat != None:
-                cat = TroveCategory.query.get(_id=cat)
-            by_cat[cat] = dict(created=cre, modified=mod)
-        return by_cat
-
     def getLastMonthTickets(self, category=None):
         from allura.model.project import TroveCategory
 
@@ -350,7 +272,7 @@ class Stats(MappedClass):
              for el in self.lastmonth.solvedtickets
              if category in el.categories + [None]],
             (0, 0))
-        if category != None:
+        if category is not None:
             category = TroveCategory.query.get(_id=category)
         if s > 0:
             time = time / s
@@ -378,7 +300,7 @@ class Stats(MappedClass):
             s, time = reduce(addtuple, [(1, el.solvingtime)
                                         for el in self.lastmonth.solvedtickets
                                         if cat in el.categories + [None]], (0, 0))
-            if cat != None:
+            if cat is not None:
                 cat = TroveCategory.query.get(_id=cat)
             if s > 0:
                 time = time / s
@@ -452,10 +374,12 @@ class Stats(MappedClass):
             if oldblob is None:
                 lines = len(listnew)
             elif newblob and newblob.has_html_view:
+                # py2 unified_diff can handle some unicode but not consistently, so best to do ensure_str (can drop it on py3)
                 diff = difflib.unified_diff(
-                    listold, listnew,
-                    ('old' + oldblob.path()).encode('utf-8'),
-                    ('new' + newblob.path()).encode('utf-8'))
+                    [six.ensure_str(h.really_unicode(line)) for line in listold],
+                    [six.ensure_str(h.really_unicode(line)) for line in listnew],
+                    six.ensure_str('old' + oldblob.path()),
+                    six.ensure_str('new' + newblob.path()))
                 lines = len(
                     [l for l in diff if len(l) > 0 and l[0] == '+']) - 1
             else:
@@ -601,9 +525,9 @@ def _convertTimeDiff(int_seconds):
         return None
     diff = timedelta(seconds=int_seconds)
     days, seconds = diff.days, diff.seconds
-    hours = seconds / 3600
+    hours = seconds // 3600
     seconds = seconds % 3600
-    minutes = seconds / 60
+    minutes = seconds // 60
     seconds = seconds % 60
     return dict(
         days=days,
@@ -611,4 +535,5 @@ def _convertTimeDiff(int_seconds):
         minutes=minutes,
         seconds=seconds)
 
+
 Mapper.compile_all()
