@@ -15,4 +15,4 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from scripttask import ScriptTask
+from .scripttask import ScriptTask
