@@ -16,8 +16,9 @@
 #       under the License.
 
 import inspect
+import wrapt
 
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 
 from allura import model as M
 from allura.lib.decorators import task
@@ -34,6 +35,6 @@ def install_app(*args, **kwargs):
 install_app.__doc__ += '''
     Arguments::
 
-        ''' + inspect.formatargspec(*inspect.getargspec(
+        ''' + wrapt.formatargspec(*inspect.getfullargspec(
     M.Project.install_app
 )).replace('self, ', '')
