@@ -19,6 +19,7 @@ import sys
 
 from allura.lib.decorators import task, event_handler
 from allura.lib.exceptions import CompoundError
+import six
 
 
 @task
@@ -27,10 +28,10 @@ def event(event_type, *args, **kwargs):
     for t in event_handler.listeners[event_type]:
         try:
             t(event_type, *args, **kwargs)
-        except:
+        except Exception:
             exceptions.append(sys.exc_info())
     if exceptions:
         if len(exceptions) == 1:
-            raise exceptions[0][0], exceptions[0][1], exceptions[0][2]
+            raise exceptions[0][1].with_traceback(exceptions[0][2])
         else:
             raise CompoundError(*exceptions)
