@@ -17,12 +17,14 @@
 
 import os
 from mock import patch
-from nose.tools import assert_in, assert_not_in, assert_equal, assert_false, assert_true
-
+import pytest
+from webtest.app import AppError
 from ming.odm import session
 
 from allura.tests import TestController
 from allura import model as M
+from allura.lib import helpers as h
+from tg import config
 
 
 class TestDiscussBase(TestController):
@@ -39,6 +41,24 @@ class TestDiscussBase(TestController):
         thread_link = new_post.find('form', {'id': 'edit_post'}).get('action')
         return thread_link.split('/')[-2]
 
+    def _make_post(self, text, username='root'):
+        thread_link = self._thread_link()
+        thread = self.app.get(thread_link, expect_errors=True)
+        for f in thread.html.findAll('form'):
+            if f.get('action', '').endswith('/post'):
+                break
+        params = dict()
+        inputs = f.findAll('input')
+        for field in inputs:
+            if field.has_attr('name'):
+                params[field['name']] = field.get('value') or ''
+        params[f.find('textarea')['name']] = text
+        r = self.app.post(f['action'], params=params,
+                          headers={'Referer': str(thread_link)},
+                          status=302,
+                          extra_environ=dict(username=username))
+        return self.app.get(r.response.headers['Location'], expect_errors=True)
+
 
 class TestDiscuss(TestDiscussBase):
 
@@ -53,34 +73,18 @@ class TestDiscuss(TestDiscussBase):
         # remove tool-wide subscription, so it doesn't interfere
         M.Mailbox.query.remove(dict(user_id=user._id, app_config_id=thread.app_config_id))
 
-        assert_false(self._is_subscribed(user, thread))
+        assert not self._is_subscribed(user, thread)
         link = self._thread_link()
         params = {
             'threads-0._id': thread_id,
             'threads-0.subscription': 'on'}
         r = self.app.post('/wiki/_discuss/subscribe', params=params)
-        assert_true(self._is_subscribed(user, thread))
+        assert self._is_subscribed(user, thread)
         params = {'threads-0._id': thread_id}
         r = self.app.post('/wiki/_discuss/subscribe', params=params)
-        assert_false(self._is_subscribed(user, thread))
+        assert not self._is_subscribed(user, thread)
+
 
-    def _make_post(self, text):
-        thread_link = self._thread_link()
-        thread = self.app.get(thread_link)
-        for f in thread.html.findAll('form'):
-            if f.get('action', '').endswith('/post'):
-                break
-        params = dict()
-        inputs = f.findAll('input')
-        for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
-                params[field['name']] = field.get('value') or ''
-        params[f.find('textarea')['name']] = text
-        r = self.app.post(f['action'].encode('utf-8'), params=params,
-                          headers={'Referer': thread_link.encode("utf-8")},
-                          extra_environ=dict(username='root'))
-        r = r.follow()
-        return r
 
     @patch('allura.controllers.discuss.g.spam_checker.check')
     @patch('allura.controllers.discuss.g.spam_checker.submit_spam')
@@ -88,7 +92,7 @@ class TestDiscuss(TestDiscussBase):
         thread_link = self._thread_link()
         r = self._make_post('This is a post')
         assert 'This is a post' in r, r
-        assert_equal(check_spam.call_args[0][0], 'This is a post')
+        assert check_spam.call_args[0][0] == 'This is a post'
 
         post_link = str(
             r.html.find('div', {'class': 'edit_post_form reply'}).find('form')['action'])
@@ -98,13 +102,13 @@ class TestDiscuss(TestDiscussBase):
         params = dict()
         inputs = post_form.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[post_form.find('textarea')['name']] = 'This is a new post'
         r = self.app.post(post_link,
                           params=params,
-                          headers={'Referer': thread_link.encode("utf-8")})
-        r = r.follow()
+                          headers={'Referer': str(thread_link)})
+        r = self.app.get(r.response.headers['Location'], status=404)
         assert 'This is a new post' in r, r
         r = self.app.get(post_link)
         assert str(r).count('This is a new post') == 3
@@ -112,15 +116,15 @@ class TestDiscuss(TestDiscussBase):
         params = dict()
         inputs = post_form.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[post_form.find('textarea')['name']] = 'Tis a reply'
         r = self.app.post(post_link + 'reply',
                           params=params,
-                          headers={'Referer': post_link.encode("utf-8")})
+                          headers={'Referer': str(post_link)})
         r = self.app.get(thread_link)
         assert 'Tis a reply' in r, r
-        permalinks = [post.find('form')['action'].encode('utf-8')
+        permalinks = [post.find('form')['action']
                       for post in r.html.findAll('div', {'class': 'edit_post_form reply'})]
         self.app.post(permalinks[1] + 'flag')
         self.app.post(permalinks[1] + 'moderate', params=dict(delete='delete'))
@@ -128,6 +132,15 @@ class TestDiscuss(TestDiscussBase):
         assert submit_spam.call_args[0] == (
             'This is a new post',), submit_spam.call_args[0]
 
+    def test_rate_limit_comments(self):
+        with h.push_config(config, **{'allura.rate_limits_per_user': '{"3600": 2}'}):
+            for i in range(0, 2):
+                r = self._make_post(f'This is a post {i}')
+                assert 'rate limit exceeded' not in r.text
+
+            r = self._make_post('This is a post that should fail.')
+            assert 'rate limit exceeded' in r.text
+
     def test_permissions(self):
         thread_url = self._thread_link()
         thread_id = thread_url.rstrip('/').split('/')[-1]
@@ -135,8 +148,8 @@ class TestDiscuss(TestDiscussBase):
 
         # ok initially
         non_admin = 'test-user'
-        self.app.get(thread_url, status=200,
-                     extra_environ=dict(username=non_admin))
+        self.app.get(thread_url, status=404,
+                     extra_environ=dict(username=str(non_admin)))
 
         # set wiki page private
         from forgewiki.model import Page
@@ -149,10 +162,10 @@ class TestDiscuss(TestDiscussBase):
             M.DENY_ALL,
         ]
 
-        self.app.get(thread_url, status=200,  # ok
+        self.app.get(thread_url, status=404,
                      extra_environ=dict(username='test-admin'))
         self.app.get(thread_url, status=403,  # forbidden
-                     extra_environ=dict(username=non_admin))
+                     extra_environ=dict(username=str(non_admin)))
 
     def test_spam_link(self):
         r = self._make_post('Test post')
@@ -179,11 +192,106 @@ class TestDiscuss(TestDiscussBase):
         assert M.Post.query.find().count() == 1
         assert M.Post.query.find({'deleted': False}).count() == 0
 
+    def test_comment_get_markdown(self):
+        r = self._make_post('This is a post')
+        post_id = str(
+            r.html.find('div', {'class': 'discussion-post'})['id'])
+        response = self.app.get(self._thread_link() + post_id + '/get_markdown')
+        assert 'This is a post' in response
+
+    def test_comment_update_markdown(self):
+        r = self._make_post('This is a post')
+        post_id = str(
+            r.html.find('div', {'class': 'discussion-post'})['id'])
+        update_link = str(self._thread_link() + post_id + '/update_markdown')
+        response = self.app.post(
+            update_link,
+            params={
+                'text': '- [x] checkbox'})
+        assert response.json['status'] == 'success'
+        # anon users can't edit markdown
+        response = self.app.post(
+            update_link,
+            params={
+                'text': '- [x] checkbox'},
+            extra_environ=dict(username='*anonymous'))
+        assert response.json['status'] == 'no_permission'
+
+    def test_comment_post_reaction_new(self):
+        r = self._make_post('This is a post')
+        post_id = str(
+            r.html.find('div', {'class': 'discussion-post'})['id'])
+        react_link = str(self._thread_link() + post_id + '/post_reaction')
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':+1:'})
+        assert response.json['status'] == 'ok'
+        assert response.json['counts'][':+1:'] == 1
+        response = self.app.post(
+            react_link,
+            params={
+                'r': 'invalid'})
+        assert response.json['status'] == 'error'
+        assert response.json['counts'][':+1:'] == 1
+        # anon users can't react comments
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':+1:'},
+            extra_environ=dict(username='*anonymous'))
+        assert response.json['error'] == 'no_permission'
+        # even anon can't send invalid reactions
+        response = self.app.post(
+            react_link,
+            params={
+                'r': 'invalid'},
+            extra_environ=dict(username='*anonymous'))
+        assert response.json['error'] == 'no_permission'
+
+    def test_comment_post_reaction_change(self):
+        r = self._make_post('This is a post')
+        post_id = str(
+            r.html.find('div', {'class': 'discussion-post'})['id'])
+        react_link = str(self._thread_link() + post_id + '/post_reaction')
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':-1:'})
+        assert response.json['status'] == 'ok'
+        assert response.json['counts'][':-1:'] == 1
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':+1:'})
+        assert response.json['status'] == 'ok'
+        assert response.json['counts'][':+1:'] == 1
+        assert ':-1:' not in response.json['counts']
+
+    def test_comment_post_reaction_undo(self):
+        r = self._make_post('This is a post')
+        post_id = str(
+            r.html.find('div', {'class': 'discussion-post'})['id'])
+        react_link = str(self._thread_link() + post_id + '/post_reaction')
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':tada:'})
+        assert response.json['status'] == 'ok'
+        assert response.json['counts'][':tada:'] == 1
+        response = self.app.post(
+            react_link,
+            params={
+                'r': ':tada:'})
+        assert response.json['status'] == 'ok'
+        assert ':tada:' not in response.json['counts']
+
     def test_user_filter(self):
         r = self._make_post('Test post')
         post_link = str(
             r.html.find('div', {'class': 'edit_post_form reply'}).find('form')['action'])
-        self.app.post(post_link + 'moderate', params=dict(spam='spam'))
+        r = self.app.post(post_link + 'moderate', params=dict(spam='spam'))
+        assert r.json == {"result": "success"}
         post = M.Post.query.find().first()
         post_username = post.author().username
         moderate_link = '/p/test/wiki/_discuss/moderate'
@@ -237,20 +345,20 @@ class TestDiscuss(TestDiscussBase):
         post_link = str(
             r.html.find('div', {'class': 'edit_post_form reply'}).find('form')['action'])
         post = M.Post.query.find().first()
-        assert_equal(post.status, 'pending')
+        assert post.status == 'pending'
         r = self.app.get('/wiki/feed.rss')
-        assert_not_in('Post needs moderation!', r)
+        assert 'Post needs moderation!' not in r
 
         self.app.post(post_link + 'moderate', params=dict(approve='approve'))
         post = M.Post.query.find().first()
-        assert_equal(post.status, 'ok')
+        assert post.status == 'ok'
         r = self.app.get('/wiki/feed.rss')
-        assert_in('Post needs moderation!', r)
+        assert 'Post needs moderation!' in r
 
     def test_post_paging(self):
         thread_link = self._thread_link()
         # just make sure it doesn't 500
-        self.app.get('%s?limit=50&page=0' % thread_link)
+        self.app.get('%s?limit=50&page=0' % thread_link, status=404)
 
     @patch('allura.controllers.discuss.g.director.create_activity')
     def test_edit_post(self, create_activity):
@@ -262,23 +370,20 @@ class TestDiscuss(TestDiscussBase):
         reply_form = r.html.find(
             'div', {'class': 'edit_post_form reply'}).find('form')
         post_link = str(reply_form['action'])
-        assert 'This is a post' in str(
-            r.html.find('div', {'class': 'display_post'}))
-        assert 'Last edit:' not in str(
-            r.html.find('div', {'class': 'display_post'}))
+        assert 'This is a post' in r.html.find('div', {'class': 'display_post'}).text
+        assert 'Last edit:' not in r.html.find('div', {'class': 'display_post'}).text
         params = dict()
         inputs = reply_form.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[reply_form.find('textarea')['name']] = 'zzz'
         self.app.post(post_link, params)
         assert create_activity.call_count == 1, create_activity.call_count
         assert create_activity.call_args[0][1] == 'modified'
         r = self.app.get(thread_url)
-        assert 'zzz' in str(r.html.find('div', {'class': 'display_post'}))
-        assert 'Last edit: Test Admin ' in str(
-            r.html.find('div', {'class': 'display_post'}))
+        assert 'zzz' in r.html.find('div', {'class': 'display_post'}).text
+        assert 'Last edit: Test Admin ' in r.html.find('div', {'class': 'display_post'}).text
 
     def test_deleted_post(self):
         r = self._make_post('This is a post')
@@ -295,22 +400,22 @@ class TestDiscuss(TestDiscussBase):
 
 class TestAttachment(TestDiscussBase):
 
-    def setUp(self):
-        super(TestAttachment, self).setUp()
+    def setup_method(self, method):
+        super().setup_method(method)
         self.thread_link = self._thread_link()
-        thread = self.app.get(self.thread_link)
+        thread = self.app.get(self.thread_link, status=404)
         for f in thread.html.findAll('form'):
             if f.get('action', '').endswith('/post'):
                 break
-        self.post_form_link = f['action'].encode('utf-8')
+        self.post_form_link = f['action']
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'Test Post'
-        r = self.app.post(f['action'].encode('utf-8'), params=params,
-                          headers={'Referer': self.thread_link.encode('utf-8')})
+        r = self.app.post(f['action'], params=params,
+                          headers={'Referer': str(self.thread_link)})
         r = r.follow()
         self.post_link = str(
             r.html.find('div', {'class': 'edit_post_form reply'}).find('form')['action'])
@@ -326,7 +431,7 @@ class TestAttachment(TestDiscussBase):
 
     def test_attach(self):
         r = self.app.post(self.post_link + 'attach',
-                          upload_files=[('file_info', 'test.txt', 'HiThere!')])
+                          upload_files=[('file_info', 'test.txt', b'HiThere!')])
         r = self.app.get(self.thread_link)
         assert '<div class="attachment_holder">' in r
         alink = self.attach_link()
@@ -334,12 +439,12 @@ class TestAttachment(TestDiscussBase):
         assert r.content_type == 'text/plain'
         assert r.content_disposition == 'attachment;filename="test.txt"', 'Attachments should force download'
         r = self.app.post(self.post_link + 'attach',
-                          upload_files=[('file_info', 'test.o12', 'HiThere!')])
+                          upload_files=[('file_info', 'test.o12', b'HiThere!')])
         r = self.app.post(alink, params=dict(delete='on'))
 
     def test_attach_svg(self):
         r = self.app.post(self.post_link + 'attach',
-                          upload_files=[('file_info', 'test.svg', '<svg onclick="prompt(document.domain)"></svg>')])
+                          upload_files=[('file_info', 'test.svg', b'<svg onclick="prompt(document.domain)"></svg>')])
         alink = self.attach_link()
         r = self.app.get(alink)
         assert r.content_type == 'image/svg+xml'
@@ -348,7 +453,7 @@ class TestAttachment(TestDiscussBase):
     def test_attach_img(self):
         r = self.app.post(self.post_link + 'attach',
                           upload_files=[('file_info', 'handtinyblack.gif',
-                                         'GIF89a\x01\x00\x01\x00\x00\xff\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x00;')])
+                                         b'GIF89a\x01\x00\x01\x00\x00\xff\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x00\x02\x00;')])
         alink = self.attach_link()
         r = self.app.get(alink)
         assert r.content_type == 'image/gif'
@@ -363,18 +468,19 @@ class TestAttachment(TestDiscussBase):
         inputs = post_form.findAll('input')
 
         for field in inputs:
-            if field.has_key('name') and field['name'] != 'file_info':  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name') and field['name'] != 'file_info':
                 params[field['name']] = field.get('value') or ''
         params[post_form.find('textarea')['name']] = 'Reply'
         r = self.app.post(self.post_link + 'reply',
                           params=params,
-                          upload_files=[('file_info', 'test.txt', 'HiThere!')])
+                          upload_files=[('file_info', 'test.txt', b'HiThere!'),
+                                        ('file_info', 'test2.txt', b'HiAgain!')])
         r = self.app.get(self.thread_link)
         assert "test.txt" in r
 
     def test_deleted_post_attachment(self):
         f = os.path.join(os.path.dirname(__file__), '..', 'data', 'user.png')
-        with open(f) as f:
+        with open(f, 'rb') as f:
             pic = f.read()
         self.app.post(
             self.post_link + 'attach',
@@ -385,7 +491,7 @@ class TestAttachment(TestDiscussBase):
         self.app.get(thumblink, status=200)
         _, slug = self.post_link.rstrip('/').rsplit('/', 1)
         post = M.Post.query.get(slug=slug)
-        assert post, 'Could not find post for {} {}'.format(slug, self.post_link)
+        assert post, f'Could not find post for {slug} {self.post_link}'
         post.deleted = True
         session(post).flush(post)
         self.app.get(alink, status=404)
@@ -396,7 +502,7 @@ class TestAttachment(TestDiscussBase):
         moderator = {'username': 'test-admin'}
         # set up attachment
         f = os.path.join(os.path.dirname(__file__), '..', 'data', 'user.png')
-        with open(f) as f:
+        with open(f, 'rb') as f:
             pic = f.read()
         self.app.post(
             self.post_link + 'attach',
@@ -413,14 +519,14 @@ class TestAttachment(TestDiscussBase):
         # make post unmoderated
         _, slug = self.post_link.rstrip('/').rsplit('/', 1)
         post = M.Post.query.get(slug=slug)
-        assert post, 'Could not find post for {} {}'.format(slug, self.post_link)
+        assert post, f'Could not find post for {slug} {self.post_link}'
         post.status = 'pending'
         session(post).flush(post)
         # ... make sure attachment is not visible to ordinary user
-        r = self.app.get(self.thread_link, extra_environ=ordinary_user)
+        r = self.app.get(self.thread_link, status=404, extra_environ=ordinary_user)
         assert '<div class="attachment_holder">' not in r, 'Attachment is visible on unmoderated post'
         # ... but visible to moderator
-        r = self.app.get(self.thread_link, extra_environ=moderator)
+        r = self.app.get(self.thread_link, status=404, extra_environ=moderator)
         assert '<div class="attachment_holder">' in r
         # ... and ordinary user can't access it
         self.app.get(alink, status=403, extra_environ=ordinary_user)
@@ -428,3 +534,6 @@ class TestAttachment(TestDiscussBase):
         # ... but moderator can
         self.app.get(alink, status=200, extra_environ=moderator)
         self.app.get(thumblink, status=200, extra_environ=moderator)
+
+
+
