@@ -1,4 +1,3 @@
-# coding=utf-8
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -18,16 +17,17 @@
 
 import json
 import os
-from cStringIO import StringIO
-import urllib2
+from io import BytesIO
+import six.moves.urllib.parse
+import six.moves.urllib.request
+import six.moves.urllib.error
 
 import PIL
 from mock import patch
 from tg import config
-from nose.tools import assert_equal, assert_in, assert_not_equal
-from ming.orm.ormsession import ThreadLocalORMSession, session
-from paste.httpexceptions import HTTPFound
-from pylons import app_globals as g, tmpl_context as c
+from ming.odm.odmsession import ThreadLocalODMSession, session
+from paste.httpexceptions import HTTPFound, HTTPMovedPermanently
+from tg import app_globals as g, tmpl_context as c
 
 import allura
 from allura import model as M
@@ -39,8 +39,20 @@ from alluratest.controller import setup_trove_categories
 
 
 class TestNeighborhood(TestController):
+
+    def setup_method(self, method):
+        super().setup_method(method)
+        self._cleanup_audit_log()
+
+    def teardown_method(self, method):
+        super().teardown_method(method)
+        self._cleanup_audit_log()
+
+    def _cleanup_audit_log(self):
+        M.AuditLog.query.remove({})
+
     def test_home_project(self):
-        r = self.app.get('/adobe/wiki/', status=302)
+        r = self.app.get('/adobe/wiki/', status=301)
         assert r.location.endswith('/adobe/wiki/Home/')
         r = r.follow()
         assert 'This is the "Adobe" neighborhood' in str(r), str(r)
@@ -96,18 +108,16 @@ class TestNeighborhood(TestController):
             'show_title': 'false',
             'allow_browse': 'false',
             'css': '.class { border: 1px; }',
-            'tracking_id': 'U-123456',
             'homepage': '[Homepage]',
             'project_list_url': 'http://fake.org/project_list',
             'project_template': '{"name": "template"}',
+            'tracking_id': 'U-123456',
+            'prohibited_tools': 'wiki, tickets',
             'anchored_tools': 'wiki:Wiki',
-            'prohibited_tools': 'wiki, tickets'
-
         }
+
         self.app.post('/p/_admin/update', params=params,
                       extra_environ=dict(username='root'))
-        # must get as many log records as many values are updated
-        assert M.AuditLog.query.find().count() == len(params)
 
         assert check_log('change neighborhood name to Pjs')
         assert check_log('change neighborhood redirect to http://fake.org/')
@@ -122,6 +132,10 @@ class TestNeighborhood(TestController):
                          '{"name": "template"}')
         assert check_log('update neighborhood tracking_id')
         assert check_log('update neighborhood prohibited tools')
+        assert check_log('update neighborhood anchored tools')
+
+        # must get as many log records as many values are updated
+        assert M.AuditLog.query.find().count() == len(params)
 
     def test_prohibited_tools(self):
         self.app.post('/p/_admin/update',
@@ -159,21 +173,21 @@ class TestNeighborhood(TestController):
                                       anchored_tools='w!iki:Wiki, tickets:Ticket'),
                           extra_environ=dict(username='root'))
         assert 'error' in self.webflash(r)
-        assert_equal(neighborhood.anchored_tools, 'wiki:Wiki, tickets:Ticket')
+        assert neighborhood.anchored_tools == 'wiki:Wiki, tickets:Ticket'
 
         r = self.app.post('/p/_admin/update',
                           params=dict(name='Projects',
                                       anchored_tools='wiki:Wiki,'),
                           extra_environ=dict(username='root'))
         assert 'error' in self.webflash(r)
-        assert_equal(neighborhood.anchored_tools, 'wiki:Wiki, tickets:Ticket')
+        assert neighborhood.anchored_tools == 'wiki:Wiki, tickets:Ticket'
 
         r = self.app.post('/p/_admin/update',
                           params=dict(name='Projects',
                                       anchored_tools='badname,'),
                           extra_environ=dict(username='root'))
         assert 'error' in self.webflash(r)
-        assert_equal(neighborhood.anchored_tools, 'wiki:Wiki, tickets:Ticket')
+        assert neighborhood.anchored_tools == 'wiki:Wiki, tickets:Ticket'
 
         r = self.app.get('/p/test/admin/overview')
         top_nav = r.html.find(id='top_nav')
@@ -219,7 +233,7 @@ class TestNeighborhood(TestController):
         neighborhood = M.Neighborhood.query.get(name='Adobe')
         proj = M.Project.query.get(neighborhood_id=neighborhood._id)
         proj.deleted = True
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         r = self.app.get('/adobe/_admin/stats/',
                          extra_environ=dict(username='root'))
         assert 'Deleted: 1' in r
@@ -230,7 +244,7 @@ class TestNeighborhood(TestController):
         proj = M.Project.query.get(neighborhood_id=neighborhood._id)
         proj.deleted = False
         proj.private = True
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         r = self.app.get('/adobe/_admin/stats/',
                          extra_environ=dict(username='root'))
         assert 'Deleted: 0' in r
@@ -240,7 +254,7 @@ class TestNeighborhood(TestController):
         neighborhood = M.Neighborhood.query.get(name='Adobe')
         proj = M.Project.query.get(neighborhood_id=neighborhood._id)
         proj.private = False
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         r = self.app.get('/adobe/_admin/stats/adminlist',
                          extra_environ=dict(username='root'))
         pq = M.Project.query.find(
@@ -263,7 +277,7 @@ class TestNeighborhood(TestController):
         file_name = 'neo-icon-set-454545-256x350.png'
         file_path = os.path.join(
             allura.__path__[0], 'nf', 'allura', 'images', file_name)
-        file_data = file(file_path).read()
+        file_data = open(file_path, 'rb').read()
         upload = ('icon', file_name, file_data)
 
         r = self.app.get('/adobe/_admin/', extra_environ=dict(username='root'))
@@ -272,7 +286,7 @@ class TestNeighborhood(TestController):
                                       homepage='# MozQ1'),
                           extra_environ=dict(username='root'), upload_files=[upload])
         r = self.app.get('/adobe/icon')
-        image = PIL.Image.open(StringIO(r.body))
+        image = PIL.Image.open(BytesIO(r.body))
         assert image.size == (48, 48)
 
         r = self.app.get('/adobe/icon?foo=bar')
@@ -283,34 +297,34 @@ class TestNeighborhood(TestController):
         neighborhood.features['google_analytics'] = True
         r = self.app.get('/adobe/_admin/overview',
                          extra_environ=dict(username='root'))
-        assert 'Analytics Tracking ID' in r
+        r.mustcontain('Google Analytics ID')
         r = self.app.get('/adobe/adobe-1/admin/overview',
                          extra_environ=dict(username='root'))
-        assert 'Analytics Tracking ID' in r
+        r.mustcontain('Google Analytics ID')
         r = self.app.post('/adobe/_admin/update',
                           params=dict(name='Adobe', css='',
                                       homepage='# MozQ1', tracking_id='U-123456'),
-                          extra_environ=dict(username='root'))
+                          extra_environ=dict(username='root'), status=302)
         r = self.app.post('/adobe/adobe-1/admin/update',
                           params=dict(tracking_id='U-654321'),
-                          extra_environ=dict(username='root'))
+                          extra_environ=dict(username='root'), status=302)
         r = self.app.get('/adobe/adobe-1/admin/overview',
                          extra_environ=dict(username='root'))
-        assert "_add_tracking('nbhd', 'U-123456');" in r, r
-        assert "_add_tracking('proj', 'U-654321');" in r
+        r.mustcontain("_add_tracking('nbhd', 'U-123456');")
+        r.mustcontain("_add_tracking('proj', 'U-654321');")
         # analytics not allowed
         neighborhood = M.Neighborhood.query.get(name='Adobe')
         neighborhood.features['google_analytics'] = False
         r = self.app.get('/adobe/_admin/overview',
                          extra_environ=dict(username='root'))
-        assert 'Analytics Tracking ID' not in r
+        r.mustcontain(no='Google Analytics ID')
         r = self.app.get('/adobe/adobe-1/admin/overview',
                          extra_environ=dict(username='root'))
-        assert 'Analytics Tracking ID' not in r
+        r.mustcontain(no='Google Analytics ID')
         r = self.app.get('/adobe/adobe-1/admin/overview',
                          extra_environ=dict(username='root'))
-        assert "_add_tracking('nbhd', 'U-123456');" not in r
-        assert "_add_tracking('proj', 'U-654321');" not in r
+        r.mustcontain(no="_add_tracking('nbhd', 'U-123456');")
+        r.mustcontain(no="_add_tracking('proj', 'U-654321');")
 
     def test_custom_css(self):
         test_css = '.test{color:red;}'
@@ -328,7 +342,7 @@ class TestNeighborhood(TestController):
         neighborhood = M.Neighborhood.query.get(name='Adobe')
         neighborhood.features['css'] = 'picker'
         r = self.app.get('/adobe/')
-        while isinstance(r.response, HTTPFound):
+        while isinstance(r.response, HTTPFound) or isinstance(r.response, HTTPMovedPermanently):
             r = r.follow()
         assert test_css in r
         r = self.app.get('/adobe/_admin/overview',
@@ -338,7 +352,7 @@ class TestNeighborhood(TestController):
         neighborhood = M.Neighborhood.query.get(name='Adobe')
         neighborhood.features['css'] = 'custom'
         r = self.app.get('/adobe/')
-        while isinstance(r.response, HTTPFound):
+        while isinstance(r.response, HTTPFound) or isinstance(r.response, HTTPMovedPermanently):
             r = r.follow()
         assert test_css in r
         r = self.app.get('/adobe/_admin/overview',
@@ -583,6 +597,7 @@ class TestNeighborhood(TestController):
         neighborhood = M.Neighborhood.query.get(name='Projects')
         neighborhood.features['private_projects'] = False
         r = self.app.get('/p/add_project', extra_environ=dict(username='root'))
+        assert 'canonical' in r
         assert 'private_project' not in r
 
         r = self.app.post(
@@ -596,7 +611,7 @@ class TestNeighborhood(TestController):
             antispam=True,
             extra_environ=dict(username='root'))
         cookies = r.headers.getall('Set-Cookie')
-        flash_msg_cookies = map(urllib2.unquote, cookies)
+        flash_msg_cookies = list(map(six.moves.urllib.parse.unquote, cookies))
 
         assert any('Internal Error' in cookie for cookie in flash_msg_cookies)
 
@@ -747,17 +762,17 @@ class TestNeighborhood(TestController):
         # check the labels and trove cats
         r = self.app.get('/adobe/testtemp/admin/trove')
         assert 'mmi' in r
-        assert u'Communications » Telephony' in r
+        assert 'Communications » Telephony' in r
         assert '5 - Production/Stable' in r
         # check the wiki text
         r = self.app.get('/adobe/testtemp/wiki/').follow()
         assert "My home text!" in r
         # check tool options
         opts = p.app_config('wiki').options
-        assert_equal(False, opts.show_discussion)
-        assert_equal(False, opts.show_left_bar)
-        assert_equal(False, opts.show_right_bar)
-        assert_equal("http://foo.com/testtemp/", opts.some_url)
+        assert opts.show_discussion is False
+        assert opts.show_left_bar is False
+        assert opts.show_right_bar is False
+        assert "http://foo.com/testtemp/" == opts.some_url
         # check that custom groups/perms/users were setup correctly
         roles = p.named_roles
         for group in test_groups:
@@ -831,23 +846,23 @@ class TestNeighborhood(TestController):
         for name in ('My+Moz', 'Te%st!', 'ab', 'a' * 16):
             r = self.app.get(
                 '/p/check_names?neighborhood=Projects&project_unixname=%s' % name)
-            assert_equal(
-                r.json,
+            assert (
+                r.json ==
                 {'project_unixname': 'Please use 3-15 small letters, numbers, and dashes.'})
         r = self.app.get(
             '/p/check_names?neighborhood=Projects&project_unixname=mymoz')
-        assert_equal(r.json, {})
+        assert r.json == {}
         r = self.app.get(
             '/p/check_names?neighborhood=Projects&project_unixname=test')
-        assert_equal(r.json,
+        assert (r.json ==
                      {'project_unixname': 'This project name is taken.'})
 
     @td.with_tool('test/sub1', 'Wiki', 'wiki')
     def test_neighborhood_project(self):
         self.app.get('/adobe/adobe-1/admin/', status=200)
         self.app.get('/p/test/sub1/wiki/')
-        self.app.get('/p/test/sub1/', status=302)
-        self.app.get('/p/test/no-such-app/', status=404)
+        self.app.get('/p/test/sub1/', status=301)
+        self.app.get('/p/test/no-such-app/', status=301)
 
     def test_neighborhood_namespace(self):
         # p/test exists, so try creating adobe/test
@@ -860,13 +875,14 @@ class TestNeighborhood(TestController):
                           extra_environ=dict(username='root'))
         assert r.status_int == 302, r.html.find(
             'div', {'class': 'error'}).string
+        assert not r.location.endswith('/add_project'), self.webflash(r)
         r = self.app.get('/adobe/test/wiki/').follow(status=200)
 
     def test_neighborhood_awards(self):
         file_name = 'adobe_icon.png'
         file_path = os.path.join(
             allura.__path__[0], 'public', 'nf', 'images', file_name)
-        file_data = file(file_path).read()
+        file_data = open(file_path, 'rb').read()
         upload = ('icon', file_name, file_data)
 
         r = self.app.get('/adobe/_admin/awards',
@@ -890,7 +906,7 @@ class TestNeighborhood(TestController):
                          foo_id, extra_environ=dict(username='root'))
         r = self.app.get('/adobe/_admin/awards/%s/icon' %
                          foo_id, extra_environ=dict(username='root'))
-        image = PIL.Image.open(StringIO(r.body))
+        image = PIL.Image.open(BytesIO(r.body))
         assert image.size == (48, 48)
         self.app.post('/adobe/_admin/awards/grant',
                       params=dict(grant='FOO', recipient='adobe-1',
@@ -898,8 +914,8 @@ class TestNeighborhood(TestController):
                       extra_environ=dict(username='root'))
         r = self.app.get('/adobe/_admin/accolades',
                          extra_environ=dict(username='root'))
-        assert_in('Winner!', r)
-        assert_in('http://award.org', r)
+        assert 'Winner!' in r
+        assert 'http://award.org' in r
         self.app.get('/adobe/_admin/awards/%s/adobe-1' %
                      foo_id, extra_environ=dict(username='root'))
         self.app.post('/adobe/_admin/awards/%s/adobe-1/revoke' % foo_id,
@@ -908,7 +924,7 @@ class TestNeighborhood(TestController):
                       extra_environ=dict(username='root'))
 
     def test_add_a_project_link(self):
-        from pylons import tmpl_context as c
+        from tg import tmpl_context as c
         # Install Home tool for all neighborhoods
         for nb in M.Neighborhood.query.find().all():
             p = nb.neighborhood_project
@@ -934,12 +950,11 @@ class TestNeighborhood(TestController):
     def test_profile_tools(self):
         r = self.app.get('/u/test-user/',
                          extra_environ=dict(username='test-user')).follow()
-        assert r.html.find('div', 'profile-section tools').find(
-            'a', href='/u/test-user/profile/'), r.html
+        assert r.html.select('div.profile-section.tools a[href="/u/test-user/profile/"]'), r.html
 
     def test_user_project_creates_on_demand(self):
         M.User.register(dict(username='donald-duck'), make_project=False)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         self.app.get('/u/donald-duck/')
 
     def test_disabled_user_has_no_user_project(self):
@@ -947,7 +962,9 @@ class TestNeighborhood(TestController):
         self.app.get('/u/donald-duck/')  # assert it's there
         M.User.query.update(dict(username='donald-duck'),
                             {'$set': {'disabled': True}})
-        self.app.get('/u/donald-duck/', status=404)
+        self.app.get('/u/donald-duck/', status=404, extra_environ={'username': '*anonymous'})
+        self.app.get('/u/donald-duck/', status=404, extra_environ={'username': 'test-user'})
+        self.app.get('/u/donald-duck/', status=301, extra_environ={'username': 'test-admin'})  # site admin user
 
     def test_more_projects_link(self):
         r = self.app.get('/adobe/adobe-1/admin/')
@@ -974,11 +991,11 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             r = self.app.get('/p/verify_phone', {'number': '1234567890'})
             expected = {'status': 'error',
                         'error': 'Phone service is not configured'}
-            assert_equal(r.json, expected)
+            assert r.json == expected
             rid = r.session.get('phone_verification.request_id')
             hash = r.session.get('phone_verification.number_hash')
-            assert_equal(rid, None)
-            assert_equal(hash, None)
+            assert rid is None
+            assert hash is None
 
     @patch.object(g, 'phone_service', autospec=True)
     def test_verify_phone(self, phone_service):
@@ -987,11 +1004,11 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
                 'request_id': 'request-id', 'status': 'ok'}
             r = self.app.get('/p/verify_phone', {'number': '1-555-444-3333'})
             phone_service.verify.assert_called_once_with('15554443333')
-            assert_equal(r.json, {'status': 'ok'})
+            assert r.json == {'status': 'ok'}
             rid = r.session.get('phone_verification.request_id')
             hash = r.session.get('phone_verification.number_hash')
-            assert_equal(rid, 'request-id')
-            assert_equal(hash, 'f9ac49faef45d18746ced08d001e23b179107940')
+            assert rid == 'request-id'
+            assert hash == 'f9ac49faef45d18746ced08d001e23b179107940'
 
     @patch.object(g, 'phone_service', autospec=True)
     def test_verify_phone_escapes_error(self, phone_service):
@@ -1003,9 +1020,9 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             r = self.app.get('/p/verify_phone', {'number': '555-444-3333'})
         expected = {
             'status': 'error',
-            'error': u'&lt;script&gt;alert(&#34;hacked&#34;);&lt;/script&gt;',
+            'error': '&lt;script&gt;alert(&#34;hacked&#34;);&lt;/script&gt;',
         }
-        assert_equal(r.json, expected)
+        assert r.json == expected
 
     @patch.object(g, 'phone_service', autospec=True)
     def test_verify_phone_already_used(self, phone_service):
@@ -1015,10 +1032,10 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             session(u).flush(u)
             phone_service.verify.return_value = {'request_id': 'request-id', 'status': 'ok'}
             r = self.app.get('/p/verify_phone', {'number': '1-555-444-9999'})
-            assert_equal(r.json, {
+            assert r.json == {
                 'status': 'error',
-                'error': u'That phone number has already been used.'
-            })
+                'error': 'That phone number has already been used.'
+            }
 
     def test_check_phone_verification_no_params(self):
         with h.push_config(config, **{'project.verify_phone': 'true'}):
@@ -1036,12 +1053,12 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             r = self.app.get('/p/verify_phone', {'number': '1234567890'})
 
             r = self.app.get('/p/check_phone_verification', {'pin': '1234'})
-            assert_equal(r.json, {'status': 'error'})
+            assert r.json == {'status': 'error'}
             phone_service.check.assert_called_once_with(req_id, '1234')
 
             user = M.User.by_username('test-admin')
             hash = user.get_tool_data('phone_verification', 'number_hash')
-            assert_equal(hash, None)
+            assert hash is None
 
     @patch.object(g, 'phone_service', autospec=True)
     def test_check_phone_verification_ok(self, phone_service):
@@ -1055,12 +1072,12 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             r = self.app.get('/p/verify_phone', {'number': '11234567890'})
 
             r = self.app.get('/p/check_phone_verification', {'pin': '1234'})
-            assert_equal(r.json, {'status': 'ok'})
+            assert r.json == {'status': 'ok'}
             phone_service.check.assert_called_once_with(req_id, '1234')
 
             user = M.User.by_username('test-admin')
             hash = user.get_tool_data('phone_verification', 'number_hash')
-            assert_equal(hash, '54c61c96d5d5aea5254c2d4f41508a938e5501b4')
+            assert hash == '54c61c96d5d5aea5254c2d4f41508a938e5501b4'
 
     @patch.object(g, 'phone_service', autospec=True)
     def test_check_phone_verification_escapes_error(self, phone_service):
@@ -1072,9 +1089,9 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
             r = self.app.get('/p/check_phone_verification', {'pin': '1234'})
         expected = {
             'status': 'error',
-            'error': u'&lt;script&gt;alert(&#34;hacked&#34;);&lt;/script&gt;',
+            'error': '&lt;script&gt;alert(&#34;hacked&#34;);&lt;/script&gt;',
         }
-        assert_equal(r.json, expected)
+        assert r.json == expected
 
     def test_register_phone_not_verified(self):
         with h.push_config(config, **{'project.verify_phone': 'true'}):
@@ -1088,16 +1105,17 @@ class TestPhoneVerificationOnProjectRegistration(TestController):
                 extra_environ=dict(username='test-user'),
                 antispam=True)
             overlay = r.html.find('div', {'id': 'phone_verification_overlay'})
-            assert_not_equal(overlay, None)
+            assert overlay is not None
             header = overlay.find('h2')
             iframe = overlay.find('iframe')
-            assert_equal(header.getText(), 'Phone Verification Required')
-            assert_equal(iframe.get('src'), '/p/phone_verification_fragment')
+            assert header.getText() == 'Phone Verification Required'
+            assert iframe.get('src') == '/p/phone_verification_fragment'
 
 
 class TestProjectImport(TestController):
 
     def test_not_found(self):
         self.app.get('/p/import_project/asdf/', status=404)
+        self.app.get('/p/import_project/', status=404)
 
     # positive tests exist within ForgeImporter package
