@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -17,9 +15,13 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from pylons import app_globals as g
+
+import json
+
+import tg
+from bson import ObjectId
+from tg import app_globals as g
 import mock
-from nose.tools import assert_equal, assert_in, assert_not_in
 from ming.odm import ThreadLocalODMSession
 from tg import config
 
@@ -43,6 +45,7 @@ class TestRestHome(TestRestApiBase):
         request.headers = {}
         request.params = {'access_token': 'foo'}
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         self._patch_token(OAuthAccessToken)
         access_token = OAuthAccessToken.query.get.return_value
         access_token.is_bearer = False
@@ -55,6 +58,7 @@ class TestRestHome(TestRestApiBase):
         request.headers = {}
         request.params = {'access_token': 'foo'}
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         self._patch_token(OAuthAccessToken)
         OAuthAccessToken.query.get.return_value = None
         r = self.api_post('/rest/p/test/wiki', access_token='foo', status=401)
@@ -84,8 +88,9 @@ class TestRestHome(TestRestApiBase):
         request.headers = {}
         request.params = {'access_token': access_token.api_key}
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         r = self.api_post('/rest/p/test/wiki', access_token='foo')
-        assert_equal(r.status_int, 200)
+        assert r.status_int == 200
 
     @mock.patch('allura.controllers.rest.M.OAuthAccessToken')
     @mock.patch('allura.controllers.rest.request')
@@ -94,6 +99,7 @@ class TestRestHome(TestRestApiBase):
             'Authorization': 'Bearer foo'
         }
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         self._patch_token(OAuthAccessToken)
         access_token = OAuthAccessToken.query.get.return_value
         access_token.is_bearer = False
@@ -107,6 +113,7 @@ class TestRestHome(TestRestApiBase):
             'Authorization': 'Bearer foo'
         }
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         self._patch_token(OAuthAccessToken)
         OAuthAccessToken.query.get.return_value = None
         r = self.api_post('/rest/p/test/wiki', access_token='foo', status=401)
@@ -136,9 +143,10 @@ class TestRestHome(TestRestApiBase):
         ThreadLocalODMSession.flush_all()
         token = access_token.api_key
         request.headers = {
-            'Authorization': 'Bearer {}'.format(token)
+            'Authorization': f'Bearer {token}'
         }
         request.scheme = 'https'
+        request.path = '/rest/p/test/wiki'
         r = self.api_post('/rest/p/test/wiki', access_token='foo', status=200)
         # reverse proxy situation
         request.scheme = 'http'
@@ -172,13 +180,13 @@ class TestRestHome(TestRestApiBase):
 
     def test_project_data(self):
         r = self.api_get('/rest/p/test/')
-        assert_equal(r.json['shortname'], 'test')
-        assert_equal(r.json['name'], 'Test Project')
-        assert_equal(len(r.json['developers']), 1)
+        assert r.json['shortname'] == 'test'
+        assert r.json['name'] == 'Test Project'
+        assert len(r.json['developers']) == 1
         admin_dev = r.json['developers'][0]
-        assert_equal(admin_dev['username'], 'test-admin')
-        assert_equal(admin_dev['name'], 'Test Admin')
-        assert_equal(admin_dev['url'], 'http://localhost/u/test-admin/')
+        assert admin_dev['username'] == 'test-admin'
+        assert admin_dev['name'] == 'Test Admin'
+        assert admin_dev['url'] == 'http://localhost/u/test-admin/'
 
     @td.with_tool('test', 'Tickets', 'bugs')
     @td.with_tool('test', 'Tickets', 'private-bugs')
@@ -193,18 +201,18 @@ class TestRestHome(TestRestApiBase):
 
         # admin sees both 'Tickets' tools
         r = self.api_get('/rest/p/test/')
-        assert_equal(r.json['shortname'], 'test')
+        assert r.json['shortname'] == 'test'
         tool_mounts = [t['mount_point'] for t in r.json['tools']]
-        assert_in('bugs', tool_mounts)
-        assert_in('private-bugs', tool_mounts)
+        assert 'bugs' in tool_mounts
+        assert 'private-bugs' in tool_mounts
 
         # anonymous sees only non-private tool
         r = self.app.get('/rest/p/test/',
                          extra_environ={'username': '*anonymous'})
-        assert_equal(r.json['shortname'], 'test')
+        assert r.json['shortname'] == 'test'
         tool_mounts = [t['mount_point'] for t in r.json['tools']]
-        assert_in('bugs', tool_mounts)
-        assert_not_in('private-bugs', tool_mounts)
+        assert 'bugs' in tool_mounts
+        assert 'private-bugs' not in tool_mounts
 
     def test_neighborhood_has_access_no_params(self):
         r = self.api_get('/rest/p/has_access', status=404)
@@ -216,13 +224,13 @@ class TestRestHome(TestRestApiBase):
         r = self.api_get(
             '/rest/p/has_access?user=babadook&perm=read',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
         r = self.api_get(
             '/rest/p/has_access?user=test-admin&perm=jump',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
 
     def test_neighborhood_has_access_not_admin(self):
         """
@@ -238,26 +246,26 @@ class TestRestHome(TestRestApiBase):
         r = self.api_get(
             '/rest/p/has_access?user=root&perm=update',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], True)
+        assert r.status_int == 200
+        assert r.json['result'] is True
         r = self.api_get(
             '/rest/p/has_access?user=test-user&perm=update',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
 
     def test_neighborhood(self):
         self.api_get('/rest/p/', status=404)
 
     def test_neighborhood_tools(self):
         r = self.api_get('/rest/p/wiki/Home/')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['title'], 'Home')
+        assert r.status_int == 200
+        assert r.json['title'] == 'Home'
 
         r = self.api_get('/rest/p/admin/installable_tools', status=403)
 
         r = self.api_get('/rest/p/admin/installable_tools', user='root')
-        assert_equal(r.status_int, 200)
+        assert r.status_int == 200
         assert [t for t in r.json['tools'] if t['tool_label'] == 'Wiki'], r.json
 
     def test_project_has_access_no_params(self):
@@ -270,13 +278,13 @@ class TestRestHome(TestRestApiBase):
         r = self.api_get(
             '/rest/p/test/has_access?user=babadook&perm=read',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
         r = self.api_get(
             '/rest/p/test/has_access?user=test-admin&perm=jump',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
 
     def test_project_has_access_not_admin(self):
         """
@@ -292,32 +300,32 @@ class TestRestHome(TestRestApiBase):
         r = self.api_get(
             '/rest/p/test/has_access?user=test-admin&perm=update',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], True)
+        assert r.status_int == 200
+        assert r.json['result'] is True
         r = self.api_get(
             '/rest/p/test/has_access?user=test-user&perm=update',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], False)
+        assert r.status_int == 200
+        assert r.json['result'] is False
 
     def test_subproject_has_access(self):
         r = self.api_get(
             '/rest/p/test/sub1/has_access?user=test-admin&perm=update',
             user='root')
-        assert_equal(r.status_int, 200)
-        assert_equal(r.json['result'], True)
+        assert r.status_int == 200
+        assert r.json['result'] is True
 
     def test_unicode(self):
         self.app.post(
-            '/wiki/tést/update',
+            h.urlquote('/wiki/tést/update'),
             params={
-                'title': 'tést',
+                'title': 'tést'.encode(),
                 'text': 'sometext',
                 'labels': '',
-                'viewable_by-0.id': 'all'})
-        r = self.api_get('/rest/p/test/wiki/tést/')
+                })
+        r = self.api_get(h.urlquote('/rest/p/test/wiki/tést/'))
         assert r.status_int == 200
-        assert r.json['title'].encode('utf-8') == 'tést', r.json
+        assert r.json['title'] == 'tést', r.json
 
     @td.with_wiki
     def test_deny_access(self):
@@ -348,13 +356,13 @@ class TestRestHome(TestRestApiBase):
         }
         with mock.patch.dict(g.entry_points, eps):
             response = self.app.get('/rest/')
-            assert_equal(response.json, {
+            assert response.json == {
                 'site_stats': {
                     'foo_24hr': 42,
                     'bar_24hr': 84,
                     'qux_24hr': 0,
                 },
-            })
+            }
 
     def test_name_validation(self):
         r = self.api_get('/rest/p/test/')
@@ -386,7 +394,7 @@ class TestRestHome(TestRestApiBase):
     def test_notification(self, _get_site_notification):
         user = M.User.by_username('test-admin')
         note = M.SiteNotification()
-        cookie = '{}-1-False'.format(note._id)
+        cookie = f'{note._id}-1-False'
         g.theme._get_site_notification = mock.Mock(return_value=(note, cookie))
 
         r = self.app.get('/rest/notification?url=test_url&cookie=test_cookie&tool_name=test_tool')
@@ -406,6 +414,147 @@ class TestRestHome(TestRestApiBase):
         assert r.json == {}
 
 
+class TestRestNbhdAddProject(TestRestApiBase):
+
+    def setup_method(self, method):
+        super().setup_method(method)
+        # create some troves we'll need
+        M.TroveCategory(fullname="Root", trove_cat_id=1, trove_parent_id=0)
+        M.TroveCategory(fullname="License", trove_cat_id=2, trove_parent_id=1)
+        M.TroveCategory(fullname="Apache License V2.0", fullpath="License :: Apache License V2.0",
+                                     trove_cat_id=4, trove_parent_id=2)
+        M.TroveCategory(fullname="Public Domain", fullpath="License :: Public Domain",
+                                      trove_cat_id=5, trove_parent_id=2)
+        p_nbhd = M.Neighborhood.query.get(url_prefix='/p/')
+        p_nbhd.features['private_projects'] = False
+
+    def test_add_project_disallowed(self):
+        self.api_post('/rest/p/add_project', user='*anonymous', status=401)
+        self.api_post('/rest/adobe/add_project', user='test-user', status=403)
+        self.api_post('/rest/p/add_project', user='test-admin', status=403)  # not a nbhd admin
+
+    def test_add_project_missing_data(self):
+        project_data = {
+            "shortname": "my-new-proj",
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert 'Required' in r.json['error']
+
+    def test_add_project_bad_data(self):
+        project_data = {
+            "shortname": "my-new-proj",
+            "admin": 'test-admin',
+            "name": "asdf",
+            "trove_licenses": ["asdfasdf"]
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert 'is not a valid trove category' in r.json['error']
+
+        # local icon paths are not allowed through web (have to use icon_url)
+        project_data = {
+            "shortname": "my-new-proj",
+            "admin": 'test-admin',
+            "name": "asdf",
+            "icon": "/local/file/path.png"
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert 'Unrecognized keys in mapping' in r.json['error']
+
+    def test_add_project_name_error(self):
+        project_data = {
+            "shortname": "test",
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert 'This project name is taken.' == r.json['error']
+
+        project_data = {
+            "shortname": "/?xyz",
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert 'Please use' in r.json['error']
+
+    def test_add_project_create_error(self):
+        project_data = {
+            "shortname": "private-project",
+            "name": "private-project",
+            "admin": 'test-admin',
+            "private": True
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=400)
+        assert "You can't create private projects in the Projects neighborhood" == r.json['error']
+
+    def test_add_project(self):
+        project_data = {
+            "shortname": "my-new-proj",
+            "admin": "test-admin",
+            "name": "My New Project",
+            "summary": "My summary.",
+            "description": "My description.",
+            "external_homepage": "http://wwww.example.com/test",
+            "labels": ["label1", "label2"],
+            "trove_licenses": [
+                "Apache License V2.0",
+                "Public Domain"],
+            "awards": [],
+            "tool_data": {
+                "allura": {
+                    "grouping_threshold": 5
+                }
+            },
+            # any image url can work here, just don't want to rely on any other websites:
+            "icon_url": "https://allura.apache.org/theme/img/logo_white.png",
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=201)
+        assert r.json == {
+            'status': 'success',
+            'html_url': 'http://localhost/p/my-new-proj/',
+            'url': 'http://localhost/rest/p/my-new-proj/',
+        }
+        p = M.Project.query.get(shortname='my-new-proj')
+        assert p.name == 'My New Project'
+        assert len(p.trove_license) == 2
+        assert isinstance(p.trove_license[0], ObjectId), p.trove_license[0]
+        # these are sensitive fields only admins should be able to set:
+        assert p.get_tool_data('allura', 'grouping_threshold') == 5
+        assert p.admins()[0].username == 'test-admin'
+
+    def test_add_project_automatic_shortname(self):
+        # no shortname given, and name "Test" would conflict with existing "test" project
+        project_data = {
+            "admin": "test-admin",
+            "name": "Test",
+        }
+        r = self.api_post('/rest/p/add_project',
+                          params=json.dumps(project_data),
+                          user='root',
+                          status=201)
+        assert r.json == {
+            'status': 'success',
+            'html_url': 'http://localhost/p/test1/',
+            'url': 'http://localhost/rest/p/test1/',
+        }
+
 
 class TestDoap(TestRestApiBase):
     validate_skip = True
@@ -421,22 +570,22 @@ class TestDoap(TestRestApiBase):
         project.short_description = 'A Short Description'
         ThreadLocalODMSession.flush_all()
         r = self.app.get('/rest/p/test?doap')
-        assert_equal(r.content_type, 'application/rdf+xml')
+        assert r.content_type == 'application/rdf+xml'
         p = r.xml.find(self.ns + 'Project')
-        assert_equal(p.attrib[self.rdf + 'about'], 'http://localhost/rest/p/test?doap#')
-        assert_equal(p.find(self.ns + 'name').text, 'test')
-        assert_equal(p.find(self.dc + 'title').text, 'Test Project')
-        assert_equal(p.find(self.ns_sf + 'private').text, '0')
-        assert_equal(p.find(self.ns + 'shortdesc').text, 'A Summary')
-        assert_equal(p.find(self.ns + 'description').text, 'A Short Description')
-        assert_equal(p.find(self.ns + 'created').text, project._id.generation_time.strftime('%Y-%m-%d'))
+        assert p.attrib[self.rdf + 'about'] == 'http://localhost/rest/p/test?doap#'
+        assert p.find(self.ns + 'name').text == 'test'
+        assert p.find(self.dc + 'title').text == 'Test Project'
+        assert p.find(self.ns_sf + 'private').text == '0'
+        assert p.find(self.ns + 'shortdesc').text == 'A Summary'
+        assert p.find(self.ns + 'description').text == 'A Short Description'
+        assert p.find(self.ns + 'created').text == project._id.generation_time.strftime('%Y-%m-%d')
 
         maintainers = p.findall(self.ns + 'maintainer')
-        assert_equal(len(maintainers), 1)
+        assert len(maintainers) == 1
         user = maintainers[0].find(self.foaf + 'Person')
-        assert_equal(user.find(self.foaf + 'name').text, 'Test Admin')
-        assert_equal(user.find(self.foaf + 'nick').text, 'test-admin')
-        assert_equal(user.find(self.foaf + 'homepage').items()[0][1],
+        assert user.find(self.foaf + 'name').text == 'Test Admin'
+        assert user.find(self.foaf + 'nick').text == 'test-admin'
+        assert (list(user.find(self.foaf + 'homepage').items())[0][1] ==
                      'http://localhost/u/test-admin/')
 
     @td.with_tool('test', 'Tickets', 'bugs')
@@ -455,10 +604,10 @@ class TestDoap(TestRestApiBase):
         p = r.xml.find(self.ns + 'Project')
         tools = p.findall(self.ns_sf + 'feature')
         tools = [(t.find(self.ns_sf + 'Feature').find(self.ns + 'name').text,
-                  t.find(self.ns_sf + 'Feature').find(self.foaf + 'page').items()[0][1])
+                  list(t.find(self.ns_sf + 'Feature').find(self.foaf + 'page').items())[0][1])
                  for t in tools]
-        assert_in(('Tickets', 'http://localhost/p/test/bugs/'), tools)
-        assert_in(('Tickets', 'http://localhost/p/test/private-bugs/'), tools)
+        assert ('Tickets', 'http://localhost/p/test/bugs/') in tools
+        assert ('Tickets', 'http://localhost/p/test/private-bugs/') in tools
 
         # anonymous sees only non-private tool
         r = self.app.get('/rest/p/test?doap',
@@ -466,27 +615,27 @@ class TestDoap(TestRestApiBase):
         p = r.xml.find(self.ns + 'Project')
         tools = p.findall(self.ns_sf + 'feature')
         tools = [(t.find(self.ns_sf + 'Feature').find(self.ns + 'name').text,
-                  t.find(self.ns_sf + 'Feature').find(self.foaf + 'page').items()[0][1])
+                  list(t.find(self.ns_sf + 'Feature').find(self.foaf + 'page').items())[0][1])
                  for t in tools]
-        assert_in(('Tickets', 'http://localhost/p/test/bugs/'), tools)
-        assert_not_in(('Tickets', 'http://localhost/p/test/private-bugs/'), tools)
+        assert ('Tickets', 'http://localhost/p/test/bugs/') in tools
+        assert ('Tickets', 'http://localhost/p/test/private-bugs/') not in tools
 
 
 class TestUserProfile(TestRestApiBase):
     @td.with_user_project('test-admin')
     def test_profile_data(self):
         r = self.app.get('/rest/u/test-admin/profile/')
-        assert_equal(r.content_type, 'application/json')
+        assert r.content_type == 'application/json'
         json = r.json
-        assert_equal(json['username'], 'test-admin')
-        assert_equal(json['name'], 'Test Admin')
-        assert_in('availability', json)
-        assert_in('joined', json)
-        assert_in('localization', json)
-        assert_in('projects', json)
-        assert_in('sex', json)
-        assert_in('skills', json)
-        assert_in('skypeaccount', json)
-        assert_in('socialnetworks', json)
-        assert_in('telnumbers', json)
-        assert_in('webpages', json)
+        assert json['username'] == 'test-admin'
+        assert json['name'] == 'Test Admin'
+        assert 'availability' in json
+        assert 'joined' in json
+        assert 'localization' in json
+        assert 'projects' in json
+        assert 'sex' in json
+        assert 'skills' in json
+        assert 'skypeaccount' in json
+        assert 'socialnetworks' in json
+        assert 'telnumbers' in json
+        assert 'webpages' in json
