@@ -1,4 +1,3 @@
-# coding: utf-8
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -19,11 +18,11 @@
 import json
 import datetime as dt
 import bson
+import pytest
 
 from mock import patch, MagicMock
-from nose.tools import assert_equal, assert_in, assert_not_in
-from ming.odm import ThreadLocalORMSession
-from pylons import tmpl_context as c
+from ming.odm import ThreadLocalODMSession
+from tg import tmpl_context as c
 from tg import config
 from bson import ObjectId
 
@@ -55,7 +54,7 @@ class TestSiteAdmin(TestController):
         r = self.app.get('/nf/admin/stats/', extra_environ=dict(
             username='root'))
         assert 'Forge Site Admin' in r.html.find(
-            'h2', {'class': 'dark title'}).contents[0]
+            'h2', {'class': 'dark title'}).find('span').contents[0]
         stats_table = r.html.find('table')
         cells = stats_table.findAll('td')
         assert cells[0].contents[0] == 'Adobe', cells[0].contents[0]
@@ -90,23 +89,23 @@ class TestSiteAdmin(TestController):
         count = len(r.html.find('table').findAll('tr'))
         p = M.Project.query.get(shortname='test')
         p.deleted = True
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         r = self.app.get('/nf/admin/new_projects', extra_environ=dict(
             username='root'))
-        assert_equal(len(r.html.find('table').findAll('tr')), count - 1)
+        assert len(r.html.find('table').findAll('tr')) == count - 1
 
     def test_new_projects_daterange_filtering(self):
         r = self.app.get('/nf/admin/new_projects', extra_environ=dict(
             username='root'))
         count = len(r.html.find('table').findAll('tr'))
-        assert_equal(count, 7)
+        assert count == 7
 
         filtr = r.forms[0]
         filtr['start-dt'] = '2000/01/01 10:10:10'
         filtr['end-dt'] = '2000/01/01 09:09:09'
         r = filtr.submit()
         count = len(r.html.find('table').findAll('tr'))
-        assert_equal(count, 1)  # only row with headers - no results
+        assert count == 1  # only row with headers - no results
 
     def test_reclone_repo_access(self):
         r = self.app.get('/nf/admin/reclone_repo', extra_environ=dict(
@@ -131,12 +130,25 @@ class TestSiteAdmin(TestController):
 
     def test_task_view(self):
         import math
-        task = M.MonQTask.post(math.ceil, (12.5,))
+        task = M.MonQTask.post(math.isclose, (99, 100), {'abs_tol': 1.1})
         url = '/nf/admin/task_manager/view/%s' % task._id
         r = self.app.get(
             url, extra_environ=dict(username='*anonymous'), status=302)
         r = self.app.get(url)
-        assert 'math.ceil' in r, r
+        assert 'math.isclose' in r, r
+        assert '<td>99</td>' in r, r
+        assert '<td>100</td>' in r, r
+        assert '<th class="second-column-headers side-header">abs_tol</th>' in r, r
+        assert '<td>1.1</td>' in r, r
+        assert 'ready' in r, r
+
+        # test resubmit too
+        M.MonQTask.run_ready()
+        r = self.app.get(url)
+        assert 'complete' in r, r
+        r = r.forms['resubmit-task-form'].submit()
+        r = r.follow()
+        assert 'ready' in r, r
 
     def test_task_new(self):
         r = self.app.get('/nf/admin/task_manager/new')
@@ -157,7 +169,7 @@ class TestSiteAdmin(TestController):
             user='root',
             path='/p/test/admin',
         ), status=302)
-        task = M.MonQTask.query.find({}).sort('_id', -1).next()
+        task = next(M.MonQTask.query.find({}).sort('_id', -1))
         assert str(task._id) in r.location
         assert task.context['project_id'] == project._id
         assert task.context['app_config_id'] == app.config._id
@@ -168,7 +180,10 @@ class TestSiteAdmin(TestController):
     def test_task_doc(self):
         r = self.app.get('/nf/admin/task_manager/task_doc', params=dict(
             task_name='allura.tests.functional.test_site_admin.test_task'))
-        assert json.loads(r.body)['doc'] == 'test_task doc string'
+        assert json.loads(r.text)['doc'] == 'test_task doc string'
+
+
+class TestSiteAdminNotifications(TestController):
 
     def test_site_notifications_access(self):
         self.app.get('/nf/admin/site_notifications', extra_environ=dict(
@@ -184,7 +199,7 @@ class TestSiteAdmin(TestController):
                                         user_role='test2',
                                         page_regex='test3',
                                         page_tool_type='test4')
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
         assert M.notification.SiteNotification.query.find().count() == 1
 
         r = self.app.get('/nf/admin/site_notifications/', extra_environ=dict(
@@ -234,7 +249,7 @@ class TestSiteAdmin(TestController):
             user_role=user_role,
             page_regex=page_regex,
             page_tool_type=page_tool_type))
-        note = M.notification.SiteNotification.query.find().sort('_id', -1).next()
+        note = next(M.notification.SiteNotification.query.find().sort('_id', -1))
 
         assert '/nf/admin/site_notifications' in r.location
 
@@ -254,8 +269,8 @@ class TestSiteAdmin(TestController):
                                                user_role='test2',
                                                page_regex='test3',
                                                page_tool_type='test4')
-        ThreadLocalORMSession().flush_all()
-        r = self.app.get('/nf/admin/site_notifications/{}/edit'.format(note._id))
+        ThreadLocalODMSession().flush_all()
+        r = self.app.get(f'/nf/admin/site_notifications/{note._id}/edit')
 
         assert r
         assert 'checked' in r.form['active'].attrs
@@ -284,20 +299,20 @@ class TestSiteAdmin(TestController):
         note = M.notification.SiteNotification(active=False,
                                                impressions=0,
                                                content='test')
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
 
         count = M.notification.SiteNotification.query.find().count()
 
-        r = self.app.post('/nf/admin/site_notifications/{}/update'.format(note._id), params=dict(
+        r = self.app.post(f'/nf/admin/site_notifications/{note._id}/update', params=dict(
             active=active,
             impressions=impressions,
             content=content,
             user_role=user_role,
             page_regex=page_regex,
             page_tool_type=page_tool_type))
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
 
-        note = M.notification.SiteNotification.query.find().sort('_id', -1).next()
+        note = next(M.notification.SiteNotification.query.find().sort('_id', -1))
 
         assert '/nf/admin/site_notifications' in r.location
         assert M.notification.SiteNotification.query.find().count() == count
@@ -312,11 +327,11 @@ class TestSiteAdmin(TestController):
         note = M.notification.SiteNotification(active=False,
                                                impressions=0,
                                                content='test')
-        ThreadLocalORMSession().flush_all()
+        ThreadLocalODMSession().flush_all()
 
         count = M.notification.SiteNotification.query.find().count()
 
-        self.app.post('/nf/admin/site_notifications/{}/delete'.format(note._id))
+        self.app.post(f'/nf/admin/site_notifications/{note._id}/delete')
         assert M.notification.SiteNotification.query.find().count() == count -1
         assert M.notification.SiteNotification.query.get(_id=bson.ObjectId(note._id)) is None
 
@@ -341,8 +356,8 @@ class TestProjectsSearch(TestController):
         'id': 'allura/model/project/Project#53ccf6e8100d2b0741746e9f',
     }])
 
-    def setUp(self):
-        super(TestProjectsSearch, self).setUp()
+    def setup_method(self, method):
+        super().setup_method(method)
         # Create project that matches TEST_HIT id
         _id = ObjectId('53ccf6e8100d2b0741746e9f')
         p = M.Project.query.get(_id=_id)
@@ -352,16 +367,16 @@ class TestProjectsSearch(TestController):
                 neighborhood_id=M.Neighborhood.query.get(url_prefix='/u/')._id,
                 shortname='test-project',
             )
-            ThreadLocalORMSession().flush_all()
+            ThreadLocalODMSession().flush_all()
 
     @patch('allura.controllers.site_admin.search')
     def test_default_fields(self, search):
         search.site_admin_search.return_value = self.TEST_HIT
         r = self.app.get('/nf/admin/search_projects?q=fake&f=shortname')
         options = [o['value'] for o in r.html.findAll('option')]
-        assert_equal(options, ['shortname', 'name', '__custom__'])
+        assert options == ['shortname', 'name', '__custom__']
         ths = [th.text for th in r.html.findAll('th')]
-        assert_equal(ths, ['Short name', 'Full name', 'Registered', 'Deleted?', 'Details'])
+        assert ths == ['Short name', 'Full name', 'Registered', 'Deleted?', 'Details']
 
     @patch('allura.controllers.site_admin.search')
     def test_additional_fields(self, search):
@@ -370,9 +385,9 @@ class TestProjectsSearch(TestController):
                                       'search.project.additional_display_fields': 'url'}):
             r = self.app.get('/nf/admin/search_projects?q=fake&f=shortname')
         options = [o['value'] for o in r.html.findAll('option')]
-        assert_equal(options, ['shortname', 'name', 'private', 'url', '__custom__'])
+        assert options == ['shortname', 'name', 'private', 'url', '__custom__']
         ths = [th.text for th in r.html.findAll('th')]
-        assert_equal(ths, ['Short name', 'Full name', 'Registered', 'Deleted?', 'url', 'Details'])
+        assert ths == ['Short name', 'Full name', 'Registered', 'Deleted?', 'url', 'Details']
 
 
 class TestUsersSearch(TestController):
@@ -398,24 +413,24 @@ class TestUsersSearch(TestController):
         'user_registration_date_dt': '2014-09-09T13:17:38Z',
         'username_s': 'darth'}])
 
-    def setUp(self):
-        super(TestUsersSearch, self).setUp()
+    def setup_method(self, method):
+        super().setup_method(method)
         # Create user that matches TEST_HIT id
         _id = ObjectId('540efdf2100d2b1483155d39')
         u = M.User.query.get(_id=_id)
         if not u:
             M.User(_id=_id, username='darth')
-            ThreadLocalORMSession().flush_all()
+            ThreadLocalODMSession().flush_all()
 
     @patch('allura.controllers.site_admin.search.site_admin_search')
     def test_default_fields(self, site_admin_search):
         site_admin_search.return_value = self.TEST_HIT
         r = self.app.get('/nf/admin/search_users?q=fake&f=username')
         options = [o['value'] for o in r.html.findAll('option')]
-        assert_equal(options, ['username', 'display_name', '__custom__'])
+        assert options == ['username', 'display_name', '__custom__']
         ths = [th.text for th in r.html.findAll('th')]
-        assert_equal(ths, ['Username', 'Display name', 'Email', 'Registered',
-                           'Status', 'Details'])
+        assert ths == ['Username', 'Display name', 'Email', 'Registered',
+                           'Status', 'Details']
 
     @patch('allura.controllers.site_admin.search.site_admin_search')
     def test_additional_fields(self, site_admin_search):
@@ -424,16 +439,16 @@ class TestUsersSearch(TestController):
                                       'search.user.additional_display_fields': 'url'}):
             r = self.app.get('/nf/admin/search_users?q=fake&f=username')
         options = [o['value'] for o in r.html.findAll('option')]
-        assert_equal(options, ['username', 'display_name', 'email_addresses', 'url', '__custom__'])
+        assert options == ['username', 'display_name', 'email_addresses', 'url', '__custom__']
         ths = [th.text for th in r.html.findAll('th')]
-        assert_equal(ths, ['Username', 'Display name', 'Email', 'Registered',
-                           'Status', 'url', 'Details'])
+        assert ths == ['Username', 'Display name', 'Email', 'Registered',
+                           'Status', 'url', 'Details']
 
 
 class TestUserDetails(TestController):
 
     def test_404(self):
-        self.app.get('/nf/admin/user/does-not-exist/', status=404)
+        self.app.get('/nf/admin/user/does-not-exist', status=404)
 
     def test_general_info(self):
         user = M.User.by_username('test-admin')
@@ -444,24 +459,24 @@ class TestUserDetails(TestController):
                             'session_date': dt.datetime(2014, 9, 12, 6, 6, 6),
                             'session_ua': 'browser of the future 1.1',
                             'session_ip': '7.7.7.7'}
-        r = self.app.get('/nf/admin/user/test-admin/')
+        r = self.app.get('/nf/admin/user/test-admin')
         # general info
-        assert_in('Username: test-admin', r)
-        assert_in('Full name: Test Admin', r)
-        assert_in('Registered: 2014-09-01 09:09:09', r)
+        assert 'Username: test-admin' in r
+        assert 'Full name: Test Admin' in r
+        assert 'Registered: 2014-09-01 09:09:09' in r
         # session info
-        assert_in('Date: 2014-09-02 06:06:06', r)
-        assert_in('IP: 8.8.8.8', r)
-        assert_in('UA: browser of the future 1.0', r)
-        assert_in('Date: 2014-09-12', r)
-        assert_in('IP: 7.7.7.7', r)
-        assert_in('UA: browser of the future 1.1', r)
+        assert 'Date: 2014-09-02 06:06:06' in r
+        assert 'IP: 8.8.8.8' in r
+        assert 'UA: browser of the future 1.0' in r
+        assert 'Date: 2014-09-12' in r
+        assert 'IP: 7.7.7.7' in r
+        assert 'UA: browser of the future 1.1' in r
         # list of projects
         projects = r.html.findAll('fieldset')[-1]
         projects = [e.getText() for e in projects.findAll('li')]
-        assert_in('Test 2&ndash;Admin', projects)
-        assert_in('Test Project&ndash;Admin', projects)
-        assert_in('Adobe project 1&ndash;Admin', projects)
+        assert 'Test 2\n\u2013\nAdmin\n' in projects
+        assert 'Test Project\n\u2013\nAdmin\n' in projects
+        assert 'Adobe project 1\n\u2013\nAdmin\n' in projects
 
     @patch('allura.model.auth.request')
     @patch('allura.lib.helpers.request')
@@ -471,162 +486,162 @@ class TestUserDetails(TestController):
         h.auditlog_user('test activity user 1')
         h.auditlog_user('test activity user 2', user=M.User.by_username('test-user-2'))
         r = self.app.get('/nf/admin/user/test-admin')
-        assert_in('Add comment', r)
-        assert_not_in('test activity', r)
+        assert 'Add comment' in r
+        assert 'test activity' not in r
         r = self.app.get('/nf/admin/user/test-user-1')
-        assert_in('test activity user 1', r)
-        assert_not_in('test activity user 2', r)
+        assert 'test activity user 1' in r
+        assert 'test activity user 2' not in r
         r = self.app.get('/nf/admin/user/test-user-2')
-        assert_not_in('test activity user 1', r)
-        assert_in('test activity user 2', r)
+        assert 'test activity user 1' not in r
+        assert 'test activity user 2' in r
 
     def test_add_audit_trail_entry_access(self):
         self.app.get('/nf/admin/user/add_audit_log_entry', status=404)  # GET is not allowed
         r = self.app.post('/nf/admin/user/add_audit_log_entry',
                           extra_environ={'username': '*anonymous'},
                           status=302)
-        assert_equal(r.location, 'http://localhost/auth/')
+        assert r.location == 'http://localhost/auth/'
 
     def test_add_comment(self):
         r = self.app.get('/nf/admin/user/test-user')
-        assert_not_in(u'Comment by test-admin: I was hêre!', r)
-        form = r.forms[4]
-        assert_equal(form['username'].value, 'test-user')
-        form['comment'] = u'I was hêre!'
+        assert 'Comment by test-admin: I was hêre!' not in r
+        form = [f for f in r.forms.values() if f.action.endswith('add_audit_trail_entry')][0]
+        assert form['username'].value == 'test-user'
+        form['comment'] = 'I was hêre!'
         r = form.submit()
-        assert_in(u'Comment added', self.webflash(r))
+        assert 'Comment added' in self.webflash(r)
         r = self.app.get('/nf/admin/user/test-user')
-        assert_in(u'Comment by test-admin: I was hêre!', r)
+        assert 'Comment by test-admin: I was hêre!' in r
 
     def test_disable_user(self):
         # user was not pending
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is False
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'enable')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'enable'
         form['status'].value = 'disable'
         with td.audits('Account disabled', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'User disabled', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, True)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'User disabled' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is True
+        assert M.User.by_username('test-user-3').pending is False
 
         # user was pending
         user = M.User.by_username('test-user-3')
         user.disabled = False
         user.pending = True
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is True
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'pending')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'pending'
         form['status'].value = 'disable'
         with td.audits('Account disabled', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'User disabled', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, True)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'User disabled' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is True
+        assert M.User.by_username('test-user-3').pending is True
 
     def test_enable_user(self):
         # user was not pending
         user = M.User.by_username('test-user-3')
         user.disabled = True
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, True)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is True
+        assert M.User.by_username('test-user-3').pending is False
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'disable')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'disable'
         form['status'].value = 'enable'
         with td.audits('Account enabled', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'User enabled', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'User enabled' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is False
 
         # user was pending
         user = M.User.by_username('test-user-3')
         user.disabled = False
         user.pending = True
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is True
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'pending')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'pending'
         form['status'].value = 'enable'
-        with td.audits('Account enabled', user=True):
+        with td.audits('Account activated', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'User enabled', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'User enabled' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is False
 
         # user was pending and disabled
         user = M.User.by_username('test-user-3')
         user.disabled = True
         user.pending = True
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, True)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is True
+        assert M.User.by_username('test-user-3').pending is True
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'disable')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'disable'
         form['status'].value = 'enable'
         with td.audits('Account enabled', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'User enabled', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'User enabled' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is False
 
     def test_set_pending(self):
         # user was disabled
         user = M.User.by_username('test-user-3')
         user.disabled = True
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, True)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is True
+        assert M.User.by_username('test-user-3').pending is False
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'disable')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'disable'
         form['status'].value = 'pending'
         with td.audits('Account changed to pending', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'Set user status to pending', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'Set user status to pending' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is True
 
         # user was enabled
         user = M.User.by_username('test-user-3')
         user.pending = False
         user.disabled = False
-        ThreadLocalORMSession.flush_all()
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, False)
+        ThreadLocalODMSession.flush_all()
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is False
         r = self.app.get('/nf/admin/user/test-user-3')
         form = r.forms[0]
-        assert_equal(form['username'].value, 'test-user-3')
-        assert_equal(form['status'].value, 'enable')
+        assert form['username'].value == 'test-user-3'
+        assert form['status'].value == 'enable'
         form['status'].value = 'pending'
         with td.audits('Account changed to pending', user=True):
             r = form.submit()
-            assert_equal(M.AuditLog.query.find().count(), 1)
-        assert_in(u'Set user status to pending', self.webflash(r))
-        assert_equal(M.User.by_username('test-user-3').disabled, False)
-        assert_equal(M.User.by_username('test-user-3').pending, True)
+            assert M.AuditLog.query.find().count() == 1
+        assert 'Set user status to pending' in self.webflash(r)
+        assert M.User.by_username('test-user-3').disabled is False
+        assert M.User.by_username('test-user-3').pending is True
 
     def test_emails(self):
         # add test@example.com
@@ -638,11 +653,11 @@ class TestUserDetails(TestController):
                 'primary_addr': 'test@example.com'},
                 extra_environ=dict(username='test-admin'))
         r = self.app.get('/nf/admin/user/test-user')
-        assert_in('test@example.com', r)
+        assert 'test@example.com' in r
         em = M.EmailAddress.get(email='test@example.com')
-        assert_equal(em.confirmed, True)
+        assert em.confirmed is True
         user = M.User.query.get(username='test-user')
-        assert_equal(user.get_pref('email_address'), 'test@example.com')
+        assert user.get_pref('email_address') == 'test@example.com'
 
         # add test2@example.com
         with td.audits('New email address: test2@example.com', user=True):
@@ -653,11 +668,11 @@ class TestUserDetails(TestController):
                 'primary_addr': 'test@example.com'},
                 extra_environ=dict(username='test-admin'))
         r = self.app.get('/nf/admin/user/test-user')
-        assert_in('test2@example.com', r)
+        assert 'test2@example.com' in r
         em = M.EmailAddress.get(email='test2@example.com')
-        assert_equal(em.confirmed, True)
+        assert em.confirmed is True
         user = M.User.query.get(username='test-user')
-        assert_equal(user.get_pref('email_address'), 'test@example.com')
+        assert user.get_pref('email_address') == 'test@example.com'
 
         # change primary: test -> test2
         with td.audits('Primary email changed: test@example.com => test2@example.com', user=True):
@@ -668,7 +683,7 @@ class TestUserDetails(TestController):
                 extra_environ=dict(username='test-admin'))
         r = self.app.get('/nf/admin/user/test-user')
         user = M.User.query.get(username='test-user')
-        assert_equal(user.get_pref('email_address'), 'test2@example.com')
+        assert user.get_pref('email_address') == 'test2@example.com'
 
         # remove test2@example.com
         with td.audits('Email address deleted: test2@example.com', user=True):
@@ -684,14 +699,14 @@ class TestUserDetails(TestController):
         r = self.app.get('/nf/admin/user/test-user')
         user = M.User.query.get(username='test-user')
         # test@example.com set as primary since test2@example.com is deleted
-        assert_equal(user.get_pref('email_address'), 'test-user@allura.local')
+        assert user.get_pref('email_address') == 'test-user@allura.local'
 
     @patch.object(LocalAuthenticationProvider, 'set_password')
     def test_set_random_password(self, set_password):
         with td.audits('Set random password', user=True, actor='test-admin'):
             r = self.app.post('/nf/admin/user/set_random_password', params={'username': 'test-user'})
-        assert_in('Password is set', self.webflash(r))
-        assert_equal(set_password.call_count, 1)
+        assert 'Password is set' in self.webflash(r)
+        assert set_password.call_count == 1
 
     @patch('allura.tasks.mail_tasks.sendsimplemail')
     @patch('allura.lib.helpers.gen_message_id')
@@ -699,24 +714,31 @@ class TestUserDetails(TestController):
         user = M.User.by_username('test-user')
         user.set_pref('email_address', 'test-user@example.org')
         M.EmailAddress(email='test-user@example.org', confirmed=True, claimed_by_user_id=user._id)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         with td.audits('Password recovery link sent to: test-user@example.org', user=True):
             r = self.app.post('/nf/admin/user/send_password_reset_link', params={'username': 'test-user'})
         hash = user.get_tool_data('AuthPasswordReset', 'hash')
         text = '''Your username is test-user
 
-To reset your password on %s, please visit the following URL:
+To update your password on %s, please visit the following URL:
 
 %s/auth/forgotten_password/%s''' % (config['site_name'], config['base_url'], hash)
         sendmail.post.assert_called_once_with(
             sender='noreply@localhost',
             toaddr='test-user@example.org',
-            fromaddr=u'"{}" <{}>'.format(config['site_name'], config['forgemail.return_path']),
+            fromaddr='"{}" <{}>'.format(config['site_name'], config['forgemail.return_path']),
             reply_to=config['forgemail.return_path'],
             subject='Allura Password recovery',
             message_id=gen_message_id(),
             text=text)
 
+    def test_make_password_reset_url(self):
+        with td.audits('Generated new password reset URL and shown to admin user', user=True):
+            r = self.app.post('/nf/admin/user/make_password_reset_url', params={'username': 'test-user'})
+        user = M.User.by_username('test-user')
+        hash = user.get_tool_data('AuthPasswordReset', 'hash')
+        assert hash in r.text
+
 
 class TestDeleteProjects(TestController):
 
@@ -728,14 +750,14 @@ class TestDeleteProjects(TestController):
 
     def test_projects_populated_from_get_params(self):
         r = self.app.get('/nf/admin/delete_projects/')
-        assert_equal(self.delete_form(r)['projects'].value, u'')
+        assert self.delete_form(r)['projects'].value == ''
         link = '/nf/admin/delete_projects/?projects=/p/test/++++%23+comment%0A/adobe/adobe-1/%0A/p/test2/'
         link += '&reason=The%0AReason&disable_users=True'
         r = self.app.get(link)
         form = self.delete_form(r)
-        assert_equal(form['projects'].value, u'/p/test/    # comment\n/adobe/adobe-1/\n/p/test2/')
-        assert_equal(form['reason'].value, u'The\nReason')
-        assert_equal(form['disable_users'].value, u'on')
+        assert form['projects'].value == '/p/test/    # comment\n/adobe/adobe-1/\n/p/test2/'
+        assert form['reason'].value == 'The\nReason'
+        assert form['disable_users'].value == 'on'
 
     def test_confirm_step_values(self):
         r = self.app.get('/nf/admin/delete_projects/')
@@ -746,16 +768,16 @@ class TestDeleteProjects(TestController):
         r = form.submit()
         confirm_form = self.confirm_form(r)
         for f in ['reason', 'disable_users']:
-            assert_equal(confirm_form[f].value, form[f].value)
-        assert_equal(confirm_form['projects'].value, 'p/test    # OK: /p/test/\ndne/dne    # Neighborhood not found')
+            assert confirm_form[f].value == form[f].value
+        assert confirm_form['projects'].value == 'p/test    # OK: /p/test/\ndne/dne    # Neighborhood not found'
 
         confirm_data = r.html.find('table').findAll('td')
-        assert_equal(len(confirm_data), 4)  # 2 projects == 2 rows (2 columns each)
-        assert_equal(confirm_data[0].getText(), 'p/test')
-        assert_equal(confirm_data[1].find('a').get('href'), '/p/test/')
-        assert_equal(confirm_data[1].getText(), '/p/test/')
-        assert_equal(confirm_data[2].getText(), 'dne/dne')
-        assert_equal(confirm_data[3].getText(), 'Neighborhood not found')
+        assert len(confirm_data) == 4  # 2 projects == 2 rows (2 columns each)
+        assert confirm_data[0].getText() == 'p/test'
+        assert confirm_data[1].find('a').get('href') == '/p/test/'
+        assert confirm_data[1].getText().strip() == '/p/test/'
+        assert confirm_data[2].getText().strip() == 'dne/dne'
+        assert confirm_data[3].getText().strip() == 'Neighborhood not found'
 
     def test_confirm_step_edit_link(self):
         r = self.app.get('/nf/admin/delete_projects/')
@@ -795,7 +817,7 @@ class TestDeleteProjects(TestController):
 
     @patch('allura.controllers.site_admin.DeleteProjects', autospec=True)
     def test_admins_and_devs_are_disabled(self, dp):
-        data = {'projects': '/p/test\np/test2', 'disable_users': True}
+        data = {'projects': '/p/test\np/test2', 'disable_users': 'True'}
         self.app.post('/nf/admin/delete_projects/really_delete', data)
         dp.post.assert_called_once_with('--disable-users p/test p/test2')
 
