@@ -19,9 +19,8 @@ import unittest
 from datetime import timedelta
 import collections
 
-from pylons import tmpl_context as c, app_globals as g
-from nose.tools import assert_equal, assert_in
-from ming.orm import ThreadLocalORMSession
+from tg import tmpl_context as c, app_globals as g
+from ming.odm import ThreadLocalODMSession
 import mock
 import bson
 
@@ -35,7 +34,7 @@ from forgewiki import model as WM
 
 class TestNotification(unittest.TestCase):
 
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         self.setup_with_tools()
 
@@ -44,14 +43,14 @@ class TestNotification(unittest.TestCase):
         setup_global_objects()
         _clear_subscriptions()
         _clear_notifications()
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         M.notification.MAILBOX_QUIESCENT = None  # disable message combining
 
     def test_subscribe_unsubscribe(self):
         M.Mailbox.subscribe(type='direct')
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         subscriptions = M.Mailbox.query.find(dict(
             project_id=c.project._id,
             app_config_id=c.app.config._id,
@@ -60,8 +59,8 @@ class TestNotification(unittest.TestCase):
         assert subscriptions[0].type == 'direct'
         assert M.Mailbox.query.find().count() == 1
         M.Mailbox.unsubscribe()
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         subscriptions = M.Mailbox.query.find(dict(
             project_id=c.project._id,
             app_config_id=c.app.config._id,
@@ -105,8 +104,8 @@ class TestNotification(unittest.TestCase):
         wiki = c.project.app_instance('wiki')
         page = WM.Page.query.get(app_config_id=wiki.config._id)
         page.parent_security_context().acl = []
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         notification = M.Notification(
             _id='_id',
             ref=page.ref,
@@ -118,7 +117,7 @@ class TestNotification(unittest.TestCase):
         )
         notification.footer = lambda: ' footer'
         notification.send_direct(c.user._id)
-        assert_equal(sendmail.post.call_count, 0)
+        assert sendmail.post.call_count == 0
 
     @mock.patch('allura.tasks.mail_tasks.sendmail')
     def test_send_direct_wrong_project_context(self, sendmail):
@@ -133,7 +132,7 @@ class TestNotification(unittest.TestCase):
         """
         project1 = c.project
         project2 = M.Project.query.get(shortname='test2')
-        assert_equal(project1.shortname, 'test')
+        assert project1.shortname == 'test'
         c.user = M.User.query.get(username='test-user')
         wiki = project1.app_instance('wiki')
         page = WM.Page.query.get(app_config_id=wiki.config._id)
@@ -166,7 +165,7 @@ class TestNotification(unittest.TestCase):
 
 class TestPostNotifications(unittest.TestCase):
 
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         self.setup_with_tools()
 
@@ -176,25 +175,25 @@ class TestPostNotifications(unittest.TestCase):
         g.set_app('wiki')
         _clear_subscriptions()
         _clear_notifications()
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         self.pg = WM.Page.query.get(app_config_id=c.app.config._id)
         M.notification.MAILBOX_QUIESCENT = None  # disable message combining
         while M.MonQTask.run_ready('setup'):
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
 
     def test_post_notification(self):
         self._post_notification()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         M.MonQTask.list()
         t = M.MonQTask.get()
-        assert_equal(t.args[1], [self.pg.index_id()])
+        assert t.args[1] == [self.pg.index_id()]
 
     def test_post_user_notification(self):
         u = M.User.query.get(username='test-admin')
         M.Notification.post_user(u, self.pg, 'metadata')
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         flash_msgs = list(h.pop_user_notifications(u))
         assert len(flash_msgs) == 1, flash_msgs
         msg = flash_msgs[0]
@@ -207,7 +206,7 @@ class TestPostNotifications(unittest.TestCase):
         self._subscribe()
         self._post_notification()
         M.MonQTask.run_ready()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         assert M.Mailbox.query.find().count() == 1
         mbox = M.Mailbox.query.get()
         assert len(mbox.queue) == 1
@@ -218,36 +217,36 @@ class TestPostNotifications(unittest.TestCase):
         user2 = M.User.query.get(username='test-user-2')
         self._subscribe(user=user2)
         self._post_notification()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
 
-        assert_equal(M.Notification.query.get()
-                     ['from_address'], '"Test Admin" <test-admin@users.localhost>')
-        assert_equal(M.Mailbox.query.find().count(), 2)
+        assert (M.Notification.query.get()
+                     ['from_address'] == '"Test Admin" <test-admin@users.localhost>')
+        assert M.Mailbox.query.find().count() == 2
 
         # sends the notification out into "mailboxes", and from mailboxes into
         # email tasks
         M.MonQTask.run_ready()
         mboxes = M.Mailbox.query.find().all()
-        assert_equal(len(mboxes), 2)
-        assert_equal(len(mboxes[0].queue), 1)
+        assert len(mboxes) == 2
+        assert len(mboxes[0].queue) == 1
         assert not mboxes[0].queue_empty
-        assert_equal(len(mboxes[1].queue), 1)
+        assert len(mboxes[1].queue) == 1
         assert not mboxes[1].queue_empty
 
         email_tasks = M.MonQTask.query.find({'state': 'ready'}).all()
         # make sure both subscribers will get an email
-        assert_equal(len(email_tasks), 2)
+        assert len(email_tasks) == 2
 
         first_destinations = [e.kwargs['destinations'][0] for e in email_tasks]
-        assert_in(str(c.user._id), first_destinations)
-        assert_in(str(user2._id), first_destinations)
-        assert_equal(email_tasks[0].kwargs['fromaddr'],
+        assert str(c.user._id) in first_destinations
+        assert str(user2._id) in first_destinations
+        assert (email_tasks[0].kwargs['fromaddr'] ==
                      '"Test Admin" <test-admin@users.localhost>')
-        assert_equal(email_tasks[1].kwargs['fromaddr'],
+        assert (email_tasks[1].kwargs['fromaddr'] ==
                      '"Test Admin" <test-admin@users.localhost>')
-        assert_equal(email_tasks[0].kwargs['sender'],
+        assert (email_tasks[0].kwargs['sender'] ==
                      'wiki@test.p.in.localhost')
-        assert_equal(email_tasks[1].kwargs['sender'],
+        assert (email_tasks[1].kwargs['sender'] ==
                      'wiki@test.p.in.localhost')
         assert email_tasks[0].kwargs['text'].startswith(
             'Home modified by Test Admin')
@@ -271,14 +270,14 @@ class TestPostNotifications(unittest.TestCase):
         try:
             # this will create a notification task
             self._post_notification()
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
             # running the notification task will create a mail task if the
             # permission check passes...
             M.MonQTask.run_ready()
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
             # ...but in this case it doesn't create a mail task since we
             # forced the perm check to fail
-            assert M.MonQTask.get() == None
+            assert M.MonQTask.get() is None
         finally:
             security.has_access = orig
 
@@ -295,8 +294,8 @@ class TestPostNotifications(unittest.TestCase):
 
     def _subscribe(self, **kw):
         self.pg.subscribe(type='direct', **kw)
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
 
     def _post_notification(self):
         return M.Notification.post(self.pg, 'metadata')
@@ -304,7 +303,7 @@ class TestPostNotifications(unittest.TestCase):
 
 class TestSubscriptionTypes(unittest.TestCase):
 
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         self.setup_with_tools()
 
@@ -314,8 +313,8 @@ class TestSubscriptionTypes(unittest.TestCase):
         g.set_app('wiki')
         _clear_subscriptions()
         _clear_notifications()
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         self.pg = WM.Page.query.get(app_config_id=c.app.config._id)
         M.notification.MAILBOX_QUIESCENT = None  # disable message combining
 
@@ -323,8 +322,8 @@ class TestSubscriptionTypes(unittest.TestCase):
         self._subscribe()
         self._post_notification(text='A')
         self._post_notification(text='B')
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         M.Mailbox.fire_ready()
 
     def test_digest_sub(self):
@@ -342,10 +341,10 @@ class TestSubscriptionTypes(unittest.TestCase):
     def test_message(self):
         self._test_message()
 
-        self.setUp()
+        self.setup_method(None)
         self._test_message()
 
-        self.setUp()
+        self.setup_method(None)
         M.notification.MAILBOX_QUIESCENT = timedelta(minutes=1)
         # will raise "assert msg is not None" since the new message is not 1
         # min old:
@@ -356,10 +355,10 @@ class TestSubscriptionTypes(unittest.TestCase):
         thd = M.Thread.query.get(ref_id=self.pg.index_id())
         thd.post('This is a very cool message')
         M.MonQTask.run_ready()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         M.Mailbox.fire_ready()
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         msg = M.MonQTask.query.get(
             task_name='allura.tasks.mail_tasks.sendmail',
             state='ready')
@@ -370,13 +369,13 @@ class TestSubscriptionTypes(unittest.TestCase):
 
     def _clear_subscriptions(self):
         M.Mailbox.query.remove({})
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
 
     def _subscribe(self, type='direct', topic=None):
         self.pg.subscribe(type=type, topic=topic)
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
 
     def _post_notification(self, text=None):
         return M.Notification.post(self.pg, 'metadata', text=text)
@@ -388,13 +387,13 @@ class TestSubscriptionTypes(unittest.TestCase):
 
             def __init__(self, factory=list, *a, **kw):
                 self._factory = factory
-                super(OrderedDefaultDict, self).__init__(*a, **kw)
+                super().__init__(*a, **kw)
 
             def __getitem__(self, key):
                 if key not in self:
                     value = self[key] = self._factory()
                 else:
-                    value = super(OrderedDefaultDict, self).__getitem__(key)
+                    value = super().__getitem__(key)
                 return value
 
         notifications = mocked_notification.query.find.return_value.all.return_value = [
@@ -436,19 +435,19 @@ class TestSubscriptionTypes(unittest.TestCase):
         notification = M.Notification()
         notification.ref_id = thd.index_id()
         user.disabled = True
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         notification.send_direct(user._id)
         count = M.MonQTask.query.find(dict(
             task_name='allura.tasks.mail_tasks.sendmail',
             state='ready')).count()
-        assert_equal(count, 0)
+        assert count == 0
         user.disabled = False
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         notification.send_direct(user._id)
         count = M.MonQTask.query.find(dict(
             task_name='allura.tasks.mail_tasks.sendmail',
             state='ready')).count()
-        assert_equal(count, 1)
+        assert count == 1
 
     @mock.patch('allura.model.notification.Notification.ref')
     def test_send_digest_disabled_user(self, ref):
@@ -458,25 +457,25 @@ class TestSubscriptionTypes(unittest.TestCase):
         ref.artifact = thd
         user = M.User.by_username('test-admin')
         user.disabled = True
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         M.Notification.send_digest(
             user._id, 'test@mail.com', 'subject', [notification])
         count = M.MonQTask.query.find(dict(
             task_name='allura.tasks.mail_tasks.sendmail',
             state='ready')).count()
-        assert_equal(count, 0)
+        assert count == 0
         user.disabled = False
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         M.Notification.send_digest(
             user._id, 'test@mail.com', 'subject', [notification])
         count = M.MonQTask.query.find(dict(
             task_name='allura.tasks.mail_tasks.sendmail',
             state='ready')).count()
-        assert_equal(count, 1)
+        assert count == 1
 
 
 class TestSiteNotification(unittest.TestCase):
-    def setUp(self):
+    def setup_method(self, method):
         self.note = M.SiteNotification(
             active=True,
             impressions=0,
@@ -491,7 +490,7 @@ class TestSiteNotification(unittest.TestCase):
         assert isinstance(note_json, dict)
 
     def test_json_keys(self):
-        keys = self.note.__json__().keys()
+        keys = list(self.note.__json__().keys())
         assert 'active' in keys
         assert 'impressions' in keys
         assert 'content' in keys
@@ -501,14 +500,14 @@ class TestSiteNotification(unittest.TestCase):
 
     def test_json_values_if_missing(self):
         note_json = self.note.__json__()
-        assert note_json['page_regex'] is ''
-        assert note_json['page_tool_type'] is ''
-        assert note_json['user_role'] is ''
+        assert note_json['page_regex'] == ''
+        assert note_json['page_tool_type'] == ''
+        assert note_json['user_role'] == ''
 
 
 def _clear_subscriptions():
-        M.Mailbox.query.remove({})
+    M.Mailbox.query.remove({})
 
 
 def _clear_notifications():
-        M.Notification.query.remove({})
+    M.Notification.query.remove({})
