@@ -20,10 +20,9 @@ from collections import defaultdict, OrderedDict
 
 import unittest
 import mock
-from nose.tools import assert_equal
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 from bson import ObjectId
-from ming.orm import session
+from ming.odm import session
 from tg import config
 
 from alluratest.controller import setup_basic_test, setup_global_objects
@@ -31,15 +30,15 @@ from allura import model as M
 from allura.lib import helpers as h
 
 
-class TestGitLikeTree(object):
+class TestGitLikeTree:
     def test_set_blob(self):
         tree = M.GitLikeTree()
         tree.set_blob('/dir/dir2/file', 'file-oid')
 
-        assert_equal(tree.blobs, {})
-        assert_equal(tree.get_tree('dir').blobs, {})
-        assert_equal(tree.get_tree('dir').get_tree('dir2')
-                     .blobs, {'file': 'file-oid'})
+        assert tree.blobs == {}
+        assert tree.get_tree('dir').blobs == {}
+        assert (tree.get_tree('dir').get_tree('dir2')
+                     .blobs == {'file': 'file-oid'})
 
     def test_hex(self):
         tree = M.GitLikeTree()
@@ -47,58 +46,32 @@ class TestGitLikeTree(object):
         hex = tree.hex()
 
         # check the reprs. In case hex (below) fails, this'll be useful
-        assert_equal(repr(tree.get_tree('dir').get_tree('dir2')),
+        assert (repr(tree.get_tree('dir').get_tree('dir2')) ==
                      'b file-oid file')
-        assert_equal(repr(tree),
+        assert (repr(tree) ==
                      't 96af1772ecce1e6044e6925e595d9373ffcd2615 dir')
         # the hex() value shouldn't change, it's an important key
-        assert_equal(hex, '4abba29a43411b9b7cecc1a74f0b27920554350d')
+        assert hex == '4abba29a43411b9b7cecc1a74f0b27920554350d'
 
         # another one should be the same
         tree2 = M.GitLikeTree()
         tree2.set_blob('/dir/dir2/file', 'file-oid')
         hex2 = tree2.hex()
-        assert_equal(hex, hex2)
+        assert hex == hex2
+
+    def test_hex_with_unicode(self):
+        tree = M.GitLikeTree()
+        tree.set_blob('/dir/f•º£', 'file-oid')
+        # the hex() value shouldn't change, it's an important key
+        assert tree.hex() == '51ce65bead2f6452da61d4f6f2e42f8648bf9e4b'
 
 
-class RepoImplTestBase(object):
-    def test_commit_run(self):
-        M.repository.CommitRunDoc.m.remove()
-        commit_ids = list(self.repo.all_commit_ids())
-        # simulate building up a commit run from multiple pushes
-        for c_id in commit_ids:
-            crb = M.repo_refresh.CommitRunBuilder([c_id])
-            crb.run()
-            crb.cleanup()
-        runs = M.repository.CommitRunDoc.m.find().all()
-        self.assertEqual(len(runs), 1)
-        run = runs[0]
-        self.assertEqual(run.commit_ids, commit_ids)
-        self.assertEqual(len(run.commit_ids), len(run.commit_times))
-        self.assertEqual(run.parent_commit_ids, [])
-
-    def test_repair_commit_run(self):
-        commit_ids = list(self.repo.all_commit_ids())
-        # simulate building up a commit run from multiple pushes, but skip the
-        # last commit to simulate a broken commit run
-        for c_id in commit_ids[:-1]:
-            crb = M.repo_refresh.CommitRunBuilder([c_id])
-            crb.run()
-            crb.cleanup()
-        # now repair the commitrun by rebuilding with all commit ids
-        crb = M.repo_refresh.CommitRunBuilder(commit_ids)
-        crb.run()
-        crb.cleanup()
-        runs = M.repository.CommitRunDoc.m.find().all()
-        self.assertEqual(len(runs), 1)
-        run = runs[0]
-        self.assertEqual(run.commit_ids, commit_ids)
-        self.assertEqual(len(run.commit_ids), len(run.commit_times))
-        self.assertEqual(run.parent_commit_ids, [])
+class RepoImplTestBase:
+    pass
 
 
 class RepoTestBase(unittest.TestCase):
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
 
     @mock.patch('allura.model.repository.Repository.url')
@@ -126,16 +99,43 @@ class RepoTestBase(unittest.TestCase):
             with mock.patch.dict(config, values, clear=True):
                 self.assertEqual(result, repo.refresh_url())
 
+    def test_clone_command_categories(self):
+        c.app = mock.Mock(**{'config._id': 'deadbeef'})
+        repo = M.repository.Repository(tool='git')
+        cmd_cats = repo.clone_command_categories(anon=False)
+        assert cmd_cats == [
+            {'key': 'file', 'name': 'File', 'title': 'Filesystem'}
+        ]
+
+        cmd_cats = repo.clone_command_categories(anon=True)
+        assert cmd_cats == [
+            {'key': 'file', 'name': 'File', 'title': 'Filesystem'}
+        ]
+
+        repo = M.repository.Repository(tool='something-else')  # no "something-else" in config so will use defaults
+        cmd_cats = repo.clone_command_categories(anon=False)
+        assert cmd_cats == [
+            {'key': 'rw', 'name': 'RW', 'title': 'Read/Write'},
+            {'key': 'ro', 'name': 'RO', 'title': 'Read Only'},
+            {'key': 'https', 'name': 'HTTPS', 'title': 'HTTPS'}
+        ]
+
+        cmd_cats = repo.clone_command_categories(anon=True)
+        assert cmd_cats == [
+            {'key': 'ro', 'name': 'RO', 'title': 'Read Only'},
+            {'key': 'https_anon', 'name': 'HTTPS', 'title': 'HTTPS'}
+        ]
+
 
 class TestLastCommit(unittest.TestCase):
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         setup_global_objects()
         self.repo = mock.Mock(
             name='repo',
             _commits=OrderedDict(),
             _last_commit=None,
-            spec=M.Repository)
+        )
         self.repo.paged_diffs.return_value = {
             'added': [],
             'removed': [],
@@ -151,7 +151,8 @@ class TestLastCommit(unittest.TestCase):
         self.repo.get_changes = lambda _id: self._changes[_id]
         self._last_commits = [(None, set())]
         self.repo._get_last_commit = lambda i, p: self._last_commits.pop()
-        lcids = M.repository.RepositoryImplementation.last_commit_ids.__func__
+        lcids = M.repository.RepositoryImplementation.last_commit_ids
+        lcids = getattr(lcids, '__func__', lcids)
         self.repo.last_commit_ids = lambda *a, **k: lcids(self.repo, *a, **k)
         c.lcid_cache = {}
 
@@ -209,7 +210,7 @@ class TestLastCommit(unittest.TestCase):
         return commit
 
     def _log(self, revs, path, id_only=True, limit=None):
-        for commit_id, commit in reversed(self.repo._commits.items()):
+        for commit_id, commit in reversed(list(self.repo._commits.items())):
             if path in commit.changed_paths:
                 yield commit_id
 
@@ -363,13 +364,13 @@ class TestLastCommit(unittest.TestCase):
         commit2 = self._add_commit('Commit 2', ['file2'])
         commit2.changed_paths = []
         result = self.repo.last_commit_ids(commit2, ['file2'])
-        assert_equal(result, {'file2': commit2._id})
+        assert result == {'file2': commit2._id}
 
     def test_missing_add_record_first_commit(self):
         commit1 = self._add_commit('Commit 1', ['file1'])
         commit1.changed_paths = []
         result = self.repo.last_commit_ids(commit1, ['file1'])
-        assert_equal(result, {'file1': commit1._id})
+        assert result == {'file1': commit1._id}
 
     def test_timeout(self):
         commit1 = self._add_commit('Commit 1', ['file1'])
@@ -399,7 +400,7 @@ class TestLastCommit(unittest.TestCase):
 
 
 class TestModelCache(unittest.TestCase):
-    def setUp(self):
+    def setup_method(self, method):
         self.cache = M.repository.ModelCache()
 
     def test_normalize_query(self):
@@ -410,8 +411,8 @@ class TestModelCache(unittest.TestCase):
         q = mock.Mock(spec_set=['query'], query='foo')
         m = mock.Mock(spec_set=['m'], m='bar')
         n = mock.Mock(spec_set=['foo'], foo='qux')
-        self.assertEquals(self.cache._model_query(q), 'foo')
-        self.assertEquals(self.cache._model_query(m), 'bar')
+        self.assertEqual(self.cache._model_query(q), 'foo')
+        self.assertEqual(self.cache._model_query(m), 'bar')
         self.assertRaises(AttributeError, self.cache._model_query, [n])
 
     @mock.patch.object(M.repository.Tree.query, 'get')
@@ -500,7 +501,7 @@ class TestModelCache(unittest.TestCase):
         self.cache.set(M.repository.Tree, {'val': 'tree1'}, tree1)
         self.cache.set(M.repository.Tree, {'val': 'tree2'}, tree2)
         self.assertEqual(set(self.cache.instance_ids(M.repository.Tree)),
-                         set(['id1', 'id2']))
+                         {'id1', 'id2'})
         self.assertEqual(self.cache.instance_ids(M.repository.LastCommit), [])
 
     @mock.patch.object(M.repository.Tree.query, 'find')
@@ -676,66 +677,84 @@ class TestModelCache(unittest.TestCase):
         session.return_value.expunge.assert_called_once_with(tree1)
 
 
-class TestMergeRequest(object):
-    def setUp(self):
+class TestMergeRequest:
+
+    def setup_method(self, method):
         setup_basic_test()
         setup_global_objects()
         self.mr = M.MergeRequest(
             app_config=mock.Mock(_id=ObjectId()),
             downstream={'commit_id': '12345'},
             request_number=1,
+
         )
-        self.mr.app = mock.Mock(forkable=True, url='/mock-app-url/')
-        self.mr.app.repo.commit.return_value = mock.Mock(_id='09876')
-        self.mr.merge_allowed = mock.Mock(return_value=True)
-        self.mr.discussion_thread = mock.Mock()
+        self._set_mr_mock_attrs(self.mr)
+
+    def _set_mr_mock_attrs(self, mr):
+        mr.app = mock.Mock(forkable=True, url='/mock-app-url/', activity_name='code merge', activity_url='/fake/url', activity_extras={}, node_id=None)
+        mr.app.repo.commit.return_value = mock.Mock(_id='09876')
+        mr.merge_allowed = mock.Mock(return_value=True)
+        mr.discussion_thread = mock.Mock()
+
+    def _reload_mr_from_db(self, mr):
+        session(mr).refresh(mr)
+        mr = M.MergeRequest.query.get(_id=mr._id)
+        self._set_mr_mock_attrs(mr)
+        return mr
 
     def test_can_merge_cache_key(self):
         key = self.mr.can_merge_cache_key()
-        assert_equal(key, '12345-09876')
+        assert key == '12345-09876'
 
     def test_get_can_merge_cache(self):
         key = self.mr.can_merge_cache_key()
-        assert_equal(self.mr.get_can_merge_cache(), None)
+        assert self.mr.get_can_merge_cache() is None
         self.mr.can_merge_cache[key] = True
-        assert_equal(self.mr.get_can_merge_cache(), True)
+        assert self.mr.get_can_merge_cache() is True
 
         self.mr.can_merge_cache_key = lambda: '123-123'
         self.mr.can_merge_cache['123-123'] = False
-        assert_equal(self.mr.get_can_merge_cache(), False)
+        assert self.mr.get_can_merge_cache() is False
 
     def test_set_can_merge_cache(self):
         key = self.mr.can_merge_cache_key()
-        assert_equal(self.mr.can_merge_cache, {})
+        assert self.mr.can_merge_cache == {}
         self.mr.set_can_merge_cache(True)
-        assert_equal(self.mr.can_merge_cache, {key: True})
+        assert self.mr.can_merge_cache == {key: True}
 
         self.mr.can_merge_cache_key = lambda: '123-123'
         self.mr.set_can_merge_cache(False)
-        assert_equal(self.mr.can_merge_cache, {key: True, '123-123': False})
+        assert self.mr.can_merge_cache == {key: True, '123-123': False}
 
     def test_can_merge_merged(self):
         self.mr.status = 'merged'
-        assert_equal(self.mr.can_merge(), True)
+        assert self.mr.can_merge() is True
 
     @mock.patch('allura.tasks.repo_tasks.can_merge', autospec=True)
     def test_can_merge_cached(self, can_merge_task):
+        # this test has to flush `mr` to the db and then reload it after changes, because set_can_merge_cache
+        # does a $set to the db and doesn't update the in-memory copy
+        session(self.mr).flush(self.mr)
+
         self.mr.set_can_merge_cache(False)
-        assert_equal(self.mr.can_merge(), False)
+        self.mr = self._reload_mr_from_db(self.mr)
+        assert self.mr.can_merge() is False
+
         self.mr.set_can_merge_cache(True)
-        assert_equal(self.mr.can_merge(), True)
-        assert_equal(can_merge_task.post.call_count, 0)
+        self.mr = self._reload_mr_from_db(self.mr)
+        assert self.mr.can_merge() is True
+        assert can_merge_task.post.call_count == 0
 
     @mock.patch('allura.tasks.repo_tasks.can_merge', autospec=True)
     def test_can_merge_not_cached(self, can_merge_task):
-        assert_equal(self.mr.can_merge(), None)
+        assert self.mr.can_merge() is None
         can_merge_task.post.assert_called_once_with(self.mr._id)
 
     @mock.patch('allura.tasks.repo_tasks.can_merge', autospec=True)
     def test_can_merge_disabled(self, can_merge_task):
         self.mr.merge_allowed.return_value = False
-        assert_equal(self.mr.can_merge(), None)
-        assert_equal(can_merge_task.post.call_count, 0)
+        assert self.mr.can_merge() is None
+        assert can_merge_task.post.call_count == 0
 
     @mock.patch('allura.tasks.repo_tasks.merge', autospec=True)
     def test_merge(self, merge_task):
@@ -746,21 +765,21 @@ class TestMergeRequest(object):
         merge_task.reset_mock()
         self.mr.merge_task_status = lambda: 'ready'
         self.mr.merge()
-        assert_equal(merge_task.post.called, False)
+        assert merge_task.post.called is False
 
     def test_merge_task_status(self):
         from allura.tasks import repo_tasks
-        assert_equal(self.mr.merge_task_status(), None)
+        assert self.mr.merge_task_status() is None
         repo_tasks.merge.post(self.mr._id)
-        assert_equal(self.mr.merge_task_status(), 'ready')
+        assert self.mr.merge_task_status() == 'ready'
         M.MonQTask.run_ready()
-        assert_equal(self.mr.merge_task_status(), 'complete')
+        assert self.mr.merge_task_status() == 'complete'
 
     def test_can_merge_task_status(self):
         from allura.tasks import repo_tasks
-        assert_equal(self.mr.can_merge_task_status(), None)
+        assert self.mr.can_merge_task_status() is None
         repo_tasks.can_merge.post(self.mr._id)
-        assert_equal(self.mr.can_merge_task_status(), 'ready')
+        assert self.mr.can_merge_task_status() == 'ready'
         with mock.patch('allura.model.repository.MergeRequest.set_can_merge_cache'):
             M.MonQTask.run_ready()
-        assert_equal(self.mr.can_merge_task_status(), 'complete')
+        assert self.mr.can_merge_task_status() == 'complete'
