@@ -15,41 +15,45 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from nose.tools import assert_raises, assert_in
-from datadiff.tools import assert_equal
+
+import datetime
+
+from testfixtures import OutputCapture
 
 from ming.base import Object
-from ming.orm import ThreadLocalORMSession
+from ming.odm import ThreadLocalODMSession
 from mock import Mock, call, patch
+import pytest
 import pymongo
 import pkg_resources
 
-from alluratest.controller import setup_basic_test, setup_global_objects
+from alluratest.controller import setup_basic_test, setup_global_objects, setup_unit_test
 from allura.command import base, script, set_neighborhood_features, \
-    create_neighborhood, show_models, taskd_cleanup
+    create_neighborhood, show_models, taskd_cleanup, taskd
 from allura import model as M
 from allura.lib.exceptions import InvalidNBFeatureValueError
 from allura.tests import decorators as td
 
+
 test_config = pkg_resources.resource_filename(
     'allura', '../test.ini') + '#main'
 
 
-class EmptyClass(object):
+class EmptyClass:
     pass
 
 
-def setUp(self):
-    """Method called by nose before running each test"""
+def setup_module():
     setup_basic_test()
     setup_global_objects()
+    setup_unit_test()
 
 
 def test_script():
     cmd = script.ScriptCommand('script')
     cmd.run(
         [test_config, pkg_resources.resource_filename('allura', 'tests/tscript.py')])
-    assert_raises(ValueError, cmd.run,
+    pytest.raises(ValueError, cmd.run,
                   [test_config, pkg_resources.resource_filename('allura', 'tests/tscript_error.py')])
 
 
@@ -67,12 +71,12 @@ def test_set_neighborhood_max_projects():
     # none is also valid
     cmd.run([test_config, str(n_id), 'max_projects', 'None'])
     neighborhood = M.Neighborhood.query.get(_id=n_id)
-    assert neighborhood.features['max_projects'] == None
+    assert neighborhood.features['max_projects'] is None
 
     # check validation
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'max_projects', 'string'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'max_projects', '2.8'])
 
 
@@ -93,11 +97,11 @@ def test_set_neighborhood_private():
     assert not neighborhood.features['private_projects']
 
     # check validation
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'private_projects', 'string'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'private_projects', '1'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'private_projects', '2.8'])
 
 
@@ -118,11 +122,11 @@ def test_set_neighborhood_google_analytics():
     assert not neighborhood.features['google_analytics']
 
     # check validation
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'google_analytics', 'string'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'google_analytics', '1'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'google_analytics', '2.8'])
 
 
@@ -148,15 +152,15 @@ def test_set_neighborhood_css():
     assert neighborhood.features['css'] == 'custom'
 
     # check validation
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'css', 'string'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'css', '1'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'css', '2.8'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'css', 'None'])
-    assert_raises(InvalidNBFeatureValueError, cmd.run,
+    pytest.raises(InvalidNBFeatureValueError, cmd.run,
                   [test_config, str(n_id), 'css', 'True'])
 
 
@@ -164,19 +168,19 @@ def test_update_neighborhood():
     cmd = create_neighborhood.UpdateNeighborhoodCommand('update-neighborhood')
     cmd.run([test_config, 'Projects', 'True'])
     # make sure the app_configs get freshly queried
-    ThreadLocalORMSession.close_all()
+    ThreadLocalODMSession.close_all()
     nb = M.Neighborhood.query.get(name='Projects')
-    assert nb.has_home_tool == True
+    assert nb.has_home_tool is True
 
     cmd = create_neighborhood.UpdateNeighborhoodCommand('update-neighborhood')
     cmd.run([test_config, 'Projects', 'False'])
     # make sure the app_configs get freshly queried
-    ThreadLocalORMSession.close_all()
+    ThreadLocalODMSession.close_all()
     nb = M.Neighborhood.query.get(name='Projects')
-    assert nb.has_home_tool == False
+    assert nb.has_home_tool is False
 
 
-class TestEnsureIndexCommand(object):
+class TestEnsureIndexCommand:
 
     def test_run(self):
         cmd = show_models.EnsureIndexCommand('ensure_index')
@@ -245,7 +249,7 @@ class TestEnsureIndexCommand(object):
         cmd = show_models.EnsureIndexCommand('ensure_index')
         cmd._update_indexes(collection, indexes)
 
-        assert_equal(collection.mock_calls, [
+        assert collection.mock_calls == [
             call.index_information(),
             call.ensure_index(
                 [('foo', 1), ('bar', 1), ('temporary_extra_field_for_indexing', 1)]),
@@ -259,12 +263,78 @@ class TestEnsureIndexCommand(object):
             call.drop_index('_foo_baz_temporary_extra_field_for_indexing'),
             call.ensure_index([('foo', 1), ('baz', 1)], unique=True, sparse=False),
             call.ensure_index([('foo', 1), ('bar', 1)], unique=False, sparse=False, background=True)
+        ]
+
+
+class TestTaskCommand:
+
+    def teardown_method(self, method):
+        M.MonQTask.query.remove({})
+
+    def test_commit(self):
+        exit_code = taskd.TaskCommand('task').run([test_config, 'commit'])
+        assert M.MonQTask.query.find({'task_name': 'allura.tasks.index_tasks.commit'}).count() == 1
+        assert exit_code == 0
+
+    def test_list(self):
+        exit_code = taskd.TaskCommand('task').run([test_config, 'list'])
+        assert exit_code == 0
+
+    def test_count(self):
+        exit_code = taskd.TaskCommand('task').run([test_config, 'count'])
+        assert exit_code == 0
+
+    def test_retry(self):
+        # self.test_commit()
+        exit_code = taskd.TaskCommand('task').run([
+            test_config, 'retry',
+            '--filter-name-prefix', 'allura.tasks.index_tasks.',
+            '--filter-result-regex', 'pysolr',
         ])
+        assert exit_code == 0
+
+    def test_purge(self):
+        # create task
+        self.test_commit()
+        assert M.MonQTask.query.find().count() == 1
+        M.MonQTask.query.update({'task_name': 'allura.tasks.index_tasks.commit'}, {'$set': {'state': 'complete'}})
+        # run purge; verify 0 records
+        exit_code = taskd.TaskCommand('task').run([
+            test_config, 'purge',
+        ])
+        assert exit_code == 0
+        assert M.MonQTask.query.find().count() == 0
+
+    def test_purge_old_only(self):
+        # create task
+        self.test_commit()
+        assert M.MonQTask.query.find().count() == 1
+
+        # force task to be in complete state
+        M.MonQTask.query.update({'task_name': 'allura.tasks.index_tasks.commit'}, {'$set': {'state': 'complete'}})
+        # run purge; verify no records deleted
+        exit_code = taskd.TaskCommand('task').run([
+            test_config, 'purge', '--filter-queued-days-ago', '180',
+        ])
+        assert exit_code == 0
+        assert M.MonQTask.query.find().count() == 1
+
+        # modify task to be old
+        then = datetime.datetime.utcnow() - datetime.timedelta(days=200)
+        M.MonQTask.query.update({'task_name': 'allura.tasks.index_tasks.commit'},
+                                {'$set': {'time_queue': then, 'time_start': then, 'time_stop': then}})
+
+        # run purge; verify old tasks deleted
+        exit_code = taskd.TaskCommand('task').run([
+            test_config, 'purge', '--filter-queued-days-ago', '180',
+        ])
+        assert exit_code == 0
+        assert M.MonQTask.query.find().count() == 0
 
 
-class TestTaskdCleanupCommand(object):
+class TestTaskdCleanupCommand:
 
-    def setUp(self):
+    def setup_method(self, method):
         self.cmd_class = taskd_cleanup.TaskdCleanupCommand
         self.old_check_taskd_status = self.cmd_class._check_taskd_status
         self.cmd_class._check_taskd_status = lambda x, p: 'OK'
@@ -279,8 +349,8 @@ class TestTaskdCleanupCommand(object):
         self.old_complete_suspicious_tasks = self.cmd_class._complete_suspicious_tasks
         self.cmd_class._complete_suspicious_tasks = lambda x: []
 
-    def tearDown(self):
-        # need to clean up setUp mocking for unit tests below to work properly
+    def teardown_method(self, method):
+        # need to clean up setup_method mocking for unit tests below to work properly
         self.cmd_class._check_taskd_status = self.old_check_taskd_status
         self.cmd_class._check_task = self.old_check_task
         self.cmd_class._busy_tasks = self.old_busy_tasks
@@ -376,7 +446,18 @@ def test_status_log_retries():
     assert cmd._taskd_status.mock_calls == expected_calls
 
 
-class TestBackgroundCommand(object):
+class TestShowModels:
+
+    def test_show_models(self):
+        cmd = show_models.ShowModelsCommand('models')
+        with OutputCapture() as output:
+            cmd.run([test_config])
+        assert '''allura.model.notification.SiteNotification
+         - <FieldProperty _id>
+         - <FieldProperty content>
+        ''' in output.captured
+
+class TestReindexAsTask:
 
     cmd = 'allura.command.show_models.ReindexCommand'
     task_name = 'allura.command.base.run_command'
@@ -384,12 +465,12 @@ class TestBackgroundCommand(object):
     def test_command_post(self):
         show_models.ReindexCommand.post('-p "project 3"')
         tasks = M.MonQTask.query.find({'task_name': self.task_name}).all()
-        assert_equal(len(tasks), 1)
+        assert len(tasks) == 1
         task = tasks[0]
-        assert_equal(task.args, [self.cmd, '-p "project 3"'])
+        assert task.args == [self.cmd, '-p "project 3"']
 
     def test_command_doc(self):
-        assert_in('Usage:', show_models.ReindexCommand.__doc__)
+        assert 'Usage:' in show_models.ReindexCommand.__doc__
 
     @patch('allura.command.show_models.ReindexCommand')
     def test_run_command(self, command):
@@ -401,12 +482,16 @@ class TestBackgroundCommand(object):
     def test_invalid_args(self):
         M.MonQTask.query.remove()
         show_models.ReindexCommand.post('--invalid-option')
-        with td.raises(Exception) as e:
-            M.MonQTask.run_ready()
-        assert_in('Error parsing args', str(e.exc))
+        try:
+            with td.raises(Exception) as e:
+                M.MonQTask.run_ready()
+            assert 'Error parsing args' in str(e.exc)
+        finally:
+            # cleanup - remove bad MonQTask
+            M.MonQTask.query.remove()
 
 
-class TestReindexCommand(object):
+class TestReindexCommand:
 
     @patch('allura.command.show_models.g')
     def test_skip_solr_delete(self, g):
@@ -418,12 +503,13 @@ class TestReindexCommand(object):
         assert not g.solr.delete.called, 'solr.delete() must not be called'
 
     @patch('pysolr.Solr')
+    @td.with_wiki  # so there's some artifacts to reindex
     def test_solr_hosts_1(self, Solr):
         cmd = show_models.ReindexCommand('reindex')
         cmd.options, args = cmd.parser.parse_args([
             '-p', 'test', '--solr', '--solr-hosts=http://blah.com/solr/forge'])
         cmd._chunked_add_artifacts(list(range(10)))
-        assert_equal(Solr.call_args[0][0], 'http://blah.com/solr/forge')
+        assert Solr.call_args[0][0] == 'http://blah.com/solr/forge'
 
     @patch('pysolr.Solr')
     def test_solr_hosts_list(self, Solr):
@@ -432,11 +518,10 @@ class TestReindexCommand(object):
             '-p', 'test', '--solr', '--solr-hosts=http://blah.com/solr/forge,https://other.net/solr/forge'])
         cmd._chunked_add_artifacts(list(range(10)))
         # check constructors of first and second Solr() instantiations
-        assert_equal(
-            set([Solr.call_args_list[0][0][0], Solr.call_args_list[1][0][0]]),
-            set(['http://blah.com/solr/forge',
-                 'https://other.net/solr/forge'])
-        )
+        assert (
+            {Solr.call_args_list[0][0][0], Solr.call_args_list[1][0][0]} ==
+            {'http://blah.com/solr/forge',
+                 'https://other.net/solr/forge'})
 
     @patch('allura.command.show_models.utils')
     def test_project_regex(self, utils):
@@ -451,12 +536,12 @@ class TestReindexCommand(object):
         cmd.options = Mock(tasks=True, max_chunk=10 * 1000, ming_config=None)
         ref_ids = list(range(10 * 1000 * 2 + 20))
         cmd._chunked_add_artifacts(ref_ids)
-        assert_equal(len(add_artifacts.post.call_args_list), 3)
-        assert_equal(
-            len(add_artifacts.post.call_args_list[0][0][0]), 10 * 1000)
-        assert_equal(
-            len(add_artifacts.post.call_args_list[1][0][0]), 10 * 1000)
-        assert_equal(len(add_artifacts.post.call_args_list[2][0][0]), 20)
+        assert len(add_artifacts.post.call_args_list) == 3
+        assert (
+            len(add_artifacts.post.call_args_list[0][0][0]) == 10 * 1000)
+        assert (
+            len(add_artifacts.post.call_args_list[1][0][0]) == 10 * 1000)
+        assert len(add_artifacts.post.call_args_list[2][0][0]) == 20
 
     @patch('allura.command.show_models.add_artifacts')
     def test_post_add_artifacts_too_large(self, add_artifacts):
@@ -470,7 +555,7 @@ class TestReindexCommand(object):
         add_artifacts.post.side_effect = on_post
         cmd = show_models.ReindexCommand('reindex')
         cmd.options, args = cmd.parser.parse_args([])
-        cmd._post_add_artifacts(range(5))
+        cmd._post_add_artifacts(list(range(5)))
         kw = {'update_solr': cmd.options.solr, 'update_refs': cmd.options.refs}
         expected = [
             call([0, 1, 2, 3, 4], **kw),
@@ -483,7 +568,7 @@ class TestReindexCommand(object):
             call([3], **kw),
             call([4], **kw)
         ]
-        assert_equal(expected, add_artifacts.post.call_args_list)
+        assert expected == add_artifacts.post.call_args_list
 
     @patch('allura.command.show_models.add_artifacts')
     def test_post_add_artifacts_other_error(self, add_artifacts):
@@ -493,4 +578,9 @@ class TestReindexCommand(object):
         cmd = show_models.ReindexCommand('reindex')
         cmd.options = Mock(ming_config=None)
         with td.raises(pymongo.errors.InvalidDocument):
-            cmd._post_add_artifacts(range(5))
+            cmd._post_add_artifacts(list(range(5)))
+
+    @td.with_wiki  # so there's some artifacts to reindex
+    def test_ming_config(self):
+        cmd = show_models.ReindexCommand('reindex')
+        cmd.run([test_config, '-p', 'test', '--tasks', '--ming-config', 'test.ini'])
