@@ -27,8 +27,8 @@ import requests
 from bson import ObjectId
 from tg import expose, validate, redirect, flash, config
 from tg.decorators import with_trailing_slash, without_trailing_slash
-from pylons import tmpl_context as c
-from pylons import response, request
+from tg import tmpl_context as c
+from tg import response, request
 from formencode import validators as fev, schema, Invalid
 from ming.odm import session
 from webob import exc
@@ -38,9 +38,11 @@ from paste.deploy.converters import asint, aslist
 from allura.app import AdminControllerMixin
 from allura.controllers import BaseController
 from allura.lib import helpers as h
+from allura.lib import validators as v
 from allura.lib.decorators import require_post, task
 from allura.lib.utils import DateJSONEncoder
 from allura import model as M
+import six
 
 
 log = logging.getLogger(__name__)
@@ -50,9 +52,9 @@ class WebhookValidator(fev.FancyValidator):
     def __init__(self, sender, app, **kw):
         self.app = app
         self.sender = sender
-        super(WebhookValidator, self).__init__(**kw)
+        super().__init__(**kw)
 
-    def _to_python(self, value, state):
+    def _convert_to_python(self, value, state):
         wh = None
         if isinstance(value, M.Webhook):
             wh = value
@@ -61,21 +63,21 @@ class WebhookValidator(fev.FancyValidator):
         else:
             try:
                 wh = M.Webhook.query.get(_id=ObjectId(value))
-            except:
+            except Exception:
                 pass
         if wh and wh.type == self.sender.type and wh.app_config_id == self.app.config._id:
             return wh
-        raise Invalid(u'Invalid webhook', value, state)
+        raise Invalid('Invalid webhook', value, state)
 
 
 class WebhookCreateForm(schema.Schema):
     url = fev.URL(not_empty=True)
-    secret = fev.UnicodeString()
+    secret = v.UnicodeString()
 
 
 class WebhookEditForm(WebhookCreateForm):
     def __init__(self, sender, app):
-        super(WebhookEditForm, self).__init__()
+        super().__init__()
         self.add_field('webhook', WebhookValidator(
             sender=sender, app=app, not_empty=True))
 
@@ -88,23 +90,22 @@ class WebhookControllerMeta(type):
         if hasattr(cls, 'create'):
             cls.create = validate(
                 cls.create_form(),
-                error_handler=cls.index.__func__,
+                error_handler=getattr(cls.index, '__func__', cls.index),
             )(cls.create)
         if hasattr(cls, 'edit'):
             cls.edit = validate(
                 cls.edit_form(sender, app),
-                error_handler=cls._default.__func__,
+                error_handler=getattr(cls._default, '__func__', cls._default),
             )(cls.edit)
         return type.__call__(cls, sender, app, *args, **kw)
 
 
-class WebhookController(BaseController, AdminControllerMixin):
-    __metaclass__ = WebhookControllerMeta
+class WebhookController(BaseController, AdminControllerMixin, metaclass=WebhookControllerMeta):
     create_form = WebhookCreateForm
     edit_form = WebhookEditForm
 
     def __init__(self, sender, app):
-        super(WebhookController, self).__init__()
+        super().__init__()
         self.sender = sender()
         self.app = app
 
@@ -120,29 +121,36 @@ class WebhookController(BaseController, AdminControllerMixin):
             session(wh).flush(wh)
         except DuplicateKeyError:
             session(wh).expunge(wh)
-            msg = u'_the_form: "{}" webhook already exists for {} {}'.format(
+            msg = '_the_form: "{}" webhook already exists for {} {}'.format(
                 wh.type, self.app.config.options.mount_label, url)
             raise Invalid(msg, None, None)
 
     @with_trailing_slash
     @expose('jinja:allura:templates/webhooks/create_form.html')
     def index(self, **kw):
-        if not c.form_values and kw:
+        if not request.validation.values and kw:
             # Executes if update_webhook raises an error
-            c.form_values = {'url': kw.get('url'),
+            request.validation.values = {'url': kw.get('url'),
                              'secret': kw.get('secret')}
         return {'sender': self.sender,
                 'action': 'create',
                 'form': self.create_form()}
 
-    @expose()
+    @expose('jinja:allura:templates/webhooks/create_form.html')  # needed when we "return self.index(...)"
     @require_post()
+    # @validate set dynamically in WebhookControllerMeta
     def create(self, url, secret):
         if self.sender.enforce_limit(self.app):
             webhook = M.Webhook(
                 type=self.sender.type,
                 app_config_id=self.app.config._id)
-            self.update_webhook(webhook, url, secret)
+            try:
+                self.update_webhook(webhook, url, secret)
+            except Invalid as e:
+                # trigger error_handler directly
+                request.validation.errors['_the_form'] = e
+                return self.index(url=url, secret=secret)
+
             M.AuditLog.log('add webhook %s %s %s',
                            webhook.type, webhook.hook_url,
                            webhook.app_config.url())
@@ -152,12 +160,18 @@ class WebhookController(BaseController, AdminControllerMixin):
                   'you are allowed to create for this project/app', 'error')
         redirect(self.app.admin_url + 'webhooks')
 
-    @expose()
+    @expose('jinja:allura:templates/webhooks/create_form.html')  # needed when we "return self._default(...)"
     @require_post()
+    # @validate set dynamically in WebhookControllerMeta
     def edit(self, webhook, url, secret):
         old_url = webhook.hook_url
         old_secret = webhook.secret
-        self.update_webhook(webhook, url, secret)
+        try:
+            self.update_webhook(webhook, url, secret)
+        except Invalid as e:
+            # trigger error_handler directly
+            request.validation.errors['_the_form'] = e
+            return self._default(webhook=webhook, url=url, secret=secret)
         M.AuditLog.log('edit webhook %s\n%s => %s\n%s',
                        webhook.type, old_url, url,
                        'secret changed' if old_secret != secret else '')
@@ -185,9 +199,9 @@ class WebhookController(BaseController, AdminControllerMixin):
             wh = form.fields['webhook'].to_python(webhook)
         except Invalid:
             raise exc.HTTPNotFound()
-        c.form_values = {'url': kw.get('url') or wh.hook_url,
+        request.validation.values = {'url': kw.get('url') or wh.hook_url,
                          'secret': kw.get('secret') or wh.secret,
-                         'webhook': unicode(wh._id)}
+                         'webhook': str(wh._id)}
         return {'sender': self.sender,
                 'action': 'edit',
                 'form': form}
@@ -195,7 +209,7 @@ class WebhookController(BaseController, AdminControllerMixin):
 
 class WebhookRestController(BaseController):
     def __init__(self, sender, app):
-        super(WebhookRestController, self).__init__()
+        super().__init__()
         self.sender = sender()
         self.app = app
         self.create_form = WebhookController.create_form
@@ -205,8 +219,8 @@ class WebhookRestController(BaseController):
         error = getattr(e, 'error_dict', None)
         if error:
             _error = {}
-            for k, v in error.iteritems():
-                _error[k] = unicode(v)
+            for k, val in error.items():
+                _error[k] = str(val)
             return _error
         error = getattr(e, 'msg', None)
         if not error:
@@ -284,7 +298,7 @@ class WebhookRestController(BaseController):
         try:
             params = {'secret': kw.pop('secret', old_secret),
                       'url': kw.pop('url', old_url),
-                      'webhook': unicode(webhook._id)}
+                      'webhook': str(webhook._id)}
             valid = form.to_python(params)
         except Exception as e:
             response.status_int = 400
@@ -313,7 +327,7 @@ class WebhookRestController(BaseController):
         return {'result': 'ok'}
 
 
-class SendWebhookHelper(object):
+class SendWebhookHelper:
     def __init__(self, webhook, payload):
         self.webhook = webhook
         self.payload = payload
@@ -325,7 +339,7 @@ class SendWebhookHelper(object):
     @property
     def retries(self):
         t = aslist(config.get('webhook.retry', [60, 120, 240]))
-        return map(int, t)
+        return list(map(int, t))
 
     def sign(self, json_payload):
         signature = hmac.new(
@@ -390,7 +404,7 @@ def send_webhook(webhook_id, payload):
     SendWebhookHelper(webhook, payload).send()
 
 
-class WebhookSender(object):
+class WebhookSender:
     """Base class for webhook senders.
 
     Subclasses are required to implement :meth:`get_payload()` and set
@@ -429,7 +443,7 @@ class WebhookSender(object):
                     for payload in payloads:
                         send_webhook.post(webhook._id, payload)
                 else:
-                    log.warn('Webhook fires too often: %s. Skipping', webhook)
+                    log.warning('Webhook fires too often: %s. Skipping', webhook)
 
     def enforce_limit(self, app):
         '''
@@ -456,16 +470,16 @@ class RepoPushWebhookSender(WebhookSender):
                 # Merge commit will have multiple parents. As far as I can tell
                 # the last one will be the branch head before merge
                 return self._convert_id(parents[-1])
-        return u''
+        return ''
 
     def _after(self, commit_ids):
         if len(commit_ids) > 0:
             return self._convert_id(commit_ids[0])
-        return u''
+        return ''
 
     def _convert_id(self, _id):
         if ':' in _id:
-            _id = u'r' + _id.rsplit(':', 1)[1]
+            _id = 'r' + _id.rsplit(':', 1)[1]
         return _id
 
     def get_payload(self, commit_ids, **kw):
