@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -21,18 +19,20 @@
 
 import logging
 
-from allura.config.environment import load_environment
-
+from allura.config.app_cfg import base_config
 __all__ = ['setup_app']
 
+
 log = logging.getLogger(__name__)
 
-from schema import setup_schema
-import bootstrap
+from .schema import setup_schema
+from . import bootstrap
 
 
 def setup_app(command, conf, vars):
     """Place any commands to setup allura here"""
-    load_environment(conf.global_conf, conf.local_conf)
+    conf = base_config.configure(conf.global_conf, conf.local_conf)
+    base_config.setup(conf)
+
     setup_schema(command, conf, vars)
     bootstrap.bootstrap(command, conf, vars)
