@@ -15,13 +15,15 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from mock import patch
 from textwrap import dedent
-from tg import config
 
+from mock import patch
+from tg import config
+from bson import ObjectId
 import dateutil.parser
-from nose.tools import assert_equal
-from pylons import app_globals as g
+from tg import app_globals as g
+from activitystream.storage.mingstorage import Activity
+from ming.odm import ThreadLocalODMSession
 
 from allura import model as M
 from alluratest.controller import TestController
@@ -30,13 +32,13 @@ from allura.tests import decorators as td
 
 class TestActivityController(TestController):
 
-    def setUp(self, *args, **kwargs):
-        super(TestActivityController, self).setUp(*args, **kwargs)
+    def setup_method(self, method, *args, **kwargs):
+        super().setup_method(method, *args, **kwargs)
         self._enabled = config.get('activitystream.enabled', 'false')
         config['activitystream.enabled'] = 'true'
 
-    def tearDown(self, *args, **kwargs):
-        super(TestActivityController, self).tearDown(*args, **kwargs)
+    def teardown_method(self, method, *args, **kwargs):
+        super().teardown_method(method, *args, **kwargs)
         config['activitystream.enabled'] = self._enabled
 
     def test_index(self):
@@ -51,17 +53,14 @@ class TestActivityController(TestController):
     @td.with_user_project('test-user-1')
     def test_anon_read(self):
         r = self.app.get('/u/test-user-1',
-                extra_environ={'username': '*anonymous'}).follow().follow()
-        assert r.html.find('div', 'profile-section tools').find('a',
-                dict(href='/u/test-user-1/activity/')), \
-                        'No Activity tool in top nav'
+                         extra_environ={'username': '*anonymous'}).follow()
+        assert r.html.select('div.profile-section.tools a[href="/u/test-user-1/activity/"]'),\
+            'No Activity tool in top nav'
 
     @td.with_tool('test', 'activity')
     @patch('forgeactivity.main.g.director')
     def test_index_html(self, director):
-        from activitystream.storage.base import StoredActivity
-        from bson import ObjectId
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "_id": ObjectId("529fa331033c5e6406d8b338"),
             "obj": {
                 "activity_extras": {
@@ -95,32 +94,32 @@ class TestActivityController(TestController):
         })]
         r = self.app.get('/p/test/activity/')
         timeline = r.html.find('ul', 'timeline')
-        assert_equal(1, len(timeline.findAll('li')))
+        assert 1 == len(timeline.findAll('li'))
         activity = timeline.find('li')
-        assert_equal(activity.time['title'], "2013-12-04 21:48:19")
+        assert activity.time['title'] == "2013-12-04 21:48:19"
         h1 = """\
         <h1>
-        <img src="/u/test-admin/user_icon" alt="Administrator 1" title="Administrator 1" class="emboss x32 avatar" />
-        <a href="/u/test-admin/">
-         Administrator 1
-        </a>
-        posted
-        <a href="/p/test/tickets/34/?limit=25#ed7c">
-         a comment
-        </a>
-        on
-        <a href="/p/test/tickets/34/">
-         ticket #34
-        </a>
+         <img alt="Administrator 1" class="emboss x32 avatar" src="/u/test-admin/user_icon" title="Administrator 1"/>
+         <a href="/u/test-admin/profile/">
+          Administrator 1
+         </a>
+         posted
+         <a href="/p/test/tickets/34/?limit=25#ed7c">
+          a comment
+         </a>
+         on
+         <a href="/p/test/tickets/34/">
+          ticket #34
+         </a>
         </h1>
         """
-        assert_equal(dedent(h1), activity.h1.prettify())
+        assert dedent(h1) == activity.h1.prettify()
         p = """\
         <p>
-        Just wanted to leave a comment on this...
+         Just wanted to leave a comment on this...
         </p>
         """
-        assert_equal(dedent(p), activity.p.prettify())
+        assert dedent(p) == activity.p.prettify()
 
     @td.with_tool('u/test-user-1', 'activity')
     @td.with_user_project('test-user-1')
@@ -170,24 +169,22 @@ class TestActivityController(TestController):
             create_timeline.side_effect = orig_create_timeline
             M.MonQTask.run_ready()
             # 3 aggregations: 1 actor, 1 follower, 1 project
-            assert_equal(create_timeline.call_count, 3)
+            assert create_timeline.call_count == 3
             create_timeline.reset_mock()
             self.app.get('/u/test-admin/activity/')
             self.app.get('/u/test-user-1/activity/')
-            assert_equal(create_timeline.call_count, 0)
+            assert create_timeline.call_count == 0
 
     @td.with_tool('test', 'activity')
     @patch('forgeactivity.main.g.director')
     def test_feed_rss_project(self, director):
-        from activitystream.storage.base import StoredActivity
-        from bson import ObjectId
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "_id": ObjectId("529fa331033c5e6406d8b338"),
             "obj": {
                 "activity_extras": {
                     "summary": "Just wanted to leave a comment on this..."
                 },
-                "activity_url": "/p/test/tickets/34/?limit=25#ed7c",
+                "activity_url": "/p/test/unicode•º/?limit=25#ed7c",
                 "activity_name": "a comment"
             },
             "target": {
@@ -215,22 +212,23 @@ class TestActivityController(TestController):
         })]
         r = self.app.get('/p/test/activity/feed.rss')
         timeline = r.xml.find('channel')
-        assert_equal(1, len(timeline.findall('item')))
+        assert 1 == len(timeline.findall('item'))
         activity = timeline.find('item')
-        assert_equal(activity.find('pubDate').text,
-                     'Wed, 04 Dec 2013 21:48:19 -0000')
-        assert_equal(activity.find('title').text,
-                     'Administrator 1 posted a comment on ticket #34')
-        assert_equal(activity.find('description').text,
-                     'Just wanted to leave a comment on this...')
-        assert_equal(activity.find('link').text,
-                     'http://localhost/p/test/tickets/34/?limit=25#ed7c')
+        assert (activity.find('pubDate').text ==
+                'Wed, 04 Dec 2013 21:48:19 -0000')
+        assert (activity.find('title').text ==
+                'Administrator 1 posted a comment on ticket #34')
+        assert (activity.find('description').text ==
+                'Just wanted to leave a comment on this...')
+        assert (activity.find('guid').text ==
+                'http://localhost/p/test/unicode•º/?limit=25#ed7c')
+        assert (activity.find('link').text ==
+                'http://localhost/p/test/unicode%E2%80%A2%C2%BA/?limit=25#ed7c')
 
     @td.with_tool('test', 'activity')
     @patch('forgeactivity.main.g.director')
     def test_feed_rss_project_verb_without_activity_name(self, director):
-        from activitystream.storage.base import StoredActivity
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "obj": {
                 "activity_extras": {},
                 "activity_url": "/p/test/tickets/34/?limit=25#ed7c",
@@ -245,18 +243,16 @@ class TestActivityController(TestController):
 
         r = self.app.get('/p/test/activity/feed.rss')
         timeline = r.xml.find('channel')
-        assert_equal(1, len(timeline.findall('item')))
+        assert 1 == len(timeline.findall('item'))
         activity = timeline.find('item')
 
-        assert_equal(activity.find('title').text, 'Administrator 1 created ticket #34')
+        assert activity.find('title').text == 'Administrator 1 created ticket #34'
 
     @td.with_tool('u/test-user-1', 'activity')
     @td.with_user_project('test-user-1')
     @patch('forgeactivity.main.g.director')
     def test_feed_rss_user(self, director):
-        from activitystream.storage.base import StoredActivity
-        from bson import ObjectId
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "_id": ObjectId("529fa331033c5e6406d8b338"),
             "obj": {
                 "activity_extras": {
@@ -290,23 +286,21 @@ class TestActivityController(TestController):
         })]
         r = self.app.get('/u/test-user-1/activity/feed.rss')
         timeline = r.xml.find('channel')
-        assert_equal(1, len(timeline.findall('item')))
+        assert 1 == len(timeline.findall('item'))
         activity = timeline.find('item')
-        assert_equal(activity.find('pubDate').text,
-                     'Wed, 04 Dec 2013 21:48:19 -0000')
-        assert_equal(activity.find('title').text,
-                     'Administrator 1 posted a comment on ticket #34')
-        assert_equal(activity.find('description').text,
-                     'Just wanted to leave a comment on this...')
-        assert_equal(activity.find('link').text,
-                     'http://localhost/p/test/tickets/34/?limit=25#ed7c')
+        assert (activity.find('pubDate').text ==
+                'Wed, 04 Dec 2013 21:48:19 -0000')
+        assert (activity.find('title').text ==
+                'Administrator 1 posted a comment on ticket #34')
+        assert (activity.find('description').text ==
+                'Just wanted to leave a comment on this...')
+        assert (activity.find('link').text ==
+                'http://localhost/p/test/tickets/34/?limit=25#ed7c')
 
     @td.with_tool('test', 'activity')
     @patch('forgeactivity.main.g.director')
     def test_feed_atom_project(self, director):
-        from activitystream.storage.base import StoredActivity
-        from bson import ObjectId
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "_id": ObjectId("529fa331033c5e6406d8b338"),
             "obj": {
                 "activity_extras": {
@@ -340,29 +334,27 @@ class TestActivityController(TestController):
         })]
         r = self.app.get('/p/test/activity/feed.atom')
         timeline = r.xml
-        assert_equal(1, len(timeline.findall(
-            '{http://www.w3.org/2005/Atom}entry')))
+        assert 1 == len(timeline.findall(
+            '{http://www.w3.org/2005/Atom}entry'))
         activity = timeline.find('{http://www.w3.org/2005/Atom}entry')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}published').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}published').text ==
             '2013-12-04T21:48:19Z')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}title').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}title').text ==
             'Administrator 1 posted a comment on ticket #34')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}summary').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}summary').text ==
             'Just wanted to leave a comment on this...')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}link').get('href'),
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}link').get('href') ==
             'http://localhost/p/test/tickets/34/?limit=25#ed7c')
 
     @td.with_tool('u/test-user-1', 'activity')
     @td.with_user_project('test-user-1')
     @patch('forgeactivity.main.g.director')
     def test_feed_atom_user(self, director):
-        from activitystream.storage.base import StoredActivity
-        from bson import ObjectId
-        director.get_timeline.return_value = [StoredActivity(**{
+        director.get_timeline.return_value = [Activity(**{
             "_id": ObjectId("529fa331033c5e6406d8b338"),
             "obj": {
                 "activity_extras": {
@@ -396,18 +388,82 @@ class TestActivityController(TestController):
         })]
         r = self.app.get('/u/test-user-1/activity/feed.atom')
         timeline = r.xml
-        assert_equal(1, len(timeline.findall(
-            '{http://www.w3.org/2005/Atom}entry')))
+        assert 1 == len(timeline.findall(
+            '{http://www.w3.org/2005/Atom}entry'))
         activity = timeline.find('{http://www.w3.org/2005/Atom}entry')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}published').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}published').text ==
             '2013-12-04T21:48:19Z')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}title').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}title').text ==
             'Administrator 1 posted a comment on ticket #34')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}summary').text,
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}summary').text ==
             'Just wanted to leave a comment on this...')
-        assert_equal(
-            activity.find('{http://www.w3.org/2005/Atom}link').get('href'),
+        assert (
+            activity.find('{http://www.w3.org/2005/Atom}link').get('href') ==
             'http://localhost/p/test/tickets/34/?limit=25#ed7c')
+
+    @td.with_tool('u/test-user-1', 'activity')
+    @td.with_user_project('test-user-1')
+    def test_delete_item_denied(self):
+        self.app.post('/u/test-user-1/activity/delete_item',
+                      {'activity_id': str(ObjectId())},
+                      status=403)
+
+    @td.with_tool('u/test-user-1', 'activity')
+    @td.with_user_project('test-user-1')
+    def test_delete_item_gone(self):
+        self.app.post('/u/test-user-1/activity/delete_item',
+                      {'activity_id': str(ObjectId())},
+                      extra_environ={'username': 'root'},  # nbhd admin
+                      status=410)
+
+    @td.with_tool('u/test-user-1', 'activity')
+    @td.with_user_project('test-user-1')
+    def test_delete_item_success(self):
+        activity_data = {
+            "obj": {
+                "activity_extras": {
+                    "summary": "Sensitive private info, oops"
+                },
+                "activity_url": "/p/test/tickets/34/?limit=25#ed7c",
+                "activity_name": "a comment"
+            },
+            "target": {
+                "activity_extras": {
+                    "allura_id": "Ticket:529f57a6033c5e5985db2efa",
+                    "summary": "Make activitystream timeline look better"
+                },
+                "activity_url": "/p/test/tickets/34/",
+                "activity_name": "ticket #34"
+            },
+            "actor": {
+                "activity_extras": {
+                    "icon_url": "/u/test-admin/user_icon",
+                    "allura_id": "User:521f96cb033c5e2587adbdff"
+                },
+                "activity_url": "/u/test-admin/",
+                "activity_name": "Administrator 1",
+                "node_id": "User:521f96cb033c5e2587adbdff"
+            },
+            "verb": "posted",
+            "published": dateutil.parser.parse("2013-12-04T21:48:19.817"),
+            "score": 1386193699,
+            "node_id": "Project:527a6584033c5e62126f5a60",
+            "owner_id": "Project:527a6584033c5e62126f5a60"
+        }
+        activity = Activity(**activity_data)
+        activity2 = Activity(**dict(activity_data, node_id='Project:123', owner_id='User:456'))
+        activity3 = Activity(**dict(activity_data, node_id='User:abc', owner_id='User:abc'))
+        ThreadLocalODMSession.flush_all()
+        activity_id = str(activity._id)
+        assert Activity.query.find({'obj.activity_extras.summary': 'Sensitive private info, oops'}).count() == 3
+
+        self.app.post('/u/test-user-1/activity/delete_item',
+                      {'activity_id': activity_id},
+                      extra_environ={'username': 'root'},  # nbhd admin
+                      status=200)
+        ThreadLocalODMSession.flush_all()
+
+        assert Activity.query.find({'obj.activity_extras.summary': 'Sensitive private info, oops'}).count() == 0
