@@ -15,32 +15,35 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-#-*- python -*-
 import logging
-import urllib2
+import six.moves.urllib.parse
+import six.moves.urllib.request
+import six.moves.urllib.error
 import json
 
 # Non-stdlib imports
 import pymongo
 from tg import config, expose, validate, redirect, flash, jsonify
 from tg.decorators import with_trailing_slash, without_trailing_slash
-from pylons import tmpl_context as c
-from pylons import app_globals as g
-from pylons import request
+from tg import tmpl_context as c
+from tg import app_globals as g
+from tg import request
 from paste.deploy.converters import asbool
 import formencode
 from formencode import validators
 from webob import exc
 
-from ming.orm import session
+from ming.odm import session
 
 # Allura-specific imports
 from allura.app import Application, SitemapEntry, ConfigOption
 from allura.app import DefaultAdminController
 from allura.lib import helpers as h
+from allura.lib import validators as v
 from allura.lib.utils import JSONForExport
+from allura.tasks import notification_tasks
 from allura.lib.search import search_app
-from allura.lib.decorators import require_post
+from allura.lib.decorators import require_post, memorable_forget
 from allura.lib.security import has_access, require_access
 from allura.lib import widgets as w
 from allura.lib import exceptions as forge_exc
@@ -49,6 +52,7 @@ from allura.lib.widgets import form_fields as ffw
 from allura.lib.widgets.search import SearchResults, SearchHelp
 from allura import model as M
 from allura.controllers import BaseController, AppDiscussionController, AppDiscussionRestController
+from allura.controllers.attachments import AttachmentController, AttachmentsController
 from allura.controllers.rest import AppRestControllerMixin
 from allura.controllers.feed import FeedArgs, FeedController
 
@@ -56,6 +60,7 @@ from allura.controllers.feed import FeedArgs, FeedController
 from forgeblog import model as BM
 from forgeblog import version
 from forgeblog import widgets
+import six
 
 log = logging.getLogger(__name__)
 
@@ -68,13 +73,19 @@ class W:
     new_post_form = widgets.NewPostForm()
     edit_post_form = widgets.EditPostForm()
     view_post_form = widgets.ViewPostForm()
-    label_edit = ffw.LabelEdit()
-    attachment_add = ffw.AttachmentAdd()
     attachment_list = ffw.AttachmentList()
+    confirmation = ffw.Lightbox(name='confirm',
+                                trigger='a.post-link',
+                                options="{ modalCSS: { minHeight: 0, width: 'inherit', top: '150px'}}")
     preview_post_form = widgets.PreviewPostForm()
     subscribe_form = SubscribeForm(thing='post')
     search_results = SearchResults()
-    help_modal = SearchHelp()
+    help_modal = SearchHelp(fields={'title': 'Title',
+                                    'text': '"Post text"',
+                                    'labels_t': 'Labels',
+                                    'mod_date_dt': 'Last modified.  Example: mod_date_dt:[2018-01-01T00:00:00Z TO *]',
+                                    'author_user_name_t': 'username (for comments only)',
+                                    })
 
 
 class ForgeBlogApp(Application):
@@ -144,6 +155,26 @@ class ForgeBlogApp(Application):
             return [
                 SitemapEntry(menu_id, '.')[self.sidebar_menu()]]
 
+    def sitemap_xml(self):
+        """
+        Used for generating sitemap.xml.
+        If the root page has default content, omit it from the sitemap.xml.
+        Assumes :attr:`main_menu` will return an entry pointing to the root page.
+        :return: a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
+        """
+        if self.should_noindex():
+            return []
+        return self.main_menu()
+
+    def should_noindex(self):
+        post = BM.BlogPost.query.get(
+            app_config_id=self.config._id,
+            state='published',
+            deleted=False)
+        if post:
+            return False
+        return True
+
     @property
     def show_discussion(self):
         if 'show_discussion' in self.config.options:
@@ -173,7 +204,7 @@ class ForgeBlogApp(Application):
         admin_url = c.project.url() + 'admin/' + \
             self.config.options.mount_point + '/'
         # temporarily disabled until some bugs are fixed
-        links = super(ForgeBlogApp, self).admin_menu(force_options=True)
+        links = super().admin_menu(force_options=True)
         # We don't want external feeds in menu unless they're enabled
         if asbool(config.get('forgeblog.exfeed', 'false')):
             links.insert(0, SitemapEntry('External feeds',
@@ -183,7 +214,7 @@ class ForgeBlogApp(Application):
 
     def install(self, project):
         'Set up any default permissions and roles here'
-        super(ForgeBlogApp, self).install(project)
+        super().install(project)
 
         # Setup permissions
         role_admin = M.ProjectRole.by_name('Admin')._id
@@ -202,23 +233,23 @@ class ForgeBlogApp(Application):
 
     def uninstall(self, project):
         "Remove all the tool's artifacts from the database"
-        BM.Attachment.query.remove(dict(app_config_id=c.app.config._id))
+        BM.BlogAttachment.query.remove(dict(app_config_id=c.app.config._id))
         BM.BlogPost.query.remove(dict(app_config_id=c.app.config._id))
         BM.BlogPostSnapshot.query.remove(dict(app_config_id=c.app.config._id))
-        super(ForgeBlogApp, self).uninstall(project)
+        super().uninstall(project)
 
     def bulk_export(self, f, export_path='', with_attachments=False):
         f.write('{"posts": [')
         posts = list(BM.BlogPost.query.find(dict(app_config_id=self.config._id)))
         if with_attachments:
-            GenericJSON = JSONForExport
+            JSONEncoder = JSONForExport
             self.export_attachments(posts, export_path)
         else:
-            GenericJSON = jsonify.GenericJSON
+            JSONEncoder = jsonify.JSONEncoder
         for i, post in enumerate(posts):
             if i > 0:
                 f.write(',')
-            json.dump(post, f, cls=GenericJSON, indent=2)
+            json.dump(post, f, cls=JSONEncoder, indent=2)
         f.write(']}')
 
     def export_attachments(self, articles, export_path):
@@ -233,14 +264,6 @@ class ForgeBlogApp(Application):
                 self.save_attachments(post_path, post.attachments)
 
 
-def rate_limit():
-    if BM.BlogPost.is_limit_exceeded(c.app.config, user=c.user):
-        msg = 'Create/edit rate limit exceeded. '
-        log.warn(msg + c.app.config.url())
-        flash(msg + 'Please try again later.', 'error')
-        redirect(c.app.config.url())
-
-
 class RootController(BaseController, FeedController):
 
     def __init__(self):
@@ -269,7 +292,7 @@ class RootController(BaseController, FeedController):
 
     @with_trailing_slash
     @expose('jinja:forgeblog:templates/blog/search.html')
-    @validate(dict(q=validators.UnicodeString(if_empty=None),
+    @validate(dict(q=v.UnicodeString(if_empty=None),
                    history=validators.StringBool(if_empty=False),
                    search_comments=validators.StringBool(if_empty=False),
                    project=validators.StringBool(if_empty=False)))
@@ -293,35 +316,37 @@ class RootController(BaseController, FeedController):
     @without_trailing_slash
     def new(self, **kw):
         require_access(c.app, 'write')
-        rate_limit()
+        self.rate_limit(BM.BlogPost, 'Create/edit', c.app.config.url())
         post = dict(
             state='published')
         c.form = W.new_post_form
-        return dict(post=post)
+        return dict(post=post,
+                    subscribed_to_tool=M.Mailbox.subscribed(),
+                    )
 
+    @memorable_forget()
     @expose()
     @require_post()
-    @validate(form=W.edit_post_form, error_handler=new)
+    # both new & edit submit here, but validate with new since it adds a field (subscribe)
+    @validate(form=W.new_post_form, error_handler=new)
     @without_trailing_slash
     def save(self, **kw):
         require_access(c.app, 'write')
-        rate_limit()
+        self.rate_limit(BM.BlogPost, 'Create/edit', c.app.config.url())
+        attachment = kw.pop('attachment', None)
         post = BM.BlogPost.new(**kw)
-        g.spam_checker.check(kw['title'] + u'\n' + kw['text'], artifact=post,
+        g.spam_checker.check(kw['title'] + '\n' + kw['text'], artifact=post,
                              user=c.user, content_type='blog-post')
-        redirect(h.really_unicode(post.url()).encode('utf-8'))
-
-    @with_trailing_slash
-    @expose('jinja:allura:templates/markdown_syntax_dialog.html')
-    def markdown_syntax_dialog(self, **kw):
-        'Static dialog page about how to use markdown.'
-        return dict()
+        if attachment is not None:
+            post.add_multiple_attachments(attachment)
+        notification_tasks.send_usermentions_notification.post(post.index_id(), kw['text'])
+        redirect(h.urlquote(h.really_unicode(post.url())))
 
     @expose()
     def _lookup(self, year=None, month=None, name=None, *rest):
         if year is None or month is None or name is None:
             raise exc.HTTPNotFound()
-        slug = '/'.join((year, month, urllib2.unquote(name).decode('utf-8')))
+        slug = '/'.join((year, month, six.ensure_text(six.moves.urllib.parse.unquote(name))))
         post = BM.BlogPost.query.get(slug=slug, app_config_id=c.app.config._id)
         if post is None:
             raise exc.HTTPNotFound()
@@ -340,22 +365,32 @@ class RootController(BaseController, FeedController):
             app.url)
 
 
+class BlogAttachmentController(AttachmentController):
+    AttachmentClass = BM.BlogAttachment
+
+
+class BlogAttachmentsController(AttachmentsController):
+    AttachmentControllerClass = BlogAttachmentController
+
+
 class PostController(BaseController, FeedController):
 
-    def __init__(self, post):
+    def __init__(self, post: BM.BlogPost):
         self.post = post
+        self.attachment = BlogAttachmentsController(self.post)
 
     def _check_security(self):
         require_access(self.post, 'read')
+        if self.post.state == 'draft':
+            require_access(self.post, 'write')
 
     @expose('jinja:forgeblog:templates/blog/post.html')
     @with_trailing_slash
     @validate(dict(page=validators.Int(if_empty=0, if_invalid=0),
                    limit=validators.Int(if_empty=None, if_invalid=None)))
     def index(self, page=0, limit=None, **kw):
-        if self.post.state == 'draft':
-            require_access(self.post, 'write')
         c.form = W.view_post_form
+        c.attachment_list = W.attachment_list
         c.subscribe_form = W.subscribe_form
         c.thread = W.thread
         post_count = self.post.discussion_thread.post_count
@@ -373,16 +408,17 @@ class PostController(BaseController, FeedController):
     @without_trailing_slash
     def edit(self, **kw):
         require_access(self.post, 'write')
-        rate_limit()
+        self.rate_limit(BM.BlogPost, 'Create/edit', c.app.config.url())
         c.form = W.edit_post_form
-        c.attachment_add = W.attachment_add
         c.attachment_list = W.attachment_list
-        c.label_edit = W.label_edit
-        return dict(post=self.post)
+        return dict(post=self.post,
+                    subscribed_to_tool=M.Mailbox.subscribed(),
+                    )
 
     @without_trailing_slash
     @expose('jinja:forgeblog:templates/blog/post_history.html')
     def history(self, **kw):
+        c.confirmation = W.confirmation
         posts = self.post.history()
         return dict(title=self.post.title, posts=posts)
 
@@ -394,35 +430,63 @@ class PostController(BaseController, FeedController):
         result = h.diff_text(p1.text, p2.text)
         return dict(p1=p1, p2=p2, edits=result)
 
+    @memorable_forget()
     @expose()
     @require_post()
     @validate(form=W.edit_post_form, error_handler=edit)
     @without_trailing_slash
     def save(self, delete=None, **kw):
         require_access(self.post, 'write')
-        rate_limit()
+        self.rate_limit(BM.BlogPost, 'Create/edit', c.app.config.url())
         if delete:
             self.post.delete()
             flash('Post deleted', 'info')
-            redirect(h.really_unicode(c.app.url).encode('utf-8'))
+            redirect(h.urlquote(h.really_unicode(c.app.url)))
         else:
-            g.spam_checker.check(kw['title'] + u'\n' + kw['text'], artifact=self.post,
+            g.spam_checker.check(kw['title'] + '\n' + kw['text'], artifact=self.post,
                                  user=c.user, content_type='blog-post')
-        for k, v in kw.iteritems():
-            setattr(self.post, k, v)
+        attachment = kw.pop('attachment', None)
+        old_text = self.post.text
+        if attachment is not None:
+            self.post.add_multiple_attachments(attachment)
+        for k, val in kw.items():
+            setattr(self.post, k, val)
         self.post.commit()
+        notification_tasks.send_usermentions_notification.post(self.post.index_id(), kw['text'], old_text)
         redirect('.')
 
     @without_trailing_slash
+    @expose('json:')
     @require_post()
+    def update_markdown(self, text=None, **kw):
+        if has_access(self.post, 'edit'):
+            self.post.text = text
+            self.post.commit()
+            g.spam_checker.check(text, artifact=self.post,
+                                 user=c.user, content_type='blog-post')
+            return {
+                'status': 'success'
+            }
+        else:
+            return {
+                'status': 'no_permission'
+            }
+
     @expose()
+    @without_trailing_slash
+    def get_markdown(self):
+        return self.post.text
+
+    @without_trailing_slash
+    @require_post()
+    @expose('json:')
     def revert(self, version, **kw):
         require_access(self.post, 'write')
         orig = self._get_version(version)
         if orig:
             self.post.text = orig.text
         self.post.commit()
-        redirect('.')
+        return dict(location='.')
 
     @expose('json:')
     @require_post()
@@ -471,18 +535,26 @@ class BlogAdminController(DefaultAdminController):
     @expose()
     @require_post()
     def set_options(self, show_discussion=False, allow_email_posting=False):
-        self.app.config.options[
-            'show_discussion'] = show_discussion and True or False
-        self.app.config.options[
-            'AllowEmailPosting'] = allow_email_posting and True or False
+        mount_point = self.app.config.options['mount_point']
+
+        if self.app.config.options.get('show_discussion') != bool(show_discussion):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "show_discussion", self.app.config.options.get('show_discussion'),
+                bool(show_discussion)))
+            self.app.config.options['show_discussion'] = bool(show_discussion)
+
+        if self.app.config.options.get('AllowEmailPosting') != bool(allow_email_posting):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "AllowEmailPosting", self.app.config.options.get('AllowEmailPosting'),
+                bool(allow_email_posting)))
+            self.app.config.options['AllowEmailPosting'] = bool(allow_email_posting)
+
         flash('Blog options updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @without_trailing_slash
     @expose('jinja:forgeblog:templates/blog/admin_exfeed.html')
     def exfeed(self):
-        #self.app.external_feeds_list = ['feed1', 'feed2']
-        #log.info("EXFEED: %s" % self.app.external_feeds_list)
         feeds_list = []
         for feed in self.app.external_feeds_list:
             feeds_list.append(feed)
@@ -495,7 +567,7 @@ class BlogAdminController(DefaultAdminController):
     @require_post()
     def set_exfeed(self, new_exfeed=None, **kw):
         exfeed_val = kw.get('exfeed', [])
-        if type(exfeed_val) == unicode:
+        if isinstance(exfeed_val, str):
             tmp_exfeed_list = []
             tmp_exfeed_list.append(exfeed_val)
         else:
@@ -514,13 +586,27 @@ class BlogAdminController(DefaultAdminController):
             except formencode.api.Invalid:
                 invalid_list.append(link)
 
+        added_feeds = set(exfeed_list).difference(self.app.external_feeds_list)
+        removed_feeds = set(self.app.external_feeds_list).difference(exfeed_list)
+
+        if added_feeds:
+            M.AuditLog.log('{}: external feed list - added: {}'.format(
+                self.app.config.options['mount_point'],
+                ', '.join(sorted(added_feeds))
+            ))
+        if removed_feeds:
+            M.AuditLog.log('{}: external feed list - removed: {}'.format(
+                self.app.config.options['mount_point'],
+                ', '.join(sorted(removed_feeds))
+            ))
+
         self.app.external_feeds_list = exfeed_list
         flash('External feeds updated')
         if len(invalid_list) > 0:
             flash('Invalid link(s): %s' %
                   ','.join(link for link in invalid_list), 'error')
 
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class RootRestController(BaseController, AppRestControllerMixin):
@@ -536,7 +622,7 @@ class RootRestController(BaseController, AppRestControllerMixin):
         if request.method == 'POST':
             require_access(c.app, 'write')
             if BM.BlogPost.is_limit_exceeded(c.app.config, user=c.user):
-                log.warn('Create/edit rate limit exceeded. %s', c.app.config.url())
+                log.warning('Create/edit rate limit exceeded. %s', c.app.config.url())
                 raise forge_exc.HTTPTooManyRequests()
             post = BM.BlogPost.new(
                 title=title,
@@ -544,7 +630,7 @@ class RootRestController(BaseController, AppRestControllerMixin):
                 text=text,
                 labels=labels.split(','),
                 **kw)
-            return exc.HTTPCreated(headers=dict(Location=h.absurl('/rest' + post.url())))
+            return exc.HTTPCreated(headers=dict(Location=str(h.absurl('/rest' + post.url()))))
 
         else:
             result = RootController().index(limit=limit, page=page)
@@ -560,7 +646,7 @@ class RootRestController(BaseController, AppRestControllerMixin):
     def _lookup(self, year=None, month=None, title=None, *rest):
         if not (year and month and title):
             raise exc.HTTPNotFound()
-        slug = '/'.join((year, month, urllib2.unquote(title).decode('utf-8')))
+        slug = '/'.join((year, month, six.ensure_text(six.moves.urllib.parse.unquote(title))))
         post = BM.BlogPost.query.get(slug=slug, app_config_id=c.app.config._id)
         if not post:
             raise exc.HTTPNotFound()
@@ -589,7 +675,7 @@ class PostRestController(BaseController):
     def _update_post(self, **post_data):
         require_access(self.post, 'write')
         if BM.BlogPost.is_limit_exceeded(c.app.config, user=c.user):
-            log.warn('Create/edit rate limit exceeded. %s', c.app.config.url())
+            log.warning('Create/edit rate limit exceeded. %s', c.app.config.url())
             raise forge_exc.HTTPTooManyRequests()
         if 'delete' in post_data:
             self.post.delete()
