@@ -15,4 +15,4 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from chat import ChatChannel, ChatMessage
+from .chat import ChatChannel, ChatMessage
