@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -18,6 +16,7 @@
 #       under the License.
 
 import mock
+import pytest
 import random
 import logging
 from email.mime.text import MIMEText
@@ -26,11 +25,12 @@ from email.mime.multipart import MIMEMultipart
 
 import pkg_resources
 import pymongo
+import webtest
 
-from ming.odm import ThreadLocalORMSession
-from pylons import tmpl_context as c
+from ming.odm import ThreadLocalODMSession
+from tg import tmpl_context as c
+from tg import config
 
-from nose.tools import assert_equal, assert_in, assert_not_in, assert_true, assert_false
 import feedparser
 
 from allura import model as M
@@ -45,8 +45,8 @@ log = logging.getLogger(__name__)
 
 
 class TestForumEmail(TestController):
-    def setUp(self):
-        TestController.setUp(self)
+    def setup_method(self, method):
+        super().setup_method(method)
         c.user = M.User.by_username('test-admin')
         self.app.get('/discussion/')
         r = self.app.get('/admin/discussion/forums')
@@ -136,8 +136,8 @@ class TestForumMessageHandling(TestController):
     Tests all the "handle_message" related logic, which is what inbound emails run through
     '''
 
-    def setUp(self):
-        TestController.setUp(self)
+    def setup_method(self, method):
+        super().setup_method(method)
         self.app.get('/discussion/')
         r = self.app.get('/admin/discussion/forums')
         form = r.forms['add-forum']
@@ -156,8 +156,8 @@ class TestForumMessageHandling(TestController):
         self.user = M.User.query.get(username='root')
 
     def test_has_access(self):
-        assert_false(c.app.has_access(M.User.anonymous(), 'testforum'))
-        assert_true(c.app.has_access(M.User.query.get(username='root'), 'testforum'))
+        assert not c.app.has_access(M.User.anonymous(), 'testforum')
+        assert c.app.has_access(M.User.query.get(username='root'), 'testforum')
 
     def test_post(self):
         self._post('testforum', 'Test Thread', 'Nothing here')
@@ -168,19 +168,19 @@ class TestForumMessageHandling(TestController):
     def test_reply(self):
         self._post('testforum', 'Test Thread', 'Nothing here',
                    message_id='test_reply@domain.net')
-        assert_equal(FM.ForumThread.query.find().count(), 1)
+        assert FM.ForumThread.query.find().count() == 1
         posts = FM.ForumPost.query.find()
-        assert_equal(posts.count(), 1)
-        assert_equal(FM.ForumThread.query.get().num_replies, 1)
-        assert_equal(FM.ForumThread.query.get().first_post_id, 'test_reply@domain.net')
+        assert posts.count() == 1
+        assert FM.ForumThread.query.get().num_replies == 1
+        assert FM.ForumThread.query.get().first_post_id == 'test_reply@domain.net'
 
         post = posts.first()
         self._post('testforum', 'Test Reply', 'Nothing here, either',
                    message_id='test_reply-msg2@domain.net',
                    in_reply_to=['test_reply@domain.net'])
-        assert_equal(FM.ForumThread.query.find().count(), 1)
-        assert_equal(FM.ForumPost.query.find().count(), 2)
-        assert_equal(FM.ForumThread.query.get().first_post_id, 'test_reply@domain.net')
+        assert FM.ForumThread.query.find().count() == 1
+        assert FM.ForumPost.query.find().count() == 2
+        assert FM.ForumThread.query.get().first_post_id == 'test_reply@domain.net'
 
     def test_reply_using_references_headers(self):
         self._post('testforum', 'Test Thread', 'Nothing here',
@@ -193,8 +193,8 @@ class TestForumMessageHandling(TestController):
                    message_id='second-message-id',
                    in_reply_to=['some-other-id@not.helpful.com'],
                    references=refs)
-        assert_equal(FM.ForumThread.query.find().count(), 1)
-        assert_equal(FM.ForumPost.query.find().count(), 2)
+        assert FM.ForumThread.query.find().count() == 1
+        assert FM.ForumPost.query.find().count() == 2
 
         prev_post = FM.ForumPost.query.find().sort('timestamp', pymongo.DESCENDING).first()
         refs = M.Notification._references(thread, prev_post) + ['second-message-id']
@@ -202,43 +202,46 @@ class TestForumMessageHandling(TestController):
                    message_id='third-message-id',
                    # missing in_reply_to altogether
                    references=refs)
-        assert_equal(FM.ForumThread.query.find().count(), 1)
-        assert_equal(FM.ForumPost.query.find().count(), 3)
+        assert FM.ForumThread.query.find().count() == 1
+        assert FM.ForumPost.query.find().count() == 3
 
     def test_attach(self):
-        self._post('testforum', 'Attachment Thread', 'This is a text file',
+        # runs handle_artifact_message() with filename field
+        self._post('testforum', 'Attachment Thread', 'This is text attachment',
                    message_id='test.attach.100@domain.net',
                    filename='test.txt',
                    content_type='text/plain')
-        self._post('testforum', 'Test Thread', 'Nothing here',
+        # runs handle_artifact_message() where there's no post with given message_id yet
+        self._post('testforum', 'Test Thread', b'Nothing here',
                    message_id='test.attach.100@domain.net')
-        self._post('testforum', 'Attachment Thread', 'This is a text file',
+        # runs handle_artifact_message() where there IS a post with given message_id
+        self._post('testforum', 'Attachment Thread', 'This is binary ¶¬¡™£¢¢•º™™¶'.encode(),
                    message_id='test.attach.100@domain.net',
                    content_type='text/plain')
 
     def test_threads(self):
         self._post('testforum', 'Test', 'test')
         thd = FM.ForumThread.query.find().first()
-        url = str('/discussion/testforum/thread/%s/' % thd._id)
+        url = '/discussion/testforum/thread/%s/' % thd._id
         self.app.get(url)
-        # accessing a non-existent thread should return a 404
-        self.app.get('/discussion/testforum/thread/foobar/', status=404)
+        resp = self.app.get('/discussion/testforum/thread/foobar/', status=301)
+        assert('/discussion/testforum/' in resp.location)
 
     def test_posts(self):
         # not sure why this fails when set to root (to match self.user_id)
         c.user = M.User.by_username('test-admin')
         self._post('testforum', 'Test', 'test')
         thd = FM.ForumThread.query.find().first()
-        thd_url = str('/discussion/testforum/thread/%s/' % thd._id)
+        thd_url = '/discussion/testforum/thread/%s/' % thd._id
         r = self.app.get(thd_url)
         p = FM.ForumPost.query.find().first()
-        url = str('/discussion/testforum/thread/%s/%s/' % (thd._id, p.slug))
+        url = str(f'/discussion/testforum/thread/{thd._id}/{p.slug}/')
         r = self.app.get(url)
         f = r.html.find('form', {'action': '/p/test' + url})
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params['subject'] = 'New Subject'
         params['text'] = 'Asdf'
@@ -249,7 +252,7 @@ class TestForumMessageHandling(TestController):
         params = dict()
         inputs = post_form.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[post_form.find('textarea')['name']] = 'text'
         r = self.app.post(url + 'reply', params=params)
@@ -260,13 +263,13 @@ class TestForumMessageHandling(TestController):
         r = self.app.get(thd_url + reply.slug + '/')
         # Check attachments
         r = self.app.post(url + 'attach',
-                          upload_files=[('file_info', 'test.txt', 'This is a textfile')])
+                          upload_files=[('file_info', 'test.txt', b'This is a textfile')])
         r = self.app.post(url + 'attach',
                           upload_files=[('file_info', 'test.asdfasdtxt',
-                                         'This is a textfile')])
+                                         b'This is a textfile')])
         r = self.app.post(url + 'attach',
-                          upload_files=[('file_info', 'test1.txt', 'This is a textfile'),
-                                        ('file_info', 'test2.txt', 'This is a textfile')])
+                          upload_files=[('file_info', 'test1.txt', b'This is a textfile'),
+                                        ('file_info', 'test2.txt', b'This is a textfile')])
         r = self.app.get(url)
         assert "test1.txt" in r
         assert "test2.txt" in r
@@ -299,8 +302,8 @@ class TestForumMessageHandling(TestController):
 
 
 class TestForum(TestController):
-    def setUp(self):
-        TestController.setUp(self)
+    def setup_method(self, method):
+        super().setup_method(method)
         self.app.get('/discussion/')
         r = self.app.get('/admin/discussion/forums')
         form = r.forms['add-forum']
@@ -350,11 +353,11 @@ class TestForum(TestController):
     def test_unicode_name(self):
         r = self.app.get('/admin/discussion/forums')
         form = r.forms['add-forum']
-        form['add_forum.shortname'] = u'téstforum'.encode('utf-8')
-        form['add_forum.name'] = u'Tést Forum'.encode('utf-8')
+        form['add_forum.shortname'] = 'téstforum'.encode()
+        form['add_forum.name'] = 'Tést Forum'.encode()
         form.submit()
         r = self.app.get('/admin/discussion/forums')
-        assert u'téstforum'.encode('utf-8') in r
+        assert 'téstforum'.encode() in r
 
     def test_markdown_description(self):
         r = self.app.get('/admin/discussion/forums')
@@ -364,16 +367,12 @@ class TestForum(TestController):
         form['add_forum.description'] = '<a href="http://cnn.com">This is CNN</a>'
         form.submit()
         r = self.app.get('/discussion/')
-        assert_equal(len(r.html.findAll('a', rel='nofollow')), 1)
+        assert len(r.html.findAll('a', rel='nofollow')) == 2
 
     def test_forum_search(self):
         self.app.get('/discussion/search')
         self.app.get('/discussion/search', params=dict(q='foo'))
 
-    def test_render_markdown_syntax(self):
-        r = self.app.get('/discussion/markdown_syntax')
-        assert 'Markdown Syntax' in r
-
     def test_forum_index(self):
         self.app.get('/discussion/testforum/')
         self.app.get('/discussion/testforum/childforum/')
@@ -394,7 +393,7 @@ class TestForum(TestController):
             params = dict()
             inputs = f.findAll('input')
             for field in inputs:
-                if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+                if field.has_attr('name'):
                     params[field['name']] = field.get('value') or ''
             params[f.find('textarea')['name']] = '1st post in Zero Posts thread'
             params[f.find('select')['name']] = 'testforum'
@@ -444,13 +443,37 @@ class TestForum(TestController):
         input_field = r.html.fieldset.find('input', {'value': username})
         assert input_field is None
 
+    def test_save_moderation_bulk_user(self):
+        # create posts
+        for i in range(5):
+            r = self.app.get('/discussion/create_topic/')
+            f = r.html.find(
+                'form', {'action': '/p/test/discussion/save_new_topic'})
+            params = dict()
+            inputs = f.findAll('input')
+            for field in inputs:
+                if field.has_attr('name'):
+                    params[field['name']] = field.get('value') or ''
+            params[f.find('textarea')['name']] = 'Post text'
+            params[f.find('select')['name']] = 'testforum'
+            params[f.find('input', {'style': 'width: 90%'})['name']] = "this is my post"
+            r = self.app.post('/discussion/save_new_topic', params=params)
+
+        assert 5 == FM.ForumPost.query.find({'status': 'ok'}).count()
+
+        r = self.app.post('/discussion/testforum/moderate/save_moderation_bulk_user', params={
+            'username': 'test-admin',
+            'spam': '1'})
+        assert '5 posts marked as spam' in self.webflash(r)
+        assert 5 == FM.ForumPost.query.find({'status': 'spam'}).count()
+
     def test_posting(self):
         r = self.app.get('/discussion/create_topic/')
         f = r.html.find('form', {'action': '/p/test/discussion/save_new_topic'})
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'This is a *test thread*'
         params[f.find('select')['name']] = 'testforum'
@@ -467,6 +490,15 @@ class TestForum(TestController):
         assert 'noreply' not in n.reply_to_address, n
         assert 'testforum@discussion.test.p' in n.reply_to_address, n
 
+    def test_new_topic_rate_limit(self):
+        with h.push_config(config, **{'forgediscussion.rate_limits_per_user': '{"3600": 1}'}):
+            # first one should succeed
+            self.test_posting()
+
+            # second should fail
+            with pytest.raises(Exception):
+                self.test_posting()
+
     def test_notifications_escaping(self):
         r = self.app.get('/discussion/create_topic/')
         f = r.html.find(
@@ -474,7 +506,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'Post text'
         params[f.find('select')['name']] = 'testforum'
@@ -482,7 +514,7 @@ class TestForum(TestController):
         r = self.app.post('/discussion/save_new_topic', params=params)
         n = M.Notification.query.find(
             dict(subject="[test:discussion] this is <h2> o'clock")).first()
-        assert_in('---\n\n[this is &lt;h2&gt; o&#39;clock]', n.text)
+        assert '---\n\n[this is &lt;h2&gt; o&#39;clock]' in n.text
 
     def _set_anon_allowed(self):
         r = self.app.get('/admin/discussion/permissions')
@@ -491,57 +523,60 @@ class TestForum(TestController):
         opt_auth = select.find(text='*authenticated').parent
         opt_admin = select.find(text='Admin').parent
         r = self.app.post('/admin/discussion/update', params={
-            'card-0.value': opt_admin['value'],
             'card-0.id': 'admin',
+            'card-0.value': opt_admin['value'],
             'card-4.id': 'read',
             'card-4.value': opt_anon['value'],
+            'card-3.id': 'post',
             'card-3.value': opt_auth['value'],
             'card-3.new': opt_anon['value'],
-            'card-3.id': 'post'})
+        })
 
     @mock.patch('allura.model.discuss.g.spam_checker')
     def test_anonymous_post(self, spam_checker):
-        spam_checker.check.return_value = True
+        spam_checker.check.return_value = False
         self._set_anon_allowed()
         r = self.app.get('/discussion/create_topic/')
         f = r.html.find('form', {'action': '/p/test/discussion/save_new_topic'})
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'Post content'
         params[f.find('select')['name']] = 'testforum'
         params[f.find('input', {'style': 'width: 90%'})['name']] = 'Test Thread'
         thread = self.app.post('/discussion/save_new_topic', params=params,
-                               extra_environ=dict(username='*anonymous')).follow()
+                               extra_environ=dict(username='*anonymous'))
 
-        # assert post awaiting moderation
-        r = self.app.get(thread.request.url,
+        # assert post return 404 but content can still be seen and moderated
+        thread_url = thread.response.headers['Location']
+        r = self.app.get(thread_url, status=404,
                          extra_environ=dict(username='*anonymous'))
         assert 'Post awaiting moderation' in r
         assert 'name="delete"' not in r
         assert 'name="approve"' not in r
         assert 'name="spam"' not in r
-        assert_equal(spam_checker.check.call_args[0][0], 'Test Thread\nPost content')
+        assert "Post content" not in r
+        assert spam_checker.check.call_args[0][0] == 'Test Thread\nPost content'
 
         # assert unapproved thread replies do not appear
-        f = thread.html.find('div', {'class': 'row reply_post_form'}).find('form')
+        f = r.html.find('div', {'class': 'comment-row reply_post_form'}).find('form')
         rep_url = f.get('action')
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'anon reply to anon post content'
         r = self.app.post(str(rep_url), params=params, extra_environ=dict(username='*anonymous'))
-        r = self.app.get(thread.request.url,
+        r = self.app.get(thread_url, status=404,
                          extra_environ=dict(username='*anonymous'))
         assert 'anon reply to anon post' not in r
-        assert_equal(spam_checker.check.call_args[0][0], 'anon reply to anon post content')
+        assert spam_checker.check.call_args[0][0] == 'anon reply to anon post content'
 
         # assert moderation controls appear for admin
-        r = self.app.get(thread.request.url)
+        r = self.app.get(thread_url, extra_environ=dict(username='test-admin'), status=404)
         assert '<div class="display_post moderate">' in r
         assert '<i class="fa fa-reply"></i>' in r
 
@@ -554,11 +589,33 @@ class TestForum(TestController):
         r = self.app.get('/discussion/testforum/moderate/')
         post = FM.ForumPost.query.get(text='Post content')
         post2 = FM.ForumPost.query.get(text='anon reply to anon post content')
-        link = '<a href="%s">[%s]</a>' % (post.thread.url() + '?limit=25#' + post.slug, post.shorthand_id())
+        link = '<a href="{}">[{}]</a>'.format(post.thread.url() + '?limit=25#' + post.slug, post.shorthand_id())
         assert link in r, link
-        link = '<a href="%s">[%s]</a>' % (post2.thread.url() + '?limit=25#' + post2.slug, post2.shorthand_id())
+        link = '<a href="{}">[{}]</a>'.format(post2.thread.url() + '?limit=25#' + post2.slug, post2.shorthand_id())
         assert link in r, link
 
+        # approve posts
+        r = self.app.post('/discussion/testforum/moderate/save_moderation', params={
+            'post-0._id': post._id,
+            'post-0.checked': 'on',
+            'approve': 'Approve Marked'})
+
+        post = FM.ForumPost.query.get(text='Post content')
+        post2 = FM.ForumPost.query.get(text='anon reply to anon post content')
+        assert 'ok' == post.status
+        assert 'pending' == post2.status
+
+        # assert anon can't edit their original post
+        r = self.app.get(thread_url,
+                         extra_environ=dict(username='*anonymous'))
+        assert 'Post content' in r
+        post_container = r.html.find('div', {'id': post.slug})
+
+        btn_edit = post_container.find('a', {'title': 'Edit'})
+        assert not btn_edit
+
+
+
     @td.with_tool('test2', 'Discussion', 'discussion')
     @mock.patch('allura.model.discuss.g.spam_checker')
     def test_is_spam(self, spam_checker):
@@ -566,11 +623,11 @@ class TestForum(TestController):
         c.user = M.User.query.get(username="test-user")
         role = M.ProjectRole(project_id=c.project._id, name='TestRole')
         M.ProjectRole.by_user(c.user, upsert=True).roles.append(role._id)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         t = M.Thread()
         p = M.Post(thread=t)
-        assert_in('TestRole', [r.name for r in c.project.named_roles])
-        assert_false(t.is_spam(p))
+        assert 'TestRole' in [r.name for r in c.project.named_roles]
+        assert not t.is_spam(p)
 
     def test_thread(self):
         r = self.app.get('/discussion/create_topic/')
@@ -579,7 +636,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -588,12 +645,12 @@ class TestForum(TestController):
         url = thread.request.url
 
         # test reply to post
-        f = thread.html.find('div', {'class': 'row reply_post_form'}).find('form')
+        f = thread.html.find('div', {'class': 'comment-row reply_post_form'}).find('form')
         rep_url = f.get('action')
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'bbb'
         thread = self.app.post(str(rep_url), params=params)
@@ -603,9 +660,9 @@ class TestForum(TestController):
             'div', {'class': 'display_post'})[0].find('p').string == 'aaa'
         assert thread.html.findAll(
             'div', {'class': 'display_post'})[1].find('p').string == 'bbb'
-        assert thread.response.body.count(
-            '<div class="row reply_post_form') == 2
-        assert thread.response.body.count('<div class="edit_post_form') == 2
+        assert thread.response.text.count(
+            '<div class="comment-row reply_post_form') == 2
+        assert thread.response.text.count('<div class="edit_post_form') == 2
 
         # test edit post
         thread_url = thread.request.url
@@ -615,14 +672,13 @@ class TestForum(TestController):
         params = dict()
         inputs = reply_form.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[reply_form.find('textarea')['name']] = 'zzz'
         self.app.post(post_link, params)
         r = self.app.get(thread_url)
-        assert 'zzz' in str(r.html.find('div', {'class': 'display_post'}))
-        assert 'Last edit: Test Admin ' in str(
-            r.html.find('div', {'class': 'display_post'}))
+        assert 'zzz' in r.html.find('div', {'class': 'display_post'}).text
+        assert 'Last edit: Test Admin ' in r.html.find('div', {'class': 'display_post'}).text
 
     def test_subscription_controls(self):
         r = self.app.get('/discussion/create_topic/')
@@ -630,7 +686,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'Post text'
         params[f.find('select')['name']] = 'testforum'
@@ -644,16 +700,16 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         self.app.post(str(subscribe_url), params=params)
-        self.app.post('/discussion/general/subscribe_to_forum', {'subscribe': True})
-        f = thread.html.find('div', {'class': 'row reply_post_form'}).find('form')
+        self.app.post('/discussion/general/subscribe_to_forum', {'subscribe': 'True'})
+        f = thread.html.find('div', {'class': 'comment-row reply_post_form'}).find('form')
         rep_url = f.get('action')
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'Reply 2'
         self.app.post(str(rep_url), params=params)
@@ -668,7 +724,7 @@ class TestForum(TestController):
     def check_announcement_table(self, response, topic_name):
         assert response.html.find(text='Announcements')
         rows = self.get_table_rows(response, 'announcements')
-        assert_equal(len(rows), 1)
+        assert len(rows) == 1
         cell = rows[0].findAll('td', {'class': 'topic'})
         assert topic_name in str(cell)
 
@@ -678,7 +734,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -690,7 +746,7 @@ class TestForum(TestController):
             flags='Announcement',
             discussion='testforum'))
         thread2 = FM.ForumThread.query.get(_id=thread_id)
-        assert_equal(thread2.flags, ['Announcement'])
+        assert thread2.flags == ['Announcement']
 
         # Check that announcements are on front discussion page
         r = self.app.get('/discussion/')
@@ -709,7 +765,7 @@ class TestForum(TestController):
         url = thread.request.url
 
         # Check that the newly created topic is the most recent in the rss feed
-        f = self.app.get('/discussion/feed.rss').body
+        f = self.app.get('/discussion/feed.rss').text
         f = feedparser.parse(f)
         newest_entry = f['entries'][0]['summary_detail']['value'].split("</p>")[0].split("<p>")[-1]
         assert newest_entry == 'Test_Description'
@@ -720,7 +776,7 @@ class TestForum(TestController):
         form.submit()
 
         # Check that reply matches the newest in the rss feed
-        f = self.app.get('/discussion/feed.rss').body
+        f = self.app.get('/discussion/feed.rss').text
         f = feedparser.parse(f)
         newest_reply = f['entries'][0]['summary_detail']['value'].split("</p>")[0].split("<p>")[-1]
         assert newest_reply == 'Test_Reply'
@@ -731,7 +787,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -745,7 +801,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -755,7 +811,7 @@ class TestForum(TestController):
         # Check that threads are ordered in reverse creation order
         r = self.app.get('/discussion/testforum/')
         rows = self.get_table_rows(r, 'forum_threads')
-        assert_equal(len(rows), 2)
+        assert len(rows) == 2
         assert 'topic2' in str(rows[0])
         assert 'topic1' in str(rows[1])
 
@@ -764,28 +820,28 @@ class TestForum(TestController):
             flags='Sticky',
             discussion='testforum'))
         thread1 = FM.ForumThread.query.get(_id=tid1)
-        assert_equal(thread1.flags, ['Sticky'])
+        assert thread1.flags == ['Sticky']
 
         # Check that Sticky thread is at the top
         r = self.app.get('/discussion/testforum/')
         rows = self.get_table_rows(r, 'forum_threads')
-        assert_equal(len(rows), 2)
-        assert 'topic1' in str(rows[0])
-        assert 'topic2' in str(rows[1])
+        assert len(rows) == 2
+        assert 'topic1' in rows[0].text
+        assert 'topic2' in rows[1].text
 
         # Reset Sticky flag
         r = self.app.post(url1 + 'moderate', params=dict(
             flags='',
             discussion='testforum'))
         thread1 = FM.ForumThread.query.get(_id=tid1)
-        assert_equal(thread1.flags, [])
+        assert thread1.flags == []
 
         # Would check that threads are again in reverse creation order,
         # but so far we actually sort by mod_date, and resetting a flag
         # updates it
         r = self.app.get('/discussion/testforum/')
         rows = self.get_table_rows(r, 'forum_threads')
-        assert_equal(len(rows), 2)
+        assert len(rows) == 2
         # assert 'topic2' in str(rows[0])
         # assert 'topic1' in str(rows[1])
 
@@ -796,7 +852,7 @@ class TestForum(TestController):
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -806,12 +862,12 @@ class TestForum(TestController):
         url = thread.request.url
         # make a reply
         f = thread.html.find(
-            'div', {'class': 'row reply_post_form'}).find('form')
+            'div', {'class': 'comment-row reply_post_form'}).find('form')
         rep_url = f.get('action')
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'bbb'
         thread = self.app.post(str(rep_url), params=params)
@@ -819,7 +875,7 @@ class TestForum(TestController):
         # make sure the posts are in the original thread
         posts = thread.html.find('div', {'id': 'comment'}).findAll(
             'div', {'class': 'discussion-post'})
-        assert_equal(len(posts), 2)
+        assert len(posts) == 2
         # move the thread
         r = self.app.post(url + 'moderate', params=dict(
             flags='',
@@ -827,48 +883,79 @@ class TestForum(TestController):
         # make sure all the posts got moved
         posts = r.html.find('div', {'id': 'comment'}).findAll(
             'div', {'class': 'discussion-post'})
-        assert_equal(len(posts), 2)
+        assert len(posts) == 2
+
+    def test_rename_thread(self):
+        # make the topic
+        r = self.app.get('/discussion/create_topic/')
+        f = r.html.find('form', {'action': '/p/test/discussion/save_new_topic'})
+        params = dict()
+        inputs = f.findAll('input')
+        for field in inputs:
+            if field.has_attr('name'):
+                params[field['name']] = field.get('value') or ''
+        params[f.find('textarea')['name']] = 'aaa aaa'
+        params[f.find('select')['name']] = 'testforum'
+        params[f.find('input', {'style': 'width: 90%'})['name']] = 'first subject'
+
+        resp = self.app.post(
+            '/discussion/save_new_topic', params=params).follow()
+        url = resp.request.url
+        resp = self.app.get(url)
+
+        assert 'first subject' in resp
+
+        f = resp.html.find('div', {'id':'mod_thread_form'}).find('form')
+        params=dict(
+            flags='',
+            discussion='general',
+            subject='changed subject')
+        resp = self.app.post(str(f.get('action')), params=params).follow()
+        resp = self.app.get(url)
+
+        assert 'first subject' not in resp.html
+        assert 'changed subject' in resp
 
     def test_sidebar_menu(self):
         r = self.app.get('/discussion/')
         sidebar = r.html.find('div', {'id': 'sidebar'})
         sidebar_menu = str(sidebar)
         sidebar_links = [i['href'] for i in sidebar.findAll('a')]
-        assert_in("/p/test/discussion/create_topic/", sidebar_links)
-        assert_in("/p/test/discussion/new_forum", sidebar_links)
-        assert_in('<h3 class="">Help</h3>', sidebar_menu)
-        assert_in("/p/test/discussion/markdown_syntax", sidebar_links)
-        assert_not_in("flag_as_spam", sidebar_links)
+        assert "/p/test/discussion/create_topic/" in sidebar_links
+        assert "/p/test/discussion/new_forum" in sidebar_links
+        assert '<h3 class="">Help</h3>' in sidebar_menu
+        assert "/nf/markdown_syntax" in sidebar_links
+        assert "flag_as_spam" not in sidebar_links
         r = self.app.get('/discussion/create_topic/')
         f = r.html.find('form', {'action': '/p/test/discussion/save_new_topic'})
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa'
         params[f.find('select')['name']] = 'testforum'
         params[f.find('input', {'style': 'width: 90%'})['name']] = 'AAA'
         thread = self.app.post('/discussion/save_new_topic', params=params).follow()
         thread_sidebarmenu = str(thread.html.find('div', {'id': 'sidebar'}))
-        assert_in("flag_as_spam", thread_sidebarmenu)
+        assert "flag_as_spam" in thread_sidebarmenu
 
     def test_sidebar_menu_anon(self):
         r = self.app.get('/discussion/')
         sidebar = r.html.find('div', {'id': 'sidebar'})
         sidebar_menu = str(sidebar)
         sidebar_links = [i['href'] for i in sidebar.findAll('a')]
-        assert_in("/p/test/discussion/create_topic/", sidebar_links)
-        assert_in("/p/test/discussion/new_forum", sidebar_links)
-        assert_in('<h3 class="">Help</h3>', sidebar_menu)
-        assert_in("/p/test/discussion/markdown_syntax", sidebar_links)
-        assert_not_in("flag_as_spam", sidebar_menu)
+        assert "/p/test/discussion/create_topic/" in sidebar_links
+        assert "/p/test/discussion/new_forum" in sidebar_links
+        assert '<h3 class="">Help</h3>' in sidebar_menu
+        assert "/nf/markdown_syntax" in sidebar_links
+        assert "flag_as_spam" not in sidebar_menu
         r = self.app.get('/discussion/create_topic/')
         f = r.html.find('form', {'action': '/p/test/discussion/save_new_topic'})
         params = dict()
         inputs = f.findAll('input')
         for field in inputs:
-            if field.has_key('name'):  # nopep8 - beautifulsoup3 actually uses has_key
+            if field.has_attr('name'):
                 params[field['name']] = field.get('value') or ''
         params[f.find('textarea')['name']] = 'aaa'
         params[f.find('select')['name']] = 'testforum'
@@ -876,7 +963,7 @@ class TestForum(TestController):
         thread = self.app.post('/discussion/save_new_topic',
                                params=params).follow(extra_environ=dict(username='*anonymous'))
         thread_sidebar_menu = str(thread.html.find('div', {'id': 'sidebar'}))
-        assert_not_in("flag_as_spam", thread_sidebar_menu)
+        assert "flag_as_spam" not in thread_sidebar_menu
 
     def test_feed(self):
         for ext in ['', '.rss', '.atom']:
@@ -895,13 +982,25 @@ class TestForum(TestController):
     def test_create_topic_unicode(self):
         r = self.app.get('/admin/discussion/forums')
         form = r.forms['add-forum']
-        form['add_forum.shortname'] = u'téstforum'.encode('utf-8')
-        form['add_forum.name'] = u'Tést Forum'.encode('utf-8')
+        form['add_forum.shortname'] = 'téstforum'.encode()
+        form['add_forum.name'] = 'Tést Forum'.encode()
         form.submit()
         r = self.app.get('/admin/discussion/forums')
-        assert u'téstforum'.encode('utf-8') in r
-        r = self.app.get(u'/p/test/discussion/create_topic/téstforum/'.encode('utf-8'))
-        assert u'<option value="téstforum" selected>Tést Forum</option>' in r
+        assert 'téstforum'.encode() in r
+        r = self.app.get(h.urlquote('/p/test/discussion/create_topic/téstforum/'))
+        assert '<option value="téstforum" selected>Tést Forum</option>' in r
+
+    def test_create_topic_attachment(self):
+        r = self.app.get('/discussion/create_topic/')
+        form = self.fill_new_topic_form(r)
+        for field in form.fields.values():
+            field = field[0]
+            if field.id is None:
+                continue
+            if 'attachment' in field.id:
+                form[field.name] = ('myfile.txt', b'foo bar baz')  # webtest.Upload dooesn't work here
+        r = form.submit().follow()
+        assert 'myfile.txt' in r, r
 
     def test_viewing_a_thread_does_not_update_project_last_updated(self):
         # Create new topic/thread
@@ -914,7 +1013,7 @@ class TestForum(TestController):
         # View the thread and make sure project last_updated is not updated
         self.app.get(url)
         timestamp_after = M.Project.query.get(shortname='test').last_updated
-        assert_equal(timestamp_before, timestamp_after)
+        assert timestamp_before == timestamp_after
 
 
 class TestForumStats(TestController):
@@ -925,7 +1024,7 @@ class TestForumStats(TestController):
     @mock.patch('ming.session.Session.aggregate')
     def test_stats_data(self, aggregate):
         # partial data, some days are implicit 0
-        aggregate.return_value = {'result': [
+        aggregate.return_value = iter([
             {
                 "_id": {
                     "year": 2013,
@@ -947,10 +1046,10 @@ class TestForumStats(TestController):
                     "day": 5},
                 "posts": 2
             },
-        ]}
+        ])
         r = self.app.get(
             '/discussion/stats_data?begin=2013-01-01&end=2013-01-06')
-        assert_equal(r.json, {
+        assert r.json == {
             'begin': '2013-01-01 00:00:00',
             'end': '2013-01-06 00:00:00',
             'data': [
@@ -961,4 +1060,4 @@ class TestForumStats(TestController):
                 [1357344000000, 2],
                 [1357430400000, 0],
             ]
-        })
+        }
