@@ -15,9 +15,11 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
+
+from allura.lib.utils import permanent_redirect
 from tg import expose, redirect
 from tg.decorators import with_trailing_slash
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 
 from allura.controllers import repository
 
@@ -31,4 +33,4 @@ class BranchBrowser(repository.BranchBrowser):
         latest = c.app.repo.latest(branch=self._branch)
         if is_empty or not latest:
             return dict(allow_fork=False, log=[], is_empty=is_empty)
-        redirect(c.app.repo.url_for_commit(self._branch) + 'tree/')
+        permanent_redirect(c.app.repo.url_for_commit(self._branch) + 'tree/')
