@@ -15,4 +15,4 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from git_repo import Repository
+from .git_repo import Repository
