@@ -14,7 +14,9 @@
 #       KIND, either express or implied.  See the License for the
 #       specific language governing permissions and limitations
 #       under the License.
+from __future__ import annotations
 
+import sys
 import os
 import shutil
 import string
@@ -23,29 +25,41 @@ import tempfile
 from datetime import datetime
 from contextlib import contextmanager
 from time import time
+import typing
 
 import tg
 import git
 import gitdb
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 from pymongo.errors import DuplicateKeyError
 from paste.deploy.converters import asbool
+import six
 
 from ming.base import Object
-from ming.orm import Mapper, session
+from ming.odm import state, Mapper, session
+from ming.odm.base import ObjectState
 from ming.utils import LazyProperty
 
 from allura.lib import helpers as h
 from allura.model.repository import topological_sort, prefix_paths_union
 from allura import model as M
+import allura.tasks
+
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+
 
 log = logging.getLogger(__name__)
 
+assert sys.getfilesystemencoding() == 'utf-8', \
+    'Filesystem encoding is not set to utf-8.  You should set https://docs.python.org/3/library/os.html#utf8-mode'
+
+
 gitdb.util.mman = gitdb.util.mman.__class__(
     max_open_handles=128)
 
 
-class GitLibCmdWrapper(object):
+class GitLibCmdWrapper:
 
     def __init__(self, client):
         self.client = client
@@ -65,15 +79,17 @@ class Repository(M.Repository):
     class __mongometa__:
         name = 'git-repository'
 
+    query: Query[Repository]
+
     @LazyProperty
     def _impl(self):
         return GitImplementation(self)
 
     def suggested_clone_dest_path(self):
-        return super(Repository, self).suggested_clone_dest_path()[:-4]
+        return super().suggested_clone_dest_path()[:-4]
 
     def clone_url(self, category, username=''):
-        clone_url = super(Repository, self).clone_url(category, username)
+        clone_url = super().clone_url(category, username)
         if clone_url.endswith('.git'):
             clone_url = clone_url[:-4]
         return clone_url
@@ -92,7 +108,7 @@ class Repository(M.Repository):
             ).format(
                 merge_request.downstream_repo.clone_url_first(anon=False, username=c.user.username),
             )
-        return 'git checkout %s\n%s\ngit merge %s' % (
+        return 'git checkout {}\n{}\ngit merge {}'.format(
             merge_request.target_branch,
             fetch_command,
             merge_request.downstream.commit_id,
@@ -130,7 +146,7 @@ class Repository(M.Repository):
             author = h.really_unicode(c.user.display_name or c.user.username)
             tmp_repo.git.config('user.name', author.encode('utf8'))
             tmp_repo.git.config('user.email', 'allura@localhost')  # a public email alias could be nice here
-            msg = u'Merge {} branch {} into {}\n\n{}'.format(
+            msg = 'Merge {} branch {} into {}\n\n{}'.format(
                 mr.downstream_repo.url(),
                 mr.source_branch,
                 mr.target_branch,
@@ -165,7 +181,7 @@ class GitImplementation(M.RepositoryImplementation):
             _git = git.Repo(self._repo.full_fs_path, odbt=git.GitCmdObjectDB)
             _git.git = GitLibCmdWrapper(_git.git)
             return _git
-        except (git.exc.NoSuchPathError, git.exc.InvalidGitRepositoryError), err:
+        except (git.exc.NoSuchPathError, git.exc.InvalidGitRepositoryError) as err:
             log.error('Problem looking up repo: %r', err)
             return None
 
@@ -213,7 +229,7 @@ class GitImplementation(M.RepositoryImplementation):
                     bare=True)
             self.__dict__['_git'] = repo
             self._setup_special_files(source_url)
-        except:
+        except Exception:
             self._repo.set_status('ready')
             raise
 
@@ -231,10 +247,9 @@ class GitImplementation(M.RepositoryImplementation):
                 try:
                     from tg import request
                     url = ' at ' + request.url
-                except:
+                except Exception:
                     pass
-                log.exception('Error with rev_parse(%s)%s' %
-                              (str(rev) + '^0', url))
+                log.info(f"couldn't run rev_parse({str(rev) + '^0'}){url}", exc_info=True)
         if result:
             result.set_context(self._repo)
         return result
@@ -265,7 +280,7 @@ class GitImplementation(M.RepositoryImplementation):
                 if M.repository.Commit.query.find(dict(_id=obj.hexsha)).count():
                     graph[obj.hexsha] = set()  # mark as parentless
                     continue
-            graph[obj.hexsha] = set(p.hexsha for p in obj.parents)
+            graph[obj.hexsha] = {p.hexsha for p in obj.parents}
             to_visit += obj.parents
         return list(topological_sort(graph))
 
@@ -294,23 +309,23 @@ class GitImplementation(M.RepositoryImplementation):
         else:
             ci_doc = CommitDoc(dict(args, _id=ci.hexsha))
             try:
-                ci_doc.m.insert(safe=True)
+                ci_doc.m.insert()
             except DuplicateKeyError:
                 if lazy:
                     return False
         self.refresh_tree_info(ci.tree, seen, lazy)
         return True
 
-    def refresh_tree_info(self, tree, seen, lazy=True):
-        from allura.model.repository import TreeDoc
+    def refresh_tree_info(self, tree: git.objects.tree.Tree, seen: set, lazy=True):
+        from allura.model.repository import Tree
         if lazy and tree.binsha in seen:
             return
         seen.add(tree.binsha)
-        doc = TreeDoc(dict(
+        doc = Tree(
             _id=tree.hexsha,
             tree_ids=[],
             blob_ids=[],
-            other_ids=[]))
+            other_ids=[])
         for o in tree:
             if o.type == 'submodule':
                 continue
@@ -321,11 +336,18 @@ class GitImplementation(M.RepositoryImplementation):
                 self.refresh_tree_info(o, seen, lazy)
                 doc.tree_ids.append(obj)
             elif o.type == 'blob':
-                doc.blob_ids.append(obj)
+                if o.mode == o.link_mode:
+                    obj.type = 'symlink'
+                    doc.other_ids.append(obj)
+                else:
+                    doc.blob_ids.append(obj)
             else:
                 obj.type = o.type
                 doc.other_ids.append(obj)
-        doc.m.save(safe=False)
+        # set to 'dirty' to force save() to be used instead of insert() (which errors if doc exists in db already)
+        state(doc).status = ObjectState.dirty
+        session(doc).flush(doc)
+        session(doc).expunge(doc)
         return doc
 
     def log(self, revs=None, path=None, exclude=None, id_only=True, limit=None, **kw):
@@ -348,12 +370,10 @@ class GitImplementation(M.RepositoryImplementation):
         If id_only is True, returns only the commit ID, otherwise it returns
         detailed information about each commit.
         """
-        path = path.strip('/').encode("utf-8") if path else None
+        path = path.strip('/') if path else None
         if exclude is not None:
             revs.extend(['^%s' % e for e in exclude])
-        args = ['--name-status', revs, '--', path]
-        if path:
-            args = ['--follow'] + args
+        args = ['--follow', '--name-status', revs, '--', path or '.']
         kwargs = {}
         if limit:
             kwargs['n'] = limit
@@ -436,7 +456,7 @@ class GitImplementation(M.RepositoryImplementation):
         stream = proc.stdout
         commit_lines = []
         while True:
-            line = stream.readline()
+            line = six.ensure_text(stream.readline())
             if '\x00' in line or not(len(line)):
                 # hash line read, need to yield previous commit
                 # first, cleaning lines a bit
@@ -486,14 +506,14 @@ class GitImplementation(M.RepositoryImplementation):
                           'hooks', 'post-receive')
         with open(fn, 'w') as fp:
             fp.write(text)
-        os.chmod(fn, 0755)
+        os.chmod(fn, 0o755)
 
     def _object(self, oid):
         evens = oid[::2]
         odds = oid[1::2]
-        binsha = ''
+        binsha = b''
         for e, o in zip(evens, odds):
-            binsha += chr(int(e + o, 16))
+            binsha += bytes((int(e + o, 16),))
         return git.Object.new_from_sha(self._git, binsha)
 
     def rev_parse(self, rev):
@@ -514,6 +534,9 @@ class GitImplementation(M.RepositoryImplementation):
         return tree._id
 
     def tarball(self, commit, path=None):
+        """
+        :param path: is currently ignored.  Can't request a snapshot of a subdirectory
+        """
         if not os.path.exists(self._repo.tarball_path):
             os.makedirs(self._repo.tarball_path)
         archive_name = self._repo.tarball_filename(commit)
@@ -561,8 +584,8 @@ class GitImplementation(M.RepositoryImplementation):
         for ref in ref_list:
             try:
                 hex_sha = ref.commit.hexsha
-            except ValueError:
-                log.debug(u"Found invalid sha: {}".format(ref))
+            except (ValueError, AssertionError) as e:
+                log.debug(f"Found invalid sha: {ref}", exc_info=e)
                 continue
             refs.append(Object(name=ref.name, object_id=hex_sha))
         time_taken = time() - start_time
@@ -572,7 +595,7 @@ class GitImplementation(M.RepositoryImplementation):
             threshold = float(threshold) if threshold else None
         except ValueError:
             threshold = None
-            log.warn('Skipping reference caching - The value for config param '
+            log.warning('Skipping reference caching - The value for config param '
                      '"repo_refs_cache_threshold" must be a float.')
 
         if threshold is not None and time_taken > threshold:
@@ -585,7 +608,7 @@ class GitImplementation(M.RepositoryImplementation):
 
     @LazyProperty
     def head(self):
-        if not self._git or not self._git.heads:
+        if not self._git:
             return None
         # if the repo's HEAD file doesn't point to a valid branch, we need to select one
         # this can happen in particular with masterless repos
@@ -613,9 +636,8 @@ class GitImplementation(M.RepositoryImplementation):
     def set_default_branch(self, name):
         if not name:
             return
-        # HEAD should point to default branch
-        self._git.git.symbolic_ref('HEAD', 'refs/heads/%s' % name)
         self._repo.default_branch_name = name
+        allura.tasks.repo_tasks.update_head_reference.post(self._repo.full_fs_path, name)
         session(self._repo).flush(self._repo)
 
     def _get_last_commit(self, commit_id, paths):
@@ -630,7 +652,7 @@ class GitImplementation(M.RepositoryImplementation):
         skip = 0
         while commit_id and not files:
             output = self._git.git.log(
-                commit_id, '--', *[p.encode('utf-8') for p in paths],
+                commit_id, '--', *[p for p in paths],
                 pretty='format:%H',
                 name_only=True,
                 max_count=1,
@@ -759,7 +781,7 @@ class GitImplementation(M.RepositoryImplementation):
                 id_only=False))
 
 
-class _OpenedGitBlob(object):
+class _OpenedGitBlob:
     CHUNK_SIZE = 4096
 
     def __init__(self, stream):
@@ -772,17 +794,17 @@ class _OpenedGitBlob(object):
         '''
         Yields one line at a time, reading from the stream
         '''
-        buffer = ''
+        buffer = b''
         while True:
             # Replenish buffer until we have a line break
-            while '\n' not in buffer:
+            while b'\n' not in buffer:
                 chars = self._stream.read(self.CHUNK_SIZE)
                 if not chars:
                     break
                 buffer += chars
             if not buffer:
                 break
-            eol = buffer.find('\n')
+            eol = buffer.find(b'\n')
             if eol == -1:
                 # end without \n
                 yield buffer
