@@ -18,22 +18,22 @@
 import os
 import errno
 import logging
-import urllib
-import urllib2
+from io import BytesIO
+
+import urllib.request
+import urllib.parse
 from collections import defaultdict
 import traceback
-from urlparse import urlparse
+from urllib.parse import urlparse
+from urllib.parse import unquote
 from datetime import datetime
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
 
-from BeautifulSoup import BeautifulSoup
+from bs4 import BeautifulSoup
 from tg import expose, validate, flash, redirect, config
 from tg.decorators import with_trailing_slash
-from pylons import app_globals as g
-from pylons import tmpl_context as c
+from tg import app_globals as g
+from tg import tmpl_context as c
+from tg import request
 from formencode import validators as fev, schema
 from webob import exc
 
@@ -60,25 +60,25 @@ log = logging.getLogger(__name__)
 class ProjectImportForm(schema.Schema):
 
     def __init__(self, source):
-        super(ProjectImportForm, self).__init__()
+        super().__init__()
         provider = ProjectRegistrationProvider.get()
         self.add_field('tools', ToolsValidator(source))
         self.add_field('project_shortname', provider.shortname_validator)
         self.allow_extra_fields = True
 
     neighborhood = fev.NotEmpty()
-    project_name = fev.UnicodeString(not_empty=True, max=40)
+    project_name = v.UnicodeString(not_empty=True, max=40)
 
 
 class ToolImportForm(schema.Schema):
 
     def __init__(self, tool_class):
-        super(ToolImportForm, self).__init__()
+        super().__init__()
         self.add_field('mount_point', v.MountPointValidator(tool_class))
-    mount_label = fev.UnicodeString()
+    mount_label = v.UnicodeString()
 
 
-class ImportErrorHandler(object):
+class ImportErrorHandler:
 
     def __init__(self, importer, project_name, project):
         self.importer = importer
@@ -146,7 +146,7 @@ def import_tool(importer_path, project_name=None,
             handler.success(app)
 
 
-class ProjectExtractor(object):
+class ProjectExtractor:
 
     """Base class for project extractors.
 
@@ -157,19 +157,21 @@ class ProjectExtractor(object):
 
     PAGE_MAP = {}
 
-    def __init__(self, project_name, page_name=None, **kw):
+    def __init__(self, project_name, page_name_or_url=None, **kw):
         self.project_name = project_name
         self._page_cache = {}
         self.url = None
         self.page = None
-        if page_name:
-            self.get_page(page_name, **kw)
+        if page_name_or_url:
+            self.get_page(page_name_or_url, **kw)
 
     @staticmethod
-    def urlopen(url, retries=3, codes=(408, 500, 502, 503, 504), timeout=120, **kw):
-        req = urllib2.Request(url, **kw)
-        req.add_header(
-            'User-Agent', 'Allura Data Importer (https://allura.apache.org/)')
+    def urlopen(url, retries=3, codes=(408, 500, 502, 503, 504), timeout=120, unredirected_hdrs=None, **kw):
+        req = urllib.request.Request(url, **kw)
+        if unredirected_hdrs:
+            for key, val in unredirected_hdrs.items():
+                req.add_unredirected_header(key, val)
+        req.add_header('User-Agent', 'Allura Data Importer (https://allura.apache.org/)')
         return h.urlopen(req, retries=retries, codes=codes, timeout=timeout)
 
     def get_page(self, page_name_or_url, parser=None, **kw):
@@ -207,7 +209,7 @@ class ProjectExtractor(object):
 
         """
         return self.PAGE_MAP[page_name].format(
-            project_name=urllib.quote(self.project_name), **kw)
+            project_name=urllib.parse.quote(self.project_name), **kw)
 
     def parse_page(self, page):
         """Transforms the result of a `urlopen` call before returning it from
@@ -321,7 +323,7 @@ class ProjectImporter(BaseController):
         Ajax form validation.
 
         """
-        return c.form_errors
+        return request.validation.errors
 
     def after_project_create(self, project, **kw):
         """
@@ -331,7 +333,7 @@ class ProjectImporter(BaseController):
         tool imports happen.
 
         :param project: The newly created project.
-        :param \*\*kw: The keyword arguments that were posted to the controller
+        :param kw: The keyword arguments that were posted to the controller
             method that created the project.
 
         """
@@ -344,17 +346,17 @@ class ToolImportControllerMeta(type):
         the appropriate App for this controller's importer.
 
         """
-        if hasattr(cls, 'create') and getattr(cls.create.decoration, 'validation', None) is None:
+        if hasattr(cls, 'create') and not getattr(cls.create.decoration, 'validations', None):
+            index_meth = getattr(cls.index, '__func__', cls.index)
             cls.create = validate(cls.import_form(aslist(importer.target_app)[0]),
-                    error_handler=cls.index.__func__)(cls.create)
+                                  error_handler=index_meth)(cls.create)
         return type.__call__(cls, importer, *args, **kw)
 
 
-class ToolImportController(BaseController):
+class ToolImportController(BaseController, metaclass=ToolImportControllerMeta):
     """ Base class for ToolImporter controllers.
 
     """
-    __metaclass__ = ToolImportControllerMeta
 
     def __init__(self, importer):
         """
@@ -373,8 +375,7 @@ class ToolImporterMeta(type):
     def __init__(cls, name, bases, attrs):
         if not (hasattr(cls, 'target_app_ep_names')
                 or hasattr(cls, 'target_app')):
-            raise AttributeError("{0} must define either "
-                    "`target_app` or `target_app_ep_names`".format(name))
+            raise AttributeError(f"{name} must define either `target_app` or `target_app_ep_names`")
         return type.__init__(cls, name, bases, attrs)
 
     def __call__(cls, *args, **kw):
@@ -386,12 +387,12 @@ class ToolImporterMeta(type):
         """
         if not getattr(cls, 'target_app', None):
             cls.target_app = [g.entry_points['tool'][ep_name]
-                    for ep_name in aslist(cls.target_app_ep_names)
-                    if ep_name in g.entry_points['tool']]
+                              for ep_name in aslist(cls.target_app_ep_names)
+                              if ep_name in g.entry_points['tool']]
         return type.__call__(cls, *args, **kw)
 
 
-class ToolImporter(object):
+class ToolImporter(metaclass=ToolImporterMeta):
 
     """
     Base class for tool importers.
@@ -427,7 +428,6 @@ class ToolImporter(object):
        The controller for this importer, to handle single tool imports.
 
     """
-    __metaclass__ = ToolImporterMeta
 
     target_app = None  # app or list of apps
     source = None  # string description of source, must match project importer
@@ -535,7 +535,7 @@ class ToolImporter(object):
 
         """
         klass = self.__class__
-        importer_path = '{0}.{1}'.format(klass.__module__, klass.__name__)
+        importer_path = f'{klass.__module__}.{klass.__name__}'
         import_tool.post(importer_path, **kw)
 
 
@@ -549,11 +549,11 @@ class ToolsValidator(fev.Set):
     """
 
     def __init__(self, source, *a, **kw):
-        super(ToolsValidator, self).__init__(*a, **kw)
+        super().__init__(*a, **kw)
         self.source = source
 
     def to_python(self, value, state=None):
-        value = super(ToolsValidator, self).to_python(value, state)
+        value = super().to_python(value, state)
         valid = []
         invalid = []
         for name in value:
@@ -569,7 +569,7 @@ class ToolsValidator(fev.Set):
         return valid
 
 
-class ProjectToolsImportController(object):
+class ProjectToolsImportController:
 
     '''List all importers available'''
 
@@ -613,19 +613,19 @@ class ImportAdminExtension(AdminExtension):
         sidebar_links.append(link)
 
 
-def stringio_parser(page):
+def bytesio_parser(page):
     return {
         'content-type': page.info()['content-type'],
-        'data': StringIO(page.read()),
+        'data': BytesIO(page.read()),
     }
 
 
-class File(object):
+class File:
 
     def __init__(self, url, filename=None):
-        extractor = ProjectExtractor(None, url, parser=stringio_parser)
+        extractor = ProjectExtractor(None, url, parser=bytesio_parser)
         self.url = url
-        self.filename = filename or os.path.basename(urlparse(url).path)
+        self.filename = filename or unquote(os.path.basename(urlparse(url).path))
         # try to get the mime-type from the filename first, because
         # some files (e.g., attachements) may have the Content-Type header
         # forced to encourage the UA to download / save the file
@@ -662,6 +662,6 @@ def save_importer_upload(project, filename, data):
     except OSError as e:
         if e.errno != errno.EEXIST:
             raise
-    with open(dest_file, 'w') as fp:
+    with open(dest_file, 'w', encoding='utf-8') as fp:
         fp.write(data)
     return dest_file
