@@ -1,5 +1,3 @@
-# coding: utf-8
-
 #       Licensed to the Apache Software Foundation (ASF) under one
 #       or more contributor license agreements.  See the NOTICE file
 #       distributed with this work for additional information
@@ -17,17 +15,16 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from unittest import TestCase
-from nose.tools import assert_equal
+from unittest import TestCase, skipIf
 from mock import Mock, patch, call
-from ming.odm import ThreadLocalORMSession
+from ming.odm import ThreadLocalODMSession
 import git
 
-from IPython.testing.decorators import module_not_available, skipif
 from allura import model as M
 from allura.tests import TestController
 from allura.tests.decorators import with_tool, without_module
 from alluratest.controller import setup_basic_test
+from alluratest.tools import module_not_available
 from forgeimporters.github.wiki import GitHubWikiImporter
 from forgeimporters.github.utils import GitHubMarkdownConverter
 from forgeimporters.github import GitHubOAuthMixin
@@ -47,13 +44,13 @@ class TestGitHubWikiImporter(TestCase):
         return project
 
     @patch('forgeimporters.github.wiki.M')
-    @patch('forgeimporters.github.wiki.ThreadLocalORMSession')
+    @patch('forgeimporters.github.wiki.ThreadLocalODMSession')
     @patch('forgeimporters.github.wiki.g')
     @patch('forgeimporters.github.wiki.GitHubProjectExtractor')
     def test_import_tool_happy_path(self, ghpe, g, tlorms, M):
         with patch('forgeimporters.github.wiki.GitHubWikiImporter.import_pages'),\
-             patch('forgeimporters.github.wiki.GitHubWikiImporter.has_wiki_repo', return_value=True),\
-             patch('forgeimporters.github.wiki.c'):
+                patch('forgeimporters.github.wiki.GitHubWikiImporter.has_wiki_repo', return_value=True),\
+                patch('forgeimporters.github.wiki.c'):
             ghpe.return_value.has_wiki.return_value = True
             p = self._make_project(gh_proj_name='myproject')
             u = Mock(name='c.user')
@@ -76,7 +73,7 @@ class TestGitHubWikiImporter(TestCase):
                 project=p, user=u, url='foo')
             g.post_event.assert_called_once_with('project_updated')
 
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         self.blob1 = Mock()
         self.blob1.name = 'Home.md'
@@ -120,7 +117,7 @@ class TestGitHubWikiImporter(TestCase):
             'project_name': 'me/project',
             'source_id': 'Page',
         }
-        assert_equal(page.import_id, import_id)
+        assert page.import_id == import_id
 
     @patch('forgeimporters.github.wiki.WM.Page.upsert')
     @patch('forgeimporters.github.wiki.h.render_any_markup')
@@ -134,11 +131,11 @@ class TestGitHubWikiImporter(TestCase):
         importer.app.url = '/p/test/wiki/'
         importer.rewrite_links = Mock(return_value='')
         importer._without_history(self.commit2)
-        assert_equal(upsert.call_args_list, [call('Home2'), call('Home3')])
+        assert upsert.call_args_list == [call('Home2'), call('Home3')]
 
-        assert_equal(render.call_args_list, [
-            call('Home2.creole', u'**test message**'),
-            call('Home3.rest', u'test message')])
+        assert render.call_args_list == [
+            call('Home2.creole', '**test message**'),
+            call('Home3.rest', 'test message')]
 
     @patch('forgeimporters.github.wiki.git.Repo')
     @patch('forgeimporters.github.wiki.mkdtemp')
@@ -156,16 +153,16 @@ class TestGitHubWikiImporter(TestCase):
         repo = clone.return_value
         repo.iter_commits.return_value = [self.commit1, self.commit2]
         GitHubWikiImporter().import_pages('wiki_url', history=True)
-        assert_equal(with_history.call_count, 2)
-        assert_equal(without_history.call_count, 0)
+        assert with_history.call_count == 2
+        assert without_history.call_count == 0
 
     @patch('forgeimporters.github.wiki.GitHubWikiImporter._with_history')
     @patch('forgeimporters.github.wiki.GitHubWikiImporter._without_history')
     def test_get_commits_without_history(self, without_history, with_history):
         with patch('forgeimporters.github.wiki.git.Repo._clone'):
             GitHubWikiImporter().import_pages('wiki_url')
-            assert_equal(with_history.call_count, 0)
-            assert_equal(without_history.call_count, 1)
+            assert with_history.call_count == 0
+            assert without_history.call_count == 1
 
     @patch('forgeimporters.github.wiki.WM.Page.upsert')
     @patch('forgeimporters.github.wiki.h.render_any_markup')
@@ -180,17 +177,17 @@ class TestGitHubWikiImporter(TestCase):
         importer.app.url = '/p/test/wiki/'
         importer.rewrite_links = Mock(return_value='')
         importer._with_history(self.commit2)
-        assert_equal(upsert.call_args_list, [call('Home')])
-        assert_equal(render.call_args_list,
-                     [call('Home.rst', u'# test message')])
+        assert upsert.call_args_list == [call('Home')]
+        assert (render.call_args_list ==
+                [call('Home.rst', '# test message')])
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     @patch('forgeimporters.github.wiki.WM.Page.upsert')
     @patch('forgeimporters.github.wiki.mediawiki2markdown')
     def test_with_history_mediawiki(self, md2mkm, upsert):
         self.commit2.stats.files = {"Home.mediawiki": self.blob1}
         self.commit2.tree = {"Home.mediawiki": self.blob1}
-        md2mkm.return_value = u'# test message'
+        md2mkm.return_value = '# test message'
         importer = GitHubWikiImporter()
         importer._set_available_pages = Mock()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
@@ -198,88 +195,88 @@ class TestGitHubWikiImporter(TestCase):
         importer.app.config.options = {}
         importer.app.url = '/p/test/wiki/'
         importer.rewrite_links = Mock(return_value='')
-        importer.convert_gollum_tags = Mock(return_value=u'# test message')
+        importer.convert_gollum_tags = Mock(return_value='# test message')
         importer._with_history(self.commit2)
-        assert_equal(upsert.call_args_list, [call('Home')])
-        assert_equal(md2mkm.call_args_list, [call(u'# test message')])
+        assert upsert.call_args_list == [call('Home')]
+        assert md2mkm.call_args_list == [call('# test message')]
 
     def test_set_available_pages(self):
         importer = GitHubWikiImporter()
         commit = Mock()
         blobs = [Mock() for i in range(3)]
-        blobs[0].name = u'Home-42.md'
-        blobs[1].name = u'image.png'
-        blobs[2].name = u'code & fun.textile'
+        blobs[0].name = 'Home-42.md'
+        blobs[1].name = 'image.png'
+        blobs[2].name = 'code & fun.textile'
         commit.tree.traverse.return_value = blobs
         importer._set_available_pages(commit)
-        assert_equal(importer.available_pages, [u'Home 42', u'code & fun'])
+        assert importer.available_pages == ['Home 42', 'code & fun']
 
     def test_gollum_page_links_case_insensitive(self):
         i = GitHubWikiImporter()
-        i.available_pages = [u'Home 42', u'code & fun']
-        assert_equal(i.convert_gollum_tags(u'[[Code & Fun]]'), u'[code & fun]')
-        assert_equal(i.convert_gollum_tags(u'[[home-42]]'), u'[Home 42]')
-        assert_equal(i.convert_gollum_tags(u'[[Unknown]]'), u'[Unknown]')
+        i.available_pages = ['Home 42', 'code & fun']
+        assert i.convert_gollum_tags('[[Code & Fun]]') == '[code & fun]'
+        assert i.convert_gollum_tags('[[home-42]]') == '[Home 42]'
+        assert i.convert_gollum_tags('[[Unknown]]') == '[Unknown]'
 
     def test_convert_page_name(self):
         f = GitHubWikiImporter()._convert_page_name
-        assert_equal(f('Page Name'), 'Page Name')
-        assert_equal(f('Page-Name'), 'Page Name')
-        assert_equal(f('Page / Name'), 'Page   Name')
+        assert f('Page Name') == 'Page Name'
+        assert f('Page-Name') == 'Page Name'
+        assert f('Page / Name') == 'Page   Name'
 
     def test_convert_gollum_page_links(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        assert_equal(f(u'[[Page]]'), u'[Page]')
-        assert_equal(f(u'[[Page Title|Page]]'), u'[Page Title](Page)')
-        assert_equal(f(u'[[Pagê Nâme]]'), u'[Pagê Nâme]')
+        assert f('[[Page]]') == '[Page]'
+        assert f('[[Page Title|Page]]') == '[Page Title](Page)'
+        assert f('[[Pagê Nâme]]') == '[Pagê Nâme]'
         # Github always converts spaces and slashes in links to hyphens,
         # to lookup page in the filesystem. During import we're converting
         # all hyphens in page name to spaces, but still supporting both link
         # formats.
-        assert_equal(f(u'[[Page With Spaces]]'), u'[Page With Spaces]')
-        assert_equal(f(u'[[Page-With-Spaces]]'), u'[Page With Spaces]')
-        assert_equal(f(u'[[Page / 1]]'), u'[Page   1]')
-        assert_equal(f(u'[[Title|Page With Spaces]]'),
-                     u'[Title](Page With Spaces)')
-        assert_equal(f(u'[[Title|Page-With-Spaces]]'),
-                     u'[Title](Page With Spaces)')
-        assert_equal(f(u'[[go here|Page / 1]]'), u'[go here](Page   1)')
+        assert f('[[Page With Spaces]]') == '[Page With Spaces]'
+        assert f('[[Page-With-Spaces]]') == '[Page With Spaces]'
+        assert f('[[Page / 1]]') == '[Page   1]'
+        assert (f('[[Title|Page With Spaces]]') ==
+                '[Title](Page With Spaces)')
+        assert (f('[[Title|Page-With-Spaces]]') ==
+                '[Title](Page With Spaces)')
+        assert f('[[go here|Page / 1]]') == '[go here](Page   1)'
 
     def test_convert_gollum_page_links_escaped(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        assert_equal(f(u"'[[Page]]"), u'[[Page]]')
-        assert_equal(f(u"'[[Page Title|Page]]"), u'[[Page Title|Page]]')
-        assert_equal(f(u"'[[Page With Spaces]]"), u'[[Page With Spaces]]')
-        assert_equal(f(u"'[[Page-With-Spaces]]"), u'[[Page-With-Spaces]]')
-        assert_equal(f(u"'[[Page / 1]]"), u'[[Page / 1]]')
-        assert_equal(f(u"'[[Title|Page With Spaces]]"),
-                     u'[[Title|Page With Spaces]]')
-        assert_equal(f(u"'[[Title|Page-With-Spaces]]"),
-                     u'[[Title|Page-With-Spaces]]')
-        assert_equal(f(u"'[[go here|Page / 1]]"), u'[[go here|Page / 1]]')
+        assert f("'[[Page]]") == '[[Page]]'
+        assert f("'[[Page Title|Page]]") == '[[Page Title|Page]]'
+        assert f("'[[Page With Spaces]]") == '[[Page With Spaces]]'
+        assert f("'[[Page-With-Spaces]]") == '[[Page-With-Spaces]]'
+        assert f("'[[Page / 1]]") == '[[Page / 1]]'
+        assert (f("'[[Title|Page With Spaces]]") ==
+                '[[Title|Page With Spaces]]')
+        assert (f("'[[Title|Page-With-Spaces]]") ==
+                '[[Title|Page-With-Spaces]]')
+        assert f("'[[go here|Page / 1]]") == '[[go here|Page / 1]]'
 
     def test_convert_gollum_external_links(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        assert_equal(f(u'[[http://domain.net]]'), u'<http://domain.net>')
-        assert_equal(f(u'[[https://domain.net]]'), u'<https://domain.net>')
-        assert_equal(f(u'[[Site|http://domain.net]]'),
-                     u'[Site](http://domain.net)')
+        assert f('[[http://domain.net]]') == '<http://domain.net>'
+        assert f('[[https://domain.net]]') == '<https://domain.net>'
+        assert (f('[[Site|http://domain.net]]') ==
+                '[Site](http://domain.net)')
 
     def test_convert_gollum_external_links_escaped(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        assert_equal(f(u"'[[http://domain.net]]"), u'[[http://domain.net]]')
-        assert_equal(f(u"'[[https://domain.net]]"), u'[[https://domain.net]]')
-        assert_equal(f(u"'[[Site|http://domain.net]]"),
-                     u'[[Site|http://domain.net]]')
+        assert f("'[[http://domain.net]]") == '[[http://domain.net]]'
+        assert f("'[[https://domain.net]]") == '[[https://domain.net]]'
+        assert (f("'[[Site|http://domain.net]]") ==
+                '[[Site|http://domain.net]]')
 
     def test_convert_gollum_toc(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        assert_equal(f(u'[[_TOC_]]'), u'[TOC]')
-        assert_equal(f(u"'[[_TOC_]]"), u'[[_TOC_]]')
+        assert f('[[_TOC_]]') == '[TOC]'
+        assert f("'[[_TOC_]]") == '[[_TOC_]]'
 
     def test_convert_gollum_tags(self):
         f = GitHubWikiImporter().convert_gollum_tags
-        source = u'''Look at [[this page|Some Page]]
+        source = '''Look at [[this page|Some Page]]
 
 More info at: [[MoreInfo]] [[Even More Info]]
 
@@ -287,7 +284,7 @@ Our website is [[http://domain.net]].
 
 '[[Escaped Tag]]'''
 
-        result = u'''Look at [this page](Some Page)
+        result = '''Look at [this page](Some Page)
 
 More info at: [MoreInfo] [Even More Info]
 
@@ -295,9 +292,9 @@ Our website is <http://domain.net>.
 
 [[Escaped Tag]]'''
 
-        assert_equal(f(source), result)
+        assert f(source) == result
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_markup(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
@@ -306,7 +303,7 @@ Our website is <http://domain.net>.
         importer.github_markdown_converter = GitHubMarkdownConverter(
             'user', 'proj')
         f = importer.convert_markup
-        source = u'''Look at [[this page|Some Page]]
+        source = '''Look at [[this page|Some Page]]
 
 More info at: [[MoreInfo]] [[Even More Info]]
 
@@ -323,7 +320,7 @@ ticket #1
 #1 header
 
 sha aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'''
-        result = u'''Look at [this page](Some Page)
+        result = '''Look at [this page](Some Page)
 
 More info at: [MoreInfo] [Even More Info]
 
@@ -339,9 +336,9 @@ ticket [#1]
 [#1] header
 
 sha [aaaaaa]'''
-        assert_equal(f(source, 'test.md').strip(), result)
+        assert f(source, 'test.md').strip() == result
 
-        assert_equal(f(u'h1. Hello', 't.textile').strip(), u'# Hello')
+        assert f('h1. Hello', 't.textile').strip() == '# Hello'
 
     @without_module('html2text')
     def test_convert_markup_without_html2text(self):
@@ -350,7 +347,7 @@ sha [aaaaaa]'''
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'''Look at [[this page|Some Page]]
+        source = '''Look at [[this page|Some Page]]
 
 More info at: [[MoreInfo]] [[Even More Info]]
 
@@ -362,48 +359,52 @@ Our website is [[http://domain.net]].
 
 [External link](https://github.com/a/b/issues/1)'''
 
-        result = u'''<p>Look at [[this page|Some Page]]</p>
+        result = '''<p>Look at [[this page|Some Page]]</p>
 <p>More info at: [[MoreInfo]] [[Even More Info]]</p>
 <p>Our website is [[http://domain.net]].</p>
-<p>'[[Escaped Tag]]</p>
+<p>\u2018[[Escaped Tag]]</p>
 <p>[External link to the wiki page](https://github.com/a/b/wiki/Page)</p>
 <p>[External link](https://github.com/a/b/issues/1)</p>'''
 
-        assert_equal(f(source, 'test.textile').strip(), result)
+        assert f(source, 'test.textile').strip() == result
 
     def test_rewrite_links(self):
         f = GitHubWikiImporter().rewrite_links
         prefix = 'https://github/a/b/wiki'
         new = '/p/test/wiki/'
-        assert_equal(
-            f(u'<a href="https://github/a/b/wiki/Test Page">Test Page</a>',
-              prefix, new),
-            u'<a href="/p/test/wiki/Test Page">Test Page</a>')
-        assert_equal(
-            f(u'<a href="https://github/a/b/wiki/Test-Page">Test-Page</a>',
-              prefix, new),
-            u'<a href="/p/test/wiki/Test Page">Test Page</a>')
-        assert_equal(
-            f(u'<a href="https://github/a/b/issues/1" class="1"></a>',
-              prefix, new),
-            u'<a href="https://github/a/b/issues/1" class="1"></a>')
-        assert_equal(
-            f(u'<a href="https://github/a/b/wiki/Test Page">https://github/a/b/wiki/Test Page</a>',
-              prefix, new),
-            u'<a href="/p/test/wiki/Test Page">/p/test/wiki/Test Page</a>')
-        assert_equal(
-            f(u'<a href="https://github/a/b/wiki/Test Page">Test <b>Page</b></a>',
-              prefix, new),
-            u'<a href="/p/test/wiki/Test Page">Test <b>Page</b></a>')
+        assert (
+            f('<a href="https://github/a/b/wiki/Test Page">Test Page</a>',
+              prefix, new) ==
+            '<a href="/p/test/wiki/Test Page">Test Page</a>')
+        assert (
+            f('<a href="https://github/a/b/wiki/Test-Page">Test-Page</a>',
+              prefix, new) ==
+            '<a href="/p/test/wiki/Test Page">Test Page</a>')
+        assert (
+            f('<a href="https://github/a/b/issues/1" class="1"></a>',
+              prefix, new) ==
+            '<a class="1" href="https://github/a/b/issues/1"></a>')
+        assert (
+            f('<a href="https://github/a/b/wiki/Test Page">https://github/a/b/wiki/Test Page</a>',
+              prefix, new) ==
+            '<a href="/p/test/wiki/Test Page">/p/test/wiki/Test Page</a>')
+        assert (
+            f('<a href="https://github/a/b/wiki/Test Page">Test blah blah</a>',
+              prefix, new) ==
+            '<a href="/p/test/wiki/Test Page">Test blah blah</a>')
+        assert (
+            f('<a href="https://github/a/b/wiki/Test Page">Test <b>Page</b></a>',
+              prefix, new) ==
+            '<a href="/p/test/wiki/Test Page">Test <b>Page</b></a>')
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_markup_with_mediawiki2markdown(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'''
+        source = '''
 ''Al'fredas 235 BC''
 == See also ==
 * [https://github.com/a/b/wiki/AgentSpring-running-instructions-for-d13n-model Test1]
@@ -411,7 +412,7 @@ Our website is [[http://domain.net]].
 * [https://github.com/a/b/wiki/AgentSpring-Q&A Test3]
 * [https://github.com/a/b/wiki/Extensions Test4]'''
 
-        result = u'''_Al'fredas 235 BC_
+        result = '''_Al'fredas 235 BC_
 
 ## See also
 
@@ -419,18 +420,19 @@ Our website is [[http://domain.net]].
   * [Test2](/p/test/wiki/AgentSpring conventions)
   * [Test3](/p/test/wiki/AgentSpring Q&A)
   * [Test4](/p/test/wiki/Extensions)
+
 '''
 
-        assert_equal(f(source, 'test.mediawiki'), result)
+        assert f(source, 'test.mediawiki') == result
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_textile_no_leading_tabs(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'''h1. Header 1
+        source = '''h1. Header 1
 
 Some text 1.
 
@@ -438,28 +440,28 @@ h2. Header 2
 
 See [[Page]]'''
 
-        result = u'''# Header 1
+        result = '''# Header 1
 
 Some text 1.
 
 ## Header 2
 
 See [Page]'''
-        assert_equal(f(source, 'test.textile').strip(), result)
+        assert f(source, 'test.textile').strip() == result
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_markup_with_amp_in_links(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'[[Ticks & Leeches]]'
-        result = u'[Ticks & Leeches]'
+        source = '[[Ticks & Leeches]]'
+        result = '[Ticks & Leeches]'
         # markdown should be untouched
-        assert_equal(f(source, 'test.rst').strip(), result)
+        assert f(source, 'test.rst').strip() == result
 
-    @skipif(module_not_available('html2text'))
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_markup_textile(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
@@ -468,51 +470,53 @@ See [Page]'''
         f = importer.convert_markup
 
         # check if lists converting works properly
-        source = u'''There are good reasons for this:
+        source = '''There are good reasons for this:
 
   # Duplicate libraries regularly break builds
   # Subtle bugs emerge with duplicate libraries, and to a lesser extent, duplicate tools
   # We want you to try harder to make your formula work with what OS X comes with
 '''
-        result = u'''There are good reasons for this:
+        result = '''There are good reasons for this:
 
   1. Duplicate libraries regularly break builds
   2. Subtle bugs emerge with duplicate libraries, and to a lesser extent, duplicate tools
   3. We want you to try harder to make your formula work with what OS X comes with
+
 '''
 
-        assert_equal(f(source, 'test.textile'), result)
+        assert f(source, 'test.textile') == result
 
         # textile-style links converts normal
         source = '*"Textile":Troubleshooting*'
-        result = '**[Textile](Troubleshooting)**\n'
-        assert_equal(f(source, 'test2.textile'), result)
+        result = '**[Textile](Troubleshooting)**\n\n'
+        assert f(source, 'test2.textile') == result
 
         # links with formatting converts normal in textile now
-        source = u'''*[[this checklist|Troubleshooting]]*
+        source = '''*[[this checklist|Troubleshooting]]*
 
 some text and *[[Tips n' Tricks]]*
 
 *[[link|http://otherlink.com]]*
 '''
-        result = u'''**[this checklist](Troubleshooting)**
+        result = '''**[this checklist](Troubleshooting)**
 
-some text and **[Tips n' Tricks]**
+some text and **[Tips n\u2019 Tricks]**
 
 **[link](http://otherlink.com)**
-'''
-        assert_equal(f(source, 'test3.textile'), result)
 
-    @skipif(module_not_available('html2text'))
+'''
+        assert f(source, 'test3.textile') == result
+
+    @skipIf(module_not_available('html2text'), 'html2text required')
     def test_convert_textile_special_tag(self):
         importer = GitHubWikiImporter()
         importer.github_wiki_url = 'https://github.com/a/b/wiki'
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'*[[this checklist|Troubleshooting]]*'
-        assert_equal(f(source, 't.textile').strip(),
-                     u'**[this checklist](Troubleshooting)**')
+        source = '*[[this checklist|Troubleshooting]]*'
+        assert (f(source, 't.textile').strip() ==
+                '**[this checklist](Troubleshooting)**')
 
     @without_module('html2text')
     def test_convert_textile_special_tag_without_html2text(self):
@@ -521,9 +525,9 @@ some text and **[Tips n' Tricks]**
         importer.app = Mock()
         importer.app.url = '/p/test/wiki/'
         f = importer.convert_markup
-        source = u'*[[this checklist|Troubleshooting]]*'
-        result = u'<p><strong>[[this checklist|Troubleshooting]]</strong></p>'
-        assert_equal(f(source, 't.textile').strip(), result)
+        source = '*[[this checklist|Troubleshooting]]*'
+        result = '<p><strong>[[this checklist|Troubleshooting]]</strong></p>'
+        assert f(source, 't.textile').strip() == result
 
     @patch('forgeimporters.github.wiki.mkdtemp', autospec=True)
     @patch('forgeimporters.github.wiki.rmtree', autospec=True)
@@ -531,7 +535,7 @@ some text and **[Tips n' Tricks]**
     def test_has_wiki_repo(self, repo, rmtree, mkdtemp):
         mkdtemp.return_value = 'fake path'
         i = GitHubWikiImporter()
-        assert_equal(i.has_wiki_repo('fake url'), True)
+        assert i.has_wiki_repo('fake url') is True
         repo.clone_from.assert_called_once_with(
             'fake url', to_path='fake path', bare=True)
         rmtree.assert_called_once_with('fake path')
@@ -539,7 +543,7 @@ some text and **[Tips n' Tricks]**
         def raise_error(*args, **kw):
             raise git.GitCommandError('bam', 'bam', 'bam')
         repo.clone_from.side_effect = raise_error
-        assert_equal(i.has_wiki_repo('fake url'), False)
+        assert i.has_wiki_repo('fake url') is False
 
 
 class TestGitHubWikiImportController(TestController, TestCase):
@@ -571,11 +575,11 @@ class TestGitHubWikiImportController(TestController, TestCase):
         self.assertEqual(r.location, 'http://localhost/p/%s/admin/' %
                          test_project_with_wiki)
         args = import_tool.post.call_args[1]
-        self.assertEqual(u'GitHub Wiki', args['mount_label'])
-        self.assertEqual(u'gh-wiki', args['mount_point'])
-        self.assertEqual(u'mulder', args['project_name'])
-        self.assertEqual(u'spooky', args['user_name'])
-        self.assertEqual(u'import_history', args['tool_option'])
+        self.assertEqual('GitHub Wiki', args['mount_label'])
+        self.assertEqual('gh-wiki', args['mount_point'])
+        self.assertEqual('mulder', args['project_name'])
+        self.assertEqual('spooky', args['user_name'])
+        self.assertEqual('import_history', args['tool_option'])
         self.assertEqual(requests.head.call_count, 1)
 
     @with_wiki
@@ -593,11 +597,11 @@ class TestGitHubWikiImportController(TestController, TestCase):
         self.assertEqual(r.location, 'http://localhost/p/%s/admin/' %
                          test_project_with_wiki)
         args = import_tool.post.call_args[1]
-        self.assertEqual(u'GitHub Wiki', args['mount_label'])
-        self.assertEqual(u'gh-wiki', args['mount_point'])
-        self.assertEqual(u'mulder', args['project_name'])
-        self.assertEqual(u'spooky', args['user_name'])
-        self.assertEqual(u'', args['tool_option'])
+        self.assertEqual('GitHub Wiki', args['mount_label'])
+        self.assertEqual('gh-wiki', args['mount_point'])
+        self.assertEqual('mulder', args['project_name'])
+        self.assertEqual('spooky', args['user_name'])
+        self.assertEqual('', args['tool_option'])
         self.assertEqual(requests.head.call_count, 1)
 
     @with_wiki
@@ -607,7 +611,7 @@ class TestGitHubWikiImportController(TestController, TestCase):
         requests.head.return_value.status_code = 200
         p = M.Project.query.get(shortname=test_project_with_wiki)
         p.set_tool_data('GitHubWikiImporter', pending=1)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         params = dict(
             gh_user_name='spooky',
             gh_project_name='mulder',
