@@ -25,19 +25,21 @@ import time
 import operator as op
 from subprocess import Popen, PIPE
 from hashlib import sha1
-from cStringIO import StringIO
+from io import BytesIO
 from datetime import datetime
 import tempfile
 from shutil import rmtree
+import typing
 
+import six
 import tg
 import pysvn
 from paste.deploy.converters import asbool, asint
 from pymongo.errors import DuplicateKeyError
-from pylons import tmpl_context as c, app_globals as g
+from tg import tmpl_context as c, app_globals as g
 
 from ming.base import Object
-from ming.orm import Mapper, FieldProperty
+from ming.odm import Mapper, FieldProperty
 from ming.utils import LazyProperty
 
 from allura import model as M
@@ -46,6 +48,10 @@ from allura.model.auth import User
 from allura.model.repository import zipdir
 from allura.model import repository as RM
 
+if typing.TYPE_CHECKING:
+    from ming.odm.mapper import Query
+
+
 log = logging.getLogger(__name__)
 
 
@@ -56,6 +62,9 @@ class Repository(M.Repository):
 
     class __mongometa__:
         name = 'svn-repository'
+
+    query: 'Query[Repository]'
+
     branches = FieldProperty([dict(name=str, object_id=str)])
     _refresh_precompute = False
 
@@ -69,8 +78,8 @@ class Repository(M.Repository):
         return self._impl.commit('HEAD')
 
     def tarball_filename(self, revision, path=None):
-        fn = super(Repository, self).tarball_filename(revision, path)
-        path = self._impl._path_to_root(path, revision)
+        fn = super().tarball_filename('r'+revision, path)
+        path = self._impl._tarball_path_clean(path, revision)
         fn += ('-' + '-'.join(path.split('/'))) if path else ''
         return fn
 
@@ -104,7 +113,7 @@ def svn_path_exists(path, rev=None):
         return False
 
 
-class SVNLibWrapper(object):
+class SVNLibWrapper:
 
     """Wrapper around pysvn, used for instrumentation."""
 
@@ -157,7 +166,7 @@ class SVNImplementation(M.RepositoryImplementation):
 
     @LazyProperty
     def _url(self):
-        return 'file://%s%s' % (self._repo.fs_path, self._repo.name)
+        return f'file://{self._repo.fs_path}{self._repo.name}'
 
     def shorthand_for_commit(self, oid):
         return '[r%d]' % self._revno(self.rev_parse(oid))
@@ -213,12 +222,12 @@ class SVNImplementation(M.RepositoryImplementation):
         # check for svn version 1.7 or later
         stdout, stderr, returncode = self.check_call(['svn', '--version'])
         pattern = r'version (?P<maj>\d+)\.(?P<min>\d+)'
-        m = re.search(pattern, stdout)
+        m = re.search(pattern, six.ensure_text(stdout))
         return m and (int(m.group('maj')) * 10 + int(m.group('min'))) >= 17
 
     def check_call(self, cmd, fail_on_error=True):
         p = Popen(cmd, stdin=PIPE, stdout=PIPE, stderr=PIPE)
-        stdout, stderr = p.communicate(input='p\n')
+        stdout, stderr = p.communicate(input=b'p\n')
         if p.returncode != 0 and fail_on_error:
             self._repo.set_status('ready')
             raise SVNCalledProcessError(cmd, p.returncode, stdout, stderr)
@@ -231,9 +240,9 @@ class SVNImplementation(M.RepositoryImplementation):
         def set_hook(hook_name):
             fn = os.path.join(self._repo.fs_path, self._repo.name,
                               'hooks', hook_name)
-            with open(fn, 'wb') as fp:
+            with open(fn, 'w') as fp:
                 fp.write('#!/bin/sh\n')
-            os.chmod(fn, 0755)
+            os.chmod(fn, 0o755)
 
         def clear_hook(hook_name):
             fn = os.path.join(self._repo.fs_path, self._repo.name,
@@ -262,7 +271,7 @@ class SVNImplementation(M.RepositoryImplementation):
             clear_hook('pre-revprop-change')
         else:
             def retry_cmd(cmd, fail_count=0):
-                max_fail = asint(tg.config.get('scm.import.retry_count', 50))
+                max_fail = asint(tg.config.get('scm.import.retry_count', 3))
                 returncode = -1
                 while returncode != 0 and fail_count < max_fail:
                     stdout, stderr, returncode = self.check_call(cmd, fail_on_error=False)
@@ -294,13 +303,13 @@ class SVNImplementation(M.RepositoryImplementation):
 
         """
         opts = self._repo.app.config.options
-        if not svn_path_exists('file://{0}{1}/{2}'.format(self._repo.fs_path,
+        if not svn_path_exists('file://{}{}/{}'.format(self._repo.fs_path,
                                                           self._repo.name, opts['checkout_url'])):
             opts['checkout_url'] = ''
 
         if (not opts['checkout_url'] and
                 svn_path_exists(
-                    'file://{0}{1}/trunk'.format(self._repo.fs_path,
+                    'file://{}{}/trunk'.format(self._repo.fs_path,
                                                  self._repo.name))):
             opts['checkout_url'] = 'trunk'
 
@@ -324,7 +333,7 @@ class SVNImplementation(M.RepositoryImplementation):
         commit) and ending with the root (first commit).
         """
         head_revno = self.head
-        return map(self._oid, range(head_revno, 0, -1))
+        return list(map(self._oid, list(range(head_revno, 0, -1))))
 
     def new_commits(self, all_commits=False):
         head_revno = self.head
@@ -387,7 +396,7 @@ class SVNImplementation(M.RepositoryImplementation):
         else:
             ci_doc = CommitDoc(dict(args, _id=oid))
             try:
-                ci_doc.m.insert(safe=True)
+                ci_doc.m.insert()
             except DuplicateKeyError:
                 if lazy:
                     return False
@@ -426,7 +435,7 @@ class SVNImplementation(M.RepositoryImplementation):
                     id=self._tree_oid(commit._id, path),
                     name=path))
             else:
-                assert False
+                raise AssertionError()
             lcd_entries.append(dict(
                 name=path,
                 commit_id=self._oid(info.last_changed_rev.number),
@@ -447,11 +456,11 @@ class SVNImplementation(M.RepositoryImplementation):
         return tree_id
 
     def _tree_oid(self, commit_id, path):
-        data = 'tree\n%s\n%s' % (commit_id, h.really_unicode(path))
+        data = f'tree\n{commit_id}\n{h.really_unicode(path)}'
         return sha1(data.encode('utf-8')).hexdigest()
 
     def _blob_oid(self, commit_id, path):
-        data = 'blob\n%s\n%s' % (commit_id, h.really_unicode(path))
+        data = f'blob\n{commit_id}\n{h.really_unicode(path)}'
         return sha1(data.encode('utf-8')).hexdigest()
 
     def _obj_oid(self, commit_id, info):
@@ -488,11 +497,11 @@ class SVNImplementation(M.RepositoryImplementation):
         if revs is None:
             revno = self.head
         else:
-            revno = max([self._revno(self.rev_parse(r)) for r in revs])
+            revno = max(self._revno(self.rev_parse(r)) for r in revs)
         if exclude is None:
             exclude = 0
         else:
-            exclude = max([self._revno(self.rev_parse(r)) for r in exclude])
+            exclude = max(self._revno(self.rev_parse(r)) for r in exclude)
         if path is None:
             url = self._url
         else:
@@ -504,7 +513,7 @@ class SVNImplementation(M.RepositoryImplementation):
                     url, revision_start=rev, peg_revision=rev, limit=limit,
                     discover_changed_paths=True)
             except pysvn.ClientError as e:
-                if 'Unable to connect' in e.message:
+                if 'Unable to connect' in e.args[0]:
                     raise  # repo error
                 return  # no (more) history for this path
             for ci in logs:
@@ -537,8 +546,7 @@ class SVNImplementation(M.RepositoryImplementation):
         size = None
         if path:
             try:
-                size = int(
-                    self._svn.list(url, revision=rev, peg_revision=rev)[0][0].size)
+                size = self._svn.list(url, revision=rev, peg_revision=rev)[0][0].size
             except pysvn.ClientError:
                 pass
         rename_details = {}
@@ -572,9 +580,9 @@ class SVNImplementation(M.RepositoryImplementation):
 
     def open_blob(self, blob):
         data = self._svn.cat(
-            self._url + blob.path(),
+            self._url + h.urlquote(blob.path()),
             revision=self._revision(blob.commit._id))
-        return StringIO(data)
+        return BytesIO(data)
 
     def blob_size(self, blob):
         try:
@@ -607,9 +615,9 @@ class SVNImplementation(M.RepositoryImplementation):
             url=self._repo.refresh_url())
         fn = os.path.join(self._repo.fs_path, self._repo.name,
                           'hooks', 'post-commit')
-        with open(fn, 'wb') as fp:
+        with open(fn, 'w') as fp:
             fp.write(text)
-        os.chmod(fn, 0755)
+        os.chmod(fn, 0o755)
 
     def _revno(self, oid):
         return int(oid.split(':')[1])
@@ -620,7 +628,7 @@ class SVNImplementation(M.RepositoryImplementation):
             self._revno(oid))
 
     def _oid(self, revno):
-        return '%s:%s' % (self._repo._id, revno)
+        return f'{self._repo._id}:{revno}'
 
     def last_commit_ids(self, commit, paths):
         '''
@@ -672,53 +680,48 @@ class SVNImplementation(M.RepositoryImplementation):
             log_entry = Object(date='', message='', changed_paths=[])
         return [p.path for p in log_entry.changed_paths]
 
-    def _path_to_root(self, path, rev=None):
-        '''Return tag/branch/trunk root for given path inside svn repo'''
+    def _tarball_path_clean(self, path, rev=None):
         if path:
-            path = path.strip('/').split('/')
-            idx = None
-            if 'tags' in path:
-                idx = path.index('tags')
-            elif 'branches' in path:
-                idx = path.index('branches')
-            # e.g. path/tags/tag-1.0/...
-            if idx is not None and idx < len(path) - 1:
-                return '/'.join(path[:idx + 2])  # path/tags/tag-1.0
-            if 'trunk' in path:
-                idx = path.index('trunk')
-                return '/'.join(path[:idx + 1])  # path/trunk
-        # no tag/brach/trunk in path
-        trunk_exists = svn_path_exists(
-            'file://%s%s/%s' % (self._repo.fs_path, self._repo.name, 'trunk'), rev)
-        if trunk_exists:
-            return 'trunk'
-        return ''
+            return path.strip('/')
+        else:
+            trunk_exists = svn_path_exists('file://{}{}/{}'.format(self._repo.fs_path, self._repo.name, 'trunk'), rev)
+            if trunk_exists:
+                return 'trunk'
+            return ''
 
     def tarball(self, commit, path=None):
-        path = self._path_to_root(path, commit)
+        """
+        Makes a svn export at `tmpdest`
+            then zips that into `dest/tmpfilename`
+            then renames that to `dest/filename`
+        """
+        path = self._tarball_path_clean(path, commit)
         if not os.path.exists(self._repo.tarball_path):
             os.makedirs(self._repo.tarball_path)
+        if not os.path.exists(self._repo.tarball_tmpdir):
+            os.makedirs(self._repo.tarball_tmpdir)
         archive_name = self._repo.tarball_filename(commit, path)
         dest = os.path.join(self._repo.tarball_path, archive_name)
-        filename = os.path.join(self._repo.tarball_path, '%s%s' %
-                                (archive_name, '.zip'))
-        tmpfilename = os.path.join(self._repo.tarball_path, '%s%s' %
-                                   (archive_name, '.tmp'))
+        tmpdest = os.path.join(self._repo.tarball_tmpdir, archive_name)
+        filename = os.path.join(self._repo.tarball_path, '{}{}'.format(archive_name, '.zip')).encode('utf-8')
+        tmpfilename = os.path.join(self._repo.tarball_path, '{}{}'.format(archive_name, '.tmp')).encode('utf-8')
         rmtree(dest.encode('utf8'), ignore_errors=True)  # must encode into bytes or it'll fail on non-ascii filenames
+        rmtree(tmpdest.encode('utf8'), ignore_errors=True)
         path = os.path.join(self._url, path)
         try:
             # need to set system locale to handle all symbols in filename
             import locale
             locale.setlocale(locale.LC_ALL, 'en_US.UTF-8')
             self._svn.export(path,
-                             dest,
+                             tmpdest,
                              revision=pysvn.Revision(
                                  pysvn.opt_revision_kind.number, commit),
                              ignore_externals=True)
-            zipdir(dest, tmpfilename)
+            zipdir(tmpdest, tmpfilename)
             os.rename(tmpfilename, filename)
         finally:
             rmtree(dest.encode('utf8'), ignore_errors=True)
+            rmtree(tmpdest.encode('utf8'), ignore_errors=True)
             if os.path.exists(tmpfilename):
                 os.remove(tmpfilename)
 
