@@ -18,6 +18,6 @@
 import logging
 
 from tg import config
-from pylons import app_globals as g
+from tg import app_globals as g
 
 log = logging.getLogger(__name__)
