@@ -15,24 +15,23 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 from datetime import datetime
-import urllib2
+from io import BytesIO
+import os
+import six.moves.urllib.parse
+import six.moves.urllib.request
+import six.moves.urllib.error
 
 import mock
-from ming.orm.ormsession import ThreadLocalORMSession
-from ming.orm import session
+import pytest
+from ming.odm.odmsession import ThreadLocalODMSession
+from ming.odm import session
 from ming import schema
-from nose.tools import (
-    raises,
-    assert_equal,
-    assert_in,
-    assert_true,
-    assert_false,
-)
 from forgetracker.model import Ticket, TicketAttachment
 from forgetracker.tests.unit import TrackerTestWithModel
 from forgetracker.import_support import ResettableStream
+import allura
 from allura.model import Feed, Post, User
 from allura.lib import helpers as h
 from allura.tests import decorators as td
@@ -58,7 +57,7 @@ class TestTicketModel(TrackerTestWithModel):
         # create and save the tickets
         t1 = _test_ticket()
         t2 = _test_ticket2()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
 
         # test label query results
         label_count1 = t1.artifacts_labeled_with(
@@ -70,13 +69,13 @@ class TestTicketModel(TrackerTestWithModel):
     def test_that_it_has_ordered_custom_fields(self):
         custom_fields = dict(my_field='my value')
         Ticket(summary='my ticket', custom_fields=custom_fields, ticket_num=3)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         ticket = Ticket.query.get(summary='my ticket')
         assert ticket.custom_fields == dict(my_field='my value')
 
-    @raises(schema.Invalid)
     def test_ticket_num_required(self):
-        Ticket(summary='my ticket')
+        with pytest.raises(schema.Invalid):
+            Ticket(summary='my ticket')
 
     def test_ticket_num_required2(self):
         t = Ticket(summary='my ticket', ticket_num=12)
@@ -89,27 +88,60 @@ class TestTicketModel(TrackerTestWithModel):
 
     def test_activity_extras(self):
         t = Ticket(summary='my ticket', ticket_num=12)
-        assert_in('allura_id', t.activity_extras)
-        assert_equal(t.activity_extras['summary'], t.summary)
+        assert 'allura_id' in t.activity_extras
+        assert t.activity_extras['summary'] == t.summary
 
     def test_has_activity_access(self):
         t = Ticket(summary='ticket', ticket_num=666)
-        assert_true(t.has_activity_access('read', c.user, 'activity'))
+        assert t.has_activity_access('read', c.user, 'activity')
         t.deleted = True
-        assert_false(t.has_activity_access('read', c.user, 'activity'))
+        assert not t.has_activity_access('read', c.user, 'activity')
 
     def test_comment_has_activity_access(self):
         t = Ticket(summary='ticket', ticket_num=666, deleted=True)
         p = t.discussion_thread.add_post(text='test post')
-        assert_equal(p.status, 'ok')
-        assert_true(p.has_activity_access('read', c.user, 'activity'))
+        assert p.status == 'ok'
+        assert p.has_activity_access('read', c.user, 'activity')
         p.status = 'spam'
-        assert_false(p.has_activity_access('read', c.user, 'activity'))
+        assert not p.has_activity_access('read', c.user, 'activity')
         p.status = 'pending'
-        assert_false(p.has_activity_access('read', c.user, 'activity'))
+        assert not p.has_activity_access('read', c.user, 'activity')
         p.status = 'ok'
         p.deleted = True
-        assert_false(p.has_activity_access('read', c.user, 'activity'))
+        assert not p.has_activity_access('read', c.user, 'activity')
+
+    def test_delete_ticket(self):
+        att_path = os.path.join(allura.__path__[0], 'tests', 'data', 'user.png')
+
+        ticket = Ticket(summary='ticket', ticket_num=666)
+        with open(att_path, 'rb') as fp:
+            ticket.attach('ticket.png', fp,
+                          discussion_id=ticket.discussion_thread.discussion._id,
+                          thread_id=ticket.discussion_thread._id)
+
+        post = ticket.discussion_thread.add_post(text='test post')
+        with open(att_path, 'rb') as fp:
+            post.attach('post.png', fp,
+                        discussion_id=ticket.discussion_thread.discussion._id,
+                        thread_id=ticket.discussion_thread._id,
+                        post_id=post._id)
+
+        ThreadLocalODMSession.flush_all()
+
+        assert Ticket.query.get(ticket_num=666)
+        assert Post.query.get(_id=post._id)
+        assert len(Ticket.attachment_class().query.find(dict(filename='ticket.png')).all()) == 2
+        assert len(Post.attachment_class().query.find(dict(filename='post.png')).all()) == 2
+
+        ticket.delete()
+
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
+
+        assert not Ticket.query.get(ticket_num=666)
+        assert not Post.query.get(_id=post._id)
+        assert not Ticket.attachment_class().query.find(dict(filename='ticket.png')).all()
+        assert not Post.attachment_class().query.find(dict(filename='post.png')).all()
 
     def test_private_ticket(self):
         from allura.model import ProjectRole
@@ -132,24 +164,24 @@ class TestTicketModel(TrackerTestWithModel):
         role_creator = ProjectRole.by_user(t.reported_by, upsert=True)._id
         ProjectRole.by_user(
             developer, upsert=True).roles.append(role_developer)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         cred = Credentials.get().clear()
 
         t.private = True
-        assert_equal(t.acl, [
-            ACE.allow(role_developer, 'save_searches'),
-            ACE.allow(role_developer, 'read'),
+        assert t.acl == [
             ACE.allow(role_developer, 'create'),
-            ACE.allow(role_developer, 'update'),
-            ACE.allow(role_developer, 'unmoderated_post'),
-            ACE.allow(role_developer, 'post'),
-            ACE.allow(role_developer, 'moderate'),
             ACE.allow(role_developer, 'delete'),
-            ACE.allow(role_creator, 'read'),
-            ACE.allow(role_creator, 'post'),
+            ACE.allow(role_developer, 'moderate'),
+            ACE.allow(role_developer, 'post'),
+            ACE.allow(role_developer, 'read'),
+            ACE.allow(role_developer, 'save_searches'),
+            ACE.allow(role_developer, 'unmoderated_post'),
+            ACE.allow(role_developer, 'update'),
             ACE.allow(role_creator, 'create'),
+            ACE.allow(role_creator, 'post'),
+            ACE.allow(role_creator, 'read'),
             ACE.allow(role_creator, 'unmoderated_post'),
-            DENY_ALL])
+            DENY_ALL]
         assert has_access(t, 'read', user=admin)()
         assert has_access(t, 'create', user=admin)()
         assert has_access(t, 'update', user=admin)()
@@ -190,15 +222,15 @@ class TestTicketModel(TrackerTestWithModel):
             summary='test ticket',
             description='test description',
             created_date=datetime(2012, 10, 29, 9, 57, 21, 465000))
-        assert_equal(t.created_date, datetime(2012, 10, 29, 9, 57, 21, 465000))
+        assert t.created_date == datetime(2012, 10, 29, 9, 57, 21, 465000)
         f = Feed.post(
             t,
             title=t.summary,
             description=t.description,
             pubdate=t.created_date)
-        assert_equal(f.pubdate, datetime(2012, 10, 29, 9, 57, 21, 465000))
-        assert_equal(f.title, 'test ticket')
-        assert_equal(f.description,
+        assert f.pubdate == datetime(2012, 10, 29, 9, 57, 21, 465000)
+        assert f.title == 'test ticket'
+        assert (f.description ==
                      '<div class="markdown_content"><p>test description</p></div>')
 
     @td.with_tool('test', 'Tickets', 'bugs', username='test-user')
@@ -213,33 +245,33 @@ class TestTicketModel(TrackerTestWithModel):
             ticket.assigned_to_id = User.by_username('test-user')._id
             ticket.discussion_thread.add_post(text='test comment')
 
-        assert_equal(
-            Ticket.query.find({'app_config_id': app1.config._id}).count(), 1)
-        assert_equal(
-            Ticket.query.find({'app_config_id': app2.config._id}).count(), 0)
-        assert_equal(
-            Post.query.find(dict(thread_id=ticket.discussion_thread._id)).count(), 1)
+        assert (
+            Ticket.query.find({'app_config_id': app1.config._id}).count() == 1)
+        assert (
+            Ticket.query.find({'app_config_id': app2.config._id}).count() == 0)
+        assert (
+            Post.query.find(dict(thread_id=ticket.discussion_thread._id)).count() == 1)
 
         t = ticket.move(app2.config)
-        assert_equal(
-            Ticket.query.find({'app_config_id': app1.config._id}).count(), 0)
-        assert_equal(
-            Ticket.query.find({'app_config_id': app2.config._id}).count(), 1)
-        assert_equal(t.summary, 'test ticket')
-        assert_equal(t.description, 'test description')
-        assert_equal(t.assigned_to.username, 'test-user')
-        assert_equal(t.url(), '/p/test/bugs2/1/')
+        assert (
+            Ticket.query.find({'app_config_id': app1.config._id}).count() == 0)
+        assert (
+            Ticket.query.find({'app_config_id': app2.config._id}).count() == 1)
+        assert t.summary == 'test ticket'
+        assert t.description == 'test description'
+        assert t.assigned_to.username == 'test-user'
+        assert t.url() == '/p/test/bugs2/1/'
 
         post = Post.query.find(dict(thread_id=ticket.discussion_thread._id,
                                     text={'$ne': 'test comment'})).first()
         assert post is not None, 'No comment about ticket moving'
         message = 'Ticket moved from /p/test/bugs/1/'
-        assert_equal(post.text, message)
+        assert post.text == message
 
         post = Post.query.find(dict(text='test comment')).first()
-        assert_equal(post.thread.discussion_id, app2.config.discussion_id)
-        assert_equal(post.thread.app_config_id, app2.config._id)
-        assert_equal(post.app_config_id, app2.config._id)
+        assert post.thread.discussion_id == app2.config.discussion_id
+        assert post.thread.app_config_id == app2.config._id
+        assert post.app_config_id == app2.config._id
 
     @td.with_tool('test', 'Tickets', 'bugs', username='test-user')
     @td.with_tool('test', 'Tickets', 'bugs2', username='test-user')
@@ -251,8 +283,8 @@ class TestTicketModel(TrackerTestWithModel):
             {'name': '_test2', 'type': 'string', 'label': 'Test field 2'}])
         app2.globals.custom_fields.append(
             {'name': '_test', 'type': 'string', 'label': 'Test field'})
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         with h.push_context(c.project._id, app_config_id=app1.config._id):
             ticket = Ticket.new()
             ticket.summary = 'test ticket'
@@ -261,16 +293,16 @@ class TestTicketModel(TrackerTestWithModel):
             ticket.custom_fields['_test2'] = 'test val 2'
 
         t = ticket.move(app2.config)
-        assert_equal(t.summary, 'test ticket')
-        assert_equal(t.description, 'test description')
-        assert_equal(t.custom_fields['_test'], 'test val')
+        assert t.summary == 'test ticket'
+        assert t.description == 'test description'
+        assert t.custom_fields['_test'] == 'test val'
         post = Post.query.find(
             dict(thread_id=ticket.discussion_thread._id)).first()
         assert post is not None, 'No comment about ticket moving'
         message = 'Ticket moved from /p/test/bugs/1/'
         message += '\n\nCan\'t be converted:\n'
         message += '\n- **_test2**: test val 2'
-        assert_equal(post.text, message)
+        assert post.text == message
 
     @td.with_tool('test', 'Tickets', 'bugs', username='test-user')
     @td.with_tool('test', 'Tickets', 'bugs2', username='test-user')
@@ -283,8 +315,8 @@ class TestTicketModel(TrackerTestWithModel):
         app2.globals.custom_fields.extend([
             {'name': '_user_field', 'type': 'user', 'label': 'User field'},
             {'name': '_user_field_2', 'type': 'user', 'label': 'User field 2'}])
-        ThreadLocalORMSession.flush_all()
-        ThreadLocalORMSession.close_all()
+        ThreadLocalODMSession.flush_all()
+        ThreadLocalODMSession.close_all()
         from allura.websetup import bootstrap
         bootstrap.create_user('test-user-0')
         with h.push_context(c.project._id, app_config_id=app1.config._id):
@@ -298,9 +330,9 @@ class TestTicketModel(TrackerTestWithModel):
             ticket.assigned_to_id = User.by_username('test-user-0')._id
 
         t = ticket.move(app2.config)
-        assert_equal(t.assigned_to_id, None)
-        assert_equal(t.custom_fields['_user_field'], 'test-user')
-        assert_equal(t.custom_fields['_user_field_2'], '')
+        assert t.assigned_to_id is None
+        assert t.custom_fields['_user_field'] == 'test-user'
+        assert t.custom_fields['_user_field_2'] == ''
         post = Post.query.find(
             dict(thread_id=ticket.discussion_thread._id)).first()
         assert post is not None, 'No comment about ticket moving'
@@ -308,7 +340,7 @@ class TestTicketModel(TrackerTestWithModel):
         message += '\n\nCan\'t be converted:\n'
         message += '\n- **_user_field_2**: test-user-0 (user not in project)'
         message += '\n- **assigned_to**: test-user-0 (user not in project)'
-        assert_equal(post.text, message)
+        assert post.text == message
 
     @td.with_tool('test', 'Tickets', 'bugs', username='test-user')
     def test_attach_with_resettable_stream(self):
@@ -316,24 +348,24 @@ class TestTicketModel(TrackerTestWithModel):
             ticket = Ticket.new()
             ticket.summary = 'test ticket'
             ticket.description = 'test description'
-        assert_equal(len(ticket.attachments), 0)
-        f = urllib2.urlopen('file://%s' % __file__)
+        assert len(ticket.attachments) == 0
+        f = six.moves.urllib.request.urlopen('file://%s' % __file__)
         TicketAttachment.save_attachment(
             'test_ticket_model.py', ResettableStream(f),
             artifact_id=ticket._id)
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         # need to refetch since attachments are cached
         session(ticket).expunge(ticket)
         ticket = Ticket.query.get(_id=ticket._id)
-        assert_equal(len(ticket.attachments), 1)
-        assert_equal(ticket.attachments[0].filename, 'test_ticket_model.py')
+        assert len(ticket.attachments) == 1
+        assert ticket.attachments[0].filename == 'test_ticket_model.py'
 
     def test_json_parents(self):
         ticket = Ticket.new()
-        json_keys = ticket.__json__().keys()
-        assert_in('related_artifacts', json_keys)  # from Artifact
-        assert_in('votes_up', json_keys)  # VotableArtifact
-        assert_in('ticket_num', json_keys)  # Ticket
+        json_keys = list(ticket.__json__().keys())
+        assert 'related_artifacts' in json_keys  # from Artifact
+        assert 'votes_up' in json_keys  # VotableArtifact
+        assert 'ticket_num' in json_keys  # Ticket
         assert ticket.__json__()['assigned_to'] is None
 
     @mock.patch('forgetracker.model.ticket.tsearch')
@@ -347,20 +379,20 @@ class TestTicketModel(TrackerTestWithModel):
         filter = None
         Ticket.paged_query_or_search(app_cfg, user, mongo_query, solr_query, filter, **kw)
         query.assert_called_once_with(app_cfg, user, mongo_query, sort=None, limit=None, page=0, **kw)
-        assert_equal(tsearch.query_filter_choices.call_count, 1)
-        assert_equal(tsearch.query_filter_choices.call_args[0][0], 'solr query')
-        assert_equal(search.call_count, 0)
+        assert tsearch.query_filter_choices.call_count == 1
+        assert tsearch.query_filter_choices.call_args[0][0] == 'solr query'
+        assert search.call_count == 0
         query.reset_mock(), search.reset_mock(), tsearch.reset_mock()
 
         filter = {'status': 'unread'}
         Ticket.paged_query_or_search(app_cfg, user, mongo_query, solr_query, filter, **kw)
         search.assert_called_once_with(app_cfg, user, solr_query, filter=filter, sort=None, limit=None, page=0, **kw)
-        assert_equal(query.call_count, 0)
-        assert_equal(tsearch.query_filter_choices.call_count, 0)
+        assert query.call_count == 0
+        assert tsearch.query_filter_choices.call_count == 0
 
     def test_index(self):
         idx = Ticket(ticket_num=2, summary="ticket2", labels=["mylabel", "other"]).index()
-        assert_equal(idx['summary_t'], 'ticket2')
-        assert_equal(idx['labels_t'], 'mylabel other')
-        assert_equal(idx['reported_by_s'], 'test-user')
-        assert_equal(idx['assigned_to_s'], None)  # must exist at least
+        assert idx['summary_t'] == 'ticket2'
+        assert idx['labels_t'] == 'mylabel other'
+        assert idx['reported_by_s'] == 'test-user'
+        assert idx['assigned_to_s'] is None  # must exist at least
