@@ -18,34 +18,34 @@
 import logging
 import re
 from datetime import datetime, timedelta
-from functools import partial
-from urllib import urlencode, unquote
+from urllib.parse import urlencode, unquote
 from webob import exc
 import json
 import os
 
 # Non-stdlib imports
-import pkg_resources
 from tg import expose, validate, redirect, flash, url, jsonify
 from tg.decorators import with_trailing_slash, without_trailing_slash
 from paste.deploy.converters import aslist
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request, response
+from tg import tmpl_context as c, app_globals as g
+from tg import request, response
 from formencode import validators
 from bson import ObjectId
 from bson.son import SON
 from bson.errors import InvalidId
-from webhelpers import feedgenerator as FG
+import feedgenerator as FG
 
 from ming import schema
 from ming.odm import session
-from ming.orm.ormsession import ThreadLocalORMSession
+from ming.odm.odmsession import ThreadLocalODMSession
 from ming.utils import LazyProperty
 
 # Pyforge-specific imports
 from allura import model as M
 from allura.lib import helpers as h
+from allura.lib import validators as v
 from allura.lib import utils
+from allura.tasks import notification_tasks
 from allura.app import (
     Application,
     SitemapEntry,
@@ -55,11 +55,12 @@ from allura.app import (
 )
 from allura.lib.search import search_artifact, SearchError
 from allura.lib.solr import escape_solr_arg
-from allura.lib.decorators import require_post
+from allura.lib.decorators import require_post, memorable_forget
 from allura.lib.security import (require_access, has_access, require,
                                  require_authenticated)
 from allura.lib import widgets as w
 from allura.lib import validators as V
+from allura.lib.utils import permanent_redirect
 from allura.lib.widgets import form_fields as ffw
 from allura.lib.widgets.subscriptions import SubscribeForm
 from allura.lib.plugin import ImportIdConverter
@@ -80,17 +81,18 @@ from forgetracker.widgets.ticket_form import TicketForm, TicketCustomField
 from forgetracker.widgets.bin_form import BinForm
 from forgetracker.widgets.ticket_search import TicketSearchResults, MassEdit, MassEditForm, MassMoveForm
 from forgetracker.widgets.admin_custom_fields import TrackerFieldAdmin, TrackerFieldDisplay
+import six
 
 log = logging.getLogger(__name__)
 
 search_validators = dict(
-    q=validators.UnicodeString(if_empty=None),
+    q=v.UnicodeString(if_empty=None),
     history=validators.StringBool(if_empty=False),
     project=validators.StringBool(if_empty=False),
     limit=validators.Int(if_invalid=None),
     page=validators.Int(if_empty=0, if_invalid=0),
-    sort=validators.UnicodeString(if_empty=None),
-    filter=V.JsonConverter(if_empty={}),
+    sort=v.UnicodeString(if_empty=None),
+    filter=V.JsonConverter(if_empty={}, if_invalid={}),
     deleted=validators.StringBool(if_empty=False))
 
 
@@ -147,10 +149,8 @@ def get_change_text(name, new_value, old_value):
         new_value = ', '.join(new_value)
     changes[name] = old_value
     changes[name] = new_value
-    tpl_fn = pkg_resources.resource_filename(
-        'forgetracker', 'data/ticket_changed_tmpl')
-    return h.render_genshi_plaintext(
-        tpl_fn,
+    tmpl = g.jinja2_env.get_template('forgetracker:data/ticket_changed.html')
+    return tmpl.render(
         changelist=changes.get_changed(),
         comment=None)
 
@@ -158,11 +158,11 @@ def get_change_text(name, new_value, old_value):
 def attachments_info(attachments):
     text = []
     for attach in attachments:
-        text.append(u"{} ({}; {})".format(
+        text.append("{} ({}; {})".format(
             h.really_unicode(attach.filename),
             h.do_filesizeformat(attach.length),
             attach.content_type))
-    return u"\n".join(text)
+    return "\n".join(text)
 
 
 def render_changes(changes, comment=None):
@@ -171,14 +171,10 @@ def render_changes(changes, comment=None):
 
     Returns tuple (post_text, notification_text)
     """
-    template = pkg_resources.resource_filename(
-        'forgetracker', 'data/ticket_changed_tmpl')
-    render = partial(
-        h.render_genshi_plaintext,
-        template,
-        changelist=changes.get_changed())
-    post_text = render(comment=None)
-    notification_text = render(comment=comment) if comment else None
+    tmpl = g.jinja2_env.get_template('forgetracker:data/ticket_changed.html')
+    changelist = changes.get_changed()
+    post_text = tmpl.render(h=h, changelist=changelist, comment=None)
+    notification_text = tmpl.render(h=h, changelist=changelist, comment=comment) if comment else None
     return post_text, notification_text
 
 
@@ -194,7 +190,7 @@ def _my_trackers(user, current_tracker_app_config):
         for ac in p.app_configs:
             if ac.tool_name.lower() == 'tickets':
                 trac = (str(ac._id),
-                        '%s/%s' % (p.shortname, ac.options['mount_point']),
+                        '{}/{}'.format(p.shortname, ac.options['mount_point']),
                         bool(current_tracker_app_config == ac))
                 trackers.append(trac)
     return trackers
@@ -284,10 +280,12 @@ class ForgeTrackerApp(Application):
             ticket = TM.Ticket.query.get(
                 app_config_id=self.config._id,
                 ticket_num=int(topic))
-        except:
+        except Exception:
             log.exception('Error getting ticket %s', topic)
             return
-        if ticket.discussion_disabled:
+        if not ticket:
+            log.info('No such ticket num: %s', topic)
+        elif ticket.discussion_disabled:
             log.info('Discussion disabled for ticket %s', ticket.ticket_num)
         else:
             self.handle_artifact_message(ticket, message)
@@ -300,6 +298,17 @@ class ForgeTrackerApp(Application):
             self.config.options.mount_label,
             '.')]
 
+    def sitemap_xml(self):
+        """
+        Used for generating sitemap.xml.
+        If the root page has default content, omit it from the sitemap.xml.
+        Assumes :attr:`main_menu` will return an entry pointing to the root page.
+        :return: a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
+        """
+        if self.should_noindex():
+            return []
+        return self.main_menu()
+
     @property
     @h.exceptionless([], log)
     def sitemap(self):
@@ -308,12 +317,16 @@ class ForgeTrackerApp(Application):
             return [
                 SitemapEntry(menu_id, '.')[self.sidebar_menu()]]
 
+    def should_noindex(self):
+        has_ticket = TM.Ticket.query.get(app_config_id=self.config._id, deleted=False)
+        return has_ticket is None
+
     def admin_menu(self):
         admin_url = c.project.url() + 'admin/' + \
             self.config.options.mount_point + '/'
         links = [SitemapEntry('Field Management', admin_url + 'fields'),
                  SitemapEntry('Edit Searches', admin_url + 'bins/')]
-        links += super(ForgeTrackerApp, self).admin_menu()
+        links += super().admin_menu()
         # change Options menu html class
         for link in links:
             if link.label == 'Options':
@@ -358,7 +371,7 @@ class ForgeTrackerApp(Application):
             links.append(SitemapEntry('Edit Searches', c.project.url() + 'admin/' +
                          c.app.config.options.mount_point + '/bins/', ui_icon=g.icons['search']))
         links.append(SitemapEntry('View Stats', self.config.url()
-                     + 'stats', ui_icon=g.icons['stats']))
+                     + 'stats/', ui_icon=g.icons['stats']))
         discussion = c.app.config.discussion
         pending_mod_count = M.Post.query.find({
             'discussion_id': discussion._id,
@@ -378,7 +391,7 @@ class ForgeTrackerApp(Application):
             links = links + search_bins
         links.append(SitemapEntry('Help'))
         links.append(
-            SitemapEntry('Formatting Help', self.config.url() + 'markdown_syntax'))
+            SitemapEntry('Formatting Help', '/nf/markdown_syntax', extra_html_attrs={'target': '_blank'}))
         return links
 
     def sidebar_menu_js(self):
@@ -425,7 +438,7 @@ class ForgeTrackerApp(Application):
 
     def install(self, project):
         'Set up any default permissions and roles here'
-        super(ForgeTrackerApp, self).install(project)
+        super().install(project)
         # Setup permissions
         role_admin = M.ProjectRole.by_name('Admin')._id
         role_developer = M.ProjectRole.by_name('Developer')._id
@@ -472,9 +485,10 @@ class ForgeTrackerApp(Application):
         app_config_id = {'app_config_id': c.app.config._id}
         TM.TicketAttachment.query.remove(app_config_id)
         TM.Ticket.query.remove(app_config_id)
+        TM.Ticket.__mongometa__.history_class.query.remove(app_config_id)
         TM.Bin.query.remove(app_config_id)
         TM.Globals.query.remove(app_config_id)
-        super(ForgeTrackerApp, self).uninstall(project)
+        super().uninstall(project)
 
     def bulk_export(self, f, export_path='', with_attachments=False):
         f.write('{"tickets": [')
@@ -487,7 +501,7 @@ class ForgeTrackerApp(Application):
             GenericClass = utils.JSONForExport
             self.export_attachments(tickets, export_path)
         else:
-            GenericClass = jsonify.GenericJSON
+            GenericClass = jsonify.JSONEncoder
 
         for i, ticket in enumerate(tickets):
             if i > 0:
@@ -536,7 +550,7 @@ class ForgeTrackerApp(Application):
             if fld.name == '_milestone':
                 for m in fld.milestones:
                     d = self.globals.milestone_count(
-                        '%s:%s' % (fld.name, m.name))
+                        f'{fld.name}:{m.name}')
                     milestones.append(dict(
                         name=m.name,
                         due_date=m.get('due_date'),
@@ -652,13 +666,6 @@ class RootController(BaseController, FeedController):
     def _check_security(self):
         require_access(c.app, 'read')
 
-    def rate_limit(self, redir='..'):
-        if TM.Ticket.is_limit_exceeded(c.app.config, user=c.user):
-            msg = 'Ticket creation rate limit exceeded. '
-            log.warn(msg + c.app.config.url())
-            flash(msg + 'Please try again later.', 'error')
-            redirect(redir)
-
     @expose('json:')
     def bin_counts(self, *args, **kw):
         bin_counts = []
@@ -682,7 +689,7 @@ class RootController(BaseController, FeedController):
                 if m.complete:
                     continue
                 count = c.app.globals.milestone_count(
-                    '%s:%s' % (fld.name, m.name))['hits']
+                    f'{fld.name}:{m.name}')['hits']
                 name = h.text.truncate(m.name, 72)
                 milestone_counts.append({'name': name, 'count': count})
         return {'milestone_counts': milestone_counts}
@@ -708,8 +715,8 @@ class RootController(BaseController, FeedController):
             {'$match': {'labels': {'$regex': '^%s' % term, '$options': 'i'}}},
             {'$group': {'_id': '$labels', 'count': {'$sum': 1}}},
             {'$sort': SON([('count', -1), ('_id', 1)])}
-        ])
-        return json.dumps([tag['_id'] for tag in tags.get('result', [])])
+        ], cursor={})
+        return json.dumps([tag['_id'] for tag in tags])
 
     @with_trailing_slash
     @h.vardec
@@ -723,10 +730,6 @@ class RootController(BaseController, FeedController):
         elif deleted and not has_access(c.app, 'delete'):
             deleted = False
 
-        if not isinstance(filter, dict):
-            # JsonConverter above can return an int, string, etc, if users give bad inputs, but it needs to be a dict
-            filter = {}
-
         # it's just our original query mangled and sent back to us
         kw.pop('q', None)
         result = TM.Ticket.paged_query_or_search(c.app.config, c.user,
@@ -794,7 +797,7 @@ class RootController(BaseController, FeedController):
                                 m.complete = new['complete'] == 'Closed'
                                 m.default = new.get('default', False)
                                 if new['old_name'] != m.name:
-                                    q = '%s:"%s"' % (fld.name, new['old_name'])
+                                    q = '{}:"{}"'.format(fld.name, new['old_name'])
                                     # search_artifact() limits results to 10
                                     # rows by default, so give it a high upper
                                     # bound to make sure we get all tickets
@@ -867,13 +870,13 @@ class RootController(BaseController, FeedController):
         response.headers['Content-Type'] = ''
         response.content_type = 'application/xml'
         d = dict(title='Ticket search results', link=h.absurl(c.app.url),
-                 description='You searched for %s' % q, language=u'en')
+                 description='You searched for %s' % q, language='en')
         if request.environ['PATH_INFO'].endswith('.atom'):
             feed = FG.Atom1Feed(**d)
         else:
             feed = FG.Rss201rev2Feed(**d)
         for t in result['tickets']:
-            url = h.absurl(t.url().encode('utf-8'))
+            url = h.absurl(t.url())
             feed_kwargs = dict(title=t.summary,
                                link=url,
                                pubdate=t.mod_date,
@@ -899,29 +902,28 @@ class RootController(BaseController, FeedController):
     @expose('jinja:forgetracker:templates/tracker/search_help.html')
     def search_help(self, **kw):
         'Static page with search help'
-        return dict()
+        return dict(
+            custom_fields=[fld for fld in (c.app.globals.custom_fields or []) if fld.name != '_milestone'],
+        )
 
     @with_trailing_slash
     @expose('jinja:forgetracker:templates/tracker/new_ticket.html')
-    def new(self, description=None, summary=None, labels=None, **kw):
+    def new(self, **kw):
         require_access(c.app, 'create')
-        self.rate_limit(redir='..')
+        self.rate_limit(TM.Ticket, 'Ticket creation', redir='..')
         c.ticket_form = W.ticket_form
         help_msg = c.app.config.options.get('TicketHelpNew', '').strip()
         return dict(action=c.app.config.url() + 'save_ticket',
                     help_msg=help_msg,
-                    description=description, summary=summary, labels=labels)
+                    url_params=kw,
+                    subscribed_to_tool=M.Mailbox.subscribed(),
+                    )
 
-    @expose('jinja:allura:templates/markdown_syntax.html')
+    @expose()
     def markdown_syntax(self, **kw):
-        """Static page explaining markdown."""
-        return dict()
-
-    @expose('jinja:allura:templates/markdown_syntax_dialog.html')
-    def markdown_syntax_dialog(self, **kw):
-        """Static page explaining markdown."""
-        return dict()
+        permanent_redirect('/nf/markdown_syntax')
 
+    @memorable_forget()
     @expose()
     @h.vardec
     @require_post()
@@ -940,25 +942,27 @@ class RootController(BaseController, FeedController):
             if not ticket:
                 raise Exception('Ticket number not found.')
             require_access(ticket, 'update')
+            ticket.update_fields_basics(ticket_form)
         else:
             require_access(c.app, 'create')
-            self.rate_limit(redir='.')
-            ticket = TM.Ticket.new()
-        g.spam_checker.check(ticket_form['summary'] + u'\n' + ticket_form.get('description', ''), artifact=ticket,
+            self.rate_limit(TM.Ticket, 'Ticket creation', redir='.')
+            ticket = TM.Ticket.new(form_fields=ticket_form)
+        ticket.update_fields_finish(ticket_form)
+        g.spam_checker.check(ticket_form['summary'] + '\n' + ticket_form.get('description', ''), artifact=ticket,
                              user=c.user, content_type='ticket')
-        ticket.update(ticket_form)
         c.app.globals.invalidate_bin_counts()
+        notification_tasks.send_usermentions_notification.post(ticket.index_id(), ticket_form.get('description', ''))
         g.director.create_activity(c.user, 'created', ticket,
                                    related_nodes=[c.project], tags=['ticket'])
         redirect(str(ticket.ticket_num) + '/')
 
     @with_trailing_slash
     @expose('jinja:forgetracker:templates/tracker/mass_edit.html')
-    @validate(dict(q=validators.UnicodeString(if_empty=None),
+    @validate(dict(q=v.UnicodeString(if_empty=None),
                    filter=V.JsonConverter(if_empty={}),
                    limit=validators.Int(if_empty=10, if_invalid=10),
                    page=validators.Int(if_empty=0, if_invalid=0),
-                   sort=validators.UnicodeString(if_empty='ticket_num_i asc')))
+                   sort=v.UnicodeString(if_empty='ticket_num_i asc')))
     def edit(self, q=None, limit=None, page=None, sort=None, filter=None, **kw):
         require_access(c.app, 'update')
         result = TM.Ticket.paged_search(c.app.config, c.user, q, filter=filter,
@@ -982,11 +986,11 @@ class RootController(BaseController, FeedController):
 
     @with_trailing_slash
     @expose('jinja:forgetracker:templates/tracker/mass_move.html')
-    @validate(dict(q=validators.UnicodeString(if_empty=None),
+    @validate(dict(q=v.UnicodeString(if_empty=None),
                    filter=V.JsonConverter(if_empty={}),
                    limit=validators.Int(if_empty=10, if_invalid=10),
                    page=validators.Int(if_empty=0, if_invalid=0),
-                   sort=validators.UnicodeString(if_empty='ticket_num_i asc')))
+                   sort=v.UnicodeString(if_empty='ticket_num_i asc')))
     def move(self, q=None, limit=None, page=None, sort=None, filter=None, **kw):
         require_access(c.app, 'admin')
         result = TM.Ticket.paged_search(c.app.config, c.user, q, filter=filter,
@@ -1034,7 +1038,7 @@ class RootController(BaseController, FeedController):
         tasks.move_tickets.post(ticket_ids, destination_tracker_id)
 
         c.app.globals.invalidate_bin_counts()
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
         count = len(tickets)
         flash('Move scheduled ({} ticket{})'.format(
             count, 's' if count != 1 else ''), 'ok')
@@ -1103,7 +1107,7 @@ class RootController(BaseController, FeedController):
         c.user_select = ffw.ProjectUserCombo()
         if dates is None:
             today = datetime.utcnow()
-            dates = "%s to %s" % ((today - timedelta(days=61))
+            dates = "{} to {}".format((today - timedelta(days=61))
                                   .strftime('%Y-%m-%d'), today.strftime('%Y-%m-%d'))
         return dict(
             now=str(now),
@@ -1155,23 +1159,11 @@ class BinController(BaseController, AdminControllerMixin):
         count = len(self.app.bins)
         return dict(bins=self.app.bins, count=count, app=self.app)
 
-    @with_trailing_slash
-    @expose('jinja:forgetracker:templates/tracker/bin.html')
-    def bins(self):
-        count = len(self.app.bins)
-        return dict(bins=self.app.bins, count=count, app=self.app)
-
-    @with_trailing_slash
-    @expose('jinja:forgetracker:templates/tracker/new_bin.html')
-    def newbin(self, q=None, **kw):
-        c.bin_form = W.bin_form
-        return dict(q=q or '', bin=bin or '', modelname='Bin', page='New Bin', globals=self.app.globals)
-
     @with_trailing_slash
     @h.vardec
     @expose('jinja:forgetracker:templates/tracker/bin.html')
     @require_post()
-    @validate(W.bin_form, error_handler=newbin)
+    @validate(W.bin_form, error_handler=index)
     def save_bin(self, **bin_form):
         """Update existing search bin or create a new one.
 
@@ -1200,7 +1192,7 @@ class BinController(BaseController, AdminControllerMixin):
             # Search threw an error.
             # Save the error on the bin object for displaying
             # in the template.
-            setattr(bin, 'error', str(e))
+            bin.error = str(e)
             # Expunge the bin object so we don't save the
             # errant search terms to mongo.
             M.session.artifact_orm_session.expunge(bin)
@@ -1210,15 +1202,6 @@ class BinController(BaseController, AdminControllerMixin):
         self.app.globals.invalidate_bin_counts()
         redirect('.')
 
-    @with_trailing_slash
-    @expose()
-    @require_post()
-    @validate(validators=dict(bin=V.Ming(TM.Bin)))
-    def delbin(self, bin=None):
-        require(lambda: bin.app_config_id == self.app.config._id)
-        bin.delete()
-        redirect(request.referer)
-
     @without_trailing_slash
     @h.vardec
     @expose('jinja:forgetracker:templates/tracker/bin.html')
@@ -1273,7 +1256,7 @@ class BinController(BaseController, AdminControllerMixin):
                             # Search threw an error.
                             # Save the error on the bin object for displaying
                             # in the template.
-                            setattr(bin, 'error', str(e))
+                            bin.error = str(e)
                             errors = True
                             # Expunge the bin object so we don't save the
                             # errant search terms to mongo.
@@ -1296,7 +1279,7 @@ class BinController(BaseController, AdminControllerMixin):
         redirect('.')
 
 
-class changelog(object):
+class changelog:
 
     """
     A dict-like object which keeps log about what keys have been changed.
@@ -1406,7 +1389,7 @@ class TicketController(BaseController, FeedController):
                         subscribed=subscribed, voting_enabled=voting_enabled,
                         page=page, limit=limit, count=post_count)
         else:
-            raise exc.HTTPNotFound, 'Ticket #%s does not exist.' % self.ticket_num
+            raise exc.HTTPNotFound('Ticket #%s does not exist.' % self.ticket_num)
 
     def get_feed(self, project, app, user):
         """Return a :class:`allura.controllers.feed.FeedArgs` object describing
@@ -1415,6 +1398,8 @@ class TicketController(BaseController, FeedController):
         Overrides :meth:`allura.controllers.feed.FeedController.get_feed`.
 
         """
+        if not self.ticket:
+            return None
         title = 'Recent changes to %d: %s' % (
             self.ticket.ticket_num, self.ticket.summary)
         return FeedArgs(
@@ -1435,6 +1420,7 @@ class TicketController(BaseController, FeedController):
             post_data['labels'] = []
         self._update_ticket(post_data)
 
+    @memorable_forget()
     @expose()
     @require_post()
     @h.vardec
@@ -1451,14 +1437,8 @@ class TicketController(BaseController, FeedController):
     @expose('json:')
     @require_post()
     def delete(self, **kw):
-        require_access(self.ticket, 'delete')
-        M.Shortlink.query.remove(dict(ref_id=self.ticket.index_id()))
-        self.ticket.deleted = True
-        suffix = " {dt.hour}:{dt.minute}:{dt.second} {dt.day}-{dt.month}-{dt.year}".format(
-            dt=datetime.utcnow())
-        self.ticket.summary += suffix
+        self.ticket.soft_delete()
         flash('Ticket successfully deleted')
-        c.app.globals.invalidate_bin_counts()
         return dict(location='../' + str(self.ticket.ticket_num))
 
     @without_trailing_slash
@@ -1467,8 +1447,7 @@ class TicketController(BaseController, FeedController):
     def undelete(self, **kw):
         require_access(self.ticket, 'delete')
         self.ticket.deleted = False
-        self.ticket.summary = re.sub(
-            ' \d+:\d+:\d+ \d+-\d+-\d+$', '', self.ticket.summary)
+        self.ticket.summary = re.sub(r' \d+:\d+:\d+ \d+-\d+-\d+$', '', self.ticket.summary)
         M.Shortlink.from_artifact(self.ticket)
         flash('Ticket successfully restored')
         c.app.globals.invalidate_bin_counts()
@@ -1477,7 +1456,9 @@ class TicketController(BaseController, FeedController):
     @require_post()
     def _update_ticket(self, post_data):
         require_access(self.ticket, 'update')
-        g.spam_checker.check(post_data.get('summary', '') + u'\n' + post_data.get('description', ''),
+        old_text = self.ticket.description
+        new_text = post_data.get('description', '')
+        g.spam_checker.check(post_data.get('summary', '') + '\n' + post_data.get('description', ''),
                              artifact=self.ticket, user=c.user, content_type='ticket')
         changes = changelog()
         comment = post_data.pop('comment', None)
@@ -1557,11 +1538,34 @@ class TicketController(BaseController, FeedController):
         self.ticket.commit()
         if comment:
             thread.post(text=comment, notify=False)
+        notification_tasks.send_usermentions_notification.post(self.ticket.index_id(), new_text, old_text)
         g.director.create_activity(c.user, 'modified', self.ticket,
                                    related_nodes=[c.project], tags=['ticket'])
         c.app.globals.invalidate_bin_counts()
         redirect('.')
 
+    @without_trailing_slash
+    @expose('json:')
+    @require_post()
+    def update_markdown(self, text=None, **kw):
+        if has_access(self.ticket, 'update'):
+            self.ticket.description = text
+            self.ticket.commit()
+            g.director.create_activity(c.user, 'modified', self.ticket, related_nodes=[c.project], tags=['ticket'])
+            g.spam_checker.check(text, artifact=self.ticket, user=c.user, content_type='ticket')
+            return {
+                'status': 'success'
+            }
+        else:
+            return {
+                'status': 'no_permission'
+            }
+
+    @expose()
+    @without_trailing_slash
+    def get_markdown(self):
+        return self.ticket.description
+
     @expose('json:')
     @require_post()
     @validate(W.subscribe_form)
@@ -1608,16 +1612,16 @@ class TicketController(BaseController, FeedController):
             tracker = M.AppConfig.query.get(_id=t_id)
             if tracker is None:
                 flash('Select valid tracker', 'error')
-                redirect(request.referer)
+                redirect(six.ensure_text(request.referer or '/'))
 
             if tracker == self.ticket.app.config:
                 flash('Ticket already in a selected tracker', 'info')
-                redirect(request.referer)
+                redirect(six.ensure_text(request.referer or '/'))
 
             if not has_access(tracker, 'admin')():
                 flash('You should have admin access to destination tracker',
                       'error')
-                redirect(request.referer)
+                redirect(six.ensure_text(request.referer or '/'))
 
             new_ticket = self.ticket.move(tracker)
             c.app.globals.invalidate_bin_counts()
@@ -1637,7 +1641,7 @@ class AttachmentController(att.AttachmentController):
 
     def handle_post(self, delete, **kw):
         old_attachments = attachments_info(self.artifact.attachments)
-        super(AttachmentController, self).handle_post(delete, **kw)
+        super().handle_post(delete, **kw)
         if delete:
             session(self.artifact.attachment_class()).flush()
             # self.artifact.attachments is ming's LazyProperty, we need to reset
@@ -1657,13 +1661,14 @@ class AttachmentController(att.AttachmentController):
 class AttachmentsController(att.AttachmentsController):
     AttachmentControllerClass = AttachmentController
 
+
 NONALNUM_RE = re.compile(r'\W+')
 
 
 class TrackerAdminController(DefaultAdminController):
 
     def __init__(self, app):
-        super(TrackerAdminController, self).__init__(app)
+        super().__init__(app)
         self.bins = BinController(app=self.app)
         # if self.app.globals and self.app.globals.milestone_names is None:
         #     self.app.globals.milestone_names = ''
@@ -1679,8 +1684,8 @@ class TrackerAdminController(DefaultAdminController):
     @expose('jinja:forgetracker:templates/tracker/admin_fields.html')
     def fields(self, **kw):
         c.form = W.field_admin
-        columns = dict((column, get_label(column))
-                       for column in self.app.globals['show_in_search'].keys())
+        columns = {column: get_label(column)
+                       for column in self.app.globals['show_in_search'].keys()}
         return dict(app=self.app, globals=self.app.globals, columns=columns)
 
     @expose('jinja:forgetracker:templates/tracker/admin_options.html')
@@ -1702,20 +1707,24 @@ class TrackerAdminController(DefaultAdminController):
     @validate(W.options_admin, error_handler=options)
     def set_options(self, **kw):
         require_access(self.app, 'configure')
-        for k, v in kw.iteritems():
-            self.app.config.options[k] = v
+        mount_point = self.app.config.options['mount_point']
+        for k, val in kw.items():
+            if self.app.config.options.get(k) != val:
+                M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                    mount_point, k, self.app.config.options.get(k), bool(val)))
+                self.app.config.options[k] = val
         flash('Options updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     @require_post()
     def allow_default_field(self, **post_data):
-        for column in self.app.globals['show_in_search'].keys():
+        for column in list(self.app.globals['show_in_search'].keys()):
             if post_data.get(column) == 'on':
                 self.app.globals['show_in_search'][column] = True
             else:
                 self.app.globals['show_in_search'][column] = False
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose()
     def update_tickets(self, **post_data):
@@ -1736,10 +1745,10 @@ class TrackerAdminController(DefaultAdminController):
             if field['type'] == 'milestone':
                 field.setdefault('milestones', [])
 
-        existing_milestone_fld_names = set(
-            mf.name for mf in self.app.globals.milestone_fields)
-        posted_milestone_fld_names = set(
-            cf['name'] for cf in custom_fields if cf['type'] == 'milestone')
+        existing_milestone_fld_names = {
+            mf.name for mf in self.app.globals.milestone_fields}
+        posted_milestone_fld_names = {
+            cf['name'] for cf in custom_fields if cf['type'] == 'milestone'}
         deleted_milestone_fld_names = existing_milestone_fld_names -\
             posted_milestone_fld_names
         added_milestone_fld_names = posted_milestone_fld_names -\
@@ -1774,13 +1783,13 @@ class TrackerAdminController(DefaultAdminController):
                     cf for cf in custom_fields
                     if cf['type'] == 'milestone'
                     and cf['name'] == milestone_fld_name][0]
-                existing_milestone_names = set(
+                existing_milestone_names = {
                     m.name for m in
-                    existing_milestone_fld.get('milestones', []))
-                old_posted_milestone_names = set(
+                    existing_milestone_fld.get('milestones', [])}
+                old_posted_milestone_names = {
                     m['old_name']
                     for m in posted_milestone_fld.get('milestones', [])
-                    if m.get('old_name', None))
+                    if m.get('old_name', None)}
                 deleted_milestone_names = existing_milestone_names -\
                     old_posted_milestone_names
 
@@ -1808,7 +1817,7 @@ class TrackerAdminController(DefaultAdminController):
 
         self.app.globals.custom_fields = custom_fields
         flash('Fields updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class RootRestController(BaseController, AppRestControllerMixin):
@@ -1846,7 +1855,7 @@ class RootRestController(BaseController, AppRestControllerMixin):
         require_access(c.app, 'create')
         if TM.Ticket.is_limit_exceeded(c.app.config, user=c.user):
             msg = 'Ticket creation rate limit exceeded. '
-            log.warn(msg + c.app.config.url())
+            log.warning(msg + c.app.config.url())
             raise forge_exc.HTTPTooManyRequests()
         if c.app.globals.milestone_names is None:
             c.app.globals.milestone_names = ''
@@ -1866,7 +1875,7 @@ class RootRestController(BaseController, AppRestControllerMixin):
 
         results = TM.Ticket.paged_search(
             c.app.config, c.user, q, limit, page, sort, show_deleted=False)
-        results['tickets'] = map(_convert_ticket, results['tickets'])
+        results['tickets'] = list(map(_convert_ticket, results['tickets']))
         return results
 
     @expose()
@@ -1923,7 +1932,7 @@ class MilestoneController(BaseController):
         else:
             raise exc.HTTPNotFound()
         for m in fld.milestones:
-            if m.name == unquote(milestone).decode('utf-8'):
+            if m.name == six.ensure_text(unquote(milestone)):
                 break
         else:
             raise exc.HTTPNotFound()
@@ -1931,9 +1940,9 @@ class MilestoneController(BaseController):
         self.field = fld
         self.milestone = m
         escaped_name = escape_solr_arg(m.name)
-        self.progress_key = '%s:%s' % (fld.name, escaped_name)
+        self.progress_key = f'{fld.name}:{escaped_name}'
         self.mongo_query = {'custom_fields.%s' % fld.name: m.name}
-        self.solr_query = '%s:%s' % (_mongo_col_to_solr_col(fld.name), escaped_name)
+        self.solr_query = f'{_mongo_col_to_solr_col(fld.name)}:{escaped_name}'
 
     @with_trailing_slash
     @h.vardec
@@ -1941,7 +1950,7 @@ class MilestoneController(BaseController):
     @validate(validators=dict(
         limit=validators.Int(if_invalid=None),
         page=validators.Int(if_empty=0, if_invalid=0),
-        sort=validators.UnicodeString(if_empty=''),
+        sort=v.UnicodeString(if_empty=''),
         filter=V.JsonConverter(if_empty={}),
         deleted=validators.StringBool(if_empty=False)))
     def index(self, q=None, columns=None, page=0, query=None, sort=None,
