@@ -19,7 +19,7 @@ import pkg_resources
 import unittest
 from datetime import datetime, timedelta
 
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 from tg import config
 import mock
 
@@ -38,7 +38,7 @@ with_git = td.with_tool('test', 'Git', 'git-userstats-model', 'Git', type='git')
 
 class TestUserStats(unittest.TestCase):
 
-    def setUp(self):
+    def setup_method(self, method):
         setup_basic_test()
         setup_global_objects()
         self.user = User.by_username('test-user-2')
@@ -441,12 +441,12 @@ class TestUserStats(unittest.TestCase):
                    login_datetime) < timedelta(seconds=1)
 
     def test_start_date(self):
-        stats = USM.UserStats(registration_date=datetime(2012, 04, 01))
-        self.assertEqual(stats.start_date, datetime(2012, 04, 01))
+        stats = USM.UserStats(registration_date=datetime(2012, 4, 1))
+        self.assertEqual(stats.start_date, datetime(2012, 4, 1))
         with h.push_config(config, **{'userstats.start_date': '2013-04-01'}):
-            self.assertEqual(stats.start_date, datetime(2013, 04, 01))
+            self.assertEqual(stats.start_date, datetime(2013, 4, 1))
         with h.push_config(config, **{'userstats.start_date': '2011-04-01'}):
-            self.assertEqual(stats.start_date, datetime(2012, 04, 01))
+            self.assertEqual(stats.start_date, datetime(2012, 4, 1))
 
     @mock.patch('allura.model.stats.difflib.unified_diff')
     def test_count_loc(self, unified_diff):
@@ -461,12 +461,11 @@ class TestUserStats(unittest.TestCase):
         )
         unified_diff.return_value = ['+++', '---', '+line']
         newcommit.tree.get_blob_by_path.return_value = mock.MagicMock()
-        newcommit.tree.get_blob_by_path.return_value.__iter__.return_value = [
-            'one']
-        newcommit.repo.commit(
-        ).tree.get_blob_by_path.return_value = mock.MagicMock()
-        newcommit.repo.commit().tree.get_blob_by_path.return_value.__iter__.return_value = [
-            'two']
+        newcommit.tree.get_blob_by_path.return_value.__iter__.return_value = ['one']
+        newcommit.tree.get_blob_by_path.return_value.path.return_value = 'onepath'
+        newcommit.repo.commit().tree.get_blob_by_path.return_value = mock.MagicMock()
+        newcommit.repo.commit().tree.get_blob_by_path.return_value.__iter__.return_value = ['two']
+        newcommit.repo.commit().tree.get_blob_by_path.return_value.path.return_value = 'twopath'
         commit_datetime = datetime.utcnow()
         project = mock.Mock(
             trove_topic=[],
