@@ -15,30 +15,33 @@
 #       specific language governing permissions and limitations
 #       under the License.
 
-#-*- python -*-
 import json
 import logging
 import os
 from pprint import pformat
-from urllib import unquote
+
+import six
+from urllib.parse import unquote, urlencode
 
 # Non-stdlib imports
 from tg import expose, validate, redirect, flash, jsonify
 from tg.decorators import with_trailing_slash, without_trailing_slash
-from pylons import tmpl_context as c, app_globals as g
-from pylons import request
+from tg import tmpl_context as c, app_globals as g
+from tg import request
 from formencode import validators
 from webob import exc
-from ming.orm import session
+from ming.odm import session
 
 # Pyforge-specific imports
 from allura import model as M
 from allura.lib import helpers as h
+from allura.lib import validators as v
 from allura.app import Application, SitemapEntry, DefaultAdminController, ConfigOption
 from allura.lib.search import search_app
-from allura.lib.decorators import require_post
+from allura.lib.decorators import require_post, memorable_forget
 from allura.lib.security import require_access, has_access
-from allura.lib.utils import is_ajax, JSONForExport
+from allura.lib.utils import is_ajax, JSONForExport, permanent_redirect
+from allura.tasks import notification_tasks
 from allura.lib import exceptions as forge_exc
 from allura.controllers import AppDiscussionController, BaseController, AppDiscussionRestController
 from allura.controllers import DispatchIndex
@@ -73,7 +76,12 @@ class W:
     page_list = ffw.PageList()
     page_size = ffw.PageSize()
     search_results = SearchResults()
-    help_modal = SearchHelp()
+    help_modal = SearchHelp(fields={'title': 'title',
+                                    'text': '"Page text"',
+                                    'mod_date_dt': 'Last modified.  Example: mod_date_dt:[2018-01-01T00:00:00Z TO *]',
+                                    'labels_t': 'labels',
+                                    'author_user_name_t': 'username (for comments only)',
+                                    })
     icons = {
         24: 'images/wiki_24.png',
         32: 'images/wiki_32.png',
@@ -107,7 +115,7 @@ class ForgeWikiApp(Application):
     default_mount_label = 'Wiki'
     default_mount_point = 'wiki'
     ordinal = 5
-    default_root_page_name = u'Home'
+    default_root_page_name = 'Home'
     icons = {
         24: 'images/wiki_24.png',
         32: 'images/wiki_32.png',
@@ -127,11 +135,13 @@ class ForgeWikiApp(Application):
         log.info('Message from %s (%s)',
                  topic, self.config.options.mount_point)
         log.info('Headers are: %s', message['headers'])
-        try:
-            page = WM.Page.upsert(topic)
-        except:
+        page = WM.Page.find_page(topic)
+        if page is None:
+            page = WM.Page.find_page(topic.replace('_', ' '))
+        if page is not None:
+            self.handle_artifact_message(page, message)
+        else:
             log.exception('Error getting artifact %s', topic)
-        self.handle_artifact_message(page, message)
 
     @property
     def root_page_name(self):
@@ -196,13 +206,25 @@ The wiki uses [Markdown](%s) syntax.
         self.config.options['AllowEmailPosting'] = bool(show)
 
     def main_menu(self):
-        '''Apps should provide their entries to be added to the main nav
+        """Apps should provide their entries to be added to the main nav
         :return: a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
-        '''
+        """
         return [SitemapEntry(
             self.config.options.mount_label,
             '.')]
 
+    def sitemap_xml(self):
+        """
+        Used for generating sitemap.xml.
+        If the root page has default content, omit it from the sitemap.xml.
+        Assumes :attr:`main_menu` will return an entry pointing to the root page.
+        :return: a list of :class:`SitemapEntries <allura.app.SitemapEntry>`
+        """
+        root_page = WM.Page.query.get(app_config_id=self.config._id, title=self.root_page_name)
+        if self.should_noindex_page(root_page):
+            return []
+        return self.main_menu()
+
     @property
     @h.exceptionless([], log)
     def sitemap(self):
@@ -216,6 +238,21 @@ The wiki uses [Markdown](%s) syntax.
             return [
                 SitemapEntry(menu_id, '.')[SitemapEntry('Pages')[pages]]]
 
+    def should_noindex_page(self, page: WM.Page) -> bool:
+        """Checks whether a page should not be indexed."""
+        if not page:
+            # this shouldn't happen; just a safeguard for using `page` below
+            return False
+        elif (page['title'] == self.default_root_page_name and page['version'] == 1) \
+                or page['text'] in ('You can edit this description', ):
+            # If page has default name (i.e. 'Home') and has not been edited, noindex.
+            # or old default text
+            # but not if comments are visible
+            visible_comments = page.discussion_thread.find_posts(status='ok', limit=1)
+            return not visible_comments
+        else:
+            return False
+
     def create_common_wiki_menu(self, has_create_access, admin_menu=False):
         links = []
         if has_create_access:
@@ -249,7 +286,8 @@ The wiki uses [Markdown](%s) syntax.
             links.append(SitemapEntry(subscribe_title, subscribe_url, ui_icon=g.icons['mail']))
         if not admin_menu:
             links += [SitemapEntry(''),
-                      SitemapEntry('Formatting Help', self.url + 'markdown_syntax/')]
+                      SitemapEntry('Formatting Help', '/nf/markdown_syntax',
+                                   extra_html_attrs={'target': '_blank', 'rel': 'nofollow'})]
         return links
 
     def admin_menu(self, skip_common_menu=False):
@@ -259,8 +297,7 @@ The wiki uses [Markdown](%s) syntax.
 
         if not self.show_left_bar and not skip_common_menu:
             links += self.create_common_wiki_menu(has_create_access=True, admin_menu=True)
-        links += super(ForgeWikiApp, self).admin_menu(force_options=True)
-
+        links += super().admin_menu(force_options=True)
         return links
 
     @h.exceptionless([], log)
@@ -300,7 +337,7 @@ The wiki uses [Markdown](%s) syntax.
     def install(self, project):
         'Set up any default permissions and roles here'
         self.config.options['project_name'] = project.name
-        super(ForgeWikiApp, self).install(project)
+        super().install(project)
         # Setup permissions
         role_admin = M.ProjectRole.by_name('Admin')._id
         role_developer = M.ProjectRole.by_name('Developer')._id
@@ -328,16 +365,15 @@ The wiki uses [Markdown](%s) syntax.
         if p is None:
             with h.push_config(c, app=self), h.notifications_disabled(c.project, disabled=not notify):
                 p = WM.Page.upsert(new_root)
-                p.viewable_by = ['all']
                 p.text = self.default_root_page_text()
                 p.commit()
 
     def uninstall(self, project):
         "Remove all the tool's artifacts from the database"
-        WM.WikiAttachment.query.remove(dict(app_config_id=self.config._id))
-        WM.Page.query.remove(dict(app_config_id=self.config._id))
+        for page in WM.Page.query.find(dict(app_config_id=self.config._id)):
+            page.delete()
         WM.Globals.query.remove(dict(app_config_id=self.config._id))
-        super(ForgeWikiApp, self).uninstall(project)
+        super().uninstall(project)
 
     def bulk_export(self, f, export_path='', with_attachments=False):
         f.write('{"pages": [')
@@ -348,7 +384,7 @@ The wiki uses [Markdown](%s) syntax.
             GenericClass = JSONForExport
             self.export_attachments(pages, export_path)
         else:
-            GenericClass = jsonify.GenericJSON
+            GenericClass = jsonify.JSONEncoder
         for i, page in enumerate(pages):
             if i > 0:
                 f.write(',')
@@ -374,13 +410,17 @@ class RootController(BaseController, DispatchIndex, FeedController):
     def __init__(self):
         self._discuss = AppDiscussionController()
 
+    def catch_all(self, *args, **kw):
+        url = f'/{request.controller_url}/?{urlencode(kw)}'
+        redirect(h.urlquote(h.really_unicode(url)))
+
     def _check_security(self):
         require_access(c.app, 'read')
 
     @with_trailing_slash
     @expose()
     def index(self, **kw):
-        redirect(h.really_unicode(c.app.root_page_name).encode('utf-8') + '/')
+        permanent_redirect(h.urlquote(h.really_unicode(c.app.root_page_name) + '/'))
 
     @expose()
     def _lookup(self, pname, *remainder):
@@ -389,11 +429,11 @@ class RootController(BaseController, DispatchIndex, FeedController):
 
     @expose()
     def new_page(self, title):
-        redirect(h.really_unicode(title).encode('utf-8') + '/')
+        redirect(h.urlquote(h.really_unicode(title) + '/'))
 
     @with_trailing_slash
     @expose('jinja:forgewiki:templates/wiki/search.html')
-    @validate(dict(q=validators.UnicodeString(if_empty=None),
+    @validate(dict(q=v.UnicodeString(if_empty=None),
                    history=validators.StringBool(if_empty=False),
                    search_comments=validators.StringBool(if_empty=False),
                    project=validators.StringBool(if_empty=False)))
@@ -415,10 +455,11 @@ class RootController(BaseController, DispatchIndex, FeedController):
 
     @with_trailing_slash
     @expose('jinja:forgewiki:templates/wiki/browse.html')
-    @validate(dict(sort=validators.UnicodeString(if_empty='alpha'),
+    @validate(dict(sort=v.UnicodeString(if_empty='alpha'),
                    show_deleted=validators.StringBool(if_empty=False),
                    page=validators.Int(if_empty=0, if_invalid=0),
-                   limit=validators.Int(if_empty=None, if_invalid=None)))
+                   limit=validators.Int(if_empty=None, if_invalid=None)),
+              error_handler=catch_all)
     def browse_pages(self, sort='alpha', show_deleted=False, page=0, limit=None, **kw):
         'list of all pages in the wiki'
         c.page_list = W.page_list
@@ -453,13 +494,14 @@ class RootController(BaseController, DispatchIndex, FeedController):
         if sort == 'recent':
             pages.sort(reverse=True, key=lambda x: (x['updated']))
             pages = pages + uv_pages
+        h1_text = f"{c.project.name} {c.app.config.options.mount_label} - Browse Pages"
         return dict(
             pages=pages, can_delete=can_delete, show_deleted=show_deleted,
-            limit=limit, count=count, page=pagenum)
+            limit=limit, count=count, page=pagenum, h1_text=h1_text)
 
     @with_trailing_slash
     @expose('jinja:forgewiki:templates/wiki/browse_tags.html')
-    @validate(dict(sort=validators.UnicodeString(if_empty='alpha'),
+    @validate(dict(sort=v.UnicodeString(if_empty='alpha'),
                    page=validators.Int(if_empty=0, if_invalid=0),
                    limit=validators.Int(if_empty=None, if_invalid=None)))
     def browse_tags(self, sort='alpha', page=0, limit=None, **kw):
@@ -479,30 +521,23 @@ class RootController(BaseController, DispatchIndex, FeedController):
                         page_tags[label] = []
                     page_tags[label].append(page)
         count = len(page_tags)
-        name_labels = list(page_tags)
-        name_labels.sort()
+        name_labels = sorted(page_tags)
+        h1_text = f"{c.project.name} {c.app.config.options.mount_label} - Browse Labels"
         return dict(labels=page_tags,
                     limit=limit,
                     count=count,
                     page=pagenum,
-                    name_labels=name_labels[start:start + limit])
+                    name_labels=name_labels[start:start + limit],
+                    h1_text=h1_text)
 
     @with_trailing_slash
     @expose('jinja:forgewiki:templates/wiki/create_page.html')
     def create_wiki_page(self, **kw):
         return {}
 
-    @with_trailing_slash
-    @expose('jinja:allura:templates/markdown_syntax.html')
+    @expose()
     def markdown_syntax(self, **kw):
-        'Display a page about how to use markdown.'
-        return dict(example=MARKDOWN_EXAMPLE)
-
-    @with_trailing_slash
-    @expose('jinja:allura:templates/markdown_syntax_dialog.html')
-    def markdown_syntax_dialog(self, **kw):
-        'Display a page about how to use markdown.'
-        return dict(example=MARKDOWN_EXAMPLE)
+        permanent_redirect('/nf/markdown_syntax')
 
     @expose()
     @require_post()
@@ -512,7 +547,7 @@ class RootController(BaseController, DispatchIndex, FeedController):
             M.Mailbox.subscribe(type='direct')
         elif unsubscribe:
             M.Mailbox.unsubscribe()
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
 
 class PageController(BaseController, FeedController):
@@ -530,17 +565,10 @@ class PageController(BaseController, FeedController):
             if self.page.deleted:
                 require_access(self.page, 'delete')
         elif has_access(c.app, 'create'):
-            self.rate_limit()
+            self.rate_limit(WM.Page, 'Page create/edit')
         else:
             raise exc.HTTPNotFound
 
-    def rate_limit(self):
-        if WM.Page.is_limit_exceeded(c.app.config, user=c.user):
-            msg = 'Page create/edit rate limit exceeded. '
-            log.warn(msg + c.app.config.url())
-            flash(msg + 'Please try again later.', 'error')
-            redirect('..')
-
     def fake_page(self):
         return dict(
             title=self.title,
@@ -587,8 +615,6 @@ class PageController(BaseController, FeedController):
                 redirect('.?version=%d' % (version - 1))
             else:
                 redirect('.')
-        elif 'all' not in page.viewable_by and c.user.username not in page.viewable_by:
-            raise exc.HTTPForbidden(detail="You may not view this page.")
         cur = page.version
         if cur > 1:
             prev = cur - 1
@@ -597,12 +623,16 @@ class PageController(BaseController, FeedController):
         next = cur + 1
         hide_left_bar = not (c.app.show_left_bar)
         subscribed_to_page = M.Mailbox.subscribed(artifact=self.page)
+        latest_version = self.page.history().limit(1).first()
+        is_latest_version = cur == latest_version.version
         return dict(
             page=page,
             cur=cur, prev=prev, next=next,
             page_subscribed=subscribed_to_page,
             hide_left_bar=hide_left_bar, show_meta=c.app.show_right_bar,
-            pagenum=pagenum, limit=limit, count=post_count)
+            pagenum=pagenum, limit=limit, count=post_count,
+            noindex=c.app.should_noindex_page(self.page),
+            is_latest_version=is_latest_version,)
 
     @without_trailing_slash
     @expose('jinja:forgewiki:templates/wiki/page_edit.html')
@@ -613,22 +643,29 @@ class PageController(BaseController, FeedController):
             page = self.page
         else:
             page = self.fake_page()
-        self.rate_limit()  # check before trying to save
+        self.rate_limit(WM.Page, 'Page create/edit')  # check before trying to save
         c.confirmation = W.confirmation
         c.markdown_editor = W.markdown_editor
         c.attachment_add = W.attachment_add
         c.attachment_list = W.attachment_list
         c.label_edit = W.label_edit
         hide_left_bar = not c.app.show_left_bar
-        return dict(page=page, page_exists=page_exists,
-                    hide_left_bar=hide_left_bar)
+        return dict(page=page,
+                    page_exists=page_exists,
+                    hide_left_bar=hide_left_bar,
+                    subscribed_to_tool=M.Mailbox.subscribed(),
+                    )
 
     @without_trailing_slash
     @expose('json:')
     @require_post()
     def delete(self, **kw):
         require_access(self.page, 'delete')
-        self.page.delete()
+        M.AuditLog.log('{}: deleted wiki page "{}"'.format(
+            c.app.config.options['mount_point'],
+            self.page.title,
+        ))
+        self.page.soft_delete()
         return dict(location='../' + self.page.title + '/?deleted=True')
 
     @without_trailing_slash
@@ -637,6 +674,10 @@ class PageController(BaseController, FeedController):
     def undelete(self, **kw):
         require_access(self.page, 'delete')
         self.page.deleted = False
+        M.AuditLog.log('{}: undeleted wiki page "{}"'.format(
+            c.app.config.options['mount_point'],
+            self.page.title,
+        ))
         M.Shortlink.from_artifact(self.page)
         return dict(location='./edit')
 
@@ -706,24 +747,28 @@ class PageController(BaseController, FeedController):
         self.page.commit()
         return dict(location='.')
 
+    @memorable_forget()
     @without_trailing_slash
     @h.vardec
     @expose()
     @require_post()
     def update(self, title=None, text=None,
                labels=None,
-               viewable_by=None,
-               new_viewable_by=None, **kw):
+               subscribe=False,
+               **kw):
         activity_verb = 'created'
         if not title:
             flash('You must provide a title for the page.', 'error')
             redirect('edit')
         title = title.replace('/', '-')
-        self.rate_limit()
+        self.rate_limit(WM.Page, 'Page create/edit')
         if not self.page:
             # the page doesn't exist yet, so create it
             self.page = WM.Page.upsert(self.title)
-            self.page.viewable_by = ['all']
+            M.AuditLog.log('{}: created new wiki page "{}"'.format(
+                c.app.config.options['mount_point'],
+                self.page.title,
+            ))
         else:
             require_access(self.page, 'edit')
             activity_verb = 'modified'
@@ -737,36 +782,51 @@ class PageController(BaseController, FeedController):
                 if self.page.title == c.app.root_page_name:
                     WM.Globals.query.get(
                         app_config_id=c.app.config._id).root = title
+                M.AuditLog.log('{}: renamed wiki page "{}" => "{}"'.format(
+                    c.app.config.options['mount_point'],
+                    self.page.title,
+                    title,
+                ))
                 self.page.title = title
                 activity_verb = 'renamed'
+        old_text = self.page.text
         self.page.text = text
         if labels:
             self.page.labels = labels.split(',')
         else:
             self.page.labels = []
-        self.page.commit()
-        g.spam_checker.check(title + u'\n' + text, artifact=self.page,
+        self.page.commit(subscribe=subscribe)
+        g.spam_checker.check(title + '\n' + text, artifact=self.page,
                              user=c.user, content_type='wiki')
+        if activity_verb == 'created':
+            notification_tasks.send_usermentions_notification.post(self.page.index_id(), text)
+        elif activity_verb == 'modified':
+            notification_tasks.send_usermentions_notification.post(self.page.index_id(), text, old_text)
         g.director.create_activity(c.user, activity_verb, self.page,
                                    related_nodes=[c.project], tags=['wiki'])
-        if new_viewable_by:
-            if new_viewable_by == 'all':
-                self.page.viewable_by.append('all')
-            else:
-                user = c.project.user_in_project(str(new_viewable_by))
-                if user:
-                    self.page.viewable_by.append(user.username)
-        if viewable_by:
-            for u in viewable_by:
-                if u.get('delete'):
-                    if u['id'] == 'all':
-                        self.page.viewable_by.remove('all')
-                    else:
-                        user = M.User.by_username(str(u['id']))
-                        if user:
-                            self.page.viewable_by.remove(user.username)
-        redirect('../' + h.really_unicode(self.page.title)
-                 .encode('utf-8') + ('/' if not name_conflict else '/edit'))
+        redirect('../' + h.urlquote(h.really_unicode(self.page.title)) + ('/' if not name_conflict else '/edit'))
+
+    @without_trailing_slash
+    @expose('json:')
+    @require_post()
+    def update_markdown(self, text=None, **kw):
+        if has_access(self.page, 'edit'):
+            self.page.text = text
+            self.page.commit()
+            g.spam_checker.check(text, artifact=self.page, user=c.user, content_type='wiki')
+            g.director.create_activity(c.user, 'modified', self.page, related_nodes=[c.project], tags=['wiki'])
+            return {
+                'status': 'success'
+            }
+        else:
+            return {
+                'status': 'no_permission'
+            }
+
+    @without_trailing_slash
+    @expose()
+    def get_markdown(self):
+        return self.page.text
 
     @without_trailing_slash
     @expose()
@@ -778,7 +838,7 @@ class PageController(BaseController, FeedController):
         self.page.add_multiple_attachments(file_info)
         if is_ajax(request):
             return
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
 
     @expose('json:')
     @require_post()
@@ -806,15 +866,6 @@ class WikiAttachmentController(ac.AttachmentController):
 class WikiAttachmentsController(ac.AttachmentsController):
     AttachmentControllerClass = WikiAttachmentController
 
-MARKDOWN_EXAMPLE = '''
-# First-level heading
-
-Some *emphasized* and **strong** text
-
-#### Fourth-level heading
-
-'''
-
 
 class RootRestController(BaseController, AppRestControllerMixin):
 
@@ -867,7 +918,7 @@ class PageRestController(BaseController):
             if not self.page:
                 require_access(c.app, 'create')
                 if WM.Page.is_limit_exceeded(c.app.config, user=c.user):
-                    log.warn('Page create/edit rate limit exceeded. %s',
+                    log.warning('Page create/edit rate limit exceeded. %s',
                              c.app.config.url())
                     raise forge_exc.HTTPTooManyRequests()
                 self.page = WM.Page.upsert(title)
@@ -907,23 +958,46 @@ class WikiAdminController(DefaultAdminController):
     @expose()
     @require_post()
     def set_home(self, new_home):
+        old_home = self.app.root_page_name
         self.app.root_page_name = new_home
         self.app.upsert_root(new_home)
         flash('Home updated')
         mount_base = c.project.url() + \
             self.app.config.options.mount_point + '/'
-        url = h.really_unicode(mount_base).encode('utf-8') + \
-            h.really_unicode(new_home).encode('utf-8') + '/'
-        redirect(url)
+        url = h.really_unicode(mount_base) + h.really_unicode(new_home) + '/'
+        M.AuditLog.log('{}: set home page "{}" => "{}"'.format(
+            self.app.config.options['mount_point'],
+            old_home,
+            new_home,
+        ))
+        redirect(h.urlquote(url))
 
     @without_trailing_slash
     @expose()
     @require_post()
     def set_options(self, show_discussion=False, show_left_bar=False, show_right_bar=False,
                     allow_email_posting=False):
-        self.app.show_discussion = show_discussion
-        self.app.show_left_bar = show_left_bar
-        self.app.show_right_bar = show_right_bar
-        self.app.allow_email_posting = allow_email_posting
+        mount_point = self.app.config.options['mount_point']
+
+        if self.app.show_discussion != bool(show_discussion):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "Show Discussion", self.app.show_discussion, bool(show_discussion)))
+            self.app.show_discussion = show_discussion
+
+        if self.app.show_left_bar != bool(show_left_bar):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "Show left Bar", self.app.show_left_bar, bool(show_left_bar)))
+            self.app.show_left_bar = show_left_bar
+
+        if self.app.show_right_bar != bool(show_right_bar):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "Show metadata", self.app.show_right_bar, bool(show_right_bar)))
+            self.app.show_right_bar = show_right_bar
+
+        if self.app.allow_email_posting != bool(allow_email_posting):
+            M.AuditLog.log('{}: set option "{}" {} => {}'.format(
+                mount_point, "Allow posting replies via email", self.app.allow_email_posting, bool(allow_email_posting)))
+            self.app.allow_email_posting = allow_email_posting
+
         flash('Wiki options updated')
-        redirect(request.referer)
+        redirect(six.ensure_text(request.referer or '/'))
