@@ -17,9 +17,9 @@
 
 import logging
 
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 
-from ming.orm import ThreadLocalORMSession
+from ming.odm import ThreadLocalODMSession
 
 from forgewiki import model as WM
 
@@ -28,12 +28,12 @@ log = logging.getLogger(__name__)
 
 def main():
     c.project = None
-    pages = WM.Page.query.find({'title': {'$regex': '\/'}}).all()
-    print 'Found %s wiki titles containing "/"...' % len(pages)
+    pages = WM.Page.query.find({'title': {'$regex': r'\/'}}).all()
+    print('Found %s wiki titles containing "/"...' % len(pages))
     for page in pages:
         page.title = page.title.replace('/', '-')
-        print 'Updated: %s' % page.title
-    ThreadLocalORMSession.flush_all()
+        print('Updated: %s' % page.title)
+    ThreadLocalODMSession.flush_all()
 
 if __name__ == '__main__':
     main()
