@@ -17,7 +17,7 @@
 
 import logging
 
-from ming.odm import ThreadLocalORMSession, state
+from ming.odm import ThreadLocalODMSession, state
 
 from allura.lib import utils
 from allura import model as M
@@ -28,12 +28,12 @@ log = logging.getLogger(__name__)
 def main():
     for chunk in utils.chunked_find(M.User):
         for user in chunk:
-            print 'Processing {0}'.format(user.username)
+            print(f'Processing {user.username}')
             user.pending = False
             # Ming doesn't mark document for update, since pending is False
             # by default, even if field is missing from mongo
             state(user).status = state(user).dirty
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
 
 if __name__ == '__main__':
     main()
