@@ -17,7 +17,7 @@
 
 import logging
 
-from ming.odm import ThreadLocalORMSession, state
+from ming.odm import ThreadLocalODMSession, state
 
 from allura.lib import utils
 from allura import model as M
@@ -29,10 +29,10 @@ def main():
     for chunk in utils.chunked_find(M.MergeRequest):
         for mr in chunk:
             try:
-                print 'Processing {0}'.format(mr.url())
+                print(f'Processing {mr.url()}')
                 mr.subscribe(user=mr.creator)
-                ThreadLocalORMSession.flush_all()
-            except:
+                ThreadLocalODMSession.flush_all()
+            except Exception:
                 log.exception('Error on %s', mr)
 
 
