@@ -18,29 +18,31 @@
 import logging
 from getpass import getpass
 from optparse import OptionParser
-from pylons import tmpl_context as c
+from tg import tmpl_context as c
 import re
 import os
 from time import mktime
 import time
 import json
-from urlparse import urlparse
-import urllib2
-import urllib
-from cookielib import CookieJar
+from urllib.parse import urlparse
+import six.moves.urllib.request
+import six.moves.urllib.parse
+import six.moves.urllib.error
+from http.cookiejar import CookieJar
 from datetime import datetime
-from ConfigParser import ConfigParser
+from configparser import ConfigParser
 import random
 import string
 
 import sqlalchemy
 from suds.client import Client
-from ming.orm.ormsession import ThreadLocalORMSession
+from ming.odm.odmsession import ThreadLocalODMSession
 from ming.base import Object
 
 from allura import model as M
 from allura.lib import helpers as h
 from allura.lib import utils
+import six
 
 log = logging.getLogger('teamforge-import')
 
@@ -58,7 +60,7 @@ client = None  # main api client
 users = {}
 
 cj = CookieJar()
-loggedInOpener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cj))
+loggedInOpener = six.moves.urllib.request.build_opener(six.moves.urllib.request.HTTPCookieProcessor(cj))
 
 
 def make_client(api_url, app):
@@ -117,7 +119,7 @@ def main():
             return
         try:
             nbhd = M.Neighborhood.query.get(name=options.neighborhood)
-        except:
+        except Exception:
             log.exception('error querying mongo')
             log.error(
                 'This should be run as "paster script production.ini ../scripts/teamforge-import.py -- ...options.."')
@@ -132,7 +134,7 @@ def main():
         project_ids = [p.id for p in projects.dataRows]
 
     if options.list_project_ids:
-        print ' '.join(project_ids)
+        print(' '.join(project_ids))
         return
 
     if not os.path.exists(options.output_dir):
@@ -155,15 +157,15 @@ def main():
                 get_news(project)
                 if not options.skip_unsupported_check:
                     check_unsupported_tools(project)
-                with open(os.path.join(options.output_dir, 'users.json'), 'w') as user_file:
+                with open(os.path.join(options.output_dir, 'users.json'), 'w', encoding='utf-8') as user_file:
                     json.dump(users, user_file, default=str)
-            except:
+            except Exception:
                 log.exception('Error extracting %s' % pid)
 
         if options.load:
             try:
                 project = create_project(pid, nbhd)
-            except:
+            except Exception:
                 log.exception('Error creating %s' % pid)
 
 
@@ -178,7 +180,7 @@ def load_users():
 
 
 def save_user(usernames):
-    if isinstance(usernames, basestring):
+    if isinstance(usernames, str):
         usernames = [usernames]
 
     load_users()
@@ -188,8 +190,8 @@ def save_user(usernames):
             user_data = client.service.getUserData(s, username)
             users[username] = Object(user_data)
             if users[username].status != 'Active':
-                log.warn('user: %s status: %s' %
-                         (username, users[username].status))
+                log.warning('user: %s status: %s' %
+                            (username, users[username].status))
 
 
 def get_project(project):
@@ -207,20 +209,20 @@ def get_project(project):
     save(json.dumps(dict(
         data=dict(data),
         access_level=access_level,
-        admins=map(dict, admins),
-        members=map(dict, members),
-        groups=map(dict, groups),
-        categories=map(dict, categories),
+        admins=list(map(dict, admins)),
+        members=list(map(dict, members)),
+        groups=list(map(dict, groups)),
+        categories=list(map(dict, categories)),
     ), default=str),
         project, project.id + '.json')
 
     if len(groups):
-        log.warn('Project has groups %s' % groups)
+        log.warning('Project has groups %s' % groups)
     for u in admins:
         if not u.status != 'active':
-            log.warn('inactive admin %s' % u)
+            log.warning('inactive admin %s' % u)
         if u.superUser:
-            log.warn('super user admin %s' % u)
+            log.warning('super user admin %s' % u)
 
     save_user(data.createdBy)
     save_user(u.userName for u in admins)
@@ -237,7 +239,7 @@ def get_user(orig_username):
         load_users()
         user = users[orig_username]
         if user.status != 'Active':
-            log.warn('Inactive user %s %s' % (orig_username, user.status))
+            log.warning(f'Inactive user {orig_username} {user.status}')
 
         if not 3 <= len(user.fullName) <= 32:
             raise Exception('invalid fullName length: %s' % user.fullName)
@@ -304,23 +306,23 @@ def convert_project_shortname(teamforge_path):
 
 
 # FIXME hardcoded
-skip_perms_usernames = set([
+skip_perms_usernames = {
     'username1', 'username2', 'username3'
-])
+}
 
 
 def create_project(pid, nbhd):
     M.session.artifact_orm_session._get().skip_mod_date = True
     data = loadjson(pid, pid + '.json')
     # pprint(data)
-    log.info('Loading: %s %s %s' % (pid, data.data.title, data.data.path))
+    log.info(f'Loading: {pid} {data.data.title} {data.data.path}')
     shortname = convert_project_shortname(data.data.path)
 
     project = M.Project.query.get(
         shortname=shortname, neighborhood_id=nbhd._id)
     if not project:
         private = (data.access_level == 'private')
-        log.debug('Creating %s private=%s' % (shortname, private))
+        log.debug(f'Creating {shortname} private={private}')
         one_admin = [
             u.userName for u in data.admins if u.status == 'Active'][0]
         project = nbhd.register_project(shortname,
@@ -346,7 +348,7 @@ def create_project(pid, nbhd):
         c.user = user
         pr = M.ProjectRole.by_user(user, project=project, upsert=True)
         pr.roles = [role_admin._id]
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
     role_developer = M.ProjectRole.by_name('Developer', project)
     for member in data.members:
         # FIXME: skip non-active users
@@ -357,7 +359,7 @@ def create_project(pid, nbhd):
         user = get_user(member.userName)
         pr = M.ProjectRole.by_user(user, project=project, upsert=True)
         pr.roles = [role_developer._id]
-        ThreadLocalORMSession.flush_all()
+        ThreadLocalODMSession.flush_all()
     project.labels = [cat.path.split('projects/categorization.root.')[1]
                       for cat in data.categories]
     icon_file = 'emsignia-MOBILITY-red.png'
@@ -365,12 +367,12 @@ def create_project(pid, nbhd):
         icon_file = 'emsignia-SOLUTIONS-blue.gif'
     if project.icon:
         M.ProjectFile.remove(dict(project_id=project._id, category='icon'))
-    with open(os.path.join('..', 'scripts', icon_file)) as fp:
+    with open(os.path.join('..', 'scripts', icon_file), 'rb') as fp:
         M.ProjectFile.save_image(
             icon_file, fp, content_type=utils.guess_mime_type(icon_file),
             square=True, thumbnail_size=(48, 48),
             thumbnail_meta=dict(project_id=project._id, category='icon'))
-    ThreadLocalORMSession.flush_all()
+    ThreadLocalODMSession.flush_all()
 
     dirs = os.listdir(os.path.join(options.output_dir, pid))
 
@@ -386,7 +388,7 @@ def create_project(pid, nbhd):
         import_news(project, pid, frs_mapping, shortname, nbhd)
 
     project.notifications_disabled = False
-    ThreadLocalORMSession.flush_all()
+    ThreadLocalODMSession.flush_all()
     return project
 
 
@@ -459,13 +461,12 @@ def import_wiki(project, pid, nbhd):
                         globals = WM.Globals(
                             app_config_id=wiki_app.config._id, root=page_data.title)
                 p = WM.Page.upsert(page_data.title)
-                p.viewable_by = ['all']
                 p.text = wiki2markdown(content)
                 # upload attachments
                 upload_attachments(p, pid, beginning)
                 if not p.history().first():
                     p.commit()
-    ThreadLocalORMSession.flush_all()
+    ThreadLocalODMSession.flush_all()
 
 
 def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
@@ -487,7 +488,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
         M.ACE.allow(role_developer, 'moderate'),
         M.ACE.allow(role_admin, 'configure'),
         M.ACE.allow(role_admin, 'admin')]
-    ThreadLocalORMSession.flush_all()
+    ThreadLocalODMSession.flush_all()
     DM.Forum.query.remove(
         dict(app_config_id=discuss_app.config._id, shortname='general'))
     forums = os.listdir(os.path.join(options.output_dir, pid, 'forum'))
@@ -543,7 +544,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
                             post_data = loadjson(pid, 'forum',
                                                  forum_name, topic_name, post)
                             p = DM.ForumPost.query.get(
-                                _id='%s%s@import' % (
+                                _id='{}{}@import'.format(
                                     post_name, str(discuss_app.config._id)),
                                 thread_id=to._id,
                                 discussion_id=fo._id,
@@ -551,7 +552,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
 
                             if not p:
                                 p = DM.ForumPost(
-                                    _id='%s%s@import' % (
+                                    _id='{}{}@import'.format(
                                         post_name, str(
                                             discuss_app.config._id)),
                                     thread_id=to._id,
@@ -566,7 +567,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
                                 frs_mapping, sf_project_shortname, post_data.content, nbhd)
                             p.status = 'ok'
                             if post_data.replyToId:
-                                p.parent_id = '%s%s@import' % (
+                                p.parent_id = '{}{}@import'.format(
                                     post_data.replyToId, str(discuss_app.config._id))
                             slug, full_slug = p.make_slugs(
                                 parent=p.parent, timestamp=create_date)
@@ -576,7 +577,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
                                 oldest_post = p
                             if newest_post is None or newest_post.timestamp < create_date:
                                 newest_post = p
-                            ThreadLocalORMSession.flush_all()
+                            ThreadLocalODMSession.flush_all()
                     to.num_replies = to_num_replies
                     to.first_post_id = oldest_post._id
                     to.last_post_date = newest_post.timestamp
@@ -584,7 +585,7 @@ def import_discussion(project, pid, frs_mapping, sf_project_shortname, nbhd):
                     fo_num_posts += to_num_replies
             fo.num_topics = fo_num_topics
             fo.num_posts = fo_num_posts
-            ThreadLocalORMSession.flush_all()
+            ThreadLocalODMSession.flush_all()
 
 
 def import_news(project, pid, frs_mapping, sf_project_shortname, nbhd):
@@ -616,7 +617,7 @@ def import_news(project, pid, frs_mapping, sf_project_shortname, nbhd):
                     p.make_slug()
                 if not p.history().first():
                     p.commit()
-                    ThreadLocalORMSession.flush_all()
+                    ThreadLocalODMSession.flush_all()
                     M.Thread.new(discussion_id=p.app_config.discussion_id,
                                  ref_id=p.index_id(),
                                  subject='%s discussion' % p.title)
@@ -625,7 +626,7 @@ def import_news(project, pid, frs_mapping, sf_project_shortname, nbhd):
                     id=user._id,
                     username=user.username,
                     display_name=user.get_pref('display_name'))
-                ThreadLocalORMSession.flush_all()
+                ThreadLocalODMSession.flush_all()
 
 
 def check_unsupported_tools(project):
@@ -636,35 +637,35 @@ def check_unsupported_tools(project):
             continue
         doc_count += 1
     if doc_count:
-        log.warn('Migrating documents is not supported, but found %s docs' %
-                 doc_count)
+        log.warning('Migrating documents is not supported, but found %s docs' %
+                    doc_count)
 
     scm = make_client(options.api_url, 'ScmApp')
     for repo in scm.service.getRepositoryList(s, project.id).dataRows:
-        log.warn('Migrating SCM repos is not supported, but found %s' %
-                 repo.repositoryPath)
+        log.warning('Migrating SCM repos is not supported, but found %s' %
+                    repo.repositoryPath)
 
     tasks = make_client(options.api_url, 'TaskApp')
     task_count = len(
         tasks.service.getTaskList(s, project.id, filters=None).dataRows)
     if task_count:
-        log.warn('Migrating tasks is not supported, but found %s tasks' %
-                 task_count)
+        log.warning('Migrating tasks is not supported, but found %s tasks' %
+                    task_count)
 
     tracker = make_client(options.api_url, 'TrackerApp')
     tracker_count = len(
         tracker.service.getArtifactList(s, project.id, filters=None).dataRows)
     if tracker_count:
-        log.warn(
+        log.warning(
             'Migrating trackers is not supported, but found %s tracker artifacts' %
             task_count)
 
 
 def load(project_id, *paths):
     in_file = os.path.join(options.output_dir, project_id, *paths)
-    with open(in_file) as input:
+    with open(in_file, encoding='utf-8') as input:
         content = input.read()
-    return unicode(content, 'utf-8')
+    return content
 
 
 def loadjson(*args):
@@ -676,7 +677,7 @@ def save(content, project, *paths):
     out_file = os.path.join(options.output_dir, project.id, *paths)
     if not os.path.exists(os.path.dirname(out_file)):
         os.makedirs(os.path.dirname(out_file))
-    with open(out_file, 'w') as out:
+    with open(out_file, 'w', encoding='utf-8') as out:
         out.write(content.encode('utf-8'))
 
 
@@ -698,7 +699,7 @@ def download_file(tool, url_path, *filepaths):
     else:
         hostname = urlparse(options.api_url).hostname
         scheme = urlparse(options.api_url).scheme
-        url = scheme + '://' + hostname + action_url + urllib.quote(url_path)
+        url = scheme + '://' + hostname + action_url + six.moves.urllib.parse.quote(url_path)
     log.debug('fetching %s' % url)
 
     resp = loggedInOpener.open(url)
@@ -706,16 +707,13 @@ def download_file(tool, url_path, *filepaths):
     # log in to make sure the file should really be html
     if resp.headers.type == 'text/html':
         # log in and save the file
-        resp = loggedInOpener.open(scheme + '://' + hostname + "/sf/sfmain/do/login", urllib.urlencode({
-            'username': options.username,
-            'password': options.password,
-            'returnToUrl': url,
-            'sfsubmit': 'submit'
-        }))
-    with open(out_file, 'w') as out:
+        resp = loggedInOpener.open(scheme + '://' + hostname + "/sf/sfmain/do/login", six.moves.urllib.parse.urlencode(
+            {'username': options.username, 'password': options.password, 'returnToUrl': url, 'sfsubmit': 'submit'}))
+    with open(out_file, 'w', encoding='utf-8') as out:
         out.write(resp.fp.read())
     return out_file
 
+
 bracket_macro = re.compile(r'\[(.*?)\]')
 h1 = re.compile(r'^!!!', re.MULTILINE)
 h2 = re.compile(r'^!!', re.MULTILINE)
@@ -741,7 +739,7 @@ def wiki2markdown(markup):
             return '[[img src=%s]]' % filename
         elif '|' in snippet:
             text, link = snippet.split('|', 1)
-            return '[%s](%s)' % (text, link)
+            return f'[{text}]({link})'
         else:
             # regular link
             return '<%s>' % snippet
@@ -753,6 +751,7 @@ def wiki2markdown(markup):
     markup = re_stats.sub('', markup)
     return markup
 
+
 re_rel = re.compile(r'\b(rel\d+)\b')
 
 
@@ -761,7 +760,7 @@ def convert_post_content(frs_mapping, sf_project_shortname, text, nbhd):
         relno = matchobj.group(1)
         path = frs_mapping.get(relno)
         if path:
-            return '<a href="/projects/%s.%s/files/%s">%s</a>' % (
+            return '<a href="/projects/{}.{}/files/{}">{}</a>'.format(
                 sf_project_shortname, nbhd.url_prefix.strip('/'), path, path)
         else:
             return relno
@@ -848,7 +847,7 @@ def get_homepage_wiki(project):
     elif 'HomePage' in pages:
         homepage = pages.pop('HomePage')
     else:
-        log.warn('did not find homepage')
+        log.warning('did not find homepage')
 
     if homepage:
         save(homepage, project, 'wiki', 'homepage_text.markdown')
@@ -861,7 +860,7 @@ def get_homepage_wiki(project):
             download_file('wiki', img_url, project.id,
                           'wiki', 'homepage', filename)
 
-    for path, text in pages.iteritems():
+    for path, text in pages.items():
         if options.default_wiki_text in text:
             log.debug('skipping default wiki page %s' % path)
         else:
@@ -884,13 +883,13 @@ def _dir_sql(created_on, project, dir_name, rel_path):
         parent_directory = "(SELECT pfs_path FROM pfs_path WHERE path_name = '%s/')" % rel_path
     sql = """
     UPDATE pfs
-      SET file_crtime = '%s'
-      WHERE source_pk = (SELECT project.project FROM project WHERE project.project_name = '%s.%s')
+      SET file_crtime = '{}'
+      WHERE source_pk = (SELECT project.project FROM project WHERE project.project_name = '{}.{}')
       AND source_table = 'project'
       AND pfs_type = 'd'
-      AND pfs_name = '%s'
-      AND parent_directory = %s;
-    """ % (created_on, convert_project_shortname(project.path), options.neighborhood_shortname,
+      AND pfs_name = '{}'
+      AND parent_directory = {};
+    """.format(created_on, convert_project_shortname(project.path), options.neighborhood_shortname,
            dir_name, parent_directory)
     return sql
 
@@ -1022,6 +1021,7 @@ def get_parser(defaults):
 
     return optparser
 
+
 re_username = re.compile(r"^[a-z\-0-9]+$")
 
 
@@ -1059,6 +1059,7 @@ def make_valid_sf_username(orig_username):
         sf_username = adjusted_username
     return sf_username
 
+
 if __name__ == '__main__':
     logging.basicConfig(level=logging.WARN)
     log.setLevel(logging.DEBUG)
@@ -1075,7 +1076,7 @@ def test_make_valid_sf_username():
         'u012345678901234567890': 'u0123456789-mmi',
         'foo^213': 'foo213-mmi'
     }
-    for k, v in tests.iteritems():
+    for k, v in tests.items():
         assert make_valid_sf_username(k) == v
 
 
